/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis;

import org.activityinfo.analysis.map.viewModel.MapResult;
import org.activityinfo.analysis.map.viewModel.MapViewModel;
import org.activityinfo.analysis.pivot.viewModel.FormForest;
import org.activityinfo.analysis.pivot.viewModel.PivotTable;
import org.activityinfo.analysis.pivot.viewModel.PivotViewModel;
import org.activityinfo.model.analysis.AnalysisModelVisitor;
import org.activityinfo.model.analysis.TableAnalysisModel;
import org.activityinfo.model.analysis.map.MapModel;
import org.activityinfo.model.analysis.pivot.PivotModel;
import org.activityinfo.observable.Observable;
import org.activityinfo.store.query.shared.FormSource;

public class AnalysisResultComputer implements AnalysisModelVisitor<Observable<AnalysisResult>> {
    private final FormSource formSource;
    private Observable<FormForest> forest;

    public AnalysisResultComputer(FormSource formSource, Observable<FormForest> forest) {
        this.formSource = formSource;
        this.forest = forest;
    }

    @Override
    public Observable<AnalysisResult> visitPivotModel(PivotModel pivotModel) {

        PivotViewModel pivotViewModel = new PivotViewModel(formSource, forest, Observable.just(pivotModel));

        return pivotViewModel.getResultTable().transform(resultTable -> {
            PivotTable pivotTable = PivotTable.create(resultTable);
            return new PivotResult(pivotModel.getVisualization(), pivotTable);
        });
    }

    @Override
    public Observable<AnalysisResult> visitTable(TableAnalysisModel tableModel) {
        return Observable.loading();
    }

    @Override
    public Observable<AnalysisResult> visitMap(MapModel mapModel) {
        MapViewModel mapViewModel = new MapViewModel(formSource, forest, Observable.just(mapModel));
        return mapViewModel.getLayerResults().transform(layerResults -> {
            MapResult mapResult = new MapResult(layerResults, mapModel.getBaseMap(),
                mapModel.getZoomLevel(), mapModel.getCenter());
            return mapResult;
        });
    }
}
