/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis;

import org.activityinfo.analysis.pivot.viewModel.FormForest;
import org.activityinfo.analysis.pivot.viewModel.PivotTableSource;
import org.activityinfo.model.analysis.ReportSources;
import org.activityinfo.model.formTree.FormTree;
import org.activityinfo.observable.Observable;
import org.activityinfo.store.query.shared.FormSource;

import java.util.List;
import java.util.Optional;
import java.util.logging.Logger;
import java.util.stream.Collectors;

public class FormForestComputer {

    private static final Logger LOGGER = Logger.getLogger(FormForestComputer.class.getName());

    public static Observable<FormForest> compute(FormSource store, ReportSources sources) {

        LOGGER.info("Updating form forest");

        // Build a FormTree for each selected form
        List<Observable<FormTree>> trees = sources.getForms()
                .stream()
                .map(id -> store.getFormTree(id))
                .collect(Collectors.toList());

        Observable<List<FormTree>> formTreeList = Observable.flatten(trees);


        List<Observable<Optional<PivotTableSource>>> tables = sources.getReports().stream()
                .map(id -> {
                    return PivotTableSource.compute(store, id);
                })
                .collect(Collectors.toList());

        Observable<List<Optional<PivotTableSource>>> tableList = Observable.flatten(tables);

        return Observable.transform(formTreeList, tableList, FormForest::new);
    }
}
