/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis;

import org.activityinfo.analysis.pivot.viewModel.PivotTable;
import org.activityinfo.json.Json;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.analysis.pivot.VisualizationType;

public class PivotResult implements AnalysisResult {

    public static final String TYPE = "pivot";

    private final VisualizationType visualization;
    private final PivotTable table;

    public PivotResult(VisualizationType visualization, PivotTable table) {
        this.visualization = visualization;
        this.table = table;
    }

    public VisualizationType getVisualization() {
        return visualization;
    }

    public PivotTable getTable() {
        return table;
    }


    @Override
    public JsonValue toJson() {
        JsonValue object = Json.createObject();
        object.put("type", TYPE);
        object.put("visualization", visualization.name());
        object.put("table", table.toJson());
        return object;
    }


    @Override
    public <T> T accept(AnalysisResultVisitor<T> visitor) {
        return visitor.visitPivot(this);
    }

    public static PivotResult fromJson(JsonValue jsonValue) {
        return new PivotResult(
            VisualizationType.valueOf(jsonValue.getString("visualization")),
            PivotTable.fromJson(jsonValue.get("table")));
    }
}
