/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis;

import org.activityinfo.model.analysis.ReportPermission;
import org.activityinfo.model.analysis.ReportRoleGrant;
import org.activityinfo.model.database.Role;
import org.activityinfo.model.database.VisibleDatabaseTree;

import java.util.Optional;

public class ReportRoleGrantItem {

    private final VisibleDatabaseTree database;
    private final Role role;
    private final ReportPermission permission;
    private final boolean allowed;

    public ReportRoleGrantItem(VisibleDatabaseTree database, Role role, ReportPermission permission, boolean allowed) {
        this.database = database;
        this.role = role;
        this.permission = permission;
        this.allowed = allowed;
    }

    public boolean isAllowed() {
        return allowed;
    }

    public ReportPermission getPermission() {
        return permission;
    }

    public String getDatabaseLabel() {
        return database.getLabel();
    }

    public String getRoleLabel() {
        return role.getLabel();
    }

    public ReportRoleGrant buildGrant(boolean shared) {
        return new ReportRoleGrant(
            database.getDatabaseId(),
            Optional.of(role.getId()),
            shared ? ReportPermission.VIEW : ReportPermission.NONE);
    }

    public boolean isShared() {
        return permission != ReportPermission.NONE;
    }

    public ReportRoleGrant updatedGrant(boolean view) {
        return new ReportRoleGrant(database.getDatabaseId(),
            Optional.of(role.getId()),
            view ? ReportPermission.VIEW : ReportPermission.NONE);
    }
}
