/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.filter;

import org.activityinfo.model.formula.FormulaNode;
import org.activityinfo.model.formula.FunctionCallNode;
import org.activityinfo.model.formula.functions.IsBlankFunction;
import org.activityinfo.model.formula.functions.NotFunction;
import org.activityinfo.model.formula.resolved.ResolvedCallNode;
import org.activityinfo.model.formula.resolved.ResolvedNode;
import org.activityinfo.model.type.FieldValue;

import java.util.List;
import java.util.Optional;

public class BlankPredicate implements RegularPredicate {

    private final String fieldLabel;
    private final ResolvedNode field;
    private final PredicateType predicateType;

    public BlankPredicate(ResolvedNode field, String fieldLabel, PredicateType predicateType) {
        this.field = field;
        this.fieldLabel = fieldLabel;
        this.predicateType = predicateType;
    }

    @Override
    public String getFieldLabel() {
        return fieldLabel;
    }

    @Override
    public ResolvedNode getField() {
        return field;
    }

    @Override
    public PredicateType getPredicateType() {
        return predicateType;
    }

    @Override
    public Optional<FieldValue> getArgument() {
        return Optional.empty();
    }

    public static boolean match(FieldMatcher fieldMatcher, ResolvedNode node, List<FilterPredicate> predicates) {
        if (!(node instanceof ResolvedCallNode)) {
            return false;
        }
        ResolvedCallNode rootCall = (ResolvedCallNode) node;
        if (rootCall.getFunction() == NotFunction.INSTANCE) {
            if (match(PredicateType.ISNOTBLANK, fieldMatcher, rootCall.getArgument(0), predicates)) {
                return true;
            }
        }

        return match(PredicateType.ISBLANK, fieldMatcher, rootCall, predicates);
    }

    private static boolean match(PredicateType type, FieldMatcher fieldMatcher, ResolvedNode node, List<FilterPredicate> predicates) {
        if (!(node instanceof ResolvedCallNode)) {
            return false;
        }
        ResolvedCallNode call = (ResolvedCallNode) node;
        if (call.getFunction() != IsBlankFunction.INSTANCE) {
            return false;
        }
        if (!(fieldMatcher.isField(call.getArgument(0)))) {
            return false;
        }
        ResolvedNode field = call.getArgument(0);
        predicates.add(new BlankPredicate(field, fieldMatcher.labelField(field), type));

        return true;
    }

    @Override
    public <T> T accept(PredicateVisitor<T> visitor) {
        return visitor.visitMissing(this);
    }

    public static FormulaNode toFormula(FormulaNode field) {
        return new FunctionCallNode(IsBlankFunction.INSTANCE, field);
    }

    @Override
    public FormulaNode toFormula() {
        return toFormula(field.toFormula());
    }
}

