/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.filter;

import org.activityinfo.model.formula.CompoundExpr;
import org.activityinfo.model.formula.FormulaNode;
import org.activityinfo.model.formula.functions.FormulaFunction;
import org.activityinfo.model.formula.functions.NotFunction;
import org.activityinfo.model.formula.resolved.*;

public class Matching {


    public static EnumItemReference isEnumReference(ResolvedNode node) {
        if(!(node instanceof ResolvedReference)) {
            return null;
        }
        ResolvedReference reference = (ResolvedReference) node;

        ReferenceNode root = reference.getRoot();
        while(true) {
            if(root instanceof EnumItemReference) {
                return ((EnumItemReference) root);
            } else if(root instanceof JoinQueryNode) {
                JoinQueryNode join = (JoinQueryNode) root;
                root = join.getRight();
            } else {
                return null;
            }
        }
    }


    public static ResolvedCallNode isBinaryCall(ResolvedNode node, FormulaFunction function) {
        return isCall(node, function, 2);
    }

    public static ResolvedCallNode isNegated(ResolvedNode node) {
        return isCall(node, NotFunction.INSTANCE, 1);
    }

    public static ResolvedCallNode isCall(ResolvedNode node, FormulaFunction function, int arity) {
        if (!(node instanceof ResolvedCallNode)) {
            return null;
        }
        ResolvedCallNode callNode = (ResolvedCallNode) node;
        if (callNode.getFunction() != function) {
            return null;
        }
        if (callNode.getArgumentCount() != arity) {
            return null;
        }

        return callNode;
    }

    public static CompoundExpr isCompoundExpr(ResolvedNode node) {
        FormulaNode formula = node.toFormula();
        if(!(formula instanceof CompoundExpr)) {
            return null;
        }
        return (CompoundExpr) formula;
    }

}
