/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.filter;

import org.activityinfo.model.formula.SupportCheckVisitor;
import org.activityinfo.model.formula.resolved.*;

public class SimpleFieldMatcher implements FieldMatcher {

    private static final ReferenceVisitor<Boolean> VISITOR = new ReferenceVisitor<Boolean>() {
        @Override
        public Boolean visitRecordId(RecordIdNode recordIdNode) {
            return false;
        }

        @Override
        public Boolean visitFormId(FormIdNode formIdNode) {
            return false;
        }

        @Override
        public Boolean visitLastEditTime(LastEditTimeNode node) {
            return false;
        }

        @Override
        public Boolean visitField(FieldReferenceNode field) {
            return true;
        }

        @Override
        public Boolean visitCalculatedField(CalculatedFieldNode field) {
            return true;
        }

        @Override
        public Boolean visitEnumItem(EnumItemReference enumItemReference) {
            return false;
        }

        @Override
        public Boolean visitJoin(JoinQueryNode node) {
            return node.getRight().accept(this);
        }

        @Override
        public Boolean visitUnion(UnionNode node) {
            return false;
        }

        @Override
        public Boolean visitAggregation(AggregateNode node) {
            return false;
        }
    };

    @Override
    public boolean isField(ResolvedNode node) {
        if (!(node instanceof ResolvedReference)) {
            return false;
        }
        ResolvedReference reference = (ResolvedReference) node;
        return reference.getRoot().accept(VISITOR);
    }

    @Override
    public String labelField(ResolvedNode node) {
        return node.toFormula().deparse();
    }
}
