/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.form;

import org.activityinfo.model.form.FormMetadata;
import org.activityinfo.model.form.FormRecord;
import org.activityinfo.model.form.TypedFormRecord;
import org.activityinfo.model.formula.eval.FieldValueProvider;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldValue;
import org.activityinfo.model.type.RecordRef;
import org.activityinfo.observable.Observable;
import org.activityinfo.promise.Maybe;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.function.Function;

public class FieldValueProviderImpl implements FieldValueProvider {

    private final Function<ResourceId, Observable<FormMetadata>> formMetadataProvider;
    private final Function<RecordRef, Observable<Maybe<FormRecord>>> recordProvider;

    private final Map<RecordRef, Observable<Optional<TypedFormRecord>>> recordCache = new HashMap<>();

    public FieldValueProviderImpl(Function<ResourceId, Observable<FormMetadata>> formMetadataProvider,
                                  Function<RecordRef, Observable<Maybe<FormRecord>>> recordProvider) {
        this.formMetadataProvider = formMetadataProvider;
        this.recordProvider = recordProvider;
    }

    private Observable<Optional<TypedFormRecord>> getRecord(RecordRef ref) {
        return recordCache.computeIfAbsent(ref, r ->
            formMetadataProvider.apply(r.getFormId()).join(form -> {
                if (!form.isVisible()) {
                    return Observable.just(Optional.<TypedFormRecord>empty());
                } else {
                    return recordProvider.apply(ref).transform(
                        maybeRecord -> maybeRecord.getIfVisible().map(
                            record -> TypedFormRecord.toTypedFormRecord(form.getSchema(), record)));
                }
            }));
    }

    @Override
    public Observable<Optional<RecordRef>> getParentRef(RecordRef ref) {
        return getRecord(ref).transform(optional -> optional.flatMap(record -> record.getParentRecordRef()));
    }

    @Override
    public Observable<Optional<FieldValue>> getFieldValue(RecordRef ref, ResourceId fieldId) {
        return getRecord(ref).transform(optional -> optional.flatMap(record -> {
            return Optional.ofNullable(record.get(fieldId));
        }));
    }

    @Override
    public Observable<Double> getLastEditTime(RecordRef recordRef) {
        return getRecord(recordRef).transform(optional -> optional.map(record -> record.getLastEditTime()).orElse(Double.NaN));
    }
}
