/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.longFormat;

import org.activityinfo.model.type.*;
import org.activityinfo.model.type.attachment.AttachmentType;
import org.activityinfo.model.type.barcode.BarcodeType;
import org.activityinfo.model.type.enumerated.EnumType;
import org.activityinfo.model.type.expr.CalculatedFieldType;
import org.activityinfo.model.type.geo.GeoAreaType;
import org.activityinfo.model.type.geo.GeoPointType;
import org.activityinfo.model.type.number.QuantityType;
import org.activityinfo.model.type.primitive.TextType;
import org.activityinfo.model.type.subform.SubFormReferenceType;
import org.activityinfo.model.type.time.*;

/**
 * Determines whether a field type should be included as a dimension
 * in the long format export.
 */
class DimensionFilter implements FieldTypeVisitor<Boolean> {

    public static final DimensionFilter INSTANCE = new DimensionFilter();

    private DimensionFilter() {
    }

    @Override
    public Boolean visitAttachment(AttachmentType attachmentType) {
        return false;
    }

    @Override
    public Boolean visitCalculated(CalculatedFieldType calculatedFieldType) {
        return false;
    }

    @Override
    public Boolean visitReference(ReferenceType referenceType) {
        return false;
    }

    @Override
    public Boolean visitNarrative(NarrativeType narrativeType) {
        return false;
    }

    @Override
    public Boolean visitQuantity(QuantityType type) {
        return false;
    }

    @Override
    public Boolean visitGeoPoint(GeoPointType geoPointType) {
        return false;
    }

    @Override
    public Boolean visitGeoArea(GeoAreaType geoAreaType) {
        return false;
    }

    @Override
    public Boolean visitEnum(EnumType enumType) {
        return enumType.getCardinality() == Cardinality.SINGLE;
    }

    @Override
    public Boolean visitBarcode(BarcodeType barcodeType) {
        return true;
    }

    @Override
    public Boolean visitSubForm(SubFormReferenceType subFormReferenceType) {
        return false;
    }

    @Override
    public Boolean visitLocalDate(LocalDateType localDateType) {
        return true;
    }

    @Override
    public Boolean visitMonth(MonthType monthType) {
        return true;
    }

    @Override
    public Boolean visitYear(YearType yearType) {
        return true;
    }

    @Override
    public Boolean visitFortnight(FortnightType fortnightType) {
        return true;
    }

    @Override
    public Boolean visitWeek(EpiWeekType epiWeekType) {
        return true;
    }

    @Override
    public Boolean visitLocalDateInterval(LocalDateIntervalType localDateIntervalType) {
        return false;
    }

    @Override
    public Boolean visitText(TextType textType) {
        return true;
    }

    @Override
    public Boolean visitSerialNumber(SerialNumberType serialNumberType) {
        return true;
    }

    @Override
    public Boolean visitSectionHeader(SectionHeaderType sectionHeaderType) {
        return false;
    }
}
