/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.longFormat;

import org.activityinfo.io.csv.TableWriter;
import org.dhatim.fastexcel.Workbook;
import org.dhatim.fastexcel.Worksheet;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

public class LongFormatXlsxWriter implements TableWriter {

    private static final int BATCH_SIZE = 1000;

    private final OutputStream out;
    private final Workbook book;
    private final Worksheet sheet;
    private int row = 0;
    private int batchRowStart = 0;

    private static class ColumnStyle {
        private final int column;
        private final String format;

        public ColumnStyle(int column, String format) {
            this.column = column;
            this.format = format;
        }
    }

    private List<ColumnStyle> styles = new ArrayList<>();

    public LongFormatXlsxWriter(OutputStream out) {
        this.out = out;
        this.book = new Workbook(out, "ActivityInfo", "1.0");
        this.sheet = book.newWorksheet("Quantities");
    }

    @Override
    public void setColumnWidth(int column, int width) {
        sheet.width(column, width);
    }

    @Override
    public void writeLine(Object... columns) throws IOException {
        for (int i = 0; i < columns.length; i++) {
            Object value = columns[i];
            if(value != null) {
                sheet.value(row, i, value);
            }
        }
        row++;
        if(row - batchRowStart > BATCH_SIZE) {
            flush();
        }
    }

    @Override
    public void setColumnFormat(int column, String pattern) {
        // We can only apply styles to the rows that we are holiding in memory, so
        // add this format to a list that we apply each time we flush a batch out to the stream
        styles.add(new ColumnStyle(column, pattern));
    }

    private void flush() throws IOException {
        flushColumnStyles();
        sheet.flush();
        batchRowStart = row;
    }

    @Override
    public void close() throws IOException {
        flushColumnStyles();
        book.finish();
        out.close();
    }

    private void flushColumnStyles() {
        if(row > batchRowStart) {
            for (ColumnStyle style : styles) {
                sheet.range(batchRowStart, style.column, row - 1, style.column)
                    .style()
                    .format(style.format)
                    .set();
            }
        }
    }

}
