/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.map.viewModel;

import org.activityinfo.json.*;

import java.util.Objects;

public class Range implements JsonSerializable {

    public static final Range EMPTY = new Range(Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY);

    private final double min;
    private final double max;

    public Range(double min, double max) {
        this.min = min;
        this.max = max;
    }

    public double getMin() {
        return min;
    }

    public double getMax() {
        return max;
    }

    public boolean isEmpty() {
        return min > max;
    }

    public Range union(Range other) {
        return new Range(Math.min(this.min, other.min), Math.max(this.max, other.max));
    }

    @Override
    public String toString() {
        return "[" + min + ", " + max + "]";
    }

    public boolean isZeroWidth() {
        return min == max;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        Range range = (Range) o;
        return Double.compare(range.min, min) == 0 && Double.compare(range.max, max) == 0;
    }

    @Override
    public int hashCode() {
        return Objects.hash(min, max);
    }

    @Override
    public JsonValue toJson() {
        if(isEmpty()) {
            return Json.createArray();
        } else {
            JsonValue range = Json.createArray();
            range.add(Json.create(min));
            range.add(Json.create(max));
            return range;
        }
    }

    public static Range fromJson(JsonValue jsonValue) {
        if(jsonValue.length() == 2) {
            return new Range(jsonValue.getNumber(0), jsonValue.getNumber(1));
        } else {
            return Range.EMPTY;
        }
    }
}
