/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.map.viewModel;

public class SphericalMercator {

    public static final double RADIUS = 6378137;
    public static final double ORIGIN_SHIFT = 2 * Math.PI * RADIUS / 2.0;
    public static final double PROJ_MIN = - Math.PI * RADIUS;
    public static final double PROJ_MAX = + Math.PI * RADIUS;
    public static final double PROJ_SIZE = PROJ_MAX - PROJ_MIN;
    public static final int PIXELS_PER_TILE = 256;

    private final int tileCount;
    private final int zoomLevel;
    private final int worldSizePixels;
    public final double metersPerTile;

    public SphericalMercator(int zoomLevel) {
        this.zoomLevel = zoomLevel;
        this.tileCount = 2 << zoomLevel;
        this.worldSizePixels = (int) (tileCount * PIXELS_PER_TILE);
        this.metersPerTile = PROJ_SIZE / tileCount;
    }

    public static double latitudeToMeters(double latitude) {
        double my = Math.log( Math.tan((90 + latitude) * Math.PI / 360.0 )) / (Math.PI / 180.0);
        my = my * ORIGIN_SHIFT / 180.0;
        return my;
    }

    public static double longitudeToMeters(double longitude) {
        return longitude * ORIGIN_SHIFT / 180.0;
    }

    public int metersToTileX(double metersX) {
        return (int)Math.floor((metersX - PROJ_MIN) / metersPerTile);
    }

    public int metersToTileY(double metersY) {
        return (int)Math.floor( (PROJ_MAX - metersY) / metersPerTile);
    }

    public int latitudeToTileY(double latitude) {
        return metersToTileY(latitudeToMeters(latitude));
    }

    public int longitudeToTileX(double longitude) {
        return metersToTileY(longitudeToMeters(longitude));
    }

}
