/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.pivot.viewModel;

import org.activityinfo.model.analysis.pivot.DimensionMapping;
import org.activityinfo.model.analysis.pivot.DimensionModel;
import org.activityinfo.model.form.FormField;
import org.activityinfo.model.formTree.FormTree;
import org.activityinfo.model.formula.CompoundExpr;
import org.activityinfo.model.formula.FormulaParser;
import org.activityinfo.model.formula.SymbolNode;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.ReferenceType;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

public class DimensionMergeMap {


    private final FormForest formForest;
    private final Map<String, Optional<FormField>> dimensionLabelMap;

    public DimensionMergeMap(FormForest formForest) {
        this.formForest = formForest;
        this.dimensionLabelMap = new HashMap<>();
    }

    public DimensionMapping get(DimensionModel dimension, FormTree formTree) {

        return dimensionLabelMap.computeIfAbsent(dimension.getId(), id -> findDimensionField(dimension))
            .flatMap(field -> findMatchingField(formTree, field))
            .orElseGet(() -> findFreeFormula(dimension));
    }

    private Optional<FormField> findDimensionField(DimensionModel dimension) {

        // Dimensions are added to them model using the formula {formId}.{fieldId}

        return dimension.getMappings().stream()
            .filter(m -> m.getType() == DimensionMapping.Type.VALUE)
            .findFirst()
            .flatMap(mapping -> FormulaParser.tryParse(mapping.getFormula()))
            .filter(formula -> formula instanceof CompoundExpr)
            .map(formula -> ((CompoundExpr) formula))
            .filter(compound -> compound.getValue() instanceof SymbolNode)
            .flatMap(compoundExpr ->
                    formForest.getFormClass(ResourceId.valueOf((((SymbolNode) compoundExpr.getValue()).getName())))
                        .flatMap(formClass -> formClass.getFieldIfPresent(ResourceId.valueOf(compoundExpr.getField().getName())))
                        .map(field -> field));
    }

    private Optional<DimensionMapping> findMatchingField(FormTree formTree, FormField originalField) {
        return formTree.getRootFormClass().getFields()
            .stream()
            .filter(field -> field.getLabel().equalsIgnoreCase(originalField.getLabel()) && !(field.getType() instanceof ReferenceType))
            .map(matchingField -> new DimensionMapping(formTree.getRootFormId(), matchingField.getId().asString()))
            .findFirst();
    }

    private DimensionMapping findFreeFormula(DimensionModel dimension) {
        if(dimension.getMappings().isEmpty()) {
            return null;
        }
        return new DimensionMapping(dimension.getMappings().get(0).getFormula());
    }

}
