/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.pivot.viewModel;

import org.activityinfo.model.analysis.pivot.Axis;
import org.activityinfo.model.analysis.pivot.DimensionModel;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

/**
 * Immutable set of dimensions
 */
public class DimensionSet implements Iterable<DimensionModel> {

    private int count;
    private DimensionModel[] dimensions;

    public DimensionSet() {
        count = 0;
        dimensions = new DimensionModel[0];
    }

    public DimensionSet(List<DimensionModel> dimensions) {
        List<DimensionModel> activeDimensions = new ArrayList<>();
        for (DimensionModel dimension : dimensions) {
            activeDimensions.add(dimension);
        }
        this.count = activeDimensions.size();
        this.dimensions = activeDimensions.toArray(new DimensionModel[count]);
    }

    public int getCount() {
        return count;
    }

    public DimensionModel getDimension(int i) {
        return dimensions[i];
    }

    public int getIndex(DimensionModel dimensionModel) {
        for (int i = 0; i < dimensions.length; i++) {
            if(dimensions[i].getId().equals(dimensionModel.getId())) {
                return i;
            }
        }
        throw new IllegalArgumentException("Dimension " + dimensionModel.getId() + " is not a member of this set");
    }

    public List<DimensionModel> getList() {
        return Arrays.asList(dimensions);
    }

    public DimensionSet add(DimensionModel newDimension) {
        List<DimensionModel> newList = new ArrayList<>(getList());
        newList.add(newDimension);
        return new DimensionSet(newList);
    }

    public int getMapDimension() {
        for (int i = 0; i < dimensions.length; i++) {
            if(dimensions[i].getAxis() == Axis.MAP) {
                return i;
            }
        }
        return -1;
    }

    @Override
    public Iterator<DimensionModel> iterator() {
        return Arrays.asList(dimensions).iterator();
    }

    /**
     *
     * @return the index of the Dimension with the given {@code dimensionId},
     * or -1 if no such dimension is included.
     */
    public int getIndexByDimensionId(String dimensionId) {
        for (int i = 0; i < dimensions.length; i++) {
            if (dimensions[i].getId().equals(dimensionId)) {
                return i;
            }
        }
        return -1;
    }
}
