/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.pivot.viewModel;

import org.activityinfo.model.analysis.pivot.DimensionMapping;
import org.activityinfo.model.analysis.pivot.DimensionModel;
import org.activityinfo.model.formTree.FormTree;
import org.activityinfo.model.query.QueryModel;
import org.activityinfo.model.resource.ResourceId;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class EffectiveFormMeasures {

    private List<EffectiveMeasure> measures = new ArrayList<>();;
    private DimensionSet dimensionSet;
    private final FormTree formTree;

    private List<EffectiveMapping> dimensions = new ArrayList<>();


    public EffectiveFormMeasures(FormTree formTree, DimensionSet dimensionSet, DimensionMergeMap dimensionMergeMap, Collection<EffectiveMeasure> measureModels) {
        this.formTree = formTree;
        this.dimensionSet = dimensionSet;

        for (int i = 0; i < dimensionSet.getCount(); i++) {
            DimensionModel dimension = dimensionSet.getDimension(i);
            DimensionMapping mapping = findMapping(formTree, dimensionMergeMap, dimension);
            dimensions.add(new EffectiveMapping(formTree, i, dimension, mapping));
        }
        for (EffectiveMeasure measure : measureModels) {
            measures.add(measure);
        }
    }

    private static DimensionMapping findMapping(FormTree formTree, DimensionMergeMap dimensionMergeMap, DimensionModel dimension) {

        ResourceId formId = formTree.getRootFormId();

        // First try to find a mapping specifically for this form
        for (DimensionMapping mapping : dimension.getMappings()) {
            if (mapping.getFormId() != null &&
                    mapping.getFormId().equals(formId)) {
                return mapping;
            }
        }

        // If nothing specifically for this form, find another field in this form with the same name
        return dimensionMergeMap.get(dimension, formTree);
    }


    public DimensionSet getDimensionSet() {
        return dimensionSet;
    }

    public List<EffectiveMapping> getDimensions() {
        return dimensions;
    }

    public EffectiveMapping getDimension(int i) {
        return dimensions.get(i);
    }

    public List<EffectiveMeasure> getMeasures() {
        return measures;
    }

    public ResourceId getFormId() {
        return formTree.getRootFormId();
    }

    public QueryModel composeQuery() {

        QueryModel queryModel = new QueryModel(getFormId());

        for (EffectiveMeasure measure : getMeasures()) {
            queryModel.selectExpr(measure.getModel().getFormula()).as(measure.getModel().getId());
        }
        for (EffectiveMapping dim : getDimensions()) {
            queryModel.addColumns(dim.getRequiredColumns());
        }
        return queryModel;
    }
}
