/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.pivot.viewModel;

import org.activityinfo.model.analysis.pivot.MeasureModel;
import org.activityinfo.model.formTree.FormTree;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.resolved.FieldReferenceNode;
import org.activityinfo.model.formula.resolved.FormulaResolver;
import org.activityinfo.model.formula.resolved.ResolvedNode;
import org.activityinfo.model.formula.resolved.ResolvedReference;
import org.activityinfo.model.resource.ResourceId;

import java.util.Optional;

/**
 * Models a measure that is part of the analysis,
 * based on the user's model as well as metadata
 */
public class EffectiveMeasure {
    private final MeasureModel model;
    private final Optional<String> formLabel;
    private final String sourceLabel;
    private final Optional<FormulaType> formulaType;
    private final ResolvedNode formula;
    private final boolean valid;

    public EffectiveMeasure(MeasureModel model, FormTree formTree) {
        this.model = model;
        this.formLabel = Optional.of(formTree.getRootFormClass().getLabel());
        this.formula = FormulaResolver.resolve(formTree, model.getFormula());
        this.sourceLabel = computeSourceLabel(formTree, formula);
        this.formulaType = formula.getResolvedType();
        valid = true;
    }

    public EffectiveMeasure(MeasureModel model) {
        this.model = model;
        this.formLabel = Optional.empty();
        this.formula = null;
        this.sourceLabel = null;
        this.formulaType = Optional.empty();
        valid = false;
    }

    public String getSourceLabel() {
        return sourceLabel;
    }

    public ResourceId getFormId() {
        return model.getFormId();
    }

    public ResolvedNode getFormula() {
        return formula;
    }

    public MeasureModel getModel() {
        return model;
    }

    public Optional<String> getFormLabel() {
        return formLabel;
    }

    public Optional<FormulaType> getFormulaType() {
        return formulaType;
    }

    public boolean isValid(){
        return valid;
    }

    private static String computeSourceLabel(FormTree formTree, ResolvedNode formula) {

        // If this measure is a reference to a single field, then
        // Check to see if the label is different.

        if(formula instanceof ResolvedReference) {
            ResolvedReference reference = (ResolvedReference) formula;
            if (reference.getRoot() instanceof FieldReferenceNode) {
                FieldReferenceNode field = (FieldReferenceNode) reference.getRoot();
                return field.getField().getLabel();
            }
        }
        return "";
    }

}

