/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.pivot.viewModel;

import com.google.gwt.core.shared.GWT;
import com.google.gwt.i18n.client.LocaleInfo;
import com.google.gwt.i18n.client.NumberFormat;

public interface PercentFormatter {

    /**
     *
     * @param proportion a proportion between 0 and 1.0
     * @return a formatted string in the current locale
     */
    public String format(double proportion);


    public static PercentFormatter get() {
        if(GWT.isClient()) {
            // Use the localized format, which varies.
            // For example, in Turkish, percent signs should be on left e.g. %10 not 10%.
            String localizedPercentPattern = LocaleInfo.getCurrentLocale().getNumberConstants().percentPattern();
            NumberFormat percentFormat = NumberFormat.getFormat(localizedPercentPattern);
            percentFormat.overrideFractionDigits(0);

            return p -> percentFormat.format(p);
        } else {
            return p -> ((int)Math.round(p * 100d)) + "%";
        }
    }
}
