/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.pivot.viewModel;

import com.google.common.collect.Sets;
import org.activityinfo.model.analysis.pivot.PivotModel;
import org.activityinfo.model.formTree.FormTree;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.observable.Observable;
import org.activityinfo.store.query.shared.FormSource;

import java.util.*;
import java.util.logging.Logger;
import java.util.stream.Collectors;

/**
 * Describes a PivotTable analysis and its results
 */
public class PivotViewModel {

    private static final Logger LOGGER = Logger.getLogger(PivotViewModel.class.getName());

    protected static final PivotModel EMPTY = PivotModel.builder().build();

    private final FormSource formSource;

    private final Observable<FormForest> formForest;
    private final Observable<EffectiveModel> effectiveModel;
    private final Observable<PivotPointSet> resultTable;
    private final Observable<List<EffectiveDimension>> dimensions;
    private final Observable<PivotTable> pivotTable;

    public PivotViewModel(FormSource source, Observable<FormForest> formForest, Observable<PivotModel> pivotModel) {
        this(source, formForest, pivotModel, Observable.just(false));
    }

    public PivotViewModel(FormSource source, Observable<FormForest> formForest, Observable<PivotModel> pivotModel, Observable<Boolean> showHidden) {
        this.formSource = source;
        this.formForest = formForest;
        this.effectiveModel = Observable.transform(formForest, pivotModel.cacheIfEqual(), (ff, m) -> {
            LOGGER.info("Updating effective model");
            return new EffectiveModel(m, ff);
        });
        this.resultTable = Observable.join(effectiveModel, showHidden, (m, s) -> {
            LOGGER.info("Updating result table");
            return PivotPointSet.compute(formSource, m, s);
        });
        this.dimensions = effectiveModel.transform(em -> {
            LOGGER.info("Updating dimensions");
            return em.getDimensions();
        });
        this.pivotTable = resultTable.transform(t -> {
            LOGGER.info("Updating pivot table");
            return PivotTable.create(t);
        });

    }

    public Observable<EffectiveModel> getEffectiveModel() {
        return effectiveModel;
    }

    public Observable<PivotPointSet> getResultTable() {
        return resultTable;
    }

    public Observable<List<EffectiveDimension>> getDimensions() {
        return dimensions;
    }

    public Observable<PivotTable> getPivotTable() {
        return pivotTable;
    }

    public Observable<FormForest> getFormForest() {
        return formForest;
    }

}
