/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.pivot.viewModel;

import org.activityinfo.model.form.FormClass;
import org.activityinfo.model.formTree.FormTree;
import org.activityinfo.model.resource.ResourceId;

import java.util.Collections;
import java.util.Objects;
import java.util.Set;

public class ReferencedForm {
    private final ResourceId formId;
    private final ResourceId databaseId;

    public ReferencedForm(ResourceId formId, ResourceId databaseId) {
        this.formId = formId;
        this.databaseId = databaseId;
    }

    public static Set<ReferencedForm> of(FormTree formTree) {
        if(formTree.getRootState() == FormTree.State.VALID) {
            return Collections.singleton(new ReferencedForm(formTree.getRootFormId(), formTree.getRootFormClass().getDatabaseId()));
        } else {
            return Collections.emptySet();
        }
    }

    public static ReferencedForm of(FormClass formClass) {
        return new ReferencedForm(formClass.getId(), formClass.getDatabaseId());
    }

    public ResourceId getFormId() {
        return formId;
    }

    public ResourceId getDatabaseId() {
        return databaseId;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        ReferencedForm that = (ReferencedForm) o;
        return formId.equals(that.formId) &&
            databaseId.equals(that.databaseId);
    }

    @Override
    public int hashCode() {
        return Objects.hash(formId, databaseId);
    }

    @Override
    public String toString() {
        return "ReferencedForm{" +
            "formId=" + formId +
            ", databaseId=" + databaseId +
            '}';
    }
}
