/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.pivot.viewModel.chart;

import org.activityinfo.analysis.pivot.viewModel.PivotTable;
import org.activityinfo.ui.vdom.shared.html.H;
import org.activityinfo.ui.vdom.shared.html.SvgTag;
import org.activityinfo.ui.vdom.shared.tree.VAttr;
import org.activityinfo.ui.vdom.shared.tree.VAttrMap;
import org.activityinfo.ui.vdom.shared.tree.VElement;
import org.activityinfo.ui.vdom.shared.tree.VNode;

import java.util.ArrayList;
import java.util.List;

public class BarChartRenderer extends GraphChartRenderer {

    public BarChartRenderer(AxisChartLayout layout, ChartFontContext fontContext) {
        super(layout, fontContext);
    }

    @Override
    protected VNode renderChartData() {
        List<PivotTable.Node> major = layout.getMajorLeafCategories();
        List<PivotTable.Node> minor = layout.getMinorLeafCategories();

        List<VNode> rects = new ArrayList<>();

        for (int i = 0; i < major.size(); i++) {
            for (int j = 0; j < minor.size(); j++) {
                double point = major.get(i).getCellValue(minor.get(j));
                if(Double.isFinite(point) ) {
                    Rect rect = layout.getBarRect(i, j, point);
                    VAttrMap rectProps = VAttr.create();
                    rectProps.set("x", rect.getLeft());
                    rectProps.set("y", rect.getTop());
                    rectProps.set("width", rect.getWidth());
                    rectProps.set("height", rect.getHeight());
                    rectProps.set("fill", graphElementColor(j));

                    rects.add(new VElement(SvgTag.RECT, rectProps));
                }
            }
        }

        return new VElement(SvgTag.GROUP, VAttr.create(), rects);
    }

    @Override
    protected VNode renderLegend() {
        return H.t("") ;
    }
}
