/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.pivot.viewModel.chart;

import org.activityinfo.analysis.pivot.viewModel.PivotTable;

import java.util.HashMap;
import java.util.Map;

public class PieChartLayoutBuilder {

    private final PivotTable pivotTable;
    private final ChartFontContext fontContext;
    private final double[] majorCategoryLabelHeights;
    private final Map<PivotTable.Node, String[]> majorCategoryTextLinesMap;


    public PieChartLayoutBuilder(PivotTable pivotTable, ChartFontContext fontContext) {
        this.fontContext = fontContext;
        this.pivotTable = pivotTable;
        this.majorCategoryLabelHeights = new double[pivotTable.getRowDimensionCount()];
        this.majorCategoryTextLinesMap = new HashMap<>();
    }


    private double computeChartWidth(PivotTable pivotTable, double pieMargin) {
        int numMajorCategories = AxisChartLayout.getMajorLeafCategories(pivotTable).size();


        double majorCategoryWidth = PieChartLayout.getMajorCategoryWidth();

        double plotAreaWidth = (numMajorCategories * majorCategoryWidth) +
                ((numMajorCategories + 1) * pieMargin);

        return AxisChartLayout.MARGIN_LEFT +
                plotAreaWidth +
                AxisChartLayout.MARGIN_RIGHT;
    }

    public double getLegendHeight(){
        return AxisChartLayout.getMinorLeafCategories(pivotTable).size() * AxisChartLayout.MINOR_CATEGORY_LABEL_LINE_HEIGHT;
    }

    public PieChartLayout build() {
        double pieMargin = PieChartLayout.PIECHART_MARGIN;
        double chartWidth = computeChartWidth(pivotTable, pieMargin);
        TextLineOperator majorLabelOperator = new TextLineOperator(fontContext.getFontMetrics(ChartFontStyle.MAJOR_CATEGORY_LABELS), PieChartLayout.getMajorCategoryWidth());
        AxisChartLayoutBuilder.computeCategoryLabelsWrap(AxisChartLayout.getMajorCategories(pivotTable), AxisChartLayout.MAXIMUM_MAJOR_CATEGORY_LABEL_LINES,
                AxisChartLayout.MAJOR_CATEGORY_LABEL_LINES_MARGIN, pivotTable.getRowDimensionCount(),
                majorLabelOperator, majorCategoryTextLinesMap, majorCategoryLabelHeights);

        return new PieChartLayout(pivotTable, chartWidth, getLegendHeight(), AxisChartLayoutBuilder.getLengendTextWidth(pivotTable, fontContext),
                majorCategoryLabelHeights, majorCategoryTextLinesMap);
    }


}
