/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.pivot.viewModel.chart;

import com.google.gwt.core.shared.GwtIncompatible;

import java.awt.*;
import java.awt.image.BufferedImage;

/**
 * Server-side context that uses widely-available fonts to render the charts.
 */
@GwtIncompatible
public class SystemFontContext implements ChartFontContext {

    private final Graphics2D graphics;

    public SystemFontContext() {
        GraphicsEnvironment graphicsEnvironment = GraphicsEnvironment.getLocalGraphicsEnvironment();
        BufferedImage image = new BufferedImage(5, 5, BufferedImage.TYPE_INT_RGB);
        graphics = graphicsEnvironment.createGraphics(image);
    }

    @Override
    public ChartFontMetrics getFontMetrics(ChartFontStyle fontStyle) {
        switch (fontStyle) {
            case SCALE_LABELS:
            case MINOR_CATEGORY_LABELS:
                return new AwtChartFontMetrics(new Font("Arial", Font.PLAIN, 14), "normal 14px Arial");

            case MAJOR_CATEGORY_LABELS:
                return new AwtChartFontMetrics(new Font("Arial", Font.BOLD, 18), "bold 18px Arial");
        }
        throw new IllegalArgumentException("style: " + fontStyle);
    }


    private class AwtChartFontMetrics implements ChartFontMetrics {

        private final java.awt.FontMetrics delegate;
        private final String svgFontString;

        public AwtChartFontMetrics(Font font, String svgFontString) {
            this.delegate = graphics.getFontMetrics(font);
            this.svgFontString = svgFontString;
        }

        @Override
        public double getAscent() {
            return delegate.getAscent();
        }

        @Override
        public String getSvgFontString() {
            return svgFontString;
        }

        @Override
        public double measureWidth(String string) {
            return delegate.stringWidth(string);
        }
    }
}
