/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.pivot.viewModel.chart;

import java.util.ArrayList;
import java.util.List;

public class TextLineOperator {
    public static final String ELIPSES = "…";

    final ChartFontMetrics metrics;
    final double maxWidth;

    public TextLineOperator(ChartFontMetrics metrics, double width){
        this.metrics = metrics;
        this.maxWidth = width;
    }

    public String truncate(String text) {
        if(metrics.measureWidth(text) <= maxWidth) {
            return text;
        }

        int fittedLength = findText(text, maxWidth - metrics.measureWidth(ELIPSES));

        if(fittedLength == 0) {
            return "";
        } else {
            return text.substring(0, fittedLength) + ELIPSES;
        }
    }

    public List<String> wrap(String text) {
        ArrayList<String> parts = new ArrayList<>();
        if(metrics.measureWidth(text) <= maxWidth) { //text fits available space
            parts.add(text);
            return parts;
        }

        String[] words = text.split(" ");
        if(words.length == 1){
            //attempt to wrap possibly long single word
            return wrapLine(text);
        } else {
            String testLine = "";
            String line = words[0];
            for(int i = 1; i < words.length; i++){
                String word = words[i];
                testLine = line + " " + word;
                double width = metrics.measureWidth(testLine);
                if(width > maxWidth){ //adding new word causes the line to wrap?
                    List<String> wrappedLines = wrapLine(line);
                    if(wrappedLines.size() == 1){
                        //add the line to the result
                        parts.add(wrappedLines.get(0));
                        line = word;
                    } else {
                        //add everything but the last line to the result
                        String lastLine = wrappedLines.remove(wrappedLines.size()-1);
                        parts.addAll(wrappedLines);
                        //check if new word can fit with remaining bits
                        testLine = lastLine + " " + word;
                        width = metrics.measureWidth(testLine);
                        if(width < maxWidth){
                            line = lastLine + " " + word;
                        }else{
                            parts.add(lastLine);
                            line = word;
                        }

                    }
                } else { //line still fits with addition of a new word
                    line = testLine;
                }
            }
            parts.addAll(wrapLine(line));
            return parts;
        }
    }

    public static String[] trim(String[] input){
        String[] result = new String[input.length];
        for(int i = 0; i < input.length; i++){
            result[i] = input[i].trim();
        }
        return result;
    }

    private List<String> wrapLine(String text) {
        ArrayList<String> parts = new ArrayList<>();
        if(metrics.measureWidth(text) <= maxWidth) {
            parts.add(text);
            return parts;
        }

        while(text.length() > 0){
            int fittedLength = findText(text, maxWidth);
            parts.add(text.substring(0, fittedLength).trim());//fits in line

            //evalulate  the remainder
            String remainder = text.substring(fittedLength).trim();
            if(metrics.measureWidth(remainder) <= maxWidth){
                parts.add(remainder);
                text = "";
            }else {
                text = remainder;
            }
        }
        return parts;
    }

    private int findText(String string, double textWidth) {
        int n = string.length();
        int left = 0;
        int right = n - 1;
        while(left < right) {
            int middle = Math.floorDiv(left + right, 2);
            double width = metrics.measureWidth(string.substring(0, middle));
            if (width > textWidth) {
                right = middle;
            } else {
                left = middle + 1;
            }
        }
        return left - 1;
    }

    public double getFontAscent(){
        return metrics.getAscent();
    }

}
