/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.table;

import org.activityinfo.model.query.BlankColumnView;
import org.activityinfo.model.query.ColumnSet;
import org.activityinfo.model.query.ColumnType;
import org.activityinfo.model.query.ColumnView;

public abstract class AbstractRenderer<T> implements ColumnRenderer<T> {

    private final String columnId;
    private ColumnView view;

    protected AbstractRenderer(String columnId) {
        this.columnId = columnId;
    }

    @Override
    public final T render(int rowIndex) {
        if(view == null) {
            return null;
        }
        return renderRow(view, rowIndex);
    }

    protected abstract T renderRow(ColumnView view, int rowIndex);

    @Override
    public final void updateColumnSet(ColumnSet columnSet) {
        this.view = columnSet.getColumnView(columnId);

        // The ColumnSet may need to reload after a change to EffectiveTableModel
        if(this.view == null) {
            this.view = new BlankColumnView(ColumnType.STRING, columnSet.getNumRows());
        }
    }
}
