/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.table;

import org.activityinfo.model.analysis.TableColumn;
import org.activityinfo.model.formTree.FormTree;
import org.activityinfo.model.formula.FormulaNode;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.resolved.FormulaResolver;
import org.activityinfo.model.formula.resolved.ResolvedNode;
import org.activityinfo.model.query.ColumnModel;

import java.util.List;

public class EffectiveTableColumn {

    public static final int DEFAULT_COLUMN_WIDTH = ColumnFormat.DEFAULT_WIDTH;
    private TableColumn model;
    private String label;
    private ResolvedNode formula;
    private ColumnFormat format;
    private int width;

    public EffectiveTableColumn(FormTree formTree, TableColumn model) {
        this.model = model;
        this.formula = FormulaResolver.resolve(formTree, model.getFormula());
        this.label = model.getLabel().orElseGet(() -> formula.label(formTree.getRootFormId()));
        format = ColumnFormatFactory.create(model.getId(), formula);
        width = model.getWidth(format.getColumnWidth());
    }

    public ResolvedNode getResolvedFormula() {
        return formula;
    }

    public FormulaNode getFormula() {
        return formula.toFormula();
    }

    public String getFormulaString() {
        return formula.toFormula().deparse();
    }

    public FormulaType getFormulaType() {
        return formula.getResolvedType().orElse(FormulaType.STRING);
    }

    public String getId() {
        return model.getId();
    }

    public String getLabel() {
        return label;
    }

    public TableColumn getModel() {
        return model;
    }

    public List<ColumnModel> getQueryModel() {
        return format.getColumnModels();
    }

    public boolean isValid() {
        return formula.isValid();
    }

    public int getWidth() {
        return width;
    }

    public ColumnFormat getFormat() {
        return format;
    }

    public <T> T accept(TableColumnVisitor<T> visitor) {
        return format.accept(this, visitor);
    }

    public int getExportedColumnCount() {
        return format.getExportedColumnCount();
    }

    @Override
    public String toString() {
        return "EffectiveTableColumn{" + label + "}";
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        EffectiveTableColumn that = (EffectiveTableColumn) o;

        if (!model.equals(that.model)) {
            return false;
        }
        if (!label.equals(that.label)) {
            return false;
        }
        if (!formula.equals(that.formula)) {
            return false;
        }
        return format.equals(that.format);

    }

    @Override
    public int hashCode() {
        int result = model.hashCode();
        result = 31 * result + label.hashCode();
        result = 31 * result + formula.hashCode();
        result = 31 * result + format.hashCode();
        return result;
    }
}
