/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.table;

import org.activityinfo.model.formula.CompoundExpr;
import org.activityinfo.model.formula.SymbolNode;
import org.activityinfo.model.formula.resolved.ResolvedNode;
import org.activityinfo.model.query.ColumnModel;

import java.util.Arrays;
import java.util.List;

public class GeoPointFormat implements ColumnFormat {

    private String columnId;
    private ResolvedNode formula;

    public GeoPointFormat(String columnId, ResolvedNode formula) {
        this.columnId = columnId;
        this.formula = formula;
    }

    public String getLatitudeId() {
        return columnId + ":lat";
    }

    public String getLongitudeId() {
        return columnId + ":lng";
    }

    private ColumnRenderer<Double> createRenderer(String id) {
        return new DoubleRenderer(id);
    }

    public ColumnRenderer<Double> createLatitudeRenderer() {
        return createRenderer(getLatitudeId());
    }

    public ColumnRenderer<Double> createLongitudeRenderer() {
        return createRenderer(getLongitudeId());
    }

    @Override
    public List<ColumnModel> getColumnModels() {
        return Arrays.asList(getLatitudeColumnModel(), getLongitudeColumnModel());
    }

    public ColumnModel getLongitudeColumnModel() {
        return getColumnModel(getLongitudeId(), "longitude");
    }

    public ColumnModel getLatitudeColumnModel() {
        return getColumnModel(getLatitudeId(), "latitude");
    }

    public ColumnModel getColumnModel(String id, String component) {
        ColumnModel longitudeModel = new ColumnModel();
        longitudeModel.setId(id);
        longitudeModel.setFormula(new CompoundExpr(formula.toFormula(), new SymbolNode(component)));
        return longitudeModel;
    }

    @Override
    public <T> T accept(EffectiveTableColumn columnModel, TableColumnVisitor<T> visitor) {
        return visitor.visitGeoPointColumn(columnModel, this);
    }

    @Override
    public int getExportedColumnCount() {
        return 2;
    }
}
