/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.analysis.table;

import org.activityinfo.analysis.filter.FieldMatcher;
import org.activityinfo.analysis.filter.FilterPredicate;
import org.activityinfo.analysis.filter.FilterViewModel;
import org.activityinfo.analysis.filter.RegularPredicate;
import org.activityinfo.model.formula.ConstantNode;
import org.activityinfo.model.formula.FormulaNode;
import org.activityinfo.model.formula.resolved.ResolvedNode;
import org.activityinfo.model.formula.resolved.ResolvedReference;

import java.util.*;

class TableColumnMatcher implements FieldMatcher {

    private Map<FormulaNode, EffectiveTableColumn> columns = new HashMap<>();

    public TableColumnMatcher(List<EffectiveTableColumn> columns) {
        for (EffectiveTableColumn column : columns) {
            if(column.getResolvedFormula().isValid()) {
                this.columns.put(column.getResolvedFormula().toFormula(), column);
            }
        }
    }

    @Override
    public boolean isField(ResolvedNode node) {
        if(node instanceof ConstantNode) {
            return false;
        }
        if(node instanceof ResolvedReference) {
            return true;
        }
        return columns.containsKey(node.toFormula());
    }

    @Override
    public String labelField(ResolvedNode node) {
        EffectiveTableColumn column = columns.get(node.toFormula());
        if(column != null) {
            return column.getLabel();
        }
        return node.toFormula().deparse();
    }

    /**
     * Finds the ids of columns that are included in the filter.
     */
    public Set<String> findFilteredColumns(FilterViewModel filter) {
        Set<String> set = new HashSet<>();
        for (FilterPredicate predicate : filter.getPredicates()) {
            if(predicate instanceof RegularPredicate) {
                RegularPredicate regularPredicate = (RegularPredicate) predicate;
                EffectiveTableColumn column = columns.get(regularPredicate.getField().toFormula());
                if(column != null) {
                    set.add(column.getId());
                }
            }
        }
        return set;
    }
}
