/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.i18n.shared;

import java.util.Optional;

/**
 * Defines available application locales.
 */
public enum ApplicationLocale {

    // ORDER:
    // 1) English first.
    // 2) All remaining languages with Latin script, sorted alphabetically
    // 3) Languages written with Arabic script, sorted alphabetically
    // 4) Additional writing systems?

    /**
     * English
     */
    EN("English"),

    /**
     * Spanish
     */
    ES("Español"),

    /**
     * Greek
     */
    EL("Ελληνικά"),

    /**
     * French
     */
    FR("Français"),

    /**
     * Dutch
     */
    NL("Nederlands"),

    /**
     * Portugese
     */
    PT("Português"),

    /**
     * Russian
     */
    RU("русский"),

    /**
     * Turkish
     */
    TR("Türkçe"),

    /**
     * Arabic
     */
    AR("العربية"),

    /**
     * Persian
     */
    FA("فارسی"),

    /**
     * Burmese
     */
    MY("မြန်မာဘာသာ");

    private String localizedName;

    ApplicationLocale(String localizedName) {
        this.localizedName = localizedName;
    }

    public String getLocalizedName() {
        return localizedName;
    }
    
    public String getCode() {
        return name().toLowerCase();
    }

    public static ApplicationLocale fromCode(String localeCode) {
        return valueOf(localeCode.toUpperCase());
    }

    public static Optional<ApplicationLocale> validateLocale(String locale) {
        if(locale == null) {
            return Optional.empty();
        }
        try {
            return Optional.of(fromCode(locale));
        } catch (IllegalArgumentException e) {
            return Optional.empty();
        }
    }
}
