/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.i18n.shared;

import com.google.gwt.core.shared.GWT;
import com.google.gwt.i18n.client.LocaleInfo;
import com.google.gwt.i18n.shared.BidiUtils;

public enum TextDirection {
    LTR,
    RTL;

    public static TextDirection currentLocale() {
        if(GWT.isClient()) {
            return LocaleInfo.getCurrentLocale().isRTL() ? RTL : LTR;
        } else {
            // For unit tests.
            return LTR;
        }
    }

    /**
     * Determines whether the direction attribute should be set to "rtl" or to "ltr".
     *
     * <p>This is necessary for workable bidirectional input. For example, a user may be using
     * the Arabic or Persian interface, but need to enter information in English. If the dir attribute
     * is not set to "ltr", then input will be very difficult.</p>
     *
     * <p>This problem *should* be solved by setting the input "dir=auto", but unfortunately neither
     * Chrome nor Firefox properly implement this according to the specification at the moment.</p>
     *
     * @see <a href="https://bugzilla.mozilla.org/show_bug.cgi?id=1103011>Firefox bug report</a>
     *
     * @param textInput
     * @return true if this
     */
    public static TextDirection estimateDirection(String textInput) {
        return estimateDirection(textInput, currentLocale());
    }

    public static TextDirection estimateDirection(String textInput, TextDirection localeDirection) {
        if(textInput.isEmpty()) {
            // If there is no input, use the direction of the current locale.
            return localeDirection;
        } else {
            // Otherwise, detect the direction based on the first character with strong directionality.
            if (BidiUtils.get().startsWithLtr(textInput)) {
                return LTR;
            } else if(BidiUtils.get().startsWithRtl(textInput)) {
                return RTL;
            } else {
                return localeDirection;
            }
        }
    }
}
