/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.i18n.shared;

import com.google.gwt.safehtml.shared.SafeHtml;

/**
 * Messages for the application.
 */
public interface UiMessages extends com.google.gwt.i18n.client.Messages {

    @DefaultMessage("{0,number,#}W{1}")
    String week(int year, int week);

    @DefaultMessage("Are you sure you want to delete the report “{0}”")
    String confirmDeleteReport(String reportTitle);

    @DefaultMessage("No history is available for this record.")
    String siteHistoryNotAvailable();

    @DefaultMessage("Please enter a value that matches the pattern ''{0}''")
    String invalidTextInput(String mask);

    @DefaultMessage("Column “{0}”")
    String columnHeading(String columnName);

    @DefaultMessage("<strong>Form</strong> in database <strong>{0}</strong>")
    String formDatabaseLabel(String databaseName);

    @DefaultMessage("<strong>Folder</strong> in database <strong>{0}</strong>")
    String folderDatabaseLabel(String databaseName);

    @DefaultMessage("{0} records")
    @AlternateMessage({ "one", "1 record" })
    @Meaning("Provides a count of sub form records in a subform column in the datatable.")
    String recordCount(@PluralCount int count);

    @AlternateMessage({ "one", "One record is valid and ready for importing now." })
    @DefaultMessage("{0} records are valid and ready for importing now")
    String validRecordCount(@PluralCount int count);

    @DefaultMessage("{0} records match existing records and will be updated, {1} new records will be added.")
    String existingRecordCount(int existingRecordCount, int newRecords);

    @DefaultMessage("Below are {0} rows that have validation issues. If you continue importing, these rows will be ignored and not imported.")
    @AlternateMessage({ "one", "Below is one row that has validation issues. If you continue importing, this row will be ignored and not imported." })
    String invalidRecordCount(@PluralCount int count);

    @AlternateMessage({ "one", "One record processed" })
    @DefaultMessage("{0} records processed")
    String recordsProcessed(@PluralCount int recordsProcessed);

    @AlternateMessage({ "one", "One record imported." })
    @DefaultMessage("{0} records imported.")
    String recordsImported(@PluralCount int recordsCommitted);

    @AlternateMessage({ "one", "One record has been successfully deleted." })
    @DefaultMessage("{0} records have been successfully deleted.")
    String recordsDeleted(@PluralCount int recordsCommitted);

    @DefaultMessage("No results for “{0}”")
    String noSearchResults(String searchString);

    @DefaultMessage("Acquired current location to within {0} meters.")
    @AlternateMessage({ "one", "Acquired current location to the nearest meter" })
    String acquiredCurrentLocation(@PluralCount int accuracy);

    @DefaultMessage("The acquired position is only accurate to within {0} meters.")
    String acquiredCurrentLocationWarning(int accuracy);

    @DefaultMessage("Field {0} of {1}")
    String fieldProgress(int currentField, int fieldCount);

    @DefaultMessage("Select {0}")
    String selectLookupKeyPlaceholder(String keyLabel);

    @DefaultMessage("{0} ({1,number,0.0} MB) exceeds the maximum file size of {2} MB")
    String fileExceedsMaximumSize(String filename, double megabytes, int maxSizeMegabytes);

    @DefaultMessage("Upload of ‘{0}’ has failed.")
    String uploadFailedFile(String filename);

    @DefaultMessage("Add record in {0}")
    String newRecordInSubForm(String subFormLabel);

    @DefaultMessage("Edit record in {0}")
    String editRecordInSubForm(String subFormLabel);

    @DefaultMessage("Record in {0}")
    @Meaning("Label for a record in a subform, for example: 'Record in weekly subform'")
    String recordInSubForm(String subFormLabel);

    @DefaultMessage("This is a subform of “{0}”.")
    String subFormRecordHeader(String label);

    @DefaultMessage("Subform of {0}")
    String subFormMiniRecordHeader(String label);

    @DefaultMessage("Format as {0}")
    @Meaning("Placeholder for text fields with an input mask")
    String inputMaskPlaceholder(String inputMask);

    @DefaultMessage("Version {0}")
    String version(String version);

    @DefaultMessage("Original name: “{0}”")
    @Meaning("Shows the original name of field after it has been named in the pivot table")
    String originalName(String label);

    @DefaultMessage("Continue as {0}")
    String offlineLoginButton(String email);

    @DefaultMessage("You are online, but {0} changes are still queued to send to the server")
    @AlternateMessage({ "one", "You are online, but one change is still queued to send to the server" })
    String onlinePendingChanges(@PluralCount int count);

    @DefaultMessage("{0} pending changes will be sent to the server when you reconnect.")
    @AlternateMessage({ "one", "One pending change will be sent to the server when you reconnect." })
    String offlinePendingChanges(@PluralCount int count);

    @DefaultMessage("{0} pending")
    String offlinePendingChangesShort(@PluralCount int count);

    @DefaultMessage("Offline download of database “{0}” failed. It will be downloaded when a connection becomes available.")
    String offlineDownloadFailed(String label);

    @DefaultMessage("Database “{0}” is downloaded and ready to use offline.")
    String offlineDownloadSucceeded(String label);

    @DefaultMessage("{0} Users")
    @AlternateMessage({ "one", "1 User" })
    String userCount(@PluralCount int count);

    @DefaultMessage("Formula: {0}")
    String formulaLabel(String formulaString);

    @DefaultMessage("After this, you won''t be able to access {0} databases that you own.")
    @AlternateMessage({ "one", "After this, you won''t be able to access the database you own." })
    String trialEndConsequences(@PluralCount int databaseCount);

    @DefaultMessage("Your trial account is expiring {0}.")
    String trialExpiringIn(String timeString);

    @DefaultMessage("in {0} hours")
    String inHours(@PluralCount int hours);

    @DefaultMessage("in {0} days")
    String inDays(@PluralCount int days);

    @DefaultMessage("in {0} weeks")
    String inWeeks(@PluralCount int weeks);

    @DefaultMessage("Please <a href=\"{0}\">contact us</a> to extend your access.")
    SafeHtml contactUsToExtendAccess(String emailAddress);

    @DefaultMessage("And {0} more")
    String andMoreOptions(int numberOfOptions);

    @DefaultMessage("You may only add up to {0} fields")
    String maxNumberOfFields(int number);

    @DefaultMessage("You may only add up to {0} options")
    String maxNumberOfOptions(int number);

    @DefaultMessage("You may only add up to {0} key fields")
    String maxNumberOfKeyFields(int number);

    @DefaultMessage("You may only add up to {0} nested subforms")
    String maxSubformDepth(int number);

    @DefaultMessage("Please enter a label name with {0} characters or less")
    String maxFieldLabelLength(int number);

    @DefaultMessage("Please enter a form name with {0} characters or less")
    String maxFormNameLength(int number);

    @DefaultMessage("A code may contain only up to {0} characters")
    String maxFieldCodeLength(int number);

    @DefaultMessage("A description may contain only up to {0} characters")
    String maxFieldDescriptionLength(int number);

    @DefaultMessage("Please enter an option label with {0} characters or less")
    String maxOptionLabelLength(int number);

    @DefaultMessage("This links expires in {0} minutes.")
    String linkExpires(String minutes);

    @DefaultMessage("Edit {0} subform")
    String editSubFormBreadcrumb(String subformLabel);

    @DefaultMessage("Dear {0},")
    String emailGreeting(String name);

    @DefaultMessage("For security, this request was received from a {0} device using {1} in {2}.")
    String passwordResetSecurity(String deviceName, String browserName, String location);

    @DefaultMessage("All times are {0}.")
    String timeZone(String name);

    @DefaultMessage("Join the {0} database on ActivityInfo")
    String inviteHeader(String databaseLabel);

    @DefaultMessage("{0} has invited you to access the {1} database. To confirm your account, click:")
    String inviteIntro(String user, String databaseLabel);

    @DefaultMessage("You have been invited to be an owner of the ActivityInfo account “{0}”. To complete your user registration, click:")
    String inviteAccountOwner(String accountName);

    @DefaultMessage("The ActivityInfo account “{0}” has been closed.")
    String accountClosedMessage(String accountName);

    @DefaultMessage("{0} has invited you to access the {1} database.")
    String inviteExistingIntro(String user, String databaseLabel);

    @DefaultMessage("{0} has invited you to access the {1} database. Use this link to create an ActivityInfo account and get started.")
    String invitePreheader(String user, String databaseLabel);

    @DefaultMessage("{0} has invited you to access the {1} database.")
    String inviteExistingPreheader(String user, String databaseLabel);

    @DefaultMessage("{0} - invitation to join")
    String inviteSubject(String databaseLabel);

    @DefaultMessage("Passwords must be at least {0} characters.")
    String passwordMinimumLength(int minimumPasswordLength);

    @DefaultMessage("not {0}")
    String filterNot(String label);

    @DefaultMessage("Missing required column “{0}”")
    String missingColumn(String columnName);

    @DefaultMessage("Unknown column “{0}”")
    String unknownColumn(String header);

    @DefaultMessage("In row {0}, there is invalid field type “{1}”")
    String invalidFieldType(int row, String type);

    @DefaultMessage("{0,number,#}W{1}-W{2}")
    String fortnightFormat(int year, int firstWeek, int secondWeek);

    @DefaultMessage("Copy of {0}")
    String copyOf(String name);

    @DefaultMessage("“{0}” is not a valid email domain.")
    String invalidEmailDomain(String domain);

    @DefaultMessage("{0} ({1}) already has an ActivityInfo account.")
    String accountExists(String name, String email);

    @DefaultMessage("The billing account “{0}” has {1} unique users across {2} databases, exceeding its limit of {3} users. You cannot add new users.")
    String userLimitExceededMultipleDatabases(String accountName, int userCount, int databaseCount, int userLimit);

    @DefaultMessage("This trial account has {0} unique users across {1} databases, exceeding its limit of {2} users. You cannot add new users.")
    String trialUserLimitExceededMultipleDatabases(int userCount, int databaseCount, int userLimit);

    @DefaultMessage("The billing account “{0}” has exceeded its limit of {1} users. You cannot add new users.")
    String userLimitExceeded(String accountName, int userLimit);

    @DefaultMessage("This trial account has exceeded its limit of {0} users. You cannot add new users.")
    String trialUserLimitExceeded(int userLimit);

    @DefaultMessage("{0} has revoked your access to the database “{1}” on ActivityInfo. You will no longer be able to access this database.")
    String deletedFromDatabase(String deletingUserName, String databaseLabel);

    @DefaultMessage("A member of our support team, {0}, has added themselves to your database “{1}” on ActivityInfo. They will have access only as long as needed to provide you with support, and you can review their actions in the Audit Log.")
    String supportAccessClaimed(String supportUserName, String databaseLabel);

    @DefaultMessage("{0} has transferred your role in the billing account “{1}” and ownership of all of your databases to {2}. You will no longer be able to access these databases, or add new databases.")
    String billingRoleTransferred(String executingUserName, String billingAccountName, String transfereeName);

    @DefaultMessage("{0} has transferred ownership of the database “{1}” from {2} to you.")
    String databaseOwnershipTransferredTo(String executingUserName, String databaseName, String previousOwnerName);

    @DefaultMessage("{0} has transferred ownership of your database “{1}” to {2}. You will no longer be able to access this database.")
    String databaseOwnershipTransferredAway(String executingUserName, String databaseName, String newOwnerName);

    @DefaultMessage("{0} has transferred ownership of the billing account “{1}” from {2} to you.")
    String billingAccountOwnershipTransferred(String executingUserName, String billingAccountName, String previousOwnerName);

    @DefaultMessage("{0} - access revoked")
    String deletedFromDatabaseSubject(String databaseLabel);

    @DefaultMessage("{0} - support access notification")
    String supportAccessClaimedSubject(String databaseLabel);

    @DefaultMessage("If you have questions about this change, you can contact {0} by replying to this email.")
    String deletedFromDatabaseQuestions(String name);

    @DefaultMessage("The form “{0}” is referenced by role “{1}” and cannot be deleted.")
    String formReferencedByRole(String form, String role);

    @DefaultMessage("Your trial account expires on {0}. After this you will not be able to access databases you own.")
    String trialEndSummary(String expirationDate);

    @DefaultMessage("Your {0} subscription includes {1} users and will renew automatically on {2}.")
    String automaticCollectionSummary(String planName, int userLimit, String renewalDate);

    @DefaultMessage("Your {0} subscription includes {1} users and will expire on {2}. Please contact us to renew your subscription before this date in order to avoid interruption in service.")
    String manualCollectionSummary(String planName, int userLimit, String renewalDate);

    @DefaultMessage("You have added {0} unique users in {1} databases.")
    String userCountStatus(int userCount, int databaseCount);

    @DefaultMessage("Shared by {0}")
    String sharedBy(String ownerName);

    @DefaultMessage("Added {0} “{1}”")
    String addedResource(String resourceType, String label);

    @DefaultMessage("Added subform “{0}” in form “{1}”")
    String addedSubform(String label, String parentLabel);

    @DefaultMessage("Renamed {0} from “{1}” to “{2}”")
    String renamedResource(String resourceType, String old, String newLabel);

    @DefaultMessage("Updated visibility of {0} “{1}” from {2} to {3}")
    String updatedVisibility(String resourceType, String label, String oldVisibility, String newVisibility);

    @DefaultMessage("Deleted {0} “{1}”")
    String deletedResource(String resourceType, String label);

    @DefaultMessage("Updated date range of lock “{0}” on {1} “{2}”")
    String updatedLockDateRangeOnResource(String lockLabel, String resourceType, String resourceLabel);

    @DefaultMessage("Updated date range of lock “{0}”")
    String updatedLockDateRange(String lockLabel);

    @DefaultMessage("Updated role “{0}”")
    String updatedRole(String label);

    @DefaultMessage("Moved {0} “{1}” from “{2}” to “{3}”")
    String movedResource(String resourceType, String resourceLabel, String oldFolder, String updatedFolder);

    @DefaultMessage("Added user {0}")
    String addedUser(String userName);

    @DefaultMessage("Deleted user {0}")
    String deletedUser(String userName);

    @DefaultMessage("User {0} left the database")
    String userLeft(String userName);

    @DefaultMessage("Updated the permissions of user {0}")
    String updatedUserPermissions(String userName);

    @DefaultMessage("Login via your {0} account")
    String loginVia(String domain);

    @DefaultMessage("Before we get started, please log in with your {0} account.")
    String confirmSsoIntro(String idpLabel);

    @DefaultMessage("This will link your ActivityInfo account to your {0} account so that you don''t have to remember a separate password for ActivityInfo.")
    String confirmSsoExplanation(String idpLabel);

    @DefaultMessage("You have successfully connected your {0} account.")
    String ssoConnectionSuccess(String idpLabel);

    @DefaultMessage("Your ActivityInfo account is linked to your {0} account, so you do not need an ActivityInfo password. Please log in with your {0} account.")
    String passwordResetSso(String idpLabel);

    @DefaultMessage("Your ActivityInfo account is already linked to your {0} account. In order to protect your account, this cannot be changed.")
    String alreadyLinked(String idpLabel);

    @DefaultMessage("{0} additional records are selected but not shown here.")
    String moreRecordsSelectedButNotShown(int recordNumber);

    @DefaultMessage("{0} out of {1} selected records are locked and can not be deleted.")
    String lockedRecords(int size, int lockedRecordNum);

    @DefaultMessage("A personal access token ({0}) was added to your account.")
    String apiTokenAddedIntro(String tokenLabel);

    @DefaultMessage("From now on, when you log into ActivityInfo, you will be directed to the {0} login page to enter your {0} password. You will no longer need a separate ActivityInfo password.")
    String ssoNotification(String idpLabel);

    @DefaultMessage("For security reasons, we only allow {0} accounts to be connected to your {1} account.")
    String domainPolicy(String domain, String idpLabel);

    @DefaultMessage("For security reasons, we do not allow {0} accounts to be connected to {1} accounts.")
    String authoritativeDomain(String domain, String idpLabel);

    @DefaultMessage("Population Density ({0})")
    String populationDensity(int year);

    @DefaultMessage("Are you sure you want to leave database “{0}”?")
    String sureLeaveDatabase(String databaseName);

    @DefaultMessage("This code is valid for {0} minutes.")
    String oneTimeCodeValidity(int minutesValid);

    @DefaultMessage("This is a test email to verify that your ActivityInfo Server is correctly configured to send email notifications.")
    String testEmailMessage();

    @DefaultMessage("You must be invited to use ActivityInfo. Please ask a database administrator to invite you to access a database with the email address “{0}”.")
    String notInvitedExplanation(String email);

    @DefaultMessage("Added a record in {0}")
    String addedRecord(String formLabel);

    @DefaultMessage("Deleted a record in {0}")
    String deletedRecord(String formLabel);

    @DefaultMessage("Updated a record in {0}")
    String updatedRecord(String formLabel);

    @DefaultMessage("Recovered a deleted record in {0}")
    String recoveredRecord(String formLabel);
}
