/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.i18n.shared;

import java.util.Locale;

/**
 * Unicode Control Characters for overiding Bi-directional text layout.
 *
 * <p>See <a href="https://developer.mozilla.org/en-US/docs/Web/Guide/Unicode_Bidrectional_Text_Algorithm">The Unicode Bidirectional Text Algorithm</a></p>
 */
public class UnicodeBidi {

    /**
     * Right-to-left Isolate.
     *
     * <p>Sets the base direction to RTL, isolating the embedded content from the surrounding text</p>
     */
    public static final String RLI = "\u2067";


    /**
     * Left-to-right isolate (LRI)
     *
     * <p>Sets the base direction to LTR, isolating the embedded content from the surrounding text</p>
     */
    public static final String LRI = "\u2069";

    /**
     * Pop Directional Isolate (PDI)
     *
     * <P>Used for RLI, LRI, or FSI</P>
     */
    public static final String PDI = "\u2069";

    public static boolean isRTL(Locale locale) {
        switch (locale.getLanguage()) {
            case "ar":
            case "fa":
                return true;
            default:
                return false;
        }
    }
}
