/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.io.csv;

import java.io.Closeable;
import java.io.IOException;

public class CsvWriter implements TableWriter {

    /**
     * Writes a byte order mark that should help spreadsheet software detect the UTF-8 character set.
     */
    public static final char BYTEORDER_MARK = '\ufeff';

    public static final String MIME_TYPE = "text/csv";

    public static final String MIME_TYPE_UTF8 = "text/csv;charset=UTF-8";

    public static final String DELIMITER = ",";
    public static final String DOUBLE_QUOTE = "\"";
    public static final String CARRIAGE_RETURN = "\r";
    public static final String LINE_ENDING = "\n";

    public static final String ESCAPED_DOUBLE_QUOTE = "\"\"";

    private Appendable writer;

    public CsvWriter() throws IOException {
        this(new StringBuilder());
    }

    public CsvWriter(Appendable writer) throws IOException {
        this.writer = writer;
        this.writer.append(BYTEORDER_MARK);
    }

    @Override
    public void setColumnWidth(int column, int width) {
        // NOOP
    }

    @Override
    public void setColumnFormat(int column, String pattern) {
        // NOOP
    }

    @Override
    public void writeLine(Object... columns) throws IOException {
        for (int i = 0; i != columns.length; ++i) {
            if (i > 0) {
                writer.append(DELIMITER);
            }
            Object val = columns[i];
            if (val != null) {
                if (val instanceof String) {
                    String escaped = escape((String) val);
                    writer.append(enquote(escaped));
                } else {
                    writer.append(val.toString());
                }
            }
        }
        writer.append(LINE_ENDING);
    }

    private String escape(String data) {
        return data.replace(DOUBLE_QUOTE, ESCAPED_DOUBLE_QUOTE);
    }

    private String enquote(String data) {
        return DOUBLE_QUOTE + data + DOUBLE_QUOTE;
    }

    @Override
    public String toString() {
        return writer.toString();
    }

    @Override
    public void close() throws IOException {
        if(writer instanceof Closeable) {
            ((Closeable) writer).close();
        }
    }
}
