/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.io.match.shared.coord;

import org.activityinfo.i18n.shared.I18N;

public enum CoordinateAxis {
    LATITUDE {
        @Override
        public double getMaximumValue() {
            return 90;
        }

        @Override
        public String getLocalizedName() {
            return I18N.CONSTANTS.latitude();
        }

        @Override
        public String getLocalizedShortName() {
            return I18N.CONSTANTS.latitudeShort();
        }

        @Override
        public String getDegreesOutOfBoundsMessage() {
            return I18N.CONSTANTS.latitudeOutOfBounds();
        }

        @Override
        public String getPositiveHemisphereCharacters() {
            return I18N.CONSTANTS.northHemiChars();
        }

        @Override
        public String getNegativeHemisphereCharacters() {
            return I18N.CONSTANTS.southHemiChars();
        }

        @Override
        public String getLocalizedPlaceholder() {
            return I18N.CONSTANTS.enterLatitude();
        }
    },

    LONGITUDE {


        @Override
        public double getMaximumValue() {
            return 180;
        }

        @Override
        public String getLocalizedName() {
            return I18N.CONSTANTS.longitude();
        }

        @Override
        public String getLocalizedShortName() {
            return I18N.CONSTANTS.longitudeShort();
        }

        @Override
        public String getDegreesOutOfBoundsMessage() {
            return I18N.CONSTANTS.longitudeOutOfBounds();
        }

        @Override
        public String getPositiveHemisphereCharacters() {
            return I18N.CONSTANTS.eastHemiChars();
        }

        @Override
        public String getNegativeHemisphereCharacters() {
            return I18N.CONSTANTS.westHemiChars();
        }

        @Override
        public String getLocalizedPlaceholder() {
            return I18N.CONSTANTS.enterLongitude();
        }

    };
    
    public abstract double getMaximumValue();

    public abstract String getLocalizedName();

    public abstract String getDegreesOutOfBoundsMessage();

    public double getMinimumValue() {
        return -getMaximumValue();
    }

    /**
     * 
     * @return the range of characters used denote the positive hemisphere for this
     * axis in the current Locale. For example, 'N' is used to indicate the positive hemisphere for latitude.
     */
    public abstract String getPositiveHemisphereCharacters();

    /**
     * 
     * @return the range of characters used to denote the negative hemisphere for this
     * axis in the current Locale. For example, 'S', is used to indicate the negative hemisphere for latitude.
     */
    public abstract String getNegativeHemisphereCharacters();

    public abstract String getLocalizedShortName();

    public abstract String getLocalizedPlaceholder();
}
