/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.json;

import org.activityinfo.json.impl.JreJsonFactory;
import org.activityinfo.json.impl.JsonReflection;
import org.activityinfo.json.impl.JsonUtil;
import org.checkerframework.checker.nullness.qual.NonNull;
import org.checkerframework.checker.nullness.qual.Nullable;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Vends out implementation of JsonFactory.
 */
public class Json {

    public static JsonValue createFromNullable(String string) {
        return instance().createFromNullable(string);
    }

    public static JsonValue create(String string) {
        return instance().create(string);
    }

    public static JsonValue create(boolean bool) {
        return instance().create(bool);
    }

    public static JsonValue createArray() {
        return instance().createArray();
    }


    public static JsonValue createStringArray(String[] strings) {
        return instance().createStringArray(strings);
    }

    public static JsonValue createStringArray(Collection<String> strings) {
        return instance().createStringArray(strings.toArray(new String[0]));
    }

    public static JsonValue createNull() {
        return instance().createNull();
    }

    public static JsonValue create(double number) {
        return instance().create(number);
    }

    public static JsonValue create(int number) {
        return instance().create(number);
    }

    public static JsonValue createObject() {
        return instance().createObject();
    }

    public static JsonFactory instance() {
        return new JreJsonFactory();
    }

    public static JsonValue parse(String jsonString) {
        return instance().parse(jsonString);
    }

    public static String stringify(Object value) {
        return JsonUtil.stringify(Json.toJson(value));
    }

    public static String stringify(JsonValue jsonValue) {
        return JsonUtil.stringify(jsonValue);
    }

    public static String stringify(JsonValue jsonValue, int indent) {
        return JsonUtil.stringify(jsonValue, indent);
    }

    public static JsonValue toJson(@NonNull Object value) {
        if(value instanceof JsonSerializable) {
            return ((JsonSerializable) value).toJson();

        } else if(value instanceof Collection) {
            JsonValue array = Json.createArray();
            for (Object element : ((Collection) value)) {
                if(element == null) {
                    array.add(Json.createNull());
                } else {
                    array.add(Json.toJson(element));
                }
            }
            return array;

        } else {
            return JsonReflection.toJson(value);
        }
    }

    public static <T> @Nullable T fromJson(Class<T> clazz, JsonValue object) {
        return JsonReflection.fromJson(clazz, object);
    }

    public static <T> List<@Nullable T> fromJsonArray(Class<T> componentClass, JsonValue array) throws JsonMappingException {
        List<@Nullable T> list = new ArrayList<>();
        for (int i = 0; i < array.length(); i++) {
            list.add(Json.fromJson(componentClass, array.get(i)));
        }
        return list;
    }


    public static JsonValue toJsonArray(Iterable<? extends JsonSerializable> objects) {
        JsonValue array = Json.createArray();
        for (JsonSerializable object : objects) {
            array.add(object.toJson());
        }
        return array;
    }
}
