/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.json;

import org.checkerframework.checker.nullness.qual.Nullable;

/**
 * Factory interface for parsing and creating JSON objects.
 */
public interface JsonFactory {

    /**
     * Create a JsonString from a Java String.
     *
     * @param string a Java String
     * @return the parsed JsonString
     */
    JsonValue create(String string);

    /**
     * Create a JsonValue from a Java String that may be null
     *
     * @param string a Java String
     * @return either JsonNull or JsonString
     */
    JsonValue createFromNullable(@Nullable String string);

    /**
     * Create a JsonNumber from a Java double.
     *
     * @param number a Java double
     * @return the parsed JsonNumber
     */
    JsonValue create(double number);

    /**
     * Create a JsonBoolean from a Java boolean.
     *
     * @param bool a Java boolean
     * @return the parsed JsonBoolean
     */
    JsonValue create(boolean bool);

    /**
     * Create an empty JsonArray.
     *
     * @return a new JsonArray
     */
    JsonValue createArray();

    JsonValue createStringArray(String[] strings);


    /**
     * Create a JsonNull.
     *
     * @return a JsonNull instance
     */
    JsonValue createNull();

    /**
     * Create an empty JsonObject.
     *
     * @return a new JsonObject
     */
    JsonValue createObject();

    /**
     * Parse a String in JSON format and return a JsonValue of the appropriate
     * type.
     *
     * @param jsonString a String in JSON format
     * @return a parsed JsonValue
     */
    JsonValue parse(String jsonString) throws JsonException;

}
