/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.json.impl;

import org.activityinfo.json.JsonType;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 * Server-side implementation of JsonBoolean.
 */
class JreJsonBoolean extends JreJsonValue {

    private static final long serialVersionUID = 1L;

    private transient boolean bool;

    public JreJsonBoolean(boolean bool) {
        this.bool = bool;
    }

    @Override
    public boolean asBoolean() {
        return getBoolean();
    }

    @Override
    public double asNumber() {
        return getBoolean() ? 1 : 0;
    }

    @Override
    public String asString() {
        return Boolean.toString(getBoolean());
    }

    @Override
    public boolean isBoolean() {
        return true;
    }

    @Override
    public boolean isJsonPrimitive() {
        return true;
    }

    public boolean getBoolean() {
        return bool;
    }

    private Object getObject() {
        return getBoolean();
    }

    public JsonType getType() {
        return JsonType.BOOLEAN;
    }

    @Override
    public void traverse(JsonVisitor visitor, JsonContext ctx) {
        visitor.visit(getBoolean(), ctx);
    }

    public String toJson() throws IllegalStateException {
        return String.valueOf(bool);
    }

    @com.google.gwt.core.shared.GwtIncompatible
    private void readObject(ObjectInputStream stream)
            throws IOException, ClassNotFoundException {
        JreJsonBoolean instance = parseJson(stream);
        this.bool = instance.bool;
    }

    @com.google.gwt.core.shared.GwtIncompatible
    private void writeObject(ObjectOutputStream stream) throws IOException {
        stream.writeObject(toJson());
    }

}
