/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.json.impl;

import org.activityinfo.json.JsonException;
import org.activityinfo.json.JsonFactory;
import org.activityinfo.json.JsonValue;
import org.checkerframework.checker.nullness.qual.Nullable;


/**
 * Implementation of JsonFactory interface using org.json library.
 */
public class JreJsonFactory implements JsonFactory {

    public JsonValue create(String string) {
        assert string != null;
        return new JreJsonString(string);
    }

    @Override
    public JsonValue createFromNullable(@Nullable String string) {
        if(string == null) {
            return createNull();
        } else {
            return create(string);
        }
    }

    public JsonValue create(double number) {
        return new JreJsonNumber(number);
    }

    public JsonValue create(boolean bool) {
        return new JreJsonBoolean(bool);
    }

    public JsonValue createArray() {
        return new JreJsonArray(this);
    }

    @Override
    public JsonValue createStringArray(String[] strings) {
        JsonValue array = createArray();
        for (String string : strings) {
            array.add(create(string));
        }
        return array;
    }

    public JsonValue createNull() {
        return JreJsonNull.NULL_INSTANCE;
    }

    public JsonValue createObject() {
        return new JreJsonObject(this);
    }

    public JsonValue parse(String jsonString) throws JsonException {
        if (jsonString.startsWith("(") && jsonString.endsWith(")")) {
            // some clients send in (json) expecting an eval is required
            jsonString = jsonString.substring(1, jsonString.length() - 1);
        }
        return new JsonTokenizer(this, jsonString).nextValue();
    }
}
