/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.json.impl;

import org.activityinfo.json.JsonType;
import org.checkerframework.checker.nullness.qual.Nullable;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Objects;

/**
 * Server-side implementation of JsonNumber.
 */
public class JreJsonNumber extends JreJsonValue  {

    private static final long serialVersionUID = 1L;

    private transient double number;

    public JreJsonNumber(double number) {
        this.number = number;
    }

    @Override
    public boolean asBoolean() {
        return Double.isNaN(getNumber()) || Math.abs(getNumber()) == 0.0 ? false : true;
    }

    @Override
    public double asNumber() {
        return getNumber();
    }


    @Override
    public String asString() {
        return toJson();
    }

    @Override
    public boolean isJsonPrimitive() {
        return true;
    }

    public double getNumber() {
        return number;
    }


    private Object getObject() {
        return getNumber();
    }

    public JsonType getType() {
        return JsonType.NUMBER;
    }

    @Override
    public void traverse(JsonVisitor visitor, JsonContext ctx) {
        visitor.visit(getNumber(), ctx);
    }

    public String toJson() {
        if (Double.isInfinite(number) || Double.isNaN(number)) {
            return "null";
        }
        String toReturn = String.valueOf(number);
        if (toReturn.endsWith(".0")) {
            toReturn = toReturn.substring(0, toReturn.length() - 2);
        }
        return toReturn;
    }

    @Override
    public boolean equals(@Nullable Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        JreJsonNumber that = (JreJsonNumber) o;
        return Double.compare(that.number, number) == 0;
    }

    @Override
    public int hashCode() {

        return Objects.hash(number);
    }

    @Override
    public long asLong() {
        return (long)number;
    }

    @com.google.gwt.core.shared.GwtIncompatible
    private void readObject(ObjectInputStream stream)
            throws IOException, ClassNotFoundException {
        JreJsonNumber instance = parseJson(stream);
        this.number = instance.number;
    }

    @com.google.gwt.core.shared.GwtIncompatible
    private void writeObject(ObjectOutputStream stream) throws IOException {
        stream.writeObject(toJson());
    }
}
