/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.json.impl;

import org.activityinfo.json.JsonType;
import org.activityinfo.json.JsonValue;
import org.checkerframework.checker.nullness.qual.Nullable;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Objects;

/**
 * Server-side implementation of JsonString.
 */
public class JreJsonString extends JreJsonValue implements JsonValue {

    private static final long serialVersionUID = 1L;

    private transient String string;

    public JreJsonString(String string) {
        this.string = string;
    }

    @Override
    public boolean asBoolean() {
        return !getString().isEmpty();
    }

    @Override
    public double asNumber() {
        try {
            if (asString().isEmpty()) {
                return 0.0;
            } else {
                return Double.parseDouble(asString());
            }
        } catch (NumberFormatException nfe) {
            return Double.NaN;
        }
    }

    @Override
    public String asString() {
        return getString();
    }

    private Object getObject() {
        return getString();
    }

    public String getString() {
        return string;
    }

    public JsonType getType() {
        return JsonType.STRING;
    }

    @Override
    public boolean isJsonPrimitive() {
        return true;
    }

    @Override
    public void traverse(JsonVisitor visitor, JsonContext ctx) {
        visitor.visit(getString(), ctx);
    }

    public String toJson() throws IllegalStateException {
        return JsonUtil.quote(getString());
    }

    @Override
    public boolean isString() {
        return true;
    }

    @Override
    public boolean equals(@Nullable Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        JreJsonString that = (JreJsonString) o;
        return Objects.equals(string, that.string);
    }

    @Override
    public int hashCode() {

        return Objects.hash(string);
    }

    @com.google.gwt.core.shared.GwtIncompatible
    private void readObject(ObjectInputStream stream) throws IOException,
            ClassNotFoundException {
        JreJsonString instance = parseJson(stream);
        this.string = instance.string;
    }

    @com.google.gwt.core.shared.GwtIncompatible
    private void writeObject(ObjectOutputStream stream) throws IOException {
        stream.writeObject(toJson());
    }
}
