/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.json.impl;

import org.activityinfo.json.*;
import org.checkerframework.checker.nullness.qual.Nullable;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.Collections;
import java.util.Map;

/**
 * JRE (non-Client) implementation of JreJsonValue.
 */
public abstract class JreJsonValue implements JsonValue {


    @Override
    public boolean isNumber() {
        return getType() == JsonType.NUMBER;
    }

    @GwtIncompatible
    protected static <T extends JreJsonValue> T parseJson(ObjectInputStream stream)
            throws ClassNotFoundException, IOException {
        String jsonString = (String) stream.readObject();
        return (T) Json.instance().parse(jsonString);
    }

    @Override
    public String toString() {
        return toJson();
    }

    @Override
    public boolean asBoolean() {
        throw conversionException(JsonType.BOOLEAN);
    }

    @Override
    public double asNumber() {
        throw conversionException(JsonType.NUMBER);
    }

    @Override
    public final int asInt() {
        return (int)asNumber();
    }

    @Override
    public long asLong() {
        throw conversionException(JsonType.NUMBER);
    }

    @Override
    public @Nullable String asString() {
        throw conversionException(JsonType.STRING);
    }


    @Override
    public boolean isJsonNull() {
        return false;
    }

    @Override
    public boolean isJsonArray() {
        return false;
    }

    @Override
    public boolean isJsonPrimitive() {
        return false;
    }

    @Override
    public boolean isJsonObject() {
        return false;
    }

    @Override
    public boolean isString() {
        return false;
    }

    @Override
    public boolean isBoolean() {
        return false;
    }


    @Override
    public boolean getBoolean(String key) {
        throw conversionException(JsonType.OBJECT);
    }

    @Override
    public double getNumber(String key) {
        throw conversionException(JsonType.OBJECT);
    }

    @Override
    public JsonValue get(String key) {
        throw conversionException(JsonType.OBJECT);
    }

    @Override
    public @Nullable String getString(String key) {
        throw conversionException(JsonType.OBJECT);
    }

    @Override
    public void put(String key, JsonValue value) {
        throw conversionException(JsonType.OBJECT);
    }

    @Override
    public void put(String key, @Nullable String value) {
        throw conversionException(JsonType.OBJECT);
    }

    @Override
    public void put(String key, double value) {
        throw conversionException(JsonType.OBJECT);
    }

    @Override
    public void put(String key, boolean bool) {
        throw conversionException(JsonType.OBJECT);
    }

    @Override
    public boolean hasKey(String key) {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public String[] keys() {
        return new String[0];
    }

    @Override
    public void remove(String key) {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public void add(String key, JsonValue value) {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public boolean getBoolean(int index) {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public double getNumber(int index) {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public JsonValue get(int index) {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public @Nullable String getString(int index) {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public int length() {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public void set(int index, JsonValue value) {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public void set(int index, String string) {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public void set(int index, double number) {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public void set(int index, boolean bool) {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public void add(JsonValue value) {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public String getRequiredString(String key) throws JsonException {
        throw new JsonException("Cannot read string member '" + key + "' from value of type " + getType().name());
    }

    @Override
    public Iterable<JsonValue> values() {
        return Collections.emptyList();
    }

    @Override
    public Iterable<Map.Entry<String, JsonValue>> entrySet() {
        return Collections.emptyList();
    }

    public abstract void traverse(JsonVisitor visitor, JsonContext ctx);


    private JsonException conversionException(JsonType toType) {
        return new JsonException("Cannot convert " + getType().name() + " to " + toType);
    }

}
