/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.json.impl;

import com.google.gwt.core.client.JavaScriptObject;
import javaemul.internal.annotations.UncheckedCast;
import jsinterop.base.Js;
import org.activityinfo.json.JsonException;
import org.activityinfo.json.JsonFactory;
import org.activityinfo.json.JsonValue;
import org.checkerframework.checker.nullness.qual.Nullable;

/**
 * JSNI based implementation of JsonFactory.
 */
public class JsJsonFactory implements JsonFactory {


    /**
     * Performs unchecked cast to lefthand-side type. You should always prefer regular casting over
     * this (unless you know what you are doing!).
     */
    @UncheckedCast
    @SuppressWarnings({"TypeParameterUnusedInFormals", "unchecked"})
    static <T> T uncheckedCast(@Nullable Object obj) {
        return (T) obj;
    }


    private static native <T extends JsonValue> T parse0(String jsonString) /*-{
        // assume Chrome, safe and non-broken JSON.parse impl
        var value = $wnd.JSON.parse(jsonString);
        // box for DevMode, not ProdMode
        return @com.google.gwt.core.client.GWT::isScript()() || value == null ? value : Object(value);
    }-*/;

    /*
     * MAGIC: primitive boolean cast to object interface.
     */
    public static native JsJsonValue createBoolean(boolean bool) /*-{
        return bool;
    }-*/;

    /*
     * MAGIC: primitive number cast to object interface.
     */
    static native JsJsonValue createNumber(double number) /*-{
        return number;
    }-*/;


    @Override
    public JsonValue createArray() {
        return (JsJsonValue) JavaScriptObject.createArray();
    }

    @Override
    public JsonValue createStringArray(String[] strings) {
        return Js.uncheckedCast(strings);
    }

    public JsonValue create(String string) {
        return Js.uncheckedCast(string);
    }

    @Override
    public JsonValue createFromNullable(@Nullable String string) {
        return JsJsonFactory.uncheckedCast(string);
    }

    public JsonValue create(double number) {
        return createNumber(number);
    }

    public JsonValue create(boolean bool) {
        return createBoolean(bool);
    }

    @Override
    public native JsonValue createNull() /*-{
        return null;
    }-*/;

    public JsJsonValue createObject() {
        return JavaScriptObject.createObject().cast();
    }

    public JsonValue parse(String jsonString) throws JsonException {
        try {
            return parse0(jsonString);
        } catch (Exception e) {
            throw new JsonException("Can't parse " + jsonString);
        }
    }
}
