/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.account;

import org.activityinfo.promise.HttpError;

public enum AddBillingAccountUserResult implements HttpError {

    OK(200),

    BILLING_ACCOUNT_NOT_FOUND(404),

    /**
     * The billing account associated with this database has been
     * suspended so no additional users can be added.
     */
    BILLING_ACCOUNT_SUSPENDED(BILLING_402),

    /**
     * The user limit of the billing account associated with this database
     * has been exceeded so no additional users can be added.
     */
    USER_LIMIT_EXCEEDED(BILLING_402),

    /**
     * You do not have sufficient permission to add a user to the billing account
     */
    PERMISSION_DENIED(FORBIDDEN_403),

    /**
     * A user with this email address has already been added to the billing account.
     */
    USER_ALREADY_ADDED(BAD_REQUEST_400),

    INTERNAL_SERVER_ERROR(500);

    private int httpStatusCode;

    AddBillingAccountUserResult(int httpStatusCode) {
        this.httpStatusCode = httpStatusCode;
    }

    @Override
    public int getHttpStatusCode() {
        return httpStatusCode;
    }
}
