package org.activityinfo.model.account;

import java.lang.String;
import javax.annotation.Nonnull;

public final class AddTestUserRequestBuilder {
  private String email = "";

  private String password = "";

  private String name = "";

  private PlatformRole platformRole = PlatformRole.NONE;

  private String locale = "";

  private boolean trial;

  public AddTestUserRequestBuilder() {
    email = "";
    password = "";
    name = "";
    platformRole = PlatformRole.NONE;
    locale = "";
  }

  public AddTestUserRequestBuilder(AddTestUserRequest toCopy) {
    this.email = toCopy.getEmail();
    this.password = toCopy.getPassword();
    this.name = toCopy.getName();
    this.platformRole = toCopy.getPlatformRole();
    this.locale = toCopy.getLocale();
    this.trial = toCopy.isTrial();
  }

  @Nonnull
  public AddTestUserRequestBuilder email(@Nonnull String email) {
    this.email = email;
    return this;
  }

  @Nonnull
  public AddTestUserRequestBuilder password(@Nonnull String password) {
    this.password = password;
    return this;
  }

  @Nonnull
  public AddTestUserRequestBuilder name(@Nonnull String name) {
    this.name = name;
    return this;
  }

  @Nonnull
  public AddTestUserRequestBuilder platformRole(@Nonnull PlatformRole platformRole) {
    this.platformRole = platformRole;
    return this;
  }

  @Nonnull
  public AddTestUserRequestBuilder locale(@Nonnull String locale) {
    this.locale = locale;
    return this;
  }

  @Nonnull
  public AddTestUserRequestBuilder trial(boolean trial) {
    this.trial = trial;
    return this;
  }

  @Nonnull
  public AddTestUserRequest build() {
    assert this.email != null : "email is missing";
    assert this.password != null : "password is missing";
    assert this.name != null : "name is missing";
    assert this.platformRole != null : "platformRole is missing";
    assert this.locale != null : "locale is missing";
    return new AddTestUserRequest(email,password,name,platformRole,locale,trial);
  }
}
