package org.activityinfo.model.account;

import java.lang.String;
import java.util.Optional;
import javax.annotation.Nonnull;

public final class AuthenticatedDeviceBuilder {
  private String id = "";

  Optional<DeviceProfile> deviceProfile = Optional.empty();

  private String location = "";

  private double authenticationTime;

  private double expirationTime;

  public AuthenticatedDeviceBuilder() {
    id = "";
    location = "";
  }

  public AuthenticatedDeviceBuilder(AuthenticatedDevice toCopy) {
    this.id = toCopy.getId();
    this.deviceProfile = toCopy.getDeviceProfile();
    this.location = toCopy.getLocation();
    this.authenticationTime = toCopy.getAuthenticationTime();
    this.expirationTime = toCopy.getExpirationTime();
  }

  @Nonnull
  public AuthenticatedDeviceBuilder id(@Nonnull String id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public AuthenticatedDeviceBuilder deviceProfile(@Nonnull DeviceProfile value) {
    this.deviceProfile = Optional.of(value);
    return this;
  }

  @Nonnull
  public AuthenticatedDeviceBuilder deviceProfile(@Nonnull Optional<DeviceProfile> value) {
    this.deviceProfile = value;
    return this;
  }

  @Nonnull
  public AuthenticatedDeviceBuilder location(@Nonnull String location) {
    this.location = location;
    return this;
  }

  @Nonnull
  public AuthenticatedDeviceBuilder authenticationTime(double authenticationTime) {
    this.authenticationTime = authenticationTime;
    return this;
  }

  @Nonnull
  public AuthenticatedDeviceBuilder expirationTime(double expirationTime) {
    this.expirationTime = expirationTime;
    return this;
  }

  @Nonnull
  public AuthenticatedDevice build() {
    assert this.id != null : "id is missing";
    assert this.location != null : "location is missing";
    return new AuthenticatedDevice(id,deviceProfile,location,authenticationTime,expirationTime);
  }
}
