/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.account;

import org.activityinfo.i18n.shared.ApplicationLocale;
import org.activityinfo.model.resource.ResourceId;

import java.io.Serializable;

/**
 * Encapsulates user identity
 */
public class AuthenticatedUser implements Serializable {
    private ResourceId userId;
    private String userEmail;
    private String name;
    private String userLocale;
    private PlatformRole platformRole;
    private AuthenticationScope scope;

    public AuthenticatedUser(ResourceId userId, String userEmail, String name, String userLocale) {
        this(userId, userEmail, name, userLocale, AuthenticationScope.LOGIN);
    }

    public AuthenticatedUser(ResourceId userId, String userEmail, String name, String userLocale, AuthenticationScope scope) {
        this.userId = userId;
        this.userEmail = userEmail;
        this.name = name;
        this.userLocale = userLocale;
        this.platformRole = PlatformRole.NONE;
        this.scope = scope;
    }

    public AuthenticatedUser(ResourceId userId, String userEmail, String name, String userLocale, PlatformRole platformRole, AuthenticationScope scope) {
        this.userId = userId;
        this.userEmail = userEmail;
        this.name = name;
        this.userLocale = userLocale;
        this.platformRole = platformRole;
        this.scope = scope;
    }

    public AuthenticatedUser(UserAccount user) {
        this(user, AuthenticationScope.LOGIN);
    }

    public AuthenticatedUser(UserAccount user, AuthenticationScope scope) {
        this.userId = user.getId();
        this.userEmail = user.getEmail();
        this.userLocale = user.getLocale();
        this.name = user.getName();
        this.platformRole = user.getPlatformRole();
        this.scope = scope;
    }

    public AuthenticatedUser() {
    }


    public ResourceId getUserId() {
        return userId;
    }

    public String getEmail() {
        return userEmail;
    }

    public EmailAddress getEmailAddress() {
        return EmailAddressFormValidator.validate(userEmail).orElseThrow(() -> {
            return new IllegalStateException("User has invalid email adddress " + userEmail);
        });
    }

    public String getName() {
        return name;
    }

    public String getUserLocale() {
        return userLocale;
    }

    public ResourceId getId() {
        return userId;
    }

    public PlatformRole getPlatformRole() {
        return platformRole;
    }

    public AuthenticationScope getScope() {
        return scope;
    }

    public String getFeatures() {
        return "";
    }

    public ApplicationLocale getApplicationLocale() {
        return ApplicationLocale.validateLocale(getUserLocale()).orElse(ApplicationLocale.EN);
    }
}
