/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.account;

import com.google.common.base.Strings;
import com.google.gwt.core.client.JsDate;
import com.google.gwt.core.shared.GWT;
import com.google.gwt.core.shared.GwtIncompatible;
import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.type.time.LocalDate;

import java.time.Instant;
import java.util.Date;

@AutoJson
@AutoBuilder
public class BillingAccount {
    private final long id;
    private final String name;
    private final boolean trial;
    private final double expirationTime;
    private final int userLimit;
    private final int userCount;
    private final int adminUserCount;
    private final int reportingUserCount;
    private final int databaseCount;
    private final double expectedPaymentTime;
    private final BillingAccountStatus status;
    private final boolean staleCounts;
    private final String planName;
    private final boolean automaticCollection;

    public BillingAccount(long id, String name, boolean trial, double expirationTime, int userLimit, int userCount, int adminUserCount, int reportingUserCount, int databaseCount, double expectedPaymentTime, BillingAccountStatus status, boolean staleCounts, String planName, boolean automaticCollection) {
        this.id = id;
        this.name = name;
        this.trial = trial;
        this.expirationTime = expirationTime;
        this.userLimit = userLimit;
        this.userCount = userCount;
        this.adminUserCount = adminUserCount;
        this.reportingUserCount = reportingUserCount;
        this.databaseCount = databaseCount;
        this.expectedPaymentTime = expectedPaymentTime;
        this.status = status;
        this.staleCounts = staleCounts;
        this.planName = planName;
        this.automaticCollection = automaticCollection;
    }

    public long getId() {
        return id;
    }

    public String getName() {
        return Strings.nullToEmpty(name);
    }

    public boolean isTrial() {
        return trial;
    }

    public double getExpirationTime() {
        return expirationTime;
    }

    @GwtIncompatible
    public Instant getExpirationInstant() {
        return Instant.ofEpochMilli((long)expirationTime);
    }

    public String getPlanName() {
        return planName;
    }

    public LocalDate getExpirationLocalDate() {
        return new LocalDate(new Date((long)expirationTime));
    }

    public int getUserLimit() {
        return userLimit;
    }

    public int getUserCount() {
        return userCount;
    }

    public int getAdminUserCount() {
        return adminUserCount;
    }

    public int getReportingUserCount() {
        return reportingUserCount;
    }

    public int getDatabaseCount() {
        return databaseCount;
    }

    public double getExpectedPaymentTime() {
        return expectedPaymentTime;
    }

    public boolean isPaymentExpected() {
        return expectedPaymentTime != 0;
    }

    public BillingAccountStatus getStatus() {
        return status;
    }

    public boolean isStaleCounts() {
        return staleCounts;
    }

    public boolean isUserLimitReached() {
        return userCount >= userLimit;
    }

    public boolean isUserLimitExceeded() {
        return userCount > userLimit;
    }

    public boolean isAutomaticCollection() {
        return automaticCollection;
    }

    public boolean isExpired() {
        if(GWT.isScript()) {
            return expirationTime < JsDate.now();
        } else {
            return expirationTime < System.currentTimeMillis();
        }
    }
}
