package org.activityinfo.model.account;

import java.lang.String;
import javax.annotation.Nonnull;

public final class BillingAccountBuilder {
  private long id;

  private String name = "";

  private boolean trial;

  private double expirationTime;

  private int userLimit;

  private int userCount;

  private int adminUserCount;

  private int reportingUserCount;

  private int databaseCount;

  private double expectedPaymentTime;

  private BillingAccountStatus status = BillingAccountStatus.ACTIVE;

  private boolean staleCounts;

  private String planName = "";

  private boolean automaticCollection;

  public BillingAccountBuilder() {
    name = "";
    status = BillingAccountStatus.ACTIVE;
    planName = "";
  }

  public BillingAccountBuilder(BillingAccount toCopy) {
    this.id = toCopy.getId();
    this.name = toCopy.getName();
    this.trial = toCopy.isTrial();
    this.expirationTime = toCopy.getExpirationTime();
    this.userLimit = toCopy.getUserLimit();
    this.userCount = toCopy.getUserCount();
    this.adminUserCount = toCopy.getAdminUserCount();
    this.reportingUserCount = toCopy.getReportingUserCount();
    this.databaseCount = toCopy.getDatabaseCount();
    this.expectedPaymentTime = toCopy.getExpectedPaymentTime();
    this.status = toCopy.getStatus();
    this.staleCounts = toCopy.isStaleCounts();
    this.planName = toCopy.getPlanName();
    this.automaticCollection = toCopy.isAutomaticCollection();
  }

  @Nonnull
  public BillingAccountBuilder id(long id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public BillingAccountBuilder name(@Nonnull String name) {
    this.name = name;
    return this;
  }

  @Nonnull
  public BillingAccountBuilder trial(boolean trial) {
    this.trial = trial;
    return this;
  }

  @Nonnull
  public BillingAccountBuilder expirationTime(double expirationTime) {
    this.expirationTime = expirationTime;
    return this;
  }

  @Nonnull
  public BillingAccountBuilder userLimit(int userLimit) {
    this.userLimit = userLimit;
    return this;
  }

  @Nonnull
  public BillingAccountBuilder userCount(int userCount) {
    this.userCount = userCount;
    return this;
  }

  @Nonnull
  public BillingAccountBuilder adminUserCount(int adminUserCount) {
    this.adminUserCount = adminUserCount;
    return this;
  }

  @Nonnull
  public BillingAccountBuilder reportingUserCount(int reportingUserCount) {
    this.reportingUserCount = reportingUserCount;
    return this;
  }

  @Nonnull
  public BillingAccountBuilder databaseCount(int databaseCount) {
    this.databaseCount = databaseCount;
    return this;
  }

  @Nonnull
  public BillingAccountBuilder expectedPaymentTime(double expectedPaymentTime) {
    this.expectedPaymentTime = expectedPaymentTime;
    return this;
  }

  @Nonnull
  public BillingAccountBuilder status(@Nonnull BillingAccountStatus status) {
    this.status = status;
    return this;
  }

  @Nonnull
  public BillingAccountBuilder staleCounts(boolean staleCounts) {
    this.staleCounts = staleCounts;
    return this;
  }

  @Nonnull
  public BillingAccountBuilder planName(@Nonnull String planName) {
    this.planName = planName;
    return this;
  }

  @Nonnull
  public BillingAccountBuilder automaticCollection(boolean automaticCollection) {
    this.automaticCollection = automaticCollection;
    return this;
  }

  @Nonnull
  public BillingAccount build() {
    assert this.name != null : "name is missing";
    assert this.status != null : "status is missing";
    assert this.planName != null : "planName is missing";
    return new BillingAccount(id,name,trial,expirationTime,userLimit,userCount,adminUserCount,reportingUserCount,databaseCount,expectedPaymentTime,status,staleCounts,planName,automaticCollection);
  }
}
