/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.account;

import org.activityinfo.i18n.shared.ApplicationLocale;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.resource.ResourceId;

import java.util.Locale;

@AutoBuilder
public class DatabaseInvite {

    private final ServingDomain domain;
    private final ResourceId databaseId;
    private final String databaseLabel;
    private final String name;
    private final String email;
    private final ResourceId userId;
    private final ApplicationLocale locale;
    private final String invitingUserEmail;
    private final String invitingUserName;
    private final String token;
    private final ActivationStatus activationStatus;

    @Builder
    public DatabaseInvite(ServingDomain domain, ResourceId databaseId, String databaseLabel, String name, String email, ResourceId userId, ApplicationLocale locale, String invitingUserEmail, String invitingUserName, String token, ActivationStatus activationStatus) {
        this.domain = domain;
        this.databaseId = databaseId;
        this.databaseLabel = databaseLabel;
        this.name = name;
        this.email = email;
        this.userId = userId;
        this.locale = locale;
        this.invitingUserEmail = invitingUserEmail;
        this.invitingUserName = invitingUserName;
        this.token = token;
        this.activationStatus = activationStatus;
    }

    public String getName() {
        return name;
    }

    public ApplicationLocale getLocale() {
        return locale;
    }

    public Locale getLocaleObject() {
        return Locale.forLanguageTag(locale.getCode());
    }

    public String getInvitingUserEmail() {
        return invitingUserEmail;
    }

    public ResourceId getUserId() {
        return userId;
    }

    public String getDatabaseLabel() {
        return databaseLabel;
    }

    public String getInvitingUserName() {
        return invitingUserName;
    }

    public ResourceId getDatabaseId() {
        return databaseId;
    }

    public String getToken() {
        return token;
    }

    public String getEmail() {
        return email;
    }

    public ServingDomain getDomain() {
        return domain;
    }

    public String getRootUrl() {
        return getDomain().getRootUrl();
    }

    public ActivationStatus getActivationStatus() {
        return activationStatus;
    }

    public boolean isActiveUser() {
        return activationStatus == ActivationStatus.ACTIVE;
    }
}
