package org.activityinfo.model.account;

import java.lang.String;
import javax.annotation.Nonnull;
import org.activityinfo.i18n.shared.ApplicationLocale;
import org.activityinfo.model.resource.ResourceId;

public final class DatabaseInviteBuilder {
  private ServingDomain domain;

  private ResourceId databaseId;

  private String databaseLabel = "";

  private String name = "";

  private String email = "";

  private ResourceId userId;

  private ApplicationLocale locale = ApplicationLocale.EN;

  private String invitingUserEmail = "";

  private String invitingUserName = "";

  private String token = "";

  private ActivationStatus activationStatus = ActivationStatus.NO_ACCOUNT;

  public DatabaseInviteBuilder() {
    databaseLabel = "";
    name = "";
    email = "";
    locale = ApplicationLocale.EN;
    invitingUserEmail = "";
    invitingUserName = "";
    token = "";
    activationStatus = ActivationStatus.NO_ACCOUNT;
  }

  public DatabaseInviteBuilder(DatabaseInvite toCopy) {
    this.domain = toCopy.getDomain();
    this.databaseId = toCopy.getDatabaseId();
    this.databaseLabel = toCopy.getDatabaseLabel();
    this.name = toCopy.getName();
    this.email = toCopy.getEmail();
    this.userId = toCopy.getUserId();
    this.locale = toCopy.getLocale();
    this.invitingUserEmail = toCopy.getInvitingUserEmail();
    this.invitingUserName = toCopy.getInvitingUserName();
    this.token = toCopy.getToken();
    this.activationStatus = toCopy.getActivationStatus();
  }

  @Nonnull
  public DatabaseInviteBuilder domain(@Nonnull ServingDomain domain) {
    this.domain = domain;
    return this;
  }

  @Nonnull
  public DatabaseInviteBuilder databaseId(@Nonnull ResourceId databaseId) {
    this.databaseId = databaseId;
    return this;
  }

  @Nonnull
  public DatabaseInviteBuilder databaseLabel(@Nonnull String databaseLabel) {
    this.databaseLabel = databaseLabel;
    return this;
  }

  @Nonnull
  public DatabaseInviteBuilder name(@Nonnull String name) {
    this.name = name;
    return this;
  }

  @Nonnull
  public DatabaseInviteBuilder email(@Nonnull String email) {
    this.email = email;
    return this;
  }

  @Nonnull
  public DatabaseInviteBuilder userId(@Nonnull ResourceId userId) {
    this.userId = userId;
    return this;
  }

  @Nonnull
  public DatabaseInviteBuilder locale(@Nonnull ApplicationLocale locale) {
    this.locale = locale;
    return this;
  }

  @Nonnull
  public DatabaseInviteBuilder invitingUserEmail(@Nonnull String invitingUserEmail) {
    this.invitingUserEmail = invitingUserEmail;
    return this;
  }

  @Nonnull
  public DatabaseInviteBuilder invitingUserName(@Nonnull String invitingUserName) {
    this.invitingUserName = invitingUserName;
    return this;
  }

  @Nonnull
  public DatabaseInviteBuilder token(@Nonnull String token) {
    this.token = token;
    return this;
  }

  @Nonnull
  public DatabaseInviteBuilder activationStatus(@Nonnull ActivationStatus activationStatus) {
    this.activationStatus = activationStatus;
    return this;
  }

  @Nonnull
  public DatabaseInvite build() {
    assert this.domain != null : "domain is missing";
    assert this.databaseId != null : "databaseId is missing";
    assert this.databaseLabel != null : "databaseLabel is missing";
    assert this.name != null : "name is missing";
    assert this.email != null : "email is missing";
    assert this.userId != null : "userId is missing";
    assert this.locale != null : "locale is missing";
    assert this.invitingUserEmail != null : "invitingUserEmail is missing";
    assert this.invitingUserName != null : "invitingUserName is missing";
    assert this.token != null : "token is missing";
    assert this.activationStatus != null : "activationStatus is missing";
    return new DatabaseInvite(domain,databaseId,databaseLabel,name,email,userId,locale,invitingUserEmail,invitingUserName,token,activationStatus);
  }
}
