/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.account;

import org.activityinfo.json.AutoJson;

import java.io.Serializable;

/**
 * A minimal description of the user's device based on the User-Agent string.
 *
 * <p>In order to help users manage their authenticated devices, we need to be able
 * describe which device we're talking about.</p>
 */
@AutoJson
public class DeviceProfile implements Serializable {

    private static final long SerialVersionUID = 1L;

    private final boolean mobile;
    private final String deviceName;
    private final String browserSoftware;

    public DeviceProfile(boolean mobile, String deviceName, String browserSoftware) {
        this.mobile = mobile;
        this.deviceName = deviceName;
        this.browserSoftware = browserSoftware;
    }

    /**
     * Returns true if this a mobile device. We classify phones, smartphones, and tablets as mobile devices.
     */
    public boolean isMobile() {
        return mobile;
    }

    /**
     * Returns a human-readible name for this device. For mobile devices, this will hopefully be a nice description
     * like "Samsung Galaxy 7" or "Huawei P6". For non-mobile devices, this is the name of the operating system,
     * like "Windows", "Linux" or "MacOS".
     */
    public String getDeviceName() {
        return deviceName;
    }

    /**
     * Returns a human-readable name of the user's browser. For example, "Chrome", "Internet Explorer" or "Firefox"
     */
    public String getBrowserSoftware() {
        return browserSoftware;
    }

    @Override
    public String toString() {
        return "DeviceProfile{" +
            "mobile=" + mobile +
            ", deviceName='" + deviceName + '\'' +
            ", browserSoftware='" + browserSoftware + '\'' +
            '}';
    }
}
