/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.account;

import java.util.Optional;

/**
 * Defines platform wide policy for a given server domain (e.g. "gmail.com" or "unhcr.org").
 *
 */
public class DomainPolicy {

    private final String domain;
    private final String idp;
    private final DomainDeliveryStatus deliveryStatus;
    private final Optional<BillingRoleAssignment> billingRole;

    public DomainPolicy(String domain, String idp) {
        this(domain, idp, DomainDeliveryStatus.UNKNOWN, Optional.empty());
    }

    public DomainPolicy(String domain, String idp, DomainDeliveryStatus deliveryStatus, Optional<BillingRoleAssignment> billingRole) {
        this.domain = domain;
        this.idp = idp;
        this.deliveryStatus = deliveryStatus;
        this.billingRole = billingRole;
    }

    /**
     * @return the domain name
     */
    public String getDomain() {
        return domain;
    }

    /**
     * The Identity Provider (IdP) that require users with email addresses in this domain to authenticate,
     * or an empty string if there is no required IdP.
     */
    public String getIdp() {
        return idp;
    }

    /**
     * The email delivery status for this domain, based on previous attempts to send
     * email to this domain.
     */
    public DomainDeliveryStatus getDeliveryStatus() {
        return deliveryStatus;
    }

    /**
     * The billing role that is automatically granted to any user from this domain.
     */
    public Optional<BillingRoleAssignment> getBillingRole() {
        return billingRole;
    }
}
