/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.account;

import java.io.Serializable;
import java.net.URI;
import java.net.URISyntaxException;

/**
 * Provides metadata about the domain on which the application
 * is being served.
 */
public class ServingDomain implements Serializable {
    private static final long serialVersionUID = 241542892559897521L;

    public static final ServingDomain DEFAULT = new ServingDomain();

    private final URI baseUri;

    public ServingDomain() {
        this.baseUri = URI.create("https://www.activityinfo.org");
    }

    public ServingDomain(String host) {
        try {
            this.baseUri = new URI("https", host, "/", null);
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    public ServingDomain(URI baseUri) {
        this.baseUri = baseUri;
    }

    public String getTitle() {
        return "ActivityInfo";
    }

    public String getHost() {
        return baseUri.getHost();
    }

    public int getPort() {
        return baseUri.getPort();
    }

    /**
     * @return the root url for this domain, including only protocal (http/https),
     * host, and port number if necessary.
     */
    public String getRootUrl() {
        String url = baseUri.toString();
        if(url.endsWith("/")) {
            return url.substring(0, url.length() - 1);
        }
        return url;
    }

    @Override
    public String toString() {
        return baseUri.getHost();
    }

    public boolean isLocalhost() {
        return baseUri.getHost().equals("localhost") || baseUri.getHost().equals("127.0.1.1");
    }

}
