package org.activityinfo.model.account;

import java.lang.String;
import java.util.Optional;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;

public final class UserAccountBuilder {
  private ResourceId id;

  private String email = "";

  private String name = "";

  private String locale = "";

  Optional<BillingRoleAssignment> billingRole = Optional.empty();

  private PlatformRole platformRole = PlatformRole.NONE;

  private ActivationStatus activationStatus = ActivationStatus.NO_ACCOUNT;

  private DeliveryStatus deliveryStatus = DeliveryStatus.UNKNOWN;

  private String idp = "";

  public UserAccountBuilder() {
    email = "";
    name = "";
    locale = "";
    platformRole = PlatformRole.NONE;
    activationStatus = ActivationStatus.NO_ACCOUNT;
    deliveryStatus = DeliveryStatus.UNKNOWN;
    idp = "";
  }

  public UserAccountBuilder(UserAccount toCopy) {
    this.id = toCopy.getId();
    this.email = toCopy.getEmail();
    this.name = toCopy.getName();
    this.locale = toCopy.getLocale();
    this.billingRole = toCopy.getBillingRole();
    this.platformRole = toCopy.getPlatformRole();
    this.activationStatus = toCopy.getActivationStatus();
    this.deliveryStatus = toCopy.getDeliveryStatus();
    this.idp = toCopy.getIdp();
  }

  @Nonnull
  public UserAccountBuilder id(@Nonnull ResourceId id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public UserAccountBuilder email(@Nonnull String email) {
    this.email = email;
    return this;
  }

  @Nonnull
  public UserAccountBuilder name(@Nonnull String name) {
    this.name = name;
    return this;
  }

  @Nonnull
  public UserAccountBuilder locale(@Nonnull String locale) {
    this.locale = locale;
    return this;
  }

  @Nonnull
  public UserAccountBuilder billingRole(@Nonnull BillingRoleAssignment value) {
    this.billingRole = Optional.of(value);
    return this;
  }

  @Nonnull
  public UserAccountBuilder billingRole(@Nonnull Optional<BillingRoleAssignment> value) {
    this.billingRole = value;
    return this;
  }

  @Nonnull
  public UserAccountBuilder platformRole(@Nonnull PlatformRole platformRole) {
    this.platformRole = platformRole;
    return this;
  }

  @Nonnull
  public UserAccountBuilder activationStatus(@Nonnull ActivationStatus activationStatus) {
    this.activationStatus = activationStatus;
    return this;
  }

  @Nonnull
  public UserAccountBuilder deliveryStatus(@Nonnull DeliveryStatus deliveryStatus) {
    this.deliveryStatus = deliveryStatus;
    return this;
  }

  @Nonnull
  public UserAccountBuilder idp(@Nonnull String idp) {
    this.idp = idp;
    return this;
  }

  @Nonnull
  public UserAccount build() {
    assert this.id != null : "id is missing";
    assert this.email != null : "email is missing";
    assert this.name != null : "name is missing";
    assert this.locale != null : "locale is missing";
    assert this.platformRole != null : "platformRole is missing";
    assert this.activationStatus != null : "activationStatus is missing";
    assert this.deliveryStatus != null : "deliveryStatus is missing";
    assert this.idp != null : "idp is missing";
    return new UserAccount(id,email,name,locale,billingRole,platformRole,activationStatus,deliveryStatus,idp);
  }
}
