package org.activityinfo.model.account;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonArrays;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.resource.ResourceId;

public final class UserAccountJson {
  public static JsonValue toJson(UserAccount model) {
    JsonValue object = Json.createObject();
    object.put("id", model.getId().asString());
    object.put("email", model.getEmail());
    object.put("name", model.getName());
    object.put("locale", model.getLocale());
    object.put("billingRole", JsonArrays.optionalToJson(model.getBillingRole(), BillingRoleAssignmentJson::toJson));
    object.put("platformRole", model.getPlatformRole().name());
    object.put("activationStatus", model.getActivationStatus().name());
    object.put("deliveryStatus", model.getDeliveryStatus().name());
    object.put("idp", model.getIdp());
    return object;
  }

  public static UserAccount fromJson(JsonValue object) {
    return new UserAccount(ResourceId.valueOf(object.getString("id")),
        object.getString("email"),
        object.getString("name"),
        object.getString("locale"),
        JsonArrays.optionalFromJson(object.get("billingRole"), BillingRoleAssignmentJson::fromJson),
        JsonArrays.enumFromJson(object.get("platformRole"), PlatformRole::valueOf, PlatformRole.NONE),
        JsonArrays.enumFromJson(object.get("activationStatus"), ActivationStatus::valueOf, ActivationStatus.NO_ACCOUNT),
        JsonArrays.enumFromJson(object.get("deliveryStatus"), DeliveryStatus::valueOf, DeliveryStatus.UNKNOWN),
        object.getString("idp"));
  }
}
