/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonSerializable;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.analysis.map.MapModel;
import org.activityinfo.model.analysis.map.MapModelJson;
import org.activityinfo.model.analysis.pivot.PivotModel;
import org.activityinfo.model.analysis.pivot.PivotModelJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;

@AutoBuilder
public class Analysis implements JsonSerializable {
    private final String id;
    private final AnalysisModel model;

    @Builder
    public Analysis(String id, AnalysisModel model) {
        this.id = id;
        this.model = model;
    }

    public String getId() {
        return id;
    }

    public AnalysisModel getModel() {
        return model;
    }


    @Override
    public JsonValue toJson() {
        JsonValue object = Json.createObject();
        object.put("id", id);
        object.put("model", model.toJson());
        object.put("modelType", model.getTypeId());
        return object;
    }

    public static Analysis fromJson(JsonValue object) {

        return new Analysis(
                object.getString("id"),
                modelFromJson(object.getString("modelType"), object.get("model")));
    }

    public static AnalysisModel modelFromJson(String type, JsonValue model) {
        String modelType = type;
        switch (modelType) {
            case PivotModel.TYPE:
                return PivotModelJson.fromJson(model);
            case TableAnalysisModel.TYPE:
                return TableAnalysisModelJson.fromJson(model);
            case MapModel.TYPE:
                return MapModelJson.fromJson(model);
            default:
                throw new UnsupportedOperationException("type: " + modelType);
        }
    }

}
