/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.analysis.pivot.PivotModel;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.resource.ResourceId;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@AutoBuilder
@AutoJson
public class Report{
    private final String id;
    private final ResourceId ownerId;
    private final String ownerName;
    private final String label;
    private final boolean published;
    private final ReportGrantSet grants;
    private final ReportLayout layout;
    private final String icon;
    private final ReportSources sources;
    private final List<ReportComponent> components;

    @Builder
    public Report(String id, ResourceId ownerId, String ownerName, String label, boolean published,
                  ReportGrantSet grants, List<ReportComponent> components, ReportLayout layout, String icon, ReportSources sources) {
        this.id = id;
        this.ownerId = ownerId;
        this.ownerName = ownerName;
        this.label = label;
        this.published = published;
        this.grants = grants;
        this.components = components;
        this.layout = layout;
        this.icon = icon;
        this.sources = sources;
    }

    public String getId() {
        return id;
    }

    public ResourceId getOwnerId() {
        return ownerId;
    }

    public String getLabel() {
        return label;
    }

    public List<ReportComponent> getComponents() {
        return components;
    }

    public boolean isPublished() {
        return published;
    }

    public ReportGrantSet getGrants() {
        return grants;
    }

    public String getOwnerName() {
        return ownerName;
    }

    public String getIcon() {
        return icon;
    }

    public ReportLayout getLayout() {
        return layout;
    }

    public ReportSources getSources() {
        return sources;
    }

    public Optional<String> getSinglePivotAnalysisId(){
        List<ReportComponent>  pivotAnalysis = getComponents().stream()
                .filter(c -> c.getAnalysisType().equals(Optional.of(PivotModel.TYPE)))
                .collect(Collectors.toList());
        if(pivotAnalysis.size() == 1){
            return Optional.of(pivotAnalysis.get(0).getId());
        }else{
            return Optional.empty();
        }


    }
}
