package org.activityinfo.model.analysis;

import java.lang.String;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;

public final class ReportBuilder {
  private String id = "";

  private ResourceId ownerId;

  private String ownerName = "";

  private String label = "";

  private boolean published;

  private ReportGrantSet grants;

  List<ReportComponent> components = new ArrayList();

  private ReportLayout layout = ReportLayout.SINGLE;

  private String icon = "";

  private ReportSources sources;

  public ReportBuilder() {
    id = "";
    ownerName = "";
    label = "";
    this.components = new ArrayList();
    layout = ReportLayout.SINGLE;
    icon = "";
  }

  public ReportBuilder(Report toCopy) {
    this.id = toCopy.getId();
    this.ownerId = toCopy.getOwnerId();
    this.ownerName = toCopy.getOwnerName();
    this.label = toCopy.getLabel();
    this.published = toCopy.isPublished();
    this.grants = toCopy.getGrants();
    this.components = new ArrayList(toCopy.getComponents());
    this.layout = toCopy.getLayout();
    this.icon = toCopy.getIcon();
    this.sources = toCopy.getSources();
  }

  @Nonnull
  public ReportBuilder id(@Nonnull String id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public ReportBuilder ownerId(@Nonnull ResourceId ownerId) {
    this.ownerId = ownerId;
    return this;
  }

  @Nonnull
  public ReportBuilder ownerName(@Nonnull String ownerName) {
    this.ownerName = ownerName;
    return this;
  }

  @Nonnull
  public ReportBuilder label(@Nonnull String label) {
    this.label = label;
    return this;
  }

  @Nonnull
  public ReportBuilder published(boolean published) {
    this.published = published;
    return this;
  }

  @Nonnull
  public ReportBuilder grants(@Nonnull ReportGrantSet grants) {
    this.grants = grants;
    return this;
  }

  @Nonnull
  public ReportBuilder addComponents(@Nonnull Collection<ReportComponent> collection) {
    this.components.addAll(collection);
    return this;
  }

  @Nonnull
  public ReportBuilder addComponent(@Nonnull ReportComponent element) {
    this.components.add(element);
    return this;
  }

  @Nonnull
  public ReportBuilder removeComponent(@Nonnull ReportComponent element) {
    this.components.remove(element);
    return this;
  }

  public ReportBuilder addComponents(@Nonnull ReportComponent... array) {
    this.components.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public ReportBuilder components(@Nonnull ReportComponent... array) {
    return components(Arrays.asList(array));
  }

  @Nonnull
  public ReportBuilder components(@Nonnull Collection<ReportComponent> collection) {
    this.components.clear();
    this.components.addAll(collection);
    return this;
  }

  @Nonnull
  public ReportBuilder layout(@Nonnull ReportLayout layout) {
    this.layout = layout;
    return this;
  }

  @Nonnull
  public ReportBuilder icon(@Nonnull String icon) {
    this.icon = icon;
    return this;
  }

  @Nonnull
  public ReportBuilder sources(@Nonnull ReportSources sources) {
    this.sources = sources;
    return this;
  }

  @Nonnull
  public Report build() {
    assert this.id != null : "id is missing";
    assert this.ownerId != null : "ownerId is missing";
    assert this.ownerName != null : "ownerName is missing";
    assert this.label != null : "label is missing";
    assert this.grants != null : "grants is missing";
    assert this.layout != null : "layout is missing";
    assert this.icon != null : "icon is missing";
    assert this.sources != null : "sources is missing";
    return new Report(id,ownerId,ownerName,label,published,grants,components,layout,icon,sources);
  }
}
