/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis;

import org.activityinfo.i18n.shared.I18N;
import org.activityinfo.json.AutoJson;
import org.activityinfo.model.analysis.map.MapModel;
import org.activityinfo.model.analysis.pivot.PivotModel;

import java.util.List;
import java.util.Optional;

@AutoJson
public class ReportComponent {
    private String id;
    private ReportComponentType type;
    private Optional<String> content;
    private Optional<String> analysisType;
    private boolean visible;


    public ReportComponent(String id, Optional<String> content, boolean visible, ReportComponentType type, Optional<String> analysisType) {
        this.id = id;
        this.type = type;
        this.content = content;
        this.visible = visible;
        this.analysisType = analysisType;
    }

    public String getId() {
        return id;
    }

    public ReportComponentType getType() {
        return type;
    }

    /**
     * The title of the section header, the body of text, or the analysis label
     */
    public Optional<String> getContent() {
        return content;
    }

    /**
     * The type of the model if this is an analysis or empty otherwise
     */
    public Optional<String> getAnalysisType() {
        return analysisType;
    }

    public boolean isVisible() {
        return visible;
    }

    @Override
    public String toString(){
        return "["
                + this.getType().toString()
                + ", " +  this.getContent().orElse("blank") + ", "
                + this.getId()
                +"]";
    }

    public static String  debugComponents(List<ReportComponent> list){
        return list.stream().map( item -> item.toString())
                .reduce("", (result, elem) -> result + (result.length() > 0?" and ":"") + elem  );
    }

    public static String getName(ReportComponentType type, Optional<String> analysisType) {
        switch (type){
            case TEXT:
                return I18N.CONSTANTS.reportTextArea();
            case SECTION_HEADER:
                return I18N.CONSTANTS.reportSectionHeader();
        }

        switch (analysisType.get()) {
            default:
            case PivotModel.TYPE:
                return I18N.CONSTANTS.pivotTable();
            case MapModel.TYPE:
                return  I18N.CONSTANTS.mapView();
        }
    }
}
