/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.resource.ResourceId;

import java.util.Collections;
import java.util.List;

@AutoJson
@AutoBuilder
public class ReportGrantSet {
    private final List<ReportRoleGrant> roles;

    public ReportGrantSet() {
        this.roles = Collections.emptyList();
    }

    @Builder
    public ReportGrantSet(List<ReportRoleGrant> roles) {
        this.roles = roles;
    }

    public List<ReportRoleGrant> getRoles() {
        return roles;
    }

    public ReportPermission findPermission(ResourceId databaseId, ResourceId roleId) {
        for (ReportRoleGrant grant : roles) {
            if(grant.getDatabaseId().equals(databaseId)) {
                if(grant.getRoleId().map(r -> r.equals(roleId)).orElse(true)) {
                    return grant.getPermission();
                }
            }
        }
        return ReportPermission.NONE;
    }

    public boolean isShared() {
        return !roles.isEmpty();
    }
}
