/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;

@AutoBuilder
@AutoJson
public final class ReportHeader {
    private final String id;
    private final String label;
    private final String icon;
    private final boolean pivotSource;
    private final boolean published;
    private final boolean shared;
    private final boolean owned;
    private final double lastUpdateTime;

    public ReportHeader(String id, String label, String icon,  boolean pivotSource, boolean published, boolean shared, boolean owned, double lastUpdateTime) {
        this.id = id;
        this.label = label;
        this.icon = icon;
        this.published = published;
        this.shared = shared;
        this.owned = owned;
        this.lastUpdateTime = lastUpdateTime;
        this.pivotSource = pivotSource;
    }

    public boolean isPublished() {
        return published;
    }

    public boolean isShared() {
        return shared;
    }

    public String getId() {
        return id;
    }

    public String getLabel() {
        return label;
    }

    public String getIcon() {
        return icon;
    }

    public boolean isPivotSource() {
        return pivotSource;
    }

    public boolean isOwned() {
        return owned;
    }

    /**
     * @return the last time this report (definition) was updated, in milliseconds since the unix epoch.
     */
    public double getLastUpdateTime() {
        return lastUpdateTime;
    }


    @Override
    public String toString() {
        return "ReportHeader{" +
            "id='" + id + '\'' +
            ", label='" + label + '\'' +
            '}';
    }
}
