package org.activityinfo.model.analysis;

import java.lang.String;
import javax.annotation.Nonnull;

public final class ReportHeaderBuilder {
  private String id = "";

  private String label = "";

  private String icon = "";

  private boolean pivotSource;

  private boolean published;

  private boolean shared;

  private boolean owned;

  private double lastUpdateTime;

  public ReportHeaderBuilder() {
    id = "";
    label = "";
    icon = "";
  }

  public ReportHeaderBuilder(ReportHeader toCopy) {
    this.id = toCopy.getId();
    this.label = toCopy.getLabel();
    this.icon = toCopy.getIcon();
    this.pivotSource = toCopy.isPivotSource();
    this.published = toCopy.isPublished();
    this.shared = toCopy.isShared();
    this.owned = toCopy.isOwned();
    this.lastUpdateTime = toCopy.getLastUpdateTime();
  }

  @Nonnull
  public ReportHeaderBuilder id(@Nonnull String id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public ReportHeaderBuilder label(@Nonnull String label) {
    this.label = label;
    return this;
  }

  @Nonnull
  public ReportHeaderBuilder icon(@Nonnull String icon) {
    this.icon = icon;
    return this;
  }

  @Nonnull
  public ReportHeaderBuilder pivotSource(boolean pivotSource) {
    this.pivotSource = pivotSource;
    return this;
  }

  @Nonnull
  public ReportHeaderBuilder published(boolean published) {
    this.published = published;
    return this;
  }

  @Nonnull
  public ReportHeaderBuilder shared(boolean shared) {
    this.shared = shared;
    return this;
  }

  @Nonnull
  public ReportHeaderBuilder owned(boolean owned) {
    this.owned = owned;
    return this;
  }

  @Nonnull
  public ReportHeaderBuilder lastUpdateTime(double lastUpdateTime) {
    this.lastUpdateTime = lastUpdateTime;
    return this;
  }

  @Nonnull
  public ReportHeader build() {
    assert this.id != null : "id is missing";
    assert this.label != null : "label is missing";
    assert this.icon != null : "icon is missing";
    return new ReportHeader(id,label,icon,pivotSource,published,shared,owned,lastUpdateTime);
  }
}
