package org.activityinfo.model.analysis;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonArrays;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.resource.ResourceId;

public final class ReportJson {
  public static JsonValue toJson(Report model) {
    JsonValue object = Json.createObject();
    object.put("id", model.getId());
    object.put("ownerId", model.getOwnerId().asString());
    object.put("ownerName", model.getOwnerName());
    object.put("label", model.getLabel());
    object.put("published", model.isPublished());
    object.put("grants", ReportGrantSetJson.toJson(model.getGrants()));
    object.put("layout", model.getLayout().name());
    object.put("icon", model.getIcon());
    object.put("sources", ReportSourcesJson.toJson(model.getSources()));
    object.put("components", JsonArrays.toJsonArray(model.getComponents(), ReportComponentJson::toJson));
    return object;
  }

  public static Report fromJson(JsonValue object) {
    return new Report(object.getString("id"),
        ResourceId.valueOf(object.getString("ownerId")),
        object.getString("ownerName"),
        object.getString("label"),
        object.getBoolean("published"),
        ReportGrantSetJson.fromJson(object.get("grants")),
        JsonArrays.listFromJson(object.get("components"), ReportComponentJson::fromJson),
        JsonArrays.enumFromJson(object.get("layout"), ReportLayout::valueOf, ReportLayout.SINGLE),
        object.getString("icon"),
        ReportSourcesJson.fromJson(object.get("sources")));
  }
}
