/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.resource.ResourceId;

import java.util.Optional;

/**
 * Grants access to all users who have a given role in a given database access
 *  to this report.
 */
@AutoJson
public class ReportRoleGrant {
    private final ResourceId databaseId;
    private final Optional<ResourceId> roleId;
    private final ReportPermission permission;

    public ReportRoleGrant(ResourceId databaseId, ReportPermission permission) {
        this.databaseId = databaseId;
        this.roleId = Optional.empty();
        this.permission = permission;
    }
    public ReportRoleGrant(ResourceId databaseId, ResourceId roleId, ReportPermission permission) {
        this.databaseId = databaseId;
        this.roleId = Optional.of(roleId);
        this.permission = permission;
    }

    @Builder
    public ReportRoleGrant(ResourceId databaseId, Optional<ResourceId> roleId, ReportPermission permission) {
        this.databaseId = databaseId;
        this.roleId = roleId;
        this.permission = permission;
    }

    public ResourceId getDatabaseId() {
        return databaseId;
    }

    public Optional<ResourceId> getRoleId() {
        return roleId;
    }

    public ReportPermission getPermission() {
        return permission;
    }
}
