/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis;

import com.google.common.collect.Sets;
import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.resource.ResourceId;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

@AutoJson
public class ReportSources {

    private final Set<ResourceId> forms;
    private final Set<ResourceId> reports;

    @Builder
    public ReportSources(Set<ResourceId> forms, Set<ResourceId> reports) {
        this.forms = forms;
        this.reports = reports;
    }

    public ReportSources() {
        this(Collections.emptySet(), Collections.emptySet());
    }

    public ReportSources(Set<ResourceId> forms) {
        this.forms = forms;
        this.reports = Collections.emptySet();
    }

    public ReportSources(ResourceId formId) {
        this(Collections.singleton(formId));
    }

    public Set<ResourceId> getForms() {
        return forms;
    }

    public Set<ResourceId> getReports() {
        return reports;
    }

    public  boolean isEmpty(){
        return forms.isEmpty() && reports.isEmpty();
    }

}
