/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis;

import com.google.common.collect.Sets;
import org.activityinfo.model.resource.ResourceId;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

public class ReportSourcesDelta {
    private final Set<ResourceId> addedForms;
    private final Set<ResourceId> addedReports;
    private final Set<ResourceId> removedForms;
    private final Set<ResourceId> removedReports;

    public ReportSourcesDelta() {
        this.addedForms = Collections.emptySet();
        this.addedReports = Collections.emptySet();;
        this.removedForms = Collections.emptySet();;
        this.removedReports = Collections.emptySet();;
    }

    public ReportSourcesDelta(Set<ResourceId> addedForms, Set<ResourceId> removedForms, Set<ResourceId> addedReports, Set<ResourceId> removedReports) {
        this.addedForms = addedForms;
        this.addedReports = addedReports;
        this.removedForms = removedForms;
        this.removedReports = removedReports;
    }

    public ReportSourcesDelta selectForms(Set<ResourceId> forms, boolean select) {
        Set<ResourceId> addSelection = new HashSet<>(this.addedForms);
        Set<ResourceId> removeSelection = new HashSet<>(this.removedForms);;
        if (select) {
            addSelection.addAll(forms);
            removeSelection.removeAll(forms);
        } else {
            addSelection.removeAll(forms);
            removeSelection.addAll(forms);
        }
        return new ReportSourcesDelta(addSelection, removeSelection, addedReports, removedReports);
    }


    public ReportSourcesDelta selectReport(ResourceId reportId, boolean select) {
        Set<ResourceId> addSelection = new HashSet<>(this.addedReports);
        Set<ResourceId> removeSelection = new HashSet<>(this.removedReports);;
        if(select) {
            addSelection.add(reportId);
            removeSelection.remove(reportId);
        } else {
            addSelection.remove(reportId);
            removeSelection.add(reportId);
        }
        return new ReportSourcesDelta(addedForms, removedForms, addSelection, removeSelection);
    }

    public static ReportSources applyDeltas(ReportSources sources, ReportSourcesDelta deltas) {
        Set<ResourceId> forms = new HashSet<>(sources.getForms()) ;
        Set<ResourceId> reports = new HashSet<>(sources.getReports());
        forms.addAll(deltas.addedForms);
        forms.removeAll(deltas.removedForms);
        reports.addAll(deltas.addedReports);
        reports.removeAll(deltas.removedReports);
        return new ReportSources(forms, reports);
    }
}
