/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.query.SortDir;

import java.util.Objects;

@AutoJson
public class SortOrder {

    private String formula;
    private boolean ascending;

    @Builder
    public SortOrder(String formula, boolean ascending) {
        this.formula = formula;
        this.ascending = ascending;
    }

    public SortOrder(String formula) {
        this(formula, true);
    }

    public SortOrder(String formulaString, SortDir direction) {
        this(formulaString, direction == SortDir.ASC);
    }

    public String getFormula() {
        return formula;
    }

    public boolean isAscending() {
        return ascending;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof SortOrder)) {
            return false;
        }
        SortOrder sortOrder = (SortOrder) o;
        return ascending == sortOrder.ascending &&
                formula.equals(sortOrder.formula);
    }

    @Override
    public int hashCode() {
        return Objects.hash(formula, ascending);
    }
}
