package org.activityinfo.model.analysis;

import java.lang.String;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Optional;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;

public final class TableAnalysisModelBuilder {
  private ResourceId formId;

  List<TableColumn> columns = new ArrayList();

  List<SortOrder> ordering = new ArrayList();

  Optional<String> filter = Optional.empty();

  public TableAnalysisModelBuilder() {
    this.columns = new ArrayList();
    this.ordering = new ArrayList();
  }

  public TableAnalysisModelBuilder(TableAnalysisModel toCopy) {
    this.formId = toCopy.getFormId();
    this.columns = new ArrayList(toCopy.getColumns());
    this.ordering = new ArrayList(toCopy.getOrdering());
    this.filter = toCopy.getFilter();
  }

  @Nonnull
  public TableAnalysisModelBuilder formId(@Nonnull ResourceId formId) {
    this.formId = formId;
    return this;
  }

  @Nonnull
  public TableAnalysisModelBuilder addColumns(@Nonnull Collection<TableColumn> collection) {
    this.columns.addAll(collection);
    return this;
  }

  @Nonnull
  public TableAnalysisModelBuilder addColumn(@Nonnull TableColumn element) {
    this.columns.add(element);
    return this;
  }

  @Nonnull
  public TableAnalysisModelBuilder removeColumn(@Nonnull TableColumn element) {
    this.columns.remove(element);
    return this;
  }

  public TableAnalysisModelBuilder addColumns(@Nonnull TableColumn... array) {
    this.columns.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public TableAnalysisModelBuilder columns(@Nonnull TableColumn... array) {
    return columns(Arrays.asList(array));
  }

  @Nonnull
  public TableAnalysisModelBuilder columns(@Nonnull Collection<TableColumn> collection) {
    this.columns.clear();
    this.columns.addAll(collection);
    return this;
  }

  @Nonnull
  public TableAnalysisModelBuilder addOrdering(@Nonnull Collection<SortOrder> collection) {
    this.ordering.addAll(collection);
    return this;
  }

  @Nonnull
  public TableAnalysisModelBuilder addOrdering(@Nonnull SortOrder element) {
    this.ordering.add(element);
    return this;
  }

  @Nonnull
  public TableAnalysisModelBuilder removeOrdering(@Nonnull SortOrder element) {
    this.ordering.remove(element);
    return this;
  }

  public TableAnalysisModelBuilder addOrdering(@Nonnull SortOrder... array) {
    this.ordering.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public TableAnalysisModelBuilder ordering(@Nonnull SortOrder... array) {
    return ordering(Arrays.asList(array));
  }

  @Nonnull
  public TableAnalysisModelBuilder ordering(@Nonnull Collection<SortOrder> collection) {
    this.ordering.clear();
    this.ordering.addAll(collection);
    return this;
  }

  @Nonnull
  public TableAnalysisModelBuilder filter(@Nonnull String value) {
    this.filter = Optional.of(value);
    return this;
  }

  @Nonnull
  public TableAnalysisModelBuilder filter(@Nonnull Optional<String> value) {
    this.filter = value;
    return this;
  }

  @Nonnull
  public TableAnalysisModel build() {
    assert this.formId != null : "formId is missing";
    return new TableAnalysisModel(formId,columns,ordering,filter);
  }
}
