/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Undocumented;

import java.util.Objects;
import java.util.Optional;


@AutoJson
@AutoBuilder
public class TableColumn {

    /**
     * The id of the column. This is used to identify the column in the result.
     */
    private final String id;

    /**
     * The text to use as the column's header.
     */
    private final Optional<String> label;

    /**
     * The width, in pixels of the column.
     */
    @Undocumented
    private final int width;

    /**
     * The formula used to compute this column's values.
     */
    private final String formula;

    public TableColumn(String id, Optional<String> label, int width, String formula) {
        this.id = id;
        this.label = label;
        this.width = width;
        this.formula = formula;
    }

    public String getId() {
        return id;
    }

    public Optional<String> getLabel() {
        return label;
    }

    /**
     * @return the width of the column in pixels, or zero if the column
     * width is not set.
     */
    public int getWidth() {
        return width;
    }

    public int getWidth(int defaultWidth) {
        return width == 0 ? defaultWidth : width;
    }

    public String getFormula() {
        return formula;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof TableColumn)) {
            return false;
        }
        TableColumn that = (TableColumn) o;
        return width == that.width &&
                id.equals(that.id) &&
                label.equals(that.label) &&
                formula.equals(that.formula);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, label, width, formula);
    }
}
