package org.activityinfo.model.analysis;

import java.lang.String;
import java.util.Optional;
import javax.annotation.Nonnull;

public final class TableColumnBuilder {
  private String id = "";

  Optional<String> label = Optional.empty();

  private int width;

  private String formula = "";

  public TableColumnBuilder() {
    id = "";
    formula = "";
  }

  public TableColumnBuilder(TableColumn toCopy) {
    this.id = toCopy.getId();
    this.label = toCopy.getLabel();
    this.width = toCopy.getWidth();
    this.formula = toCopy.getFormula();
  }

  @Nonnull
  public TableColumnBuilder id(@Nonnull String id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public TableColumnBuilder label(@Nonnull String value) {
    this.label = Optional.of(value);
    return this;
  }

  @Nonnull
  public TableColumnBuilder label(@Nonnull Optional<String> value) {
    this.label = value;
    return this;
  }

  @Nonnull
  public TableColumnBuilder width(int width) {
    this.width = width;
    return this;
  }

  @Nonnull
  public TableColumnBuilder formula(@Nonnull String formula) {
    this.formula = formula;
    return this;
  }

  @Nonnull
  public TableColumn build() {
    assert this.id != null : "id is missing";
    assert this.formula != null : "formula is missing";
    return new TableColumn(id,label,width,formula);
  }
}
