/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis;

import org.activityinfo.promise.HttpError;

public enum UpdateReportError implements HttpError {

    /**
     * A report cannot be added without a label
     */
    MISSING_LABEL(BAD_REQUEST_400),

    /**
     * A report cannot be added without a label
     */
    MISSING_MODEL(BAD_REQUEST_400),

    /**
     * A report cannot be added without a component
     */
    MISSING_COMPONENT(BAD_REQUEST_400),

    /**
     * A report cannot be added without a data source
     */
    MISSING_SOURCE(BAD_REQUEST_400),

    /**
     * The 'single' report layout requires a single analysis component.
     */
    SINGLE_LAYOUT_REQUIRES_SINGLE_ANALYSIS(BAD_REQUEST_400),


    /**
     * You do not have permission to update this report
     */
    PERMISSION_DENIED(FORBIDDEN_403),

    /**
     * The database specified in a {@link ReportRoleGrant} cannot be found.
     */
    DATABASE_IN_GRANT_NOT_FOUND(BAD_REQUEST_400),

    /**
     * the role specified in a {@link ReportRoleGrant} cannot be found.
     */
    ROLE_NOT_FOUND(BAD_REQUEST_400),

    /**
     * You do not have permission to share this report with the given database role.
     */
    ROLE_GRANT_FORBIDDEN(FORBIDDEN_403),

    /**
     * You do not have permission to publish a report that includes the given form or database.
     */
    PUBLISHING_FORBIDDEN(FORBIDDEN_403);

    private final int statusCode;

    UpdateReportError(int statusCode) {
        this.statusCode = statusCode;
    }

    @Override
    public int getHttpStatusCode() {
        return statusCode;
    }
}
