package org.activityinfo.model.analysis;

import java.lang.Boolean;
import java.lang.String;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Optional;
import javax.annotation.Nonnull;

public final class UpdateReportRequestBuilder {
  private String id = "";

  Optional<String> label = Optional.empty();

  Optional<ReportSources> sources = Optional.empty();

  List<ReportComponent> components = new ArrayList();

  Optional<Boolean> publish = Optional.empty();

  List<ReportRoleGrant> roleGrants = new ArrayList();

  List<Analysis> analyses = new ArrayList();

  Optional<ReportLayout> layout = Optional.empty();

  public UpdateReportRequestBuilder() {
    id = "";
    this.components = new ArrayList();
    this.roleGrants = new ArrayList();
    this.analyses = new ArrayList();
  }

  public UpdateReportRequestBuilder(UpdateReportRequest toCopy) {
    this.id = toCopy.getId();
    this.label = toCopy.getLabel();
    this.sources = toCopy.getSources();
    this.components = new ArrayList(toCopy.getComponents());
    this.publish = toCopy.getPublish();
    this.roleGrants = new ArrayList(toCopy.getRoleGrants());
    this.analyses = new ArrayList(toCopy.getAnalyses());
    this.layout = toCopy.getLayout();
  }

  @Nonnull
  public UpdateReportRequestBuilder id(@Nonnull String id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder label(@Nonnull String value) {
    this.label = Optional.of(value);
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder label(@Nonnull Optional<String> value) {
    this.label = value;
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder sources(@Nonnull ReportSources value) {
    this.sources = Optional.of(value);
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder sources(@Nonnull Optional<ReportSources> value) {
    this.sources = value;
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder addComponents(@Nonnull Collection<ReportComponent> collection) {
    this.components.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder addComponent(@Nonnull ReportComponent element) {
    this.components.add(element);
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder removeComponent(@Nonnull ReportComponent element) {
    this.components.remove(element);
    return this;
  }

  public UpdateReportRequestBuilder addComponents(@Nonnull ReportComponent... array) {
    this.components.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder components(@Nonnull ReportComponent... array) {
    return components(Arrays.asList(array));
  }

  @Nonnull
  public UpdateReportRequestBuilder components(@Nonnull Collection<ReportComponent> collection) {
    this.components.clear();
    this.components.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder publish(@Nonnull Boolean value) {
    this.publish = Optional.of(value);
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder publish(@Nonnull Optional<Boolean> value) {
    this.publish = value;
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder addRoleGrants(@Nonnull Collection<ReportRoleGrant> collection) {
    this.roleGrants.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder addRoleGrant(@Nonnull ReportRoleGrant element) {
    this.roleGrants.add(element);
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder removeRoleGrant(@Nonnull ReportRoleGrant element) {
    this.roleGrants.remove(element);
    return this;
  }

  public UpdateReportRequestBuilder addRoleGrants(@Nonnull ReportRoleGrant... array) {
    this.roleGrants.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder roleGrants(@Nonnull ReportRoleGrant... array) {
    return roleGrants(Arrays.asList(array));
  }

  @Nonnull
  public UpdateReportRequestBuilder roleGrants(@Nonnull Collection<ReportRoleGrant> collection) {
    this.roleGrants.clear();
    this.roleGrants.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder addAnalyses(@Nonnull Collection<Analysis> collection) {
    this.analyses.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder addAnalyse(@Nonnull Analysis element) {
    this.analyses.add(element);
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder removeAnalyse(@Nonnull Analysis element) {
    this.analyses.remove(element);
    return this;
  }

  public UpdateReportRequestBuilder addAnalyses(@Nonnull Analysis... array) {
    this.analyses.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder analyses(@Nonnull Analysis... array) {
    return analyses(Arrays.asList(array));
  }

  @Nonnull
  public UpdateReportRequestBuilder analyses(@Nonnull Collection<Analysis> collection) {
    this.analyses.clear();
    this.analyses.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder layout(@Nonnull ReportLayout value) {
    this.layout = Optional.of(value);
    return this;
  }

  @Nonnull
  public UpdateReportRequestBuilder layout(@Nonnull Optional<ReportLayout> value) {
    this.layout = value;
    return this;
  }

  @Nonnull
  public UpdateReportRequest build() {
    assert this.id != null : "id is missing";
    return new UpdateReportRequest(id,label,sources,components,publish,roleGrants,analyses,layout);
  }
}
