package org.activityinfo.model.analysis;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonArrays;
import org.activityinfo.json.JsonValue;

public final class UpdateReportRequestJson {
  public static JsonValue toJson(UpdateReportRequest model) {
    JsonValue object = Json.createObject();
    object.put("id", model.getId());
    object.put("label", JsonArrays.optionalToJson(model.getLabel(), Json::create));
    object.put("layout", JsonArrays.optionalToJson(model.getLayout(), JsonArrays::toJsonString));
    object.put("sources", JsonArrays.optionalToJson(model.getSources(), ReportSourcesJson::toJson));
    object.put("components", JsonArrays.toJsonArray(model.getComponents(), ReportComponentJson::toJson));
    object.put("publish", JsonArrays.optionalToJson(model.getPublish(), JsonArrays::boolToJson));
    object.put("roleGrants", JsonArrays.toJsonArray(model.getRoleGrants(), ReportRoleGrantJson::toJson));
    object.put("analyses", JsonArrays.toJsonArray(model.getAnalyses(), Analysis::toJson));
    return object;
  }

  public static UpdateReportRequest fromJson(JsonValue object) {
    return new UpdateReportRequest(object.getString("id"),
        JsonArrays.optionalFromJson(object.get("label"), JsonValue::asString),
        JsonArrays.optionalFromJson(object.get("sources"), ReportSourcesJson::fromJson),
        JsonArrays.listFromJson(object.get("components"), ReportComponentJson::fromJson),
        JsonArrays.optionalFromJson(object.get("publish"), JsonValue::asBoolean),
        JsonArrays.listFromJson(object.get("roleGrants"), ReportRoleGrantJson::fromJson),
        JsonArrays.listFromJson(object.get("analyses"), Analysis::fromJson),
        JsonArrays.optionalFromJson(object.get("layout"), j -> ReportLayout.valueOf(j.asString().toUpperCase())));
  }
}
