/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis.map;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.analysis.AnalysisModel;
import org.activityinfo.model.analysis.pivot.Statistic;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.formula.resolved.FieldReferenceNode;
import org.activityinfo.model.formula.resolved.ResolvedNode;
import org.activityinfo.model.formula.resolved.ResolvedReference;
import org.activityinfo.model.resource.ResourceId;

import java.util.Optional;

@AutoJson
@AutoBuilder
public class Layer {

    public final static LayerType DEFAULT_LAYER_TYPE = LayerType.PROPORTIONAL_CIRCLE;
    public final static String DEFAULT_COLOR = "7fc97f";
    public final static double DEFAULT_MIN_RADIUS = 10;
    public final static double DEFAULT_MAX_RADIUS = 35;
    public final static double DEFAULT_OPACITY = 0.6;
    //TODO to set less specific default icon id
    public final static String DEFAULT_ICON_ID = "ocha-about";
    public final static Statistic DEFAULT_CLUSTERING = Statistic.SUM;

    private final String id;
    private final ResourceId formId;

    /**
     * The human-readable name of the layer. In most cases will default to the field name,
     * but can also be edited by the user. Should appear in the legend.
     */
    private final String label;

    /**
     * Determines the formula used to compute the quantity value for a proportional
     * circle or a shaded polygon layer.
     */
    private final String valueFormula;


    private final LayerType type;


    /**
     * The formula that gives the geographic value (GeoPoint or GeoArea) to plot.
     * See org.activityinfo.ui.client.table.viewModel.GeographySource#findSources(org.activityinfo.model.formTree.FormTree)
     */
    private final String geographyFormula;


    private final String color;

    private final double minRadius;

    private final double maxRadius;


    private final Optional<String> filterFormula;

    private final Optional<String> clusterFormula;

    private final Statistic clustering;

    private final boolean visible;

    /**
     * From 1.0 (fully opaque, to 0 fully transparent)
     */
    private final double opacity;


    /**
     * The id of the icon to use for icon layers.
     */
    private final String iconId;


    public Layer(String id, ResourceId formId, String label, String valueFormula, LayerType type,
                 String geographyFormula, String color, double minRadius, double maxRadius, Optional<String> filterFormula,
                 Optional<String> clusterFormula, Statistic clustering, boolean visible, double opacity, String iconId) {
        this.id = id;
        this.formId = formId;
        this.label = label;
        this.valueFormula = valueFormula;
        this.type = type;
        this.geographyFormula = geographyFormula;
        this.color = color;
        this.minRadius = minRadius;
        this.maxRadius = maxRadius;
        this.filterFormula = filterFormula;
        this.clusterFormula = clusterFormula;
        this.clustering = clustering;
        this.visible = visible;
        this.opacity = opacity;
        this.iconId = iconId;
    }

    public String getId() {
        return id;
    }

    public ResourceId getFormId() {
        return formId;
    }

    public String getLabel() {
        return label;
    }

    public String getValueFormula() {
        return valueFormula;
    }

    public LayerType getType() {
        return type;
    }

    public String getGeographyFormula() {
        return geographyFormula;
    }

    public String getColor() {
        return color;
    }

    public double getMinRadius() {
        return minRadius;
    }

    public double getMaxRadius() {
        return maxRadius;
    }

    public Optional<String> getFilterFormula() {
        return filterFormula;
    }

    public Optional<String> getClusterFormula() {
        return clusterFormula;
    }

    public String getIconId() {
        return iconId;
    }

    public boolean isVisible() {
        return visible;
    }

    public double getOpacity() {
        return opacity;
    }

    public Statistic getClustering() {
        return clustering;
    }


}
