package org.activityinfo.model.analysis.map;

import java.lang.String;
import java.util.Optional;
import javax.annotation.Nonnull;
import org.activityinfo.model.analysis.pivot.Statistic;
import org.activityinfo.model.resource.ResourceId;

public final class LayerBuilder {
  private String id = "";

  private ResourceId formId;

  private String label = "";

  private String valueFormula = "";

  private LayerType type = LayerType.PROPORTIONAL_CIRCLE;

  private String geographyFormula = "";

  private String color = "";

  private double minRadius;

  private double maxRadius;

  Optional<String> filterFormula = Optional.empty();

  Optional<String> clusterFormula = Optional.empty();

  private Statistic clustering = Statistic.COUNT;

  private boolean visible;

  private double opacity;

  private String iconId = "";

  public LayerBuilder() {
    id = "";
    label = "";
    valueFormula = "";
    type = LayerType.PROPORTIONAL_CIRCLE;
    geographyFormula = "";
    color = "";
    clustering = Statistic.COUNT;
    iconId = "";
  }

  public LayerBuilder(Layer toCopy) {
    this.id = toCopy.getId();
    this.formId = toCopy.getFormId();
    this.label = toCopy.getLabel();
    this.valueFormula = toCopy.getValueFormula();
    this.type = toCopy.getType();
    this.geographyFormula = toCopy.getGeographyFormula();
    this.color = toCopy.getColor();
    this.minRadius = toCopy.getMinRadius();
    this.maxRadius = toCopy.getMaxRadius();
    this.filterFormula = toCopy.getFilterFormula();
    this.clusterFormula = toCopy.getClusterFormula();
    this.clustering = toCopy.getClustering();
    this.visible = toCopy.isVisible();
    this.opacity = toCopy.getOpacity();
    this.iconId = toCopy.getIconId();
  }

  @Nonnull
  public LayerBuilder id(@Nonnull String id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public LayerBuilder formId(@Nonnull ResourceId formId) {
    this.formId = formId;
    return this;
  }

  @Nonnull
  public LayerBuilder label(@Nonnull String label) {
    this.label = label;
    return this;
  }

  @Nonnull
  public LayerBuilder valueFormula(@Nonnull String valueFormula) {
    this.valueFormula = valueFormula;
    return this;
  }

  @Nonnull
  public LayerBuilder type(@Nonnull LayerType type) {
    this.type = type;
    return this;
  }

  @Nonnull
  public LayerBuilder geographyFormula(@Nonnull String geographyFormula) {
    this.geographyFormula = geographyFormula;
    return this;
  }

  @Nonnull
  public LayerBuilder color(@Nonnull String color) {
    this.color = color;
    return this;
  }

  @Nonnull
  public LayerBuilder minRadius(double minRadius) {
    this.minRadius = minRadius;
    return this;
  }

  @Nonnull
  public LayerBuilder maxRadius(double maxRadius) {
    this.maxRadius = maxRadius;
    return this;
  }

  @Nonnull
  public LayerBuilder filterFormula(@Nonnull String value) {
    this.filterFormula = Optional.of(value);
    return this;
  }

  @Nonnull
  public LayerBuilder filterFormula(@Nonnull Optional<String> value) {
    this.filterFormula = value;
    return this;
  }

  @Nonnull
  public LayerBuilder clusterFormula(@Nonnull String value) {
    this.clusterFormula = Optional.of(value);
    return this;
  }

  @Nonnull
  public LayerBuilder clusterFormula(@Nonnull Optional<String> value) {
    this.clusterFormula = value;
    return this;
  }

  @Nonnull
  public LayerBuilder clustering(@Nonnull Statistic clustering) {
    this.clustering = clustering;
    return this;
  }

  @Nonnull
  public LayerBuilder visible(boolean visible) {
    this.visible = visible;
    return this;
  }

  @Nonnull
  public LayerBuilder opacity(double opacity) {
    this.opacity = opacity;
    return this;
  }

  @Nonnull
  public LayerBuilder iconId(@Nonnull String iconId) {
    this.iconId = iconId;
    return this;
  }

  @Nonnull
  public Layer build() {
    assert this.id != null : "id is missing";
    assert this.formId != null : "formId is missing";
    assert this.label != null : "label is missing";
    assert this.valueFormula != null : "valueFormula is missing";
    assert this.type != null : "type is missing";
    assert this.geographyFormula != null : "geographyFormula is missing";
    assert this.color != null : "color is missing";
    assert this.clustering != null : "clustering is missing";
    assert this.iconId != null : "iconId is missing";
    return new Layer(id,formId,label,valueFormula,type,geographyFormula,color,minRadius,maxRadius,filterFormula,clusterFormula,clustering,visible,opacity,iconId);
  }
}
