package org.activityinfo.model.analysis.map;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonArrays;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.analysis.pivot.Statistic;
import org.activityinfo.model.resource.ResourceId;

public final class LayerJson {
  public static JsonValue toJson(Layer model) {
    JsonValue object = Json.createObject();
    object.put("id", model.getId());
    object.put("formId", model.getFormId().asString());
    object.put("label", model.getLabel());
    object.put("valueFormula", model.getValueFormula());
    object.put("type", model.getType().name());
    object.put("geographyFormula", model.getGeographyFormula());
    object.put("color", model.getColor());
    object.put("minRadius", model.getMinRadius());
    object.put("maxRadius", model.getMaxRadius());
    object.put("filterFormula", JsonArrays.optionalToJson(model.getFilterFormula(), Json::create));
    object.put("clusterFormula", JsonArrays.optionalToJson(model.getClusterFormula(), Json::create));
    object.put("clustering", model.getClustering().name());
    object.put("visible", model.isVisible());
    object.put("opacity", model.getOpacity());
    object.put("iconId", model.getIconId());
    return object;
  }

  public static Layer fromJson(JsonValue object) {
    return new Layer(object.getString("id"),
        ResourceId.valueOf(object.getString("formId")),
        object.getString("label"),
        object.getString("valueFormula"),
        JsonArrays.enumFromJson(object.get("type"), LayerType::valueOf, LayerType.PROPORTIONAL_CIRCLE),
        object.getString("geographyFormula"),
        object.getString("color"),
        (double)object.getNumber("minRadius"),
        (double)object.getNumber("maxRadius"),
        JsonArrays.optionalFromJson(object.get("filterFormula"), JsonValue::asString),
        JsonArrays.optionalFromJson(object.get("clusterFormula"), JsonValue::asString),
        JsonArrays.enumFromJson(object.get("clustering"), Statistic::valueOf, Statistic.COUNT),
        object.getBoolean("visible"),
        (double)object.getNumber("opacity"),
        object.getString("iconId"));
  }
}
