package org.activityinfo.model.analysis.map;

import java.lang.String;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import javax.annotation.Nonnull;
import org.activityinfo.model.type.geo.GeoPoint;

public final class MapModelBuilder {
  private String baseMap = "";

  List<Layer> layers = new ArrayList();

  private double zoomLevel;

  private GeoPoint center;

  public MapModelBuilder() {
    baseMap = "";
    this.layers = new ArrayList();
  }

  public MapModelBuilder(MapModel toCopy) {
    this.baseMap = toCopy.getBaseMap();
    this.layers = new ArrayList(toCopy.getLayers());
    this.zoomLevel = toCopy.getZoomLevel();
    this.center = toCopy.getCenter();
  }

  @Nonnull
  public MapModelBuilder baseMap(@Nonnull String baseMap) {
    this.baseMap = baseMap;
    return this;
  }

  @Nonnull
  public MapModelBuilder addLayers(@Nonnull Collection<Layer> collection) {
    this.layers.addAll(collection);
    return this;
  }

  @Nonnull
  public MapModelBuilder addLayer(@Nonnull Layer element) {
    this.layers.add(element);
    return this;
  }

  @Nonnull
  public MapModelBuilder removeLayer(@Nonnull Layer element) {
    this.layers.remove(element);
    return this;
  }

  public MapModelBuilder addLayers(@Nonnull Layer... array) {
    this.layers.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public MapModelBuilder layers(@Nonnull Layer... array) {
    return layers(Arrays.asList(array));
  }

  @Nonnull
  public MapModelBuilder layers(@Nonnull Collection<Layer> collection) {
    this.layers.clear();
    this.layers.addAll(collection);
    return this;
  }

  @Nonnull
  public MapModelBuilder zoomLevel(double zoomLevel) {
    this.zoomLevel = zoomLevel;
    return this;
  }

  @Nonnull
  public MapModelBuilder center(@Nonnull GeoPoint center) {
    this.center = center;
    return this;
  }

  @Nonnull
  public MapModel build() {
    assert this.baseMap != null : "baseMap is missing";
    assert this.center != null : "center is missing";
    return new MapModel(baseMap,layers,zoomLevel,center);
  }
}
