/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis.pivot;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;

import java.util.Collections;
import java.util.Objects;
import java.util.Set;

/**
 * Renames, groups, or hides a dimension category.
 */
@AutoBuilder
@AutoJson
public class CategoryMapping {
    private final String label;
    private final Set<String> source;
    private final boolean visible;

    @Builder
    public CategoryMapping(String label, Set<String> source, boolean visible) {
        this.label = label;
        this.source = source;
        this.visible = visible;
    }

    public CategoryMapping(String category, boolean visible) {
        this.label = category;
        this.source = Collections.singleton(category);
        this.visible = visible;
    }

    public String getLabel() {
        return label;
    }

    /**
     * Returns the dimension categories that are included in this mapping.
     */
    public Set<String> getSource() {
        return source;
    }

    /**
     * Returns true if this category mapping are hidden from the visualization, but included
     * in totals.
     */
    public boolean isVisible() {
        return visible;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        CategoryMapping that = (CategoryMapping) o;
        return visible == that.visible &&
            label.equals(that.label) &&
            source.equals(that.source);
    }

    @Override
    public int hashCode() {
        return Objects.hash(label, source, visible);
    }
}
