package org.activityinfo.model.analysis.pivot;

import java.lang.String;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import javax.annotation.Nonnull;

public final class CategoryMappingBuilder {
  private String label = "";

  Set<String> source = new HashSet();

  private boolean visible;

  public CategoryMappingBuilder() {
    label = "";
    this.source = new HashSet();
  }

  public CategoryMappingBuilder(CategoryMapping toCopy) {
    this.label = toCopy.getLabel();
    this.source = new HashSet(toCopy.getSource());
    this.visible = toCopy.isVisible();
  }

  @Nonnull
  public CategoryMappingBuilder label(@Nonnull String label) {
    this.label = label;
    return this;
  }

  @Nonnull
  public CategoryMappingBuilder addSource(@Nonnull Collection<String> collection) {
    this.source.addAll(collection);
    return this;
  }

  @Nonnull
  public CategoryMappingBuilder addSource(@Nonnull String element) {
    this.source.add(element);
    return this;
  }

  @Nonnull
  public CategoryMappingBuilder removeSource(@Nonnull String element) {
    this.source.remove(element);
    return this;
  }

  public CategoryMappingBuilder addSource(@Nonnull String... array) {
    this.source.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public CategoryMappingBuilder source(@Nonnull String... array) {
    return source(Arrays.asList(array));
  }

  @Nonnull
  public CategoryMappingBuilder source(@Nonnull Collection<String> collection) {
    this.source.clear();
    this.source.addAll(collection);
    return this;
  }

  @Nonnull
  public CategoryMappingBuilder visible(boolean visible) {
    this.visible = visible;
    return this;
  }

  @Nonnull
  public CategoryMapping build() {
    assert this.label != null : "label is missing";
    return new CategoryMapping(label,source,visible);
  }
}
