package org.activityinfo.model.analysis.pivot;

import java.lang.String;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Optional;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;

public final class DimensionModelBuilder {
  private String id = "";

  private String label = "";

  List<DimensionMapping> mappings = new ArrayList();

  private Axis axis = Axis.ROW;

  private boolean totals;

  private boolean missingIncluded;

  private boolean percentage;

  Optional<DateLevel> dateLevel = Optional.empty();

  Optional<String> totalLabel = Optional.empty();

  Optional<String> missingLabel = Optional.empty();

  List<CategoryMapping> categoryMappings = new ArrayList();

  Optional<ResourceId> mapTemplateId = Optional.empty();

  public DimensionModelBuilder() {
    id = "";
    label = "";
    this.mappings = new ArrayList();
    axis = Axis.ROW;
    this.categoryMappings = new ArrayList();
  }

  public DimensionModelBuilder(DimensionModel toCopy) {
    this.id = toCopy.getId();
    this.label = toCopy.getLabel();
    this.mappings = new ArrayList(toCopy.getMappings());
    this.axis = toCopy.getAxis();
    this.totals = toCopy.isTotals();
    this.missingIncluded = toCopy.isMissingIncluded();
    this.percentage = toCopy.isPercentage();
    this.dateLevel = toCopy.getDateLevel();
    this.totalLabel = toCopy.getTotalLabel();
    this.missingLabel = toCopy.getMissingLabel();
    this.categoryMappings = new ArrayList(toCopy.getCategoryMappings());
    this.mapTemplateId = toCopy.getMapTemplateId();
  }

  @Nonnull
  public DimensionModelBuilder id(@Nonnull String id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public DimensionModelBuilder label(@Nonnull String label) {
    this.label = label;
    return this;
  }

  @Nonnull
  public DimensionModelBuilder addMappings(@Nonnull Collection<DimensionMapping> collection) {
    this.mappings.addAll(collection);
    return this;
  }

  @Nonnull
  public DimensionModelBuilder addMapping(@Nonnull DimensionMapping element) {
    this.mappings.add(element);
    return this;
  }

  @Nonnull
  public DimensionModelBuilder removeMapping(@Nonnull DimensionMapping element) {
    this.mappings.remove(element);
    return this;
  }

  public DimensionModelBuilder addMappings(@Nonnull DimensionMapping... array) {
    this.mappings.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public DimensionModelBuilder mappings(@Nonnull DimensionMapping... array) {
    return mappings(Arrays.asList(array));
  }

  @Nonnull
  public DimensionModelBuilder mappings(@Nonnull Collection<DimensionMapping> collection) {
    this.mappings.clear();
    this.mappings.addAll(collection);
    return this;
  }

  @Nonnull
  public DimensionModelBuilder axis(@Nonnull Axis axis) {
    this.axis = axis;
    return this;
  }

  @Nonnull
  public DimensionModelBuilder totals(boolean totals) {
    this.totals = totals;
    return this;
  }

  @Nonnull
  public DimensionModelBuilder missingIncluded(boolean missingIncluded) {
    this.missingIncluded = missingIncluded;
    return this;
  }

  @Nonnull
  public DimensionModelBuilder percentage(boolean percentage) {
    this.percentage = percentage;
    return this;
  }

  @Nonnull
  public DimensionModelBuilder dateLevel(@Nonnull DateLevel value) {
    this.dateLevel = Optional.of(value);
    return this;
  }

  @Nonnull
  public DimensionModelBuilder dateLevel(@Nonnull Optional<DateLevel> value) {
    this.dateLevel = value;
    return this;
  }

  @Nonnull
  public DimensionModelBuilder totalLabel(@Nonnull String value) {
    this.totalLabel = Optional.of(value);
    return this;
  }

  @Nonnull
  public DimensionModelBuilder totalLabel(@Nonnull Optional<String> value) {
    this.totalLabel = value;
    return this;
  }

  @Nonnull
  public DimensionModelBuilder missingLabel(@Nonnull String value) {
    this.missingLabel = Optional.of(value);
    return this;
  }

  @Nonnull
  public DimensionModelBuilder missingLabel(@Nonnull Optional<String> value) {
    this.missingLabel = value;
    return this;
  }

  @Nonnull
  public DimensionModelBuilder addCategoryMappings(
      @Nonnull Collection<CategoryMapping> collection) {
    this.categoryMappings.addAll(collection);
    return this;
  }

  @Nonnull
  public DimensionModelBuilder addCategoryMapping(@Nonnull CategoryMapping element) {
    this.categoryMappings.add(element);
    return this;
  }

  @Nonnull
  public DimensionModelBuilder removeCategoryMapping(@Nonnull CategoryMapping element) {
    this.categoryMappings.remove(element);
    return this;
  }

  public DimensionModelBuilder addCategoryMappings(@Nonnull CategoryMapping... array) {
    this.categoryMappings.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public DimensionModelBuilder categoryMappings(@Nonnull CategoryMapping... array) {
    return categoryMappings(Arrays.asList(array));
  }

  @Nonnull
  public DimensionModelBuilder categoryMappings(@Nonnull Collection<CategoryMapping> collection) {
    this.categoryMappings.clear();
    this.categoryMappings.addAll(collection);
    return this;
  }

  @Nonnull
  public DimensionModelBuilder mapTemplateId(@Nonnull ResourceId value) {
    this.mapTemplateId = Optional.of(value);
    return this;
  }

  @Nonnull
  public DimensionModelBuilder mapTemplateId(@Nonnull Optional<ResourceId> value) {
    this.mapTemplateId = value;
    return this;
  }

  @Nonnull
  public DimensionModel build() {
    assert this.id != null : "id is missing";
    assert this.label != null : "label is missing";
    assert this.axis != null : "axis is missing";
    return new DimensionModel(id,label,mappings,axis,totals,missingIncluded,percentage,dateLevel,totalLabel,missingLabel,categoryMappings,mapTemplateId);
  }
}
