package org.activityinfo.model.analysis.pivot;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonArrays;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.resource.ResourceId;

public final class DimensionModelJson {
  public static JsonValue toJson(DimensionModel model) {
    JsonValue object = Json.createObject();
    object.put("id", model.getId());
    object.put("label", model.getLabel());
    object.put("mappings", JsonArrays.toJsonArray(model.getMappings(), DimensionMapping::toJson));
    object.put("axis", model.getAxis().name());
    object.put("totals", model.isTotals());
    object.put("missingIncluded", model.isMissingIncluded());
    object.put("percentage", model.isPercentage());
    object.put("dateLevel", JsonArrays.optionalToJson(model.getDateLevel(), JsonArrays::toJsonString));
    object.put("totalLabel", JsonArrays.optionalToJson(model.getTotalLabel(), Json::create));
    object.put("missingLabel", JsonArrays.optionalToJson(model.getMissingLabel(), Json::create));
    object.put("categoryMappings", JsonArrays.toJsonArray(model.getCategoryMappings(), CategoryMappingJson::toJson));
    object.put("mapTemplateId", JsonArrays.optionalToJson(model.getMapTemplateId(), ResourceId::toJson));
    return object;
  }

  public static DimensionModel fromJson(JsonValue object) {
    return new DimensionModel(object.getString("id"),
        object.getString("label"),
        JsonArrays.listFromJson(object.get("mappings"), DimensionMapping::fromJson),
        JsonArrays.enumFromJson(object.get("axis"), Axis::valueOf, Axis.ROW),
        object.getBoolean("totals"),
        object.getBoolean("missingIncluded"),
        object.getBoolean("percentage"),
        JsonArrays.optionalFromJson(object.get("dateLevel"), j -> DateLevel.valueOf(j.asString().toUpperCase())),
        JsonArrays.optionalFromJson(object.get("totalLabel"), JsonValue::asString),
        JsonArrays.optionalFromJson(object.get("missingLabel"), JsonValue::asString),
        JsonArrays.listFromJson(object.get("categoryMappings"), CategoryMappingJson::fromJson),
        JsonArrays.optionalFromJson(object.get("mapTemplateId"), ResourceId::fromJson));
  }
}
