/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis.pivot;

import java.util.*;
import java.util.function.Function;

public final class ImmutableLists {

    public static <T> List<T> update(List<T> list, T updatedItem, Function<T, String> keyProvider) {

        String updatedId = keyProvider.apply(updatedItem);

        List<T> newList = new ArrayList<>(list.size());
        boolean updated = false;
        for (T item : list) {
            String itemId = keyProvider.apply(item);
            if(itemId.equals(updatedId)) {
                newList.add(updatedItem);
                updated = true;
            } else {
                newList.add(item);
            }
        }

        if (!updated) {
            newList.add(updatedItem);
        }
        return newList;
    }

    public static <T> List<T> remove(List<T> list, String id, Function<T, String> keyProvider) {

        List<T> newList = new ArrayList<>(list.size());
        for (T item : list) {
            String itemId = keyProvider.apply(item);
            if(!itemId.equals(id)) {
                newList.add(item);
            }
        }
        return newList;
    }

    public static <T> List<T> reorder(List<T> list, String afterId, List<T> items, Function<T, String> keyProvider) {

        Set<String> toReorder = new HashSet<>();
        for (T item : items) {
            if(!toReorder.add(keyProvider.apply(item))) {
                throw new IllegalArgumentException("Duplicate ids in items");
            }
        }
        boolean added = false;
        List<T> newList = new ArrayList<>(list.size());
        for (T item : list) {
            String itemId = keyProvider.apply(item);
            if(!toReorder.contains(itemId)) {
                newList.add(item);
            }
            if(itemId.equals(afterId)) {
                newList.addAll(items);
                added = true;
            }
        }
        if(!added) {
            newList.addAll(items);
        }
        return newList;
    }

    public static <T> List<T> concat(List<T> head, T tail) {
        List<T> newList = new ArrayList<>(head);
        newList.add(tail);
        return newList;
    }

    public static <T> List<T> reorder(List<T> list, String insertId, int insertIndex, Function<T, String> keyProvider) {

        T insertItem = null;
        for(T item : list){
            String itemId = keyProvider.apply(item);
            if(itemId.equals(insertId)){
                insertItem = item;
            }
        }
        if(insertItem == null){
            throw new IllegalArgumentException("No item found in list with id: " + insertId);
        }

        List<T> newList = new ArrayList<>(list.size());
        for (int i = 0; i < list.size(); i++) {
            if((i == insertIndex) && (newList.size() == insertIndex)){
                newList.add(insertItem);
            }

            T item = list.get(i);
            String itemId = keyProvider.apply(item);
            if(!itemId.equals(insertId)) {
                newList.add(item);
            }

            if((i == insertIndex) && (newList.size() == insertIndex)){
                newList.add(insertItem);
            }
        }
        if (newList.size()!= list.size()) {
            throw new IllegalStateException("Size mismatch");
        }
        return newList;
    }

    public static <T> Optional<T> get(List<T> list, String id, Function<T, String> keyProvider) {
        for (T item : list) {
            String itemId = keyProvider.apply(item);
            if(itemId.equals(id)) {
                return Optional.of(item);
            }
        }
        return Optional.empty();
    }
}
