/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis.pivot;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.resource.ResourceId;

import java.util.Objects;
import java.util.Set;

/**
 * A measure contributes quantities to an analysis.
 */
@AutoJson
@AutoBuilder
public class MeasureModel {

    private final String id;
    private final String code;
    private final String label;
    private final ResourceId formId;
    private final String formula;
    private final Set<Statistic> statistics;

    public MeasureModel(String id, String code, String label, ResourceId formId, String formula, Set<Statistic> statistics) {
        this.id = id;
        this.code = code;
        this.label = label;
        this.formId = formId;
        this.formula = formula;
        this.statistics = statistics;
    }

    public static MeasureModelBuilder builder() {
        return new MeasureModelBuilder();
    }

    public String getId() {
        return id;
    }

    /**
     * Return this measure's code. Primarily used for the long format export.
     */
    public String getCode() {
        return code;
    }

    public String getLabel() {
        return label;
    }

    /**
     *
     * @return the id of the form that is the source of this measure.
     */
    public ResourceId getFormId() {
        return formId;
    }

    public String getFormula() {
        return formula;
    }

    /**
     *
     * @return the statistics function used to combine values.
     */
    public Set<Statistic> getStatistics() {
        return statistics;
    }

    public MeasureModelBuilder with() {
        return new MeasureModelBuilder(this);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof MeasureModel)) {
            return false;
        }
        MeasureModel that = (MeasureModel) o;
        return id.equals(that.id) &&
                label.equals(that.label) &&
                formId.equals(that.formId) &&
                formula.equals(that.formula) &&
                statistics.equals(that.statistics);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, label, formId, formula, statistics);
    }

    @Override
    public String toString() {
        return "MeasureModel{" +
                "id='" + id + '\'' +
                ", label='" + label + '\'' +
                ", formId=" + formId +
                ", formula='" + formula + '\'' +
                ", statistics=" + statistics +
                '}';
    }
}
