package org.activityinfo.model.analysis.pivot;

import java.lang.String;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;

public final class MeasureModelBuilder {
  private String id = "";

  private String code = "";

  private String label = "";

  private ResourceId formId;

  private String formula = "";

  Set<Statistic> statistics = new HashSet();

  public MeasureModelBuilder() {
    id = "";
    code = "";
    label = "";
    formula = "";
    this.statistics = new HashSet();
  }

  public MeasureModelBuilder(MeasureModel toCopy) {
    this.id = toCopy.getId();
    this.code = toCopy.getCode();
    this.label = toCopy.getLabel();
    this.formId = toCopy.getFormId();
    this.formula = toCopy.getFormula();
    this.statistics = new HashSet(toCopy.getStatistics());
  }

  @Nonnull
  public MeasureModelBuilder id(@Nonnull String id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public MeasureModelBuilder code(@Nonnull String code) {
    this.code = code;
    return this;
  }

  @Nonnull
  public MeasureModelBuilder label(@Nonnull String label) {
    this.label = label;
    return this;
  }

  @Nonnull
  public MeasureModelBuilder formId(@Nonnull ResourceId formId) {
    this.formId = formId;
    return this;
  }

  @Nonnull
  public MeasureModelBuilder formula(@Nonnull String formula) {
    this.formula = formula;
    return this;
  }

  @Nonnull
  public MeasureModelBuilder addStatistics(@Nonnull Collection<Statistic> collection) {
    this.statistics.addAll(collection);
    return this;
  }

  @Nonnull
  public MeasureModelBuilder addStatistic(@Nonnull Statistic element) {
    this.statistics.add(element);
    return this;
  }

  @Nonnull
  public MeasureModelBuilder removeStatistic(@Nonnull Statistic element) {
    this.statistics.remove(element);
    return this;
  }

  public MeasureModelBuilder addStatistics(@Nonnull Statistic... array) {
    this.statistics.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public MeasureModelBuilder statistics(@Nonnull Statistic... array) {
    return statistics(Arrays.asList(array));
  }

  @Nonnull
  public MeasureModelBuilder statistics(@Nonnull Collection<Statistic> collection) {
    this.statistics.clear();
    this.statistics.addAll(collection);
    return this;
  }

  @Nonnull
  public MeasureModel build() {
    assert this.id != null : "id is missing";
    assert this.code != null : "code is missing";
    assert this.label != null : "label is missing";
    assert this.formId != null : "formId is missing";
    assert this.formula != null : "formula is missing";
    return new MeasureModel(id,code,label,formId,formula,statistics);
  }
}
