package org.activityinfo.model.analysis.pivot;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import javax.annotation.Nonnull;

public final class PivotModelBuilder {
  private VisualizationType visualization = VisualizationType.TABLE;

  List<MeasureModel> measures = new ArrayList();

  List<DimensionModel> dimensions = new ArrayList();

  public PivotModelBuilder() {
    visualization = VisualizationType.TABLE;
    this.measures = new ArrayList();
    this.dimensions = new ArrayList();
  }

  public PivotModelBuilder(PivotModel toCopy) {
    this.visualization = toCopy.getVisualization();
    this.measures = new ArrayList(toCopy.getMeasures());
    this.dimensions = new ArrayList(toCopy.getDimensions());
  }

  @Nonnull
  public PivotModelBuilder visualization(@Nonnull VisualizationType visualization) {
    this.visualization = visualization;
    return this;
  }

  @Nonnull
  public PivotModelBuilder addMeasures(@Nonnull Collection<MeasureModel> collection) {
    this.measures.addAll(collection);
    return this;
  }

  @Nonnull
  public PivotModelBuilder addMeasure(@Nonnull MeasureModel element) {
    this.measures.add(element);
    return this;
  }

  @Nonnull
  public PivotModelBuilder removeMeasure(@Nonnull MeasureModel element) {
    this.measures.remove(element);
    return this;
  }

  public PivotModelBuilder addMeasures(@Nonnull MeasureModel... array) {
    this.measures.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public PivotModelBuilder measures(@Nonnull MeasureModel... array) {
    return measures(Arrays.asList(array));
  }

  @Nonnull
  public PivotModelBuilder measures(@Nonnull Collection<MeasureModel> collection) {
    this.measures.clear();
    this.measures.addAll(collection);
    return this;
  }

  @Nonnull
  public PivotModelBuilder addDimensions(@Nonnull Collection<DimensionModel> collection) {
    this.dimensions.addAll(collection);
    return this;
  }

  @Nonnull
  public PivotModelBuilder addDimension(@Nonnull DimensionModel element) {
    this.dimensions.add(element);
    return this;
  }

  @Nonnull
  public PivotModelBuilder removeDimension(@Nonnull DimensionModel element) {
    this.dimensions.remove(element);
    return this;
  }

  public PivotModelBuilder addDimensions(@Nonnull DimensionModel... array) {
    this.dimensions.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public PivotModelBuilder dimensions(@Nonnull DimensionModel... array) {
    return dimensions(Arrays.asList(array));
  }

  @Nonnull
  public PivotModelBuilder dimensions(@Nonnull Collection<DimensionModel> collection) {
    this.dimensions.clear();
    this.dimensions.addAll(collection);
    return this;
  }

  @Nonnull
  public PivotModel build() {
    assert this.visualization != null : "visualization is missing";
    return new PivotModel(visualization,measures,dimensions);
  }
}
