/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.analysis.pivot;

import org.activityinfo.i18n.shared.I18N;
import org.activityinfo.model.formula.functions.*;

import java.util.ArrayList;
import java.util.List;

/**
 * The way in which a measure is aggregate
 */
public enum Statistic {
    COUNT {
        @Override
        public String getLabel() {
            return I18N.CONSTANTS.count();
        }

        @Override
        public StatFunction getFunction() {
            return CountFunction.INSTANCE;
        }
    },
    COUNT_DISTINCT {
        @Override
        public String getLabel() {
            return I18N.CONSTANTS.countDistinct();
        }

        @Override
        public StatFunction getFunction() {
            return CountDistinctFunction.INSTANCE;
        }
    },
    SUM {
        @Override
        public String getLabel() {
            return I18N.CONSTANTS.sum();
        }

        @Override
        public StatFunction getFunction() {
            return SumFunction.INSTANCE;
        }
    },
    AVERAGE {
        @Override
        public String getLabel() {
            return I18N.CONSTANTS.average();
        }

        @Override
        public StatFunction getFunction() {
            return AverageFunction.INSTANCE;
        }
    },
    MEDIAN {
        @Override
        public String getLabel() {
            return I18N.CONSTANTS.median();
        }

        @Override
        public StatFunction getFunction() {
            return MedianFunction.INSTANCE;
        }
    },
    MIN {
        @Override
        public String getLabel() {
            return I18N.CONSTANTS.minimum();
        }

        @Override
        public StatFunction getFunction() {
            return MinFunction.INSTANCE;
        }
    },
    MAX {
        @Override
        public String getLabel() {
            return I18N.CONSTANTS.maximum();
        }

        @Override
        public StatFunction getFunction() {
            return MaxFunction.INSTANCE;
        }
    };


    Statistic() {
    }

    public abstract String getLabel();

    public abstract StatFunction getFunction();

    public static List<String> labels() {
        List<String> labels = new ArrayList<>();
        for (Statistic statistic : values()) {
            labels.add(statistic.getLabel());
        }
        return labels;
    }
}
