/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.audit;

import org.activityinfo.i18n.shared.I18N;
import org.activityinfo.model.database.*;
import org.activityinfo.model.form.RecordChangeType;
import org.activityinfo.model.type.RecordRef;

import java.time.Instant;
import java.util.Optional;

public class RecordAuditEventFactory {

    public static final String ID_PREFIX = "record:";

    public static String composeFormLabel(VisibleDatabaseTree tree, Resource form) {
        StringBuilder label = new StringBuilder(form.getLabel());
        while(form.getType() == ResourceType.SUB_FORM) {
            form = tree.getResource(form.getParentId()).orElse(null);
            if(form == null) {
                break;
            }
            label.append(" in ").append(form.getLabel());
        }
        return label.toString();
    }

    public static AuditEvent recordEvent(RecordRef recordRef,
                                  String formLabel,
                                  long version,
                                  Optional<UserRef> user,
                                  Instant time,
                                  RecordChangeType type,
                                  boolean reverted) {
        return new AuditEventBuilder()
            .id(ID_PREFIX + ":" + recordRef.toQualifiedString() + ":" + version)
            .user(user)
            .version(version)
            .type(AuditEventType.RECORD)
            .time(time.toEpochMilli())
            .recordRef(recordRef)
            .description(auditDescription(type, formLabel))
            .reverted(reverted)
            .deleted(type == RecordChangeType.DELETED)
            .build();
    }

    private static String auditDescription(RecordChangeType type, String formLabel) {
        switch (type) {
            case ADDED:
                return I18N.MESSAGES.addedRecord(formLabel);
            case DELETED:
                return I18N.MESSAGES.deletedRecord(formLabel);
            case UPDATED:
                return I18N.MESSAGES.updatedRecord(formLabel);
            case RECOVERED:
                return I18N.MESSAGES.recoveredRecord(formLabel);
        }
        return "";
    }

}
