package org.activityinfo.model.collectionLink;

import java.lang.String;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;

public final class CollectionLinkBuilder {
  private ResourceId formId;

  private String linkId = "";

  private ResourceId databaseId;

  private CollectionLinkStatus status = CollectionLinkStatus.OPEN;

  public CollectionLinkBuilder() {
    linkId = "";
    status = CollectionLinkStatus.OPEN;
  }

  public CollectionLinkBuilder(CollectionLink toCopy) {
    this.formId = toCopy.getFormId();
    this.linkId = toCopy.getLinkId();
    this.databaseId = toCopy.getDatabaseId();
    this.status = toCopy.getStatus();
  }

  @Nonnull
  public CollectionLinkBuilder formId(@Nonnull ResourceId formId) {
    this.formId = formId;
    return this;
  }

  @Nonnull
  public CollectionLinkBuilder linkId(@Nonnull String linkId) {
    this.linkId = linkId;
    return this;
  }

  @Nonnull
  public CollectionLinkBuilder databaseId(@Nonnull ResourceId databaseId) {
    this.databaseId = databaseId;
    return this;
  }

  @Nonnull
  public CollectionLinkBuilder status(@Nonnull CollectionLinkStatus status) {
    this.status = status;
    return this;
  }

  @Nonnull
  public CollectionLink build() {
    assert this.formId != null : "formId is missing";
    assert this.linkId != null : "linkId is missing";
    assert this.databaseId != null : "databaseId is missing";
    assert this.status != null : "status is missing";
    return new CollectionLink(formId,linkId,databaseId,status);
  }
}
