/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */

package org.activityinfo.model.collectionLink;

import org.activityinfo.model.form.FormClass;
import org.activityinfo.model.form.FormMetadata;
import org.activityinfo.model.formTree.FormTree;
import org.activityinfo.model.resource.ResourceId;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class CollectionLinkPublicReferenceChecker {

    public CollectionLinkPublicReferenceChecker() {
    }

    /**
     * @param formTree
     * @param nonPublicReferenceList a list of reference non-public forms referenced by the root form and all
     *                               the reference forms which are used as the key fields in reference forms referenced
     *                               by the root form and are non-public either
     * @param isRoot is a given formTree computed for a root form
     */
    public List<FormClass> check(FormTree formTree, List<FormClass> nonPublicReferenceList, boolean isRoot) {
        // check for root reference fields: those referenced forms must be public
        List<FormTree.Node> rootFields = formTree.getRootFields();
        Set<ResourceId> visitedFormIds = new HashSet<>();
        for (FormTree.Node node : rootFields) {
            Collection<ResourceId> referenceRange = isRoot ? node.getOnlyReferenceRange() :
                    // in THOSE referenced forms, any key reference fields must ALSO reference public forms
                    node.getOnlyKeyReferenceRange();
            for (ResourceId formId : referenceRange) {
                FormMetadata formMetadata = formTree.getFormMetadata(formId);
                if (formMetadata.isVisible()) {
                    if (!formMetadata.getPermissions().isPublicForm()) {
                        nonPublicReferenceList.add(formMetadata.getSchema());
                    }
                    if (visitedFormIds.add(formMetadata.getId())) {
                        check(formTree.tree(formMetadata.getId()), nonPublicReferenceList, false);
                    }
                }
            }
        }
        return nonPublicReferenceList;
    }

}
