/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.resource.ResourceId;

import java.util.Optional;

@AutoJson
@AutoBuilder
public class AddDatabaseRequest {

    /**
     * The id of the new database.
     */
    private final ResourceId id;

    /**
     * Human readable name of the database.
     */
    private final String label;

    /**
     * The id of the template to use in creating the new database. The the two templates currently supported
     * are "reporting" (Multi-partner reporting) and "casemanagement." If ommitted, a blank database
     * with three simple roles will be added.
     */
    private final String templateId;

    /**
     * An optional, longer description for the database.
     */
    private final Optional<String> description;

    @Builder
    public AddDatabaseRequest(ResourceId id, String label, String templateId, Optional<String> description) {
        this.id = id;
        this.label = label;
        this.templateId = templateId;
        this.description = description;
    }

    public AddDatabaseRequest(ResourceId id, String label) {
        this(id, label, "", Optional.empty());
    }

    public ResourceId getId() {
        return id;
    }

    public String getTemplateId() {
        return templateId;
    }

    public String getLabel() {
        return label;
    }

    public Optional<String> getDescription() {
        return description;
    }


}
