package org.activityinfo.model.database;

import java.lang.String;
import java.util.Optional;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;

public final class AddDatabaseRequestBuilder {
  private ResourceId id;

  private String label = "";

  private String templateId = "";

  Optional<String> description = Optional.empty();

  public AddDatabaseRequestBuilder() {
    label = "";
    templateId = "";
  }

  public AddDatabaseRequestBuilder(AddDatabaseRequest toCopy) {
    this.id = toCopy.getId();
    this.label = toCopy.getLabel();
    this.templateId = toCopy.getTemplateId();
    this.description = toCopy.getDescription();
  }

  @Nonnull
  public AddDatabaseRequestBuilder id(@Nonnull ResourceId id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public AddDatabaseRequestBuilder label(@Nonnull String label) {
    this.label = label;
    return this;
  }

  @Nonnull
  public AddDatabaseRequestBuilder templateId(@Nonnull String templateId) {
    this.templateId = templateId;
    return this;
  }

  @Nonnull
  public AddDatabaseRequestBuilder description(@Nonnull String value) {
    this.description = Optional.of(value);
    return this;
  }

  @Nonnull
  public AddDatabaseRequestBuilder description(@Nonnull Optional<String> value) {
    this.description = value;
    return this;
  }

  @Nonnull
  public AddDatabaseRequest build() {
    assert this.id != null : "id is missing";
    assert this.label != null : "label is missing";
    assert this.templateId != null : "templateId is missing";
    return new AddDatabaseRequest(id,label,templateId,description);
  }
}
