/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;

import java.util.List;

@AutoJson
@AutoBuilder
public class AddDatabaseUserRequest {

    /**
     * The email of the user to add
     */
    private final String email;

    /**
     * The name of the user to add. This is only used in the email invitation to the user if there
     * is no existing ActivityInfo account with this email address.
     */
    private final String name;

    /**
     * The locale to use when sending an email invite to the user.
     */
    private final String locale;

    /**
     * The role to assign to the user.
     */
    private final RoleAssignment role;

    /**
     * Additional permissions to grant to the user.
     */
    private final List<DatabaseUserGrant> grants;

    public AddDatabaseUserRequest(String email, String name, String locale, RoleAssignment role, List<DatabaseUserGrant> grants) {
        this.email = email;
        this.name = name;
        this.locale = locale;
        this.role = role;
        this.grants = grants;
    }

    public String getEmail() {
        return email;
    }

    public String getLocale() {
        return locale;
    }

    public String getName() {
        return name;
    }

    public RoleAssignment getRole() {
        return role;
    }

    public List<DatabaseUserGrant> getGrants() {
        return grants;
    }
}
