package org.activityinfo.model.database;

import java.lang.String;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import javax.annotation.Nonnull;

public final class AddDatabaseUserRequestBuilder {
  private String email = "";

  private String name = "";

  private String locale = "";

  private RoleAssignment role;

  List<DatabaseUserGrant> grants = new ArrayList();

  public AddDatabaseUserRequestBuilder() {
    email = "";
    name = "";
    locale = "";
    this.grants = new ArrayList();
  }

  public AddDatabaseUserRequestBuilder(AddDatabaseUserRequest toCopy) {
    this.email = toCopy.getEmail();
    this.name = toCopy.getName();
    this.locale = toCopy.getLocale();
    this.role = toCopy.getRole();
    this.grants = new ArrayList(toCopy.getGrants());
  }

  @Nonnull
  public AddDatabaseUserRequestBuilder email(@Nonnull String email) {
    this.email = email;
    return this;
  }

  @Nonnull
  public AddDatabaseUserRequestBuilder name(@Nonnull String name) {
    this.name = name;
    return this;
  }

  @Nonnull
  public AddDatabaseUserRequestBuilder locale(@Nonnull String locale) {
    this.locale = locale;
    return this;
  }

  @Nonnull
  public AddDatabaseUserRequestBuilder role(@Nonnull RoleAssignment role) {
    this.role = role;
    return this;
  }

  @Nonnull
  public AddDatabaseUserRequestBuilder addGrants(
      @Nonnull Collection<DatabaseUserGrant> collection) {
    this.grants.addAll(collection);
    return this;
  }

  @Nonnull
  public AddDatabaseUserRequestBuilder addGrant(@Nonnull DatabaseUserGrant element) {
    this.grants.add(element);
    return this;
  }

  @Nonnull
  public AddDatabaseUserRequestBuilder removeGrant(@Nonnull DatabaseUserGrant element) {
    this.grants.remove(element);
    return this;
  }

  public AddDatabaseUserRequestBuilder addGrants(@Nonnull DatabaseUserGrant... array) {
    this.grants.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public AddDatabaseUserRequestBuilder grants(@Nonnull DatabaseUserGrant... array) {
    return grants(Arrays.asList(array));
  }

  @Nonnull
  public AddDatabaseUserRequestBuilder grants(@Nonnull Collection<DatabaseUserGrant> collection) {
    this.grants.clear();
    this.grants.addAll(collection);
    return this;
  }

  @Nonnull
  public AddDatabaseUserRequest build() {
    assert this.email != null : "email is missing";
    assert this.name != null : "name is missing";
    assert this.locale != null : "locale is missing";
    assert this.role != null : "role is missing";
    return new AddDatabaseUserRequest(email,name,locale,role,grants);
  }
}
