/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.form.FormClass;
import org.activityinfo.model.resource.ResourceId;

import java.util.Optional;

@AutoBuilder
@AutoJson
public class AddFormRequest {

    private final Resource formResource;
    private final FormClass formClass;

    @Builder
    public AddFormRequest(Resource formResource, FormClass formClass) {
        this.formResource = formResource;
        this.formClass = formClass;
    }

    public AddFormRequest(FormClass formClass) {
        this(formClass.getDatabaseId(), Optional.empty(), formClass);
    }

    public AddFormRequest(ResourceId databaseId, Optional<ResourceId> folderId, FormClass formClass) {
        ResourceId parentId = folderId.orElse(databaseId);
        this.formResource = new Resource(formClass.getId(), ResourceType.FORM, parentId, formClass.getLabel());
        this.formClass = formClass;
    }

    public AddFormRequest(ResourceId folderId, FormClass newForm) {
        this.formResource = new Resource(newForm.getId(), ResourceType.FORM, folderId, newForm.getLabel());
        this.formClass = newForm;
    }

    public Resource getFormResource() {
        return formResource;
    }

    public ResourceId getResourceId() {
        return formResource.getId();
    }

    public FormClass getFormClass() {
        return formClass;
    }

}
