/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.promise.HttpError;

public enum AddUserError implements HttpError {
    /**
     * A role with the requested id could not be found in this database.
     */
    ROLE_NOT_FOUND(BAD_REQUEST_400),

    /**
     * A database with this id could not be found
     */
    DATABASE_NOT_FOUND(BAD_REQUEST_400),

    /**
     * The billing account associated with this database has been
     * closed and so no additional users can be added.
     */
    BILLING_ACCOUNT_CLOSED(BILLING_402),

    /**
     * The billing account associated with this database has been
     * suspended so no additional users can be added.
     */
    BILLING_ACCOUNT_SUSPENDED(BILLING_402),

    /**
     * The user limit of the billing account associated with this database
     * has been exceeded so no additional users can be added.
     */
    USER_LIMIT_EXCEEDED(BILLING_402),

    /**
     * You do not have sufficient permission to add a user to this database.
     */
    PERMISSION_DENIED(FORBIDDEN_403),

    /**
     * You cannot add yourself to a database.
     */
    CANNOT_ADD_SELF(BAD_REQUEST_400),

    /**
     * The owner of a database cannot be added as a user.
     */
    CANNOT_ADD_OWNER(BAD_REQUEST_400),

    /**
     * A user with this email address has already been added to the database.
     */
    USER_ALREADY_ADDED(BAD_REQUEST_400),

    /**
     * The requested locale is invalid or not available.
     */
    INVALID_LOCALE(BAD_REQUEST_400),

    INTERNAL_SERVER_ERROR(500);

    private int httpStatusCode;

    AddUserError(int httpStatusCode) {
        this.httpStatusCode = httpStatusCode;
    }

    @Override
    public int getHttpStatusCode() {
        return httpStatusCode;
    }
}
