/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.RecordRef;

import java.util.Optional;

@AutoJson
@AutoBuilder
public class AuditEvent implements Comparable<AuditEvent> {

    public static final String USER_PERMISSION_UPDATE_PREFIX = "user:";

    private final String id;
    private final double time;
    private final Optional<UserRef> user;
    private final Optional<RecordRef> recordRef;
    private final long version;
    private final Optional<ResourceId> databaseUserId;
    private final String description;
    private final AuditEventType type;
    private final boolean deleted;
    private final boolean reverted;

    public AuditEvent(String id, double time, Optional<UserRef> user, Optional<RecordRef> recordRef, long version, Optional<ResourceId> databaseUserId, String description, AuditEventType type, boolean deleted, boolean reverted) {
        this.id = id;
        this.time = time;
        this.user = user;
        this.recordRef = recordRef;
        this.version = version;
        this.databaseUserId = databaseUserId;
        this.description = description;
        this.type = type;
        this.deleted = deleted;
        this.reverted = reverted;
    }

    public String getId() {
        return id;
    }

    /**
     * The time of the event, in milliseconds since the unix epoch.
     */
    public double getTime() {
        return time;
    }

    /**
     * The record ref, if this event concerns a record.
     */
    public Optional<RecordRef> getRecordRef() {
        return recordRef;
    }

    public long getVersion() {
        return version;
    }

    public Optional<ResourceId> getDatabaseUserId() {
        return databaseUserId;
    }

    public String getDescription() {
        return description;
    }

    public boolean isReverted() {
        return reverted;
    }

    public Optional<UserRef> getUser() {
        return user;
    }

    public boolean isDeleted() {
        return deleted;
    }

    public AuditEventType getType() {
        return type;
    }

    public AuditEvent revert() {
        return new AuditEventBuilder(this)
            .reverted(true)
            .build();
    }

    @Override
    public int compareTo(AuditEvent o) {
        return Double.compare(o.time, time);
    }
}
