package org.activityinfo.model.database;

import java.lang.String;
import java.util.Optional;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.RecordRef;

public final class AuditEventBuilder {
  private String id = "";

  private double time;

  Optional<UserRef> user = Optional.empty();

  Optional<RecordRef> recordRef = Optional.empty();

  private long version;

  Optional<ResourceId> databaseUserId = Optional.empty();

  private String description = "";

  private AuditEventType type = AuditEventType.DATABASE_TREE;

  private boolean deleted;

  private boolean reverted;

  public AuditEventBuilder() {
    id = "";
    description = "";
    type = AuditEventType.DATABASE_TREE;
  }

  public AuditEventBuilder(AuditEvent toCopy) {
    this.id = toCopy.getId();
    this.time = toCopy.getTime();
    this.user = toCopy.getUser();
    this.recordRef = toCopy.getRecordRef();
    this.version = toCopy.getVersion();
    this.databaseUserId = toCopy.getDatabaseUserId();
    this.description = toCopy.getDescription();
    this.type = toCopy.getType();
    this.deleted = toCopy.isDeleted();
    this.reverted = toCopy.isReverted();
  }

  @Nonnull
  public AuditEventBuilder id(@Nonnull String id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public AuditEventBuilder time(double time) {
    this.time = time;
    return this;
  }

  @Nonnull
  public AuditEventBuilder user(@Nonnull UserRef value) {
    this.user = Optional.of(value);
    return this;
  }

  @Nonnull
  public AuditEventBuilder user(@Nonnull Optional<UserRef> value) {
    this.user = value;
    return this;
  }

  @Nonnull
  public AuditEventBuilder recordRef(@Nonnull RecordRef value) {
    this.recordRef = Optional.of(value);
    return this;
  }

  @Nonnull
  public AuditEventBuilder recordRef(@Nonnull Optional<RecordRef> value) {
    this.recordRef = value;
    return this;
  }

  @Nonnull
  public AuditEventBuilder version(long version) {
    this.version = version;
    return this;
  }

  @Nonnull
  public AuditEventBuilder databaseUserId(@Nonnull ResourceId value) {
    this.databaseUserId = Optional.of(value);
    return this;
  }

  @Nonnull
  public AuditEventBuilder databaseUserId(@Nonnull Optional<ResourceId> value) {
    this.databaseUserId = value;
    return this;
  }

  @Nonnull
  public AuditEventBuilder description(@Nonnull String description) {
    this.description = description;
    return this;
  }

  @Nonnull
  public AuditEventBuilder type(@Nonnull AuditEventType type) {
    this.type = type;
    return this;
  }

  @Nonnull
  public AuditEventBuilder deleted(boolean deleted) {
    this.deleted = deleted;
    return this;
  }

  @Nonnull
  public AuditEventBuilder reverted(boolean reverted) {
    this.reverted = reverted;
    return this;
  }

  @Nonnull
  public AuditEvent build() {
    assert this.id != null : "id is missing";
    assert this.description != null : "description is missing";
    assert this.type != null : "type is missing";
    return new AuditEvent(id,time,user,recordRef,version,databaseUserId,description,type,deleted,reverted);
  }
}
