package org.activityinfo.model.database;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonArrays;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.RecordRef;

public final class AuditEventJson {
  public static JsonValue toJson(AuditEvent model) {
    JsonValue object = Json.createObject();
    object.put("id", model.getId());
    object.put("time", model.getTime());
    object.put("user", JsonArrays.optionalToJson(model.getUser(), UserRefJson::toJson));
    object.put("recordRef", JsonArrays.optionalToJson(model.getRecordRef(), RecordRef::toJson));
    object.put("version", model.getVersion());
    object.put("databaseUserId", JsonArrays.optionalToJson(model.getDatabaseUserId(), ResourceId::toJson));
    object.put("description", model.getDescription());
    object.put("type", model.getType().name());
    object.put("deleted", model.isDeleted());
    object.put("reverted", model.isReverted());
    return object;
  }

  public static AuditEvent fromJson(JsonValue object) {
    return new AuditEvent(object.getString("id"),
        (double)object.getNumber("time"),
        JsonArrays.optionalFromJson(object.get("user"), UserRefJson::fromJson),
        JsonArrays.optionalFromJson(object.get("recordRef"), RecordRef::fromJson),
        (long)object.getNumber("version"),
        JsonArrays.optionalFromJson(object.get("databaseUserId"), ResourceId::fromJson),
        object.getString("description"),
        JsonArrays.enumFromJson(object.get("type"), AuditEventType::valueOf, AuditEventType.DATABASE_TREE),
        object.getBoolean("deleted"),
        object.getBoolean("reverted"));
  }
}
