/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

@AutoJson
public class AuditEventList {
    private final List<AuditEvent> events;
    private final double startTime;
    private final double endTime;
    private final boolean moreEvents;

    @Builder
    public AuditEventList(List<AuditEvent> events, double startTime, double endTime, boolean moreEvents) {
        this.events = events;
        this.startTime = startTime;
        this.endTime = endTime;
        this.moreEvents = moreEvents;
    }

    public AuditEventList(List<AuditEventList> pages) {
        this.events = new ArrayList<>();
        this.startTime = pages.get(0).getStartTime();
        this.endTime = pages.get(pages.size() - 1).getEndTime();
        this.moreEvents = pages.get(pages.size() - 1).isMoreEvents();

        for (AuditEventList page : pages) {
            events.addAll(page.getEvents());
        }
    }

    public List<AuditEvent> getEvents() {
        return events;
    }

    public double getStartTime() {
        return startTime;
    }

    public double getEndTime() {
        return endTime;
    }

    public boolean isMoreEvents() {
        return moreEvents;
    }

    public AuditEventList transform(Function<AuditEvent, AuditEvent> transformer) {

        for (int i = 0; i < events.size(); i++) {
            AuditEvent oldEvent = events.get(i);
            AuditEvent newEvent = transformer.apply(oldEvent);
            if(newEvent != oldEvent) {
                return copyAndTransform(i, newEvent, transformer);
            }
        }

        // No changes
        return this;
    }

    private AuditEventList copyAndTransform(int firstChangeIndex, AuditEvent newEvent, Function<AuditEvent, AuditEvent> transformer) {
        List<AuditEvent> newEvents = new ArrayList<>();
        for (int i = 0; i < firstChangeIndex; i++) {
            newEvents.add(events.get(i));
        }
        newEvents.add(newEvent);
        for (int i = firstChangeIndex + 1; i < events.size(); i++) {
            newEvents.add(transformer.apply(events.get(i)));
        }

        return new AuditEventList(newEvents, startTime, endTime, moreEvents);
    }
}
