/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.resource.ResourceId;

import java.util.Arrays;
import java.util.Collections;
import java.util.Optional;
import java.util.Set;

@AutoJson
public class AuditRequest {
    private final Optional<ResourceId> resourceFilter;
    private final Set<AuditResourceType> typeFilter;
    private final long startTime;

    public AuditRequest() {
        resourceFilter = Optional.empty();
        typeFilter = Collections.emptySet();
        startTime = System.currentTimeMillis();
    }

    @Builder
    public AuditRequest(Optional<ResourceId> resourceFilter, Set<AuditResourceType> typeFilter, long startTime) {
        this.resourceFilter = resourceFilter;
        this.typeFilter = typeFilter;
        this.startTime = startTime;
    }

    /**
     * The start time of the request, in milliseconds since the unix epoch. The results will include
     * the first 100 - 150 events that occurred <strong>before</strong> this time.
     */
    public long getStartTime() {
        return startTime;
    }

    /**
     * Optionally restrict the list of events to a given folder or form.
     */
    public Optional<ResourceId> getResourceFilter() {
        return resourceFilter;
    }

    public Set<AuditResourceType> getTypeFilter() {
        return typeFilter;
    }

    public boolean isIncluded(AuditResourceType... types) {
        return isIncluded(Arrays.asList(types));
    }

    public boolean isIncluded(Iterable<AuditResourceType> types) {
        if(typeFilter.isEmpty()) {
            return true;
        }
        for (AuditResourceType type : types) {
            if(typeFilter.contains(type)) {
                return true;
            }
        }
        return false;
    }

    public AuditRequest withStartTime(long startingAt) {
        return new AuditRequest(resourceFilter, typeFilter, startingAt);
    }
}
