/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.resource.ResourceId;

@AutoJson
@AutoBuilder
public class DatabaseHeader {

    /**
     * The globally-unique id of this database.
     */
    private final ResourceId databaseId;

    /**
     * The human-readable name of this database.
     */
    private final String label;

    /**
     * A longer, optional description for this database.
     */
    private final String description;

    /**
     * The id of the user who owns this database.
     */
    private final ResourceId ownerId;

    /**
     * The id of the billing account to which this database belongs
     */
    private final long billingAccountId;

    /**
     * True if this database belongs to a billing account that has been suspended.
     */
    private final boolean suspended;

    /**
     * True if this database has been published as a template.
     */
    private final boolean publishedTemplate;

    @Builder
    public DatabaseHeader(ResourceId databaseId, String label, String description, ResourceId ownerId, long billingAccountId, boolean suspended, boolean publishedTemplate) {
        this.databaseId = databaseId;
        this.label = label;
        this.description = description;
        this.ownerId = ownerId;
        this.billingAccountId = billingAccountId;
        this.suspended = suspended;
        this.publishedTemplate = publishedTemplate;
    }

    public DatabaseHeader(VisibleDatabaseTree meta) {
        this(meta.getDatabaseId(),
                meta.getLabel(),
                meta.getDescription(),
                meta.getOwnerId(),
                meta.getBillingAccountId(),
                meta.isSuspended(),
                meta.isPublishedTemplate());
    }

    public ResourceId getDatabaseId() {
        return databaseId;
    }

    public String getLabel() {
        return label;
    }

    public String getDescription() {
        return description;
    }

    public ResourceId getOwnerId() {
        return ownerId;
    }

    public long getBillingAccountId() {
        return billingAccountId;
    }

    public boolean isSuspended() {
        return suspended;
    }

    public boolean isPublishedTemplate() {
        return publishedTemplate;
    }
}
