package org.activityinfo.model.database;

import java.lang.String;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;

public final class DatabaseHeaderBuilder {
  private ResourceId databaseId;

  private String label = "";

  private String description = "";

  private ResourceId ownerId;

  private long billingAccountId;

  private boolean suspended;

  private boolean publishedTemplate;

  public DatabaseHeaderBuilder() {
    label = "";
    description = "";
  }

  public DatabaseHeaderBuilder(DatabaseHeader toCopy) {
    this.databaseId = toCopy.getDatabaseId();
    this.label = toCopy.getLabel();
    this.description = toCopy.getDescription();
    this.ownerId = toCopy.getOwnerId();
    this.billingAccountId = toCopy.getBillingAccountId();
    this.suspended = toCopy.isSuspended();
    this.publishedTemplate = toCopy.isPublishedTemplate();
  }

  @Nonnull
  public DatabaseHeaderBuilder databaseId(@Nonnull ResourceId databaseId) {
    this.databaseId = databaseId;
    return this;
  }

  @Nonnull
  public DatabaseHeaderBuilder label(@Nonnull String label) {
    this.label = label;
    return this;
  }

  @Nonnull
  public DatabaseHeaderBuilder description(@Nonnull String description) {
    this.description = description;
    return this;
  }

  @Nonnull
  public DatabaseHeaderBuilder ownerId(@Nonnull ResourceId ownerId) {
    this.ownerId = ownerId;
    return this;
  }

  @Nonnull
  public DatabaseHeaderBuilder billingAccountId(long billingAccountId) {
    this.billingAccountId = billingAccountId;
    return this;
  }

  @Nonnull
  public DatabaseHeaderBuilder suspended(boolean suspended) {
    this.suspended = suspended;
    return this;
  }

  @Nonnull
  public DatabaseHeaderBuilder publishedTemplate(boolean publishedTemplate) {
    this.publishedTemplate = publishedTemplate;
    return this;
  }

  @Nonnull
  public DatabaseHeader build() {
    assert this.databaseId != null : "databaseId is missing";
    assert this.label != null : "label is missing";
    assert this.description != null : "description is missing";
    assert this.ownerId != null : "ownerId is missing";
    return new DatabaseHeader(databaseId,label,description,ownerId,billingAccountId,suspended,publishedTemplate);
  }
}
