package org.activityinfo.model.database;

import java.lang.String;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;

public final class DatabaseTreeBuilder {
  private ResourceId databaseId;

  private ResourceId ownerId;

  private long billingAccountId;

  private String ownerName = "";

  private String ownerEmail = "";

  private long version;

  private String label = "";

  private String description = "";

  private boolean publishedTemplate;

  private boolean suspended;

  private String storage = "";

  List<Resource> resources = new ArrayList();

  List<RecordLock> locks = new ArrayList();

  List<Role> roles = new ArrayList();

  List<SecurityCategory> securityCategories = new ArrayList();

  public DatabaseTreeBuilder() {
    ownerName = "";
    ownerEmail = "";
    label = "";
    description = "";
    storage = "";
    this.resources = new ArrayList();
    this.locks = new ArrayList();
    this.roles = new ArrayList();
    this.securityCategories = new ArrayList();
  }

  public DatabaseTreeBuilder(DatabaseTree toCopy) {
    this.databaseId = toCopy.getDatabaseId();
    this.ownerId = toCopy.getOwnerId();
    this.billingAccountId = toCopy.getBillingAccountId();
    this.ownerName = toCopy.getOwnerName();
    this.ownerEmail = toCopy.getOwnerEmail();
    this.version = toCopy.getVersion();
    this.label = toCopy.getLabel();
    this.description = toCopy.getDescription();
    this.publishedTemplate = toCopy.isPublishedTemplate();
    this.suspended = toCopy.isSuspended();
    this.storage = toCopy.getStorage();
    this.resources = new ArrayList(toCopy.getResources());
    this.locks = new ArrayList(toCopy.getLocks());
    this.roles = new ArrayList(toCopy.getRoles());
    this.securityCategories = new ArrayList(toCopy.getSecurityCategories());
  }

  @Nonnull
  public DatabaseTreeBuilder databaseId(@Nonnull ResourceId databaseId) {
    this.databaseId = databaseId;
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder ownerId(@Nonnull ResourceId ownerId) {
    this.ownerId = ownerId;
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder billingAccountId(long billingAccountId) {
    this.billingAccountId = billingAccountId;
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder ownerName(@Nonnull String ownerName) {
    this.ownerName = ownerName;
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder ownerEmail(@Nonnull String ownerEmail) {
    this.ownerEmail = ownerEmail;
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder version(long version) {
    this.version = version;
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder label(@Nonnull String label) {
    this.label = label;
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder description(@Nonnull String description) {
    this.description = description;
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder publishedTemplate(boolean publishedTemplate) {
    this.publishedTemplate = publishedTemplate;
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder suspended(boolean suspended) {
    this.suspended = suspended;
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder storage(@Nonnull String storage) {
    this.storage = storage;
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder addResources(@Nonnull Collection<Resource> collection) {
    this.resources.addAll(collection);
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder addResource(@Nonnull Resource element) {
    this.resources.add(element);
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder removeResource(@Nonnull Resource element) {
    this.resources.remove(element);
    return this;
  }

  public DatabaseTreeBuilder addResources(@Nonnull Resource... array) {
    this.resources.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder resources(@Nonnull Resource... array) {
    return resources(Arrays.asList(array));
  }

  @Nonnull
  public DatabaseTreeBuilder resources(@Nonnull Collection<Resource> collection) {
    this.resources.clear();
    this.resources.addAll(collection);
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder addLocks(@Nonnull Collection<RecordLock> collection) {
    this.locks.addAll(collection);
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder addLock(@Nonnull RecordLock element) {
    this.locks.add(element);
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder removeLock(@Nonnull RecordLock element) {
    this.locks.remove(element);
    return this;
  }

  public DatabaseTreeBuilder addLocks(@Nonnull RecordLock... array) {
    this.locks.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder locks(@Nonnull RecordLock... array) {
    return locks(Arrays.asList(array));
  }

  @Nonnull
  public DatabaseTreeBuilder locks(@Nonnull Collection<RecordLock> collection) {
    this.locks.clear();
    this.locks.addAll(collection);
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder addRoles(@Nonnull Collection<Role> collection) {
    this.roles.addAll(collection);
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder addRole(@Nonnull Role element) {
    this.roles.add(element);
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder removeRole(@Nonnull Role element) {
    this.roles.remove(element);
    return this;
  }

  public DatabaseTreeBuilder addRoles(@Nonnull Role... array) {
    this.roles.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder roles(@Nonnull Role... array) {
    return roles(Arrays.asList(array));
  }

  @Nonnull
  public DatabaseTreeBuilder roles(@Nonnull Collection<Role> collection) {
    this.roles.clear();
    this.roles.addAll(collection);
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder addSecurityCategories(
      @Nonnull Collection<SecurityCategory> collection) {
    this.securityCategories.addAll(collection);
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder addSecurityCategory(@Nonnull SecurityCategory element) {
    this.securityCategories.add(element);
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder removeSecurityCategory(@Nonnull SecurityCategory element) {
    this.securityCategories.remove(element);
    return this;
  }

  public DatabaseTreeBuilder addSecurityCategories(@Nonnull SecurityCategory... array) {
    this.securityCategories.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public DatabaseTreeBuilder securityCategories(@Nonnull SecurityCategory... array) {
    return securityCategories(Arrays.asList(array));
  }

  @Nonnull
  public DatabaseTreeBuilder securityCategories(@Nonnull Collection<SecurityCategory> collection) {
    this.securityCategories.clear();
    this.securityCategories.addAll(collection);
    return this;
  }

  @Nonnull
  public DatabaseTree build() {
    assert this.databaseId != null : "databaseId is missing";
    assert this.ownerId != null : "ownerId is missing";
    assert this.ownerName != null : "ownerName is missing";
    assert this.ownerEmail != null : "ownerEmail is missing";
    assert this.label != null : "label is missing";
    assert this.description != null : "description is missing";
    assert this.storage != null : "storage is missing";
    return new DatabaseTree(databaseId,ownerId,billingAccountId,ownerName,ownerEmail,version,label,description,publishedTemplate,suspended,storage,resources,locks,roles,securityCategories);
  }
}
