package org.activityinfo.model.database;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonArrays;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.resource.ResourceId;

public final class DatabaseTreeJson {
  public static JsonValue toJson(DatabaseTree model) {
    JsonValue object = Json.createObject();
    object.put("databaseId", model.getDatabaseId().asString());
    object.put("ownerId", model.getOwnerId().asString());
    object.put("billingAccountId", model.getBillingAccountId());
    object.put("ownerName", model.getOwnerName());
    object.put("ownerEmail", model.getOwnerEmail());
    object.put("version", model.getVersion());
    object.put("label", model.getLabel());
    object.put("description", model.getDescription());
    object.put("publishedTemplate", model.isPublishedTemplate());
    object.put("suspended", model.isSuspended());
    object.put("storage", model.getStorage());
    object.put("roles", JsonArrays.toJsonArray(model.getRoles(), RoleJson::toJson));
    object.put("securityCategories", JsonArrays.toJsonArray(model.getSecurityCategories(), SecurityCategoryJson::toJson));
    object.put("resources", JsonArrays.toJsonArray(model.getResources(), Resource::toJson));
    object.put("locks", JsonArrays.toJsonArray(model.getLocks(), RecordLockJson::toJson));
    return object;
  }

  public static DatabaseTree fromJson(JsonValue object) {
    return new DatabaseTree(ResourceId.valueOf(object.getString("databaseId")),
        ResourceId.valueOf(object.getString("ownerId")),
        (long)object.getNumber("billingAccountId"),
        object.getString("ownerName"),
        object.getString("ownerEmail"),
        (long)object.getNumber("version"),
        object.getString("label"),
        object.getString("description"),
        object.getBoolean("publishedTemplate"),
        object.getBoolean("suspended"),
        object.getString("storage"),
        JsonArrays.listFromJson(object.get("resources"), Resource::fromJson),
        JsonArrays.listFromJson(object.get("locks"), RecordLockJson::fromJson),
        JsonArrays.listFromJson(object.get("roles"), RoleJson::fromJson),
        JsonArrays.listFromJson(object.get("securityCategories"), SecurityCategoryJson::fromJson));
  }
}
