/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import com.google.common.base.Strings;
import org.activityinfo.json.AutoJson;
import org.activityinfo.model.account.DeliveryStatus;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.resource.ResourceId;

@AutoJson
@AutoBuilder
public class DatabaseUser {

    private final ResourceId databaseId;
    private final ResourceId userId;

    /**
     * The user's name, provided either when invited or by the user themselves
     * after activating their account.
     */
    private final String name;

    /**
     * This user's email address.
     */
    private final String email;

    /**
     * This user's assigned role in the database.
     */
    private final RoleAssignment role;

    /**
     * A monotonically-increasing version number. This version is incremented each time a change
     * is made to the users permissions.
     */
    private final long version;

    /**
     * The date on which the user was invited to this database.
     */
    private final String inviteDate;

    private final DeliveryStatus deliveryStatus;

    /**
     * True if the user has accept the invitation to create an ActivityInfo account.
     */
    private final boolean inviteAccepted;

    @Builder
    public DatabaseUser(ResourceId databaseId,
                        ResourceId userId,
                        String name,
                        String email,
                        RoleAssignment role, long version,
                        String inviteDate,
                        DeliveryStatus deliveryStatus,
                        boolean inviteAccepted) {
        this.databaseId = databaseId;
        this.userId = userId;
        this.name = Strings.nullToEmpty(name);
        this.email = email;
        this.role = role;
        this.version = version;
        this.inviteDate = inviteDate;
        this.deliveryStatus = deliveryStatus;
        this.inviteAccepted = inviteAccepted;
    }

    public ResourceId getDatabaseId() {
        return databaseId;
    }

    public ResourceId getUserId() {
        return userId;
    }

    public DeliveryStatus getDeliveryStatus() {
        return deliveryStatus;
    }

    public String getName() {
        return name;
    }

    public String getEmail() {
        return email;
    }

    public long getVersion() {
        return version;
    }

    public String getInviteDate() {
        return inviteDate;
    }

    public boolean isInviteAccepted() {
        return inviteAccepted;
    }

    public RoleAssignment getRole() {
        return role;
    }

}
