package org.activityinfo.model.database;

import java.lang.String;
import javax.annotation.Nonnull;
import org.activityinfo.model.account.DeliveryStatus;
import org.activityinfo.model.resource.ResourceId;

public final class DatabaseUserBuilder {
  private ResourceId databaseId;

  private ResourceId userId;

  private String name = "";

  private String email = "";

  private RoleAssignment role;

  private long version;

  private String inviteDate = "";

  private DeliveryStatus deliveryStatus = DeliveryStatus.UNKNOWN;

  private boolean inviteAccepted;

  public DatabaseUserBuilder() {
    name = "";
    email = "";
    inviteDate = "";
    deliveryStatus = DeliveryStatus.UNKNOWN;
  }

  public DatabaseUserBuilder(DatabaseUser toCopy) {
    this.databaseId = toCopy.getDatabaseId();
    this.userId = toCopy.getUserId();
    this.name = toCopy.getName();
    this.email = toCopy.getEmail();
    this.role = toCopy.getRole();
    this.version = toCopy.getVersion();
    this.inviteDate = toCopy.getInviteDate();
    this.deliveryStatus = toCopy.getDeliveryStatus();
    this.inviteAccepted = toCopy.isInviteAccepted();
  }

  @Nonnull
  public DatabaseUserBuilder databaseId(@Nonnull ResourceId databaseId) {
    this.databaseId = databaseId;
    return this;
  }

  @Nonnull
  public DatabaseUserBuilder userId(@Nonnull ResourceId userId) {
    this.userId = userId;
    return this;
  }

  @Nonnull
  public DatabaseUserBuilder name(@Nonnull String name) {
    this.name = name;
    return this;
  }

  @Nonnull
  public DatabaseUserBuilder email(@Nonnull String email) {
    this.email = email;
    return this;
  }

  @Nonnull
  public DatabaseUserBuilder role(@Nonnull RoleAssignment role) {
    this.role = role;
    return this;
  }

  @Nonnull
  public DatabaseUserBuilder version(long version) {
    this.version = version;
    return this;
  }

  @Nonnull
  public DatabaseUserBuilder inviteDate(@Nonnull String inviteDate) {
    this.inviteDate = inviteDate;
    return this;
  }

  @Nonnull
  public DatabaseUserBuilder deliveryStatus(@Nonnull DeliveryStatus deliveryStatus) {
    this.deliveryStatus = deliveryStatus;
    return this;
  }

  @Nonnull
  public DatabaseUserBuilder inviteAccepted(boolean inviteAccepted) {
    this.inviteAccepted = inviteAccepted;
    return this;
  }

  @Nonnull
  public DatabaseUser build() {
    assert this.databaseId != null : "databaseId is missing";
    assert this.userId != null : "userId is missing";
    assert this.name != null : "name is missing";
    assert this.email != null : "email is missing";
    assert this.role != null : "role is missing";
    assert this.inviteDate != null : "inviteDate is missing";
    assert this.deliveryStatus != null : "deliveryStatus is missing";
    return new DatabaseUser(databaseId,userId,name,email,role,version,inviteDate,deliveryStatus,inviteAccepted);
  }
}
