package org.activityinfo.model.database;

import java.lang.String;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import javax.annotation.Nonnull;
import org.activityinfo.model.account.ActivationStatus;
import org.activityinfo.model.account.DeliveryStatus;
import org.activityinfo.model.resource.ResourceId;

public final class DatabaseUserGrantSetBuilder {
  private ResourceId databaseId;

  private ResourceId userId;

  private long version;

  private String email = "";

  private String name = "";

  private double inviteTime;

  private ActivationStatus activationStatus = ActivationStatus.NO_ACCOUNT;

  private DeliveryStatus deliveryStatus = DeliveryStatus.UNKNOWN;

  private double lastLoginTime;

  private RoleAssignment role;

  List<DatabaseUserGrant> grants = new ArrayList();

  public DatabaseUserGrantSetBuilder() {
    email = "";
    name = "";
    activationStatus = ActivationStatus.NO_ACCOUNT;
    deliveryStatus = DeliveryStatus.UNKNOWN;
    this.grants = new ArrayList();
  }

  public DatabaseUserGrantSetBuilder(DatabaseUserGrantSet toCopy) {
    this.databaseId = toCopy.getDatabaseId();
    this.userId = toCopy.getUserId();
    this.version = toCopy.getVersion();
    this.email = toCopy.getEmail();
    this.name = toCopy.getName();
    this.inviteTime = toCopy.getInviteTime();
    this.activationStatus = toCopy.getActivationStatus();
    this.deliveryStatus = toCopy.getDeliveryStatus();
    this.lastLoginTime = toCopy.getLastLoginTime();
    this.role = toCopy.getRole();
    this.grants = new ArrayList(toCopy.getGrants());
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder databaseId(@Nonnull ResourceId databaseId) {
    this.databaseId = databaseId;
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder userId(@Nonnull ResourceId userId) {
    this.userId = userId;
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder version(long version) {
    this.version = version;
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder email(@Nonnull String email) {
    this.email = email;
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder name(@Nonnull String name) {
    this.name = name;
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder inviteTime(double inviteTime) {
    this.inviteTime = inviteTime;
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder activationStatus(@Nonnull ActivationStatus activationStatus) {
    this.activationStatus = activationStatus;
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder deliveryStatus(@Nonnull DeliveryStatus deliveryStatus) {
    this.deliveryStatus = deliveryStatus;
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder lastLoginTime(double lastLoginTime) {
    this.lastLoginTime = lastLoginTime;
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder role(@Nonnull RoleAssignment role) {
    this.role = role;
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder addGrants(@Nonnull Collection<DatabaseUserGrant> collection) {
    this.grants.addAll(collection);
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder addGrant(@Nonnull DatabaseUserGrant element) {
    this.grants.add(element);
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder removeGrant(@Nonnull DatabaseUserGrant element) {
    this.grants.remove(element);
    return this;
  }

  public DatabaseUserGrantSetBuilder addGrants(@Nonnull DatabaseUserGrant... array) {
    this.grants.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder grants(@Nonnull DatabaseUserGrant... array) {
    return grants(Arrays.asList(array));
  }

  @Nonnull
  public DatabaseUserGrantSetBuilder grants(@Nonnull Collection<DatabaseUserGrant> collection) {
    this.grants.clear();
    this.grants.addAll(collection);
    return this;
  }

  @Nonnull
  public DatabaseUserGrantSet build() {
    assert this.databaseId != null : "databaseId is missing";
    assert this.userId != null : "userId is missing";
    assert this.email != null : "email is missing";
    assert this.name != null : "name is missing";
    assert this.activationStatus != null : "activationStatus is missing";
    assert this.deliveryStatus != null : "deliveryStatus is missing";
    assert this.role != null : "role is missing";
    return new DatabaseUserGrantSet(databaseId,userId,version,email,name,inviteTime,activationStatus,deliveryStatus,lastLoginTime,role,grants);
  }
}
