package org.activityinfo.model.database;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonArrays;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.account.ActivationStatus;
import org.activityinfo.model.account.DeliveryStatus;
import org.activityinfo.model.resource.ResourceId;

public final class DatabaseUserGrantSetJson {
  public static JsonValue toJson(DatabaseUserGrantSet model) {
    JsonValue object = Json.createObject();
    object.put("databaseId", model.getDatabaseId().asString());
    object.put("userId", model.getUserId().asString());
    object.put("version", model.getVersion());
    object.put("email", model.getEmail());
    object.put("name", model.getName());
    object.put("inviteTime", model.getInviteTime());
    object.put("activationStatus", model.getActivationStatus().name());
    object.put("deliveryStatus", model.getDeliveryStatus().name());
    object.put("lastLoginTime", model.getLastLoginTime());
    object.put("role", model.getRole().toJson());
    object.put("grants", JsonArrays.toJsonArray(model.getGrants(), DatabaseUserGrant::toJson));
    return object;
  }

  public static DatabaseUserGrantSet fromJson(JsonValue object) {
    return new DatabaseUserGrantSet(ResourceId.valueOf(object.getString("databaseId")),
        ResourceId.valueOf(object.getString("userId")),
        (long)object.getNumber("version"),
        object.getString("email"),
        object.getString("name"),
        (double)object.getNumber("inviteTime"),
        JsonArrays.enumFromJson(object.get("activationStatus"), ActivationStatus::valueOf, ActivationStatus.NO_ACCOUNT),
        JsonArrays.enumFromJson(object.get("deliveryStatus"), DeliveryStatus::valueOf, DeliveryStatus.UNKNOWN),
        (double)object.getNumber("lastLoginTime"),
        RoleAssignment.fromJson(object.get("role")),
        JsonArrays.listFromJson(object.get("grants"), DatabaseUserGrant::fromJson));
  }
}
