/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.permission.Operation;
import org.activityinfo.model.resource.ResourceId;

import java.util.HashSet;
import java.util.Optional;
import java.util.Set;

@AutoJson
public class FilteredPermission {

    private Operation operation;

    /**
     * A formula which restricts a permission to specific records. Only record-level
     * permissions can be filtered.
     */
    private Optional<String> filter;

    private Set<ResourceId> securityCategories;

    @Builder
    public FilteredPermission(Operation operation, Optional<String> filter, Set<ResourceId> securityCategories) {
        this.operation = operation;
        this.filter = filter;
        this.securityCategories = securityCategories;
    }

    public FilteredPermission(Operation operation, Optional<String> filter) {
        this.operation = operation;
        this.filter = filter;
        this.securityCategories = new HashSet<>();
    }

    public FilteredPermission(Operation operation) {
        this.operation = operation;
        this.filter = Optional.empty();
        this.securityCategories = new HashSet<>();
    }

    public FilteredPermission(Operation operation, String filter) {
        this.operation = operation;
        this.filter = Optional.of(filter);
        this.securityCategories = new HashSet<>();
    }

    public FilteredPermission(Operation operation, Set<ResourceId> securityCategories) {
        this.operation = operation;
        this.filter = Optional.empty();
        this.securityCategories = securityCategories;
    }

    public Operation getOperation() {
        return operation;
    }

    public boolean isFiltered() {
        return filter.isPresent();
    }

    public Optional<String> getFilter() {
        return filter;
    }

    public Set<ResourceId> getSecurityCategories() {
        return securityCategories;
    }
}
