/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.model.permission.Operation;

import java.util.*;

public class PermissionSet implements HasPermissions {

    private Map<Operation, FilteredPermission> map = new HashMap<>();

    public PermissionSet(Iterable<FilteredPermission> permissions) {
        for (FilteredPermission permission : permissions) {
            map.put(permission.getOperation(), permission);
        }
    }


    @Override
    public Optional<FilteredPermission> getPermission(Operation operation) {
        return Optional.ofNullable(map.get(operation));
    }

    @Override
    public boolean hasPermission(Operation operation) {
        return map.containsKey(operation);
    }

    @Override
    public Collection<FilteredPermission> getPermissions() {
        return map.values();
    }

    /**
     * Returns a new {@code PermissionSet} with all of the operations permitted by the given {@code permissions},
     * <em>plus</em> the permission to view all records.
     */
    public static PermissionSet withUnfilteredView(HasPermissions permissions) {
        List<FilteredPermission> newList = new ArrayList<>(permissions.getPermissions());
        newList.add(new FilteredPermission(Operation.VIEW));
        return new PermissionSet(newList);
    }

    /**
     *
     * @param grant
     * @return
     */
    public static HasPermissions withView(HasPermissions grant) {
        return null;
    }
}
