/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.account.ActivationStatus;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.resource.ResourceId;

import java.util.Optional;

@AutoBuilder
@AutoJson
public class PreflightUserResponse {
    private final Optional<ResourceId> userId;
    private final ActivationStatus activationStatus;
    private final String name;
    private final boolean addedToDatabase;
    private final boolean validEmail;
    private final String localizedErrorMessage;

    @Builder
    public PreflightUserResponse(Optional<ResourceId> userId, ActivationStatus activationStatus, String name, boolean addedToDatabase, boolean validEmail, String localizedErrorMessage) {
        this.userId = userId;
        this.activationStatus = activationStatus;
        this.name = name;
        this.addedToDatabase = addedToDatabase;
        this.validEmail = validEmail;
        this.localizedErrorMessage = localizedErrorMessage;
    }

    /**
     * If the user is an existing user, then their user id. Otherwise zero.
     * @return
     */
    public Optional<ResourceId> getUserId() {
        return userId;
    }

    public ActivationStatus getActivationStatus() {
        return activationStatus;
    }

    public String getName() {
        return name;
    }

    public boolean isActiveUser() {
        return activationStatus != ActivationStatus.NO_ACCOUNT;
    }

    public boolean isAddedToDatabase() {
        return addedToDatabase;
    }

    public boolean isValidEmail() {
        return validEmail;
    }

    public String getLocalizedErrorMessage() {
        return localizedErrorMessage;
    }
}
