package org.activityinfo.model.database;

import java.lang.String;
import java.util.Optional;
import javax.annotation.Nonnull;
import org.activityinfo.model.account.ActivationStatus;
import org.activityinfo.model.resource.ResourceId;

public final class PreflightUserResponseBuilder {
  Optional<ResourceId> userId = Optional.empty();

  private ActivationStatus activationStatus = ActivationStatus.NO_ACCOUNT;

  private String name = "";

  private boolean addedToDatabase;

  private boolean validEmail;

  private String localizedErrorMessage = "";

  public PreflightUserResponseBuilder() {
    activationStatus = ActivationStatus.NO_ACCOUNT;
    name = "";
    localizedErrorMessage = "";
  }

  public PreflightUserResponseBuilder(PreflightUserResponse toCopy) {
    this.userId = toCopy.getUserId();
    this.activationStatus = toCopy.getActivationStatus();
    this.name = toCopy.getName();
    this.addedToDatabase = toCopy.isAddedToDatabase();
    this.validEmail = toCopy.isValidEmail();
    this.localizedErrorMessage = toCopy.getLocalizedErrorMessage();
  }

  @Nonnull
  public PreflightUserResponseBuilder userId(@Nonnull ResourceId value) {
    this.userId = Optional.of(value);
    return this;
  }

  @Nonnull
  public PreflightUserResponseBuilder userId(@Nonnull Optional<ResourceId> value) {
    this.userId = value;
    return this;
  }

  @Nonnull
  public PreflightUserResponseBuilder activationStatus(@Nonnull ActivationStatus activationStatus) {
    this.activationStatus = activationStatus;
    return this;
  }

  @Nonnull
  public PreflightUserResponseBuilder name(@Nonnull String name) {
    this.name = name;
    return this;
  }

  @Nonnull
  public PreflightUserResponseBuilder addedToDatabase(boolean addedToDatabase) {
    this.addedToDatabase = addedToDatabase;
    return this;
  }

  @Nonnull
  public PreflightUserResponseBuilder validEmail(boolean validEmail) {
    this.validEmail = validEmail;
    return this;
  }

  @Nonnull
  public PreflightUserResponseBuilder localizedErrorMessage(@Nonnull String localizedErrorMessage) {
    this.localizedErrorMessage = localizedErrorMessage;
    return this;
  }

  @Nonnull
  public PreflightUserResponse build() {
    assert this.activationStatus != null : "activationStatus is missing";
    assert this.name != null : "name is missing";
    assert this.localizedErrorMessage != null : "localizedErrorMessage is missing";
    return new PreflightUserResponse(userId,activationStatus,name,addedToDatabase,validEmail,localizedErrorMessage);
  }
}
