/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.time.LocalDateInterval;

import java.util.Objects;

@AutoJson
@AutoBuilder
public class RecordLock {

    /**
     * This lock's id.
     */
    private final ResourceId id;

    /**
     * A human-readable label for this lock
     */
    private final String label;

    /**
     * The resource (database, folder, form, or subform) to which this lock applies.
     */
    private final ResourceId resourceId;

    /**
     * The date range to which this lock applies.
     */
    private final LocalDateInterval dateRange;

    public RecordLock(ResourceId id, String label, ResourceId resourceId, LocalDateInterval dateRange) {
        this.id = id;
        this.label = label;
        this.resourceId = resourceId;
        this.dateRange = dateRange;
    }

    /**
     * @return the id of this lock itself.
     */
    public ResourceId getId() {
        return id;
    }

    /**
     * @return the name of this lock.
     */
    public String getLabel() {
        return label;
    }

    /**
     *
     * @return the id of the resource to be locked.
     */
    public ResourceId getResourceId() {
        return resourceId;
    }

    /**
     * @return the date range of records to which this lock applies.
     */
    public LocalDateInterval getDateRange() {
        return dateRange;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof RecordLock)) {
            return false;
        }
        RecordLock that = (RecordLock) o;
        return id.equals(that.id) &&
                label.equals(that.label) &&
                resourceId.equals(that.resourceId) &&
                dateRange.equals(that.dateRange);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, label, resourceId, dateRange);
    }
}
