package org.activityinfo.model.database;

import java.lang.String;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.time.LocalDateInterval;

public final class RecordLockBuilder {
  private ResourceId id;

  private String label = "";

  private ResourceId resourceId;

  private LocalDateInterval dateRange;

  public RecordLockBuilder() {
    label = "";
  }

  public RecordLockBuilder(RecordLock toCopy) {
    this.id = toCopy.getId();
    this.label = toCopy.getLabel();
    this.resourceId = toCopy.getResourceId();
    this.dateRange = toCopy.getDateRange();
  }

  @Nonnull
  public RecordLockBuilder id(@Nonnull ResourceId id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public RecordLockBuilder label(@Nonnull String label) {
    this.label = label;
    return this;
  }

  @Nonnull
  public RecordLockBuilder resourceId(@Nonnull ResourceId resourceId) {
    this.resourceId = resourceId;
    return this;
  }

  @Nonnull
  public RecordLockBuilder dateRange(@Nonnull LocalDateInterval dateRange) {
    this.dateRange = dateRange;
    return this;
  }

  @Nonnull
  public RecordLock build() {
    assert this.id != null : "id is missing";
    assert this.label != null : "label is missing";
    assert this.resourceId != null : "resourceId is missing";
    assert this.dateRange != null : "dateRange is missing";
    return new RecordLock(id,label,resourceId,dateRange);
  }
}
