/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.promise.HttpError;

public enum RecoverDatabaseResult implements HttpError {

    /**
     * The database was recovered.
     */
    RECOVERED(200),

    /**
     * No record of a database with this id could be found.
     */
    NOT_FOUND(NOT_FOUND_404),

    /**
     * A database with this id exists, but is not deleted.
     */
    NOT_DELETED(BAD_REQUEST_400),

    /**
     * A database with this id was deleted, but it has been permanently deleted and
     * cannot be recovered.
     */
    PURGED(BAD_REQUEST_400),

    /**
     * Only the owner of a database at the time it was deleted, can recover the
     * database.
     */
    PERMISSION_DENIED(FORBIDDEN_403),

    /**
     * The requesting user does not have a billing account, or the account is suspended.
     */
    SUSPENDED(402);


    private final int statusCode;

    RecoverDatabaseResult(int statusCode) {
        this.statusCode = statusCode;
    }

    @Override
    public int getHttpStatusCode() {
        return statusCode;
    }
}
