/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.i18n.shared.I18N;
import org.activityinfo.promise.HttpError;
import org.activityinfo.promise.LocalizedError;
import org.checkerframework.checker.nullness.qual.Nullable;

public enum RestoreDatabaseUserError implements HttpError, LocalizedError {
    REVERTED(200),
    DATABASE_NOT_FOUND(404),

    /**
     * You do not have the "AUDIT" permission and so cannot restore deleted users.
     */
    AUDIT_NOT_AUTHORIZED(FORBIDDEN_403),

    /**
     * The user has already been restored or re-added to the database.
     */
    USER_ALREADY_RESTORED(400) {
        @Override
        public @Nullable String getLocalizedErrorMessage() {
            return I18N.CONSTANTS.userAlreadyRestored();
        }
    },

    /**
     * This user never had access to this database.
     */
    USER_NEVER_HAD_ROLE(400),

    /**
     * This user has closed their ActivityInfo user account and cannot be added
     * to databases again.
     */
    USER_ACCOUNT_CLOSED(400),


    /**
     * The user was previously assigned a role that has since been
     * deleted. This user cannot be restored.
     */
    ROLE_DELETED(BAD_REQUEST_400) {
        @Override
        public @Nullable String getLocalizedErrorMessage() {
            return I18N.CONSTANTS.userRoleDeleted();
        }
    },

    /**
    /**
     * The billing account associated with this database has been
     * closed and so no additional users can be added.
     */
    BILLING_ACCOUNT_CLOSED(BILLING_402),

    /**
     * The billing account associated with this database has been
     * suspended so no additional users can be added.
     */
    BILLING_ACCOUNT_SUSPENDED(BILLING_402),

    /**
     * The user limit of the billing account associated with this database
     * has been exceeded so no additional users can be added.
     */
    USER_LIMIT_EXCEEDED(BILLING_402),

    /**
     * You do not have sufficient permission to restore a user with these permissions.
     */
    PERMISSION_DENIED(FORBIDDEN_403),

    /**
     * You cannot add yourself to a database.
     */
    CANNOT_ADD_SELF(BAD_REQUEST_400),

    /**
     * The owner of a database cannot be added as a user.
     */
    CANNOT_ADD_OWNER(BAD_REQUEST_400),


    INTERNAL_SERVER_ERROR(500);


    private final int statusCode;

    RestoreDatabaseUserError(int statusCode) {
        this.statusCode = statusCode;
    }

    @Override
    public int getHttpStatusCode() {
        return statusCode;
    }
}
