package org.activityinfo.model.database;

import java.lang.String;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;

public final class RoleBuilder {
  private ResourceId id;

  private String label = "";

  List<FilteredPermission> permissions = new ArrayList();

  List<RoleParameter> parameters = new ArrayList();

  List<RoleFilter> filters = new ArrayList();

  List<DatabaseUserGrant> grants = new ArrayList();

  private int version;

  private boolean grantBased;

  public RoleBuilder() {
    label = "";
    this.permissions = new ArrayList();
    this.parameters = new ArrayList();
    this.filters = new ArrayList();
    this.grants = new ArrayList();
  }

  public RoleBuilder(Role toCopy) {
    this.id = toCopy.getId();
    this.label = toCopy.getLabel();
    this.permissions = new ArrayList(toCopy.getPermissions());
    this.parameters = new ArrayList(toCopy.getParameters());
    this.filters = new ArrayList(toCopy.getFilters());
    this.grants = new ArrayList(toCopy.getGrants());
    this.version = toCopy.getVersion();
    this.grantBased = toCopy.isGrantBased();
  }

  @Nonnull
  public RoleBuilder id(@Nonnull ResourceId id) {
    this.id = id;
    return this;
  }

  @Nonnull
  public RoleBuilder label(@Nonnull String label) {
    this.label = label;
    return this;
  }

  @Nonnull
  public RoleBuilder addPermissions(@Nonnull Collection<FilteredPermission> collection) {
    this.permissions.addAll(collection);
    return this;
  }

  @Nonnull
  public RoleBuilder addPermission(@Nonnull FilteredPermission element) {
    this.permissions.add(element);
    return this;
  }

  @Nonnull
  public RoleBuilder removePermission(@Nonnull FilteredPermission element) {
    this.permissions.remove(element);
    return this;
  }

  public RoleBuilder addPermissions(@Nonnull FilteredPermission... array) {
    this.permissions.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public RoleBuilder permissions(@Nonnull FilteredPermission... array) {
    return permissions(Arrays.asList(array));
  }

  @Nonnull
  public RoleBuilder permissions(@Nonnull Collection<FilteredPermission> collection) {
    this.permissions.clear();
    this.permissions.addAll(collection);
    return this;
  }

  @Nonnull
  public RoleBuilder addParameters(@Nonnull Collection<RoleParameter> collection) {
    this.parameters.addAll(collection);
    return this;
  }

  @Nonnull
  public RoleBuilder addParameter(@Nonnull RoleParameter element) {
    this.parameters.add(element);
    return this;
  }

  @Nonnull
  public RoleBuilder removeParameter(@Nonnull RoleParameter element) {
    this.parameters.remove(element);
    return this;
  }

  public RoleBuilder addParameters(@Nonnull RoleParameter... array) {
    this.parameters.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public RoleBuilder parameters(@Nonnull RoleParameter... array) {
    return parameters(Arrays.asList(array));
  }

  @Nonnull
  public RoleBuilder parameters(@Nonnull Collection<RoleParameter> collection) {
    this.parameters.clear();
    this.parameters.addAll(collection);
    return this;
  }

  @Nonnull
  public RoleBuilder addFilters(@Nonnull Collection<RoleFilter> collection) {
    this.filters.addAll(collection);
    return this;
  }

  @Nonnull
  public RoleBuilder addFilter(@Nonnull RoleFilter element) {
    this.filters.add(element);
    return this;
  }

  @Nonnull
  public RoleBuilder removeFilter(@Nonnull RoleFilter element) {
    this.filters.remove(element);
    return this;
  }

  public RoleBuilder addFilters(@Nonnull RoleFilter... array) {
    this.filters.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public RoleBuilder filters(@Nonnull RoleFilter... array) {
    return filters(Arrays.asList(array));
  }

  @Nonnull
  public RoleBuilder filters(@Nonnull Collection<RoleFilter> collection) {
    this.filters.clear();
    this.filters.addAll(collection);
    return this;
  }

  @Nonnull
  public RoleBuilder addGrants(@Nonnull Collection<DatabaseUserGrant> collection) {
    this.grants.addAll(collection);
    return this;
  }

  @Nonnull
  public RoleBuilder addGrant(@Nonnull DatabaseUserGrant element) {
    this.grants.add(element);
    return this;
  }

  @Nonnull
  public RoleBuilder removeGrant(@Nonnull DatabaseUserGrant element) {
    this.grants.remove(element);
    return this;
  }

  public RoleBuilder addGrants(@Nonnull DatabaseUserGrant... array) {
    this.grants.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public RoleBuilder grants(@Nonnull DatabaseUserGrant... array) {
    return grants(Arrays.asList(array));
  }

  @Nonnull
  public RoleBuilder grants(@Nonnull Collection<DatabaseUserGrant> collection) {
    this.grants.clear();
    this.grants.addAll(collection);
    return this;
  }

  @Nonnull
  public RoleBuilder version(int version) {
    this.version = version;
    return this;
  }

  @Nonnull
  public RoleBuilder grantBased(boolean grantBased) {
    this.grantBased = grantBased;
    return this;
  }

  @Nonnull
  public Role build() {
    assert this.id != null : "id is missing";
    assert this.label != null : "label is missing";
    return new Role(id,label,permissions,parameters,filters,grants,version,grantBased);
  }
}
