package org.activityinfo.model.database;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonArrays;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.resource.ResourceId;

public final class RoleJson {
  public static JsonValue toJson(Role model) {
    JsonValue object = Json.createObject();
    object.put("id", model.getId().asString());
    object.put("label", model.getLabel());
    object.put("permissions", JsonArrays.toJsonArray(model.getPermissions(), FilteredPermissionJson::toJson));
    object.put("parameters", JsonArrays.toJsonArray(model.getParameters(), RoleParameterJson::toJson));
    object.put("filters", JsonArrays.toJsonArray(model.getFilters(), RoleFilterJson::toJson));
    object.put("grants", JsonArrays.toJsonArray(model.getGrants(), DatabaseUserGrant::toJson));
    object.put("version", model.getVersion());
    object.put("grantBased", model.isGrantBased());
    return object;
  }

  public static Role fromJson(JsonValue object) {
    return new Role(ResourceId.valueOf(object.getString("id")),
        object.getString("label"),
        JsonArrays.listFromJson(object.get("permissions"), FilteredPermissionJson::fromJson),
        JsonArrays.listFromJson(object.get("parameters"), RoleParameterJson::fromJson),
        JsonArrays.listFromJson(object.get("filters"), RoleFilterJson::fromJson),
        JsonArrays.listFromJson(object.get("grants"), DatabaseUserGrant::fromJson),
        (int)object.getNumber("version"),
        object.getBoolean("grantBased"));
  }
}
