/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.form.FormField;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.Cardinality;
import org.activityinfo.model.type.FieldValue;
import org.activityinfo.model.type.ReferenceType;

@AutoJson
@AutoBuilder
public class RoleParameter {
    private ResourceId parameterId;
    private String label;
    private ResourceId range;

    @Builder
    public RoleParameter(ResourceId parameterId, String label, ResourceId range) {
        this.parameterId = parameterId;
        this.label = label;
        this.range = range;
    }

    /**
     * The id of this parameter.
     */
    public ResourceId getParameterId() {
        return parameterId;
    }

    /**
     * The human-readable label for this parameter
     */
    public String getLabel() {
        return label;
    }

    /**
     * The id of the form that this parameter references
     */
    public ResourceId getRange() {
        return range;
    }

    public FieldValue parseJsonValue(JsonValue value) {
        return toFieldType().parseJsonValue(value);
    }

    public ReferenceType toFieldType() {
        return new ReferenceType(Cardinality.SINGLE, range);
    }

    public FormField toReferenceField() {
        FormField field = new FormField(parameterId);
        field.setLabel(label);
        field.setType(toFieldType());
        return field;
    }
}
