/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.promise.HttpError;

import java.util.Collections;
import java.util.List;
import java.util.Optional;

@AutoJson
public class UpdateDatabaseError implements HttpError {

    private final UpdateDatabaseErrorCode code;
    private final List<String> messages;
    private final Optional<String> localizedMessage;

    public UpdateDatabaseError(UpdateDatabaseErrorCode code, List<String> messages) {
        this.code = code;
        this.messages = messages;
        this.localizedMessage = Optional.empty();
    }

    public UpdateDatabaseError(UpdateDatabaseErrorCode code) {
        this.code = code;
        this.messages = Collections.emptyList();
        this.localizedMessage = Optional.empty();
    }

    public UpdateDatabaseError(UpdateDatabaseErrorCode code, Optional<String> localizedMessage) {
        this.code = code;
        this.localizedMessage = localizedMessage;
        this.messages = Collections.emptyList();
    }

    @Builder
    public UpdateDatabaseError(UpdateDatabaseErrorCode code, List<String> messages, Optional<String> localizedMessage) {
        this.code = code;
        this.messages = messages;
        this.localizedMessage = localizedMessage;
    }

    public UpdateDatabaseErrorCode getCode() {
        return code;
    }

    @Override
    public int getHttpStatusCode() {
        return code.getHttpStatusCode();
    }

    @Override
    public String name() {
        return code.name();
    }

    public List<String> getMessages() {
        return messages;
    }

    public Optional<String> getLocalizedMessage() {
        return localizedMessage;
    }

    @Override
    public String toString() {
        return "UpdateDatabaseError{" +
            "code=" + code +
            ", messages=" + messages +
            ", localizedMessage=" + localizedMessage +
            '}';
    }
}
