/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.annotation.Undocumented;
import org.activityinfo.model.resource.ResourceId;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@AutoJson
@AutoBuilder
public class UpdateDatabaseRequest {

    /**
     * List of resources to add or update.
     */
    private final List<Resource> resourceUpdates;

    /**
     * List of resource ids to delete from the database.
     */
    private final List<ResourceId> resourceDeletions;

    /**
     * List of locks to add or update
     */
    private final List<RecordLock> lockUpdates;

    /**
     * List of lock ids to delete
     */
    private final List<ResourceId> lockDeletions;

    /**
     * List of roles to add or update.
     */
    private final List<Role> roleUpdates;

    /**
     * List of role ids to delete.
     */
    private final List<ResourceId> roleDeletions;

    @Undocumented
    private final Optional<Boolean> publishedTemplate;

    @Builder
    public UpdateDatabaseRequest(List<Resource> resourceUpdates,
                                 List<ResourceId> resourceDeletions,
                                 List<RecordLock> lockUpdates,
                                 List<ResourceId> lockDeletions,
                                 List<Role> roleUpdates,
                                 List<ResourceId> roleDeletions,
                                 Optional<Boolean> publishedTemplate) {
        this.resourceUpdates = resourceUpdates;
        this.resourceDeletions = resourceDeletions;
        this.lockUpdates = lockUpdates;
        this.lockDeletions = lockDeletions;
        this.roleUpdates = roleUpdates;
        this.roleDeletions = roleDeletions;
        this.publishedTemplate = publishedTemplate;
    }

    public List<Resource> getResourceUpdates() {
        return resourceUpdates;
    }

    public List<ResourceId> getResourceUpdateIds() {
        return resourceUpdates.stream().map(Resource::getId).collect(Collectors.toList());
    }

    public boolean hasResourceUpdates() {
        return !resourceUpdates.isEmpty();
    }

    public List<ResourceId> getResourceDeletions() {
        return resourceDeletions;
    }

    public boolean hasResourceDeletions() {
        return !resourceDeletions.isEmpty();
    }

    public List<RecordLock> getLockUpdates() {
        return lockUpdates;
    }

    public List<ResourceId> getLockUpdateResourceIds() {
        return lockUpdates.stream().map(RecordLock::getResourceId).collect(Collectors.toList());
    }

    public boolean hasLockUpdates() {
        return !lockUpdates.isEmpty();
    }

    public List<ResourceId> getLockDeletions() {
        return lockDeletions;
    }

    public boolean hasLockDeletions() {
        return !lockDeletions.isEmpty();
    }

    public List<Role> getRoleUpdates() {
        return roleUpdates;
    }

    public List<ResourceId> getRoleDeletions() {
        return roleDeletions;
    }

    public Optional<Boolean> getPublishedTemplate() {
        return publishedTemplate;
    }
}
