package org.activityinfo.model.database;

import java.lang.Boolean;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Optional;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;

public final class UpdateDatabaseRequestBuilder {
  List<Resource> resourceUpdates = new ArrayList();

  List<ResourceId> resourceDeletions = new ArrayList();

  List<RecordLock> lockUpdates = new ArrayList();

  List<ResourceId> lockDeletions = new ArrayList();

  List<Role> roleUpdates = new ArrayList();

  List<ResourceId> roleDeletions = new ArrayList();

  Optional<Boolean> publishedTemplate = Optional.empty();

  public UpdateDatabaseRequestBuilder() {
    this.resourceUpdates = new ArrayList();
    this.resourceDeletions = new ArrayList();
    this.lockUpdates = new ArrayList();
    this.lockDeletions = new ArrayList();
    this.roleUpdates = new ArrayList();
    this.roleDeletions = new ArrayList();
  }

  public UpdateDatabaseRequestBuilder(UpdateDatabaseRequest toCopy) {
    this.resourceUpdates = new ArrayList(toCopy.getResourceUpdates());
    this.resourceDeletions = new ArrayList(toCopy.getResourceDeletions());
    this.lockUpdates = new ArrayList(toCopy.getLockUpdates());
    this.lockDeletions = new ArrayList(toCopy.getLockDeletions());
    this.roleUpdates = new ArrayList(toCopy.getRoleUpdates());
    this.roleDeletions = new ArrayList(toCopy.getRoleDeletions());
    this.publishedTemplate = toCopy.getPublishedTemplate();
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder addResourceUpdates(@Nonnull Collection<Resource> collection) {
    this.resourceUpdates.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder addResourceUpdate(@Nonnull Resource element) {
    this.resourceUpdates.add(element);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder removeResourceUpdate(@Nonnull Resource element) {
    this.resourceUpdates.remove(element);
    return this;
  }

  public UpdateDatabaseRequestBuilder addResourceUpdates(@Nonnull Resource... array) {
    this.resourceUpdates.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder resourceUpdates(@Nonnull Resource... array) {
    return resourceUpdates(Arrays.asList(array));
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder resourceUpdates(@Nonnull Collection<Resource> collection) {
    this.resourceUpdates.clear();
    this.resourceUpdates.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder addResourceDeletions(
      @Nonnull Collection<ResourceId> collection) {
    this.resourceDeletions.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder addResourceDeletion(@Nonnull ResourceId element) {
    this.resourceDeletions.add(element);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder removeResourceDeletion(@Nonnull ResourceId element) {
    this.resourceDeletions.remove(element);
    return this;
  }

  public UpdateDatabaseRequestBuilder addResourceDeletions(@Nonnull ResourceId... array) {
    this.resourceDeletions.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder resourceDeletions(@Nonnull ResourceId... array) {
    return resourceDeletions(Arrays.asList(array));
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder resourceDeletions(
      @Nonnull Collection<ResourceId> collection) {
    this.resourceDeletions.clear();
    this.resourceDeletions.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder addLockUpdates(@Nonnull Collection<RecordLock> collection) {
    this.lockUpdates.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder addLockUpdate(@Nonnull RecordLock element) {
    this.lockUpdates.add(element);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder removeLockUpdate(@Nonnull RecordLock element) {
    this.lockUpdates.remove(element);
    return this;
  }

  public UpdateDatabaseRequestBuilder addLockUpdates(@Nonnull RecordLock... array) {
    this.lockUpdates.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder lockUpdates(@Nonnull RecordLock... array) {
    return lockUpdates(Arrays.asList(array));
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder lockUpdates(@Nonnull Collection<RecordLock> collection) {
    this.lockUpdates.clear();
    this.lockUpdates.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder addLockDeletions(@Nonnull Collection<ResourceId> collection) {
    this.lockDeletions.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder addLockDeletion(@Nonnull ResourceId element) {
    this.lockDeletions.add(element);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder removeLockDeletion(@Nonnull ResourceId element) {
    this.lockDeletions.remove(element);
    return this;
  }

  public UpdateDatabaseRequestBuilder addLockDeletions(@Nonnull ResourceId... array) {
    this.lockDeletions.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder lockDeletions(@Nonnull ResourceId... array) {
    return lockDeletions(Arrays.asList(array));
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder lockDeletions(@Nonnull Collection<ResourceId> collection) {
    this.lockDeletions.clear();
    this.lockDeletions.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder addRoleUpdates(@Nonnull Collection<Role> collection) {
    this.roleUpdates.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder addRoleUpdate(@Nonnull Role element) {
    this.roleUpdates.add(element);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder removeRoleUpdate(@Nonnull Role element) {
    this.roleUpdates.remove(element);
    return this;
  }

  public UpdateDatabaseRequestBuilder addRoleUpdates(@Nonnull Role... array) {
    this.roleUpdates.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder roleUpdates(@Nonnull Role... array) {
    return roleUpdates(Arrays.asList(array));
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder roleUpdates(@Nonnull Collection<Role> collection) {
    this.roleUpdates.clear();
    this.roleUpdates.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder addRoleDeletions(@Nonnull Collection<ResourceId> collection) {
    this.roleDeletions.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder addRoleDeletion(@Nonnull ResourceId element) {
    this.roleDeletions.add(element);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder removeRoleDeletion(@Nonnull ResourceId element) {
    this.roleDeletions.remove(element);
    return this;
  }

  public UpdateDatabaseRequestBuilder addRoleDeletions(@Nonnull ResourceId... array) {
    this.roleDeletions.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder roleDeletions(@Nonnull ResourceId... array) {
    return roleDeletions(Arrays.asList(array));
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder roleDeletions(@Nonnull Collection<ResourceId> collection) {
    this.roleDeletions.clear();
    this.roleDeletions.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder publishedTemplate(@Nonnull Boolean value) {
    this.publishedTemplate = Optional.of(value);
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequestBuilder publishedTemplate(@Nonnull Optional<Boolean> value) {
    this.publishedTemplate = value;
    return this;
  }

  @Nonnull
  public UpdateDatabaseRequest build() {
    return new UpdateDatabaseRequest(resourceUpdates,resourceDeletions,lockUpdates,lockDeletions,roleUpdates,roleDeletions,publishedTemplate);
  }
}
