/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.form.FormClass;
import org.activityinfo.model.form.FormMetadata;
import org.activityinfo.model.permission.PermissionOracle;
import org.activityinfo.model.resource.ResourceId;

import java.util.List;
import java.util.stream.Collectors;

@AutoJson
public class UpdateFormResponse {
    private final VisibleDatabaseTree database;
    private final List<FormMetadata> forms;

    @Builder
    public UpdateFormResponse(VisibleDatabaseTree database, List<FormMetadata> forms) {
        this.database = database;
        this.forms = forms;
    }

    public static UpdateFormResponse from(VisibleDatabaseTree updatedTree, List<FormClass> updatedForms) {
        return new UpdateFormResponse(updatedTree, updatedForms.stream()
            .map(schema -> FormMetadata.of(schema, PermissionOracle.formPermissions(schema.getId(), updatedTree)))
            .collect(Collectors.toList()));
    }

    public ResourceId getDatabaseId() {
        return database.getDatabaseId();
    }

    public VisibleDatabaseTree getDatabase() {
        return database;
    }

    public List<FormMetadata> getForms() {
        return forms;
    }
}
