/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.database;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.resource.ResourceId;

import java.util.List;
import java.util.stream.Collectors;

@AutoBuilder
@AutoJson
public class UpdateGrantSetRequest {

    private final List<RoleAssignment> roleUpdates;
    private final List<DatabaseUserGrant> grantUpdates;
    private final List<ResourceId> grantDeletions;

    @Builder
    public UpdateGrantSetRequest(List<RoleAssignment> roleUpdates, List<DatabaseUserGrant> grantUpdates, List<ResourceId> grantDeletions) {
        this.roleUpdates = roleUpdates;
        this.grantUpdates = grantUpdates;
        this.grantDeletions = grantDeletions;
    }

    public List<DatabaseUserGrant> getGrantUpdates() {
        return grantUpdates;
    }

    public List<ResourceId> getGrantUpdateIds() {
        return grantUpdates.stream().map(DatabaseUserGrant::getResourceId).collect(Collectors.toList());
    }

    public boolean hasGrantUpdates() {
        return !grantUpdates.isEmpty();
    }

    public List<ResourceId> getGrantDeletions() {
        return grantDeletions;
    }

    public boolean hasGrantDeletions() {
        return !grantDeletions.isEmpty();
    }

    public List<RoleAssignment> getRoleUpdates() {
        return roleUpdates;
    }
}
