package org.activityinfo.model.database;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import javax.annotation.Nonnull;
import org.activityinfo.model.resource.ResourceId;

public final class UpdateGrantSetRequestBuilder {
  List<RoleAssignment> roleUpdates = new ArrayList();

  List<DatabaseUserGrant> grantUpdates = new ArrayList();

  List<ResourceId> grantDeletions = new ArrayList();

  public UpdateGrantSetRequestBuilder() {
    this.roleUpdates = new ArrayList();
    this.grantUpdates = new ArrayList();
    this.grantDeletions = new ArrayList();
  }

  public UpdateGrantSetRequestBuilder(UpdateGrantSetRequest toCopy) {
    this.roleUpdates = new ArrayList(toCopy.getRoleUpdates());
    this.grantUpdates = new ArrayList(toCopy.getGrantUpdates());
    this.grantDeletions = new ArrayList(toCopy.getGrantDeletions());
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder addRoleUpdates(
      @Nonnull Collection<RoleAssignment> collection) {
    this.roleUpdates.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder addRoleUpdate(@Nonnull RoleAssignment element) {
    this.roleUpdates.add(element);
    return this;
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder removeRoleUpdate(@Nonnull RoleAssignment element) {
    this.roleUpdates.remove(element);
    return this;
  }

  public UpdateGrantSetRequestBuilder addRoleUpdates(@Nonnull RoleAssignment... array) {
    this.roleUpdates.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder roleUpdates(@Nonnull RoleAssignment... array) {
    return roleUpdates(Arrays.asList(array));
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder roleUpdates(@Nonnull Collection<RoleAssignment> collection) {
    this.roleUpdates.clear();
    this.roleUpdates.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder addGrantUpdates(
      @Nonnull Collection<DatabaseUserGrant> collection) {
    this.grantUpdates.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder addGrantUpdate(@Nonnull DatabaseUserGrant element) {
    this.grantUpdates.add(element);
    return this;
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder removeGrantUpdate(@Nonnull DatabaseUserGrant element) {
    this.grantUpdates.remove(element);
    return this;
  }

  public UpdateGrantSetRequestBuilder addGrantUpdates(@Nonnull DatabaseUserGrant... array) {
    this.grantUpdates.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder grantUpdates(@Nonnull DatabaseUserGrant... array) {
    return grantUpdates(Arrays.asList(array));
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder grantUpdates(
      @Nonnull Collection<DatabaseUserGrant> collection) {
    this.grantUpdates.clear();
    this.grantUpdates.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder addGrantDeletions(
      @Nonnull Collection<ResourceId> collection) {
    this.grantDeletions.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder addGrantDeletion(@Nonnull ResourceId element) {
    this.grantDeletions.add(element);
    return this;
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder removeGrantDeletion(@Nonnull ResourceId element) {
    this.grantDeletions.remove(element);
    return this;
  }

  public UpdateGrantSetRequestBuilder addGrantDeletions(@Nonnull ResourceId... array) {
    this.grantDeletions.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder grantDeletions(@Nonnull ResourceId... array) {
    return grantDeletions(Arrays.asList(array));
  }

  @Nonnull
  public UpdateGrantSetRequestBuilder grantDeletions(@Nonnull Collection<ResourceId> collection) {
    this.grantDeletions.clear();
    this.grantDeletions.addAll(collection);
    return this;
  }

  @Nonnull
  public UpdateGrantSetRequest build() {
    return new UpdateGrantSetRequest(roleUpdates,grantUpdates,grantDeletions);
  }
}
