package org.activityinfo.model.database;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonArrays;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.resource.ResourceId;

public final class VisibleDatabaseTreeJson {
  public static JsonValue toJson(VisibleDatabaseTree model) {
    JsonValue object = Json.createObject();
    object.put("databaseId", model.getDatabaseId().asString());
    object.put("userId", JsonArrays.optionalToJson(model.getUserId(), ResourceId::toJson));
    object.put("version", model.getVersion());
    object.put("label", model.getLabel());
    object.put("description", model.getDescription());
    object.put("ownerRef", UserRefJson.toJson(model.getOwnerRef()));
    object.put("billingAccountId", model.getBillingAccountId());
    object.put("role", model.getRole().toJson());
    object.put("suspended", model.isSuspended());
    object.put("storage", model.getStorage());
    object.put("publishedTemplate", model.isPublishedTemplate());
    object.put("resources", JsonArrays.toJsonArray(model.getResources(), Resource::toJson));
    object.put("grants", JsonArrays.toJsonArray(model.getGrants(), DatabaseUserGrant::toJson));
    object.put("locks", JsonArrays.toJsonArray(model.getLocks(), RecordLockJson::toJson));
    object.put("roles", JsonArrays.toJsonArray(model.getRoles(), RoleJson::toJson));
    object.put("securityCategories", JsonArrays.toJsonArray(model.getSecurityCategories(), SecurityCategoryJson::toJson));
    return object;
  }

  public static VisibleDatabaseTree fromJson(JsonValue object) {
    return new VisibleDatabaseTree(ResourceId.valueOf(object.getString("databaseId")),
        JsonArrays.optionalFromJson(object.get("userId"), ResourceId::fromJson),
        object.getString("version"),
        object.getString("label"),
        object.getString("description"),
        UserRefJson.fromJson(object.get("ownerRef")),
        object.getBoolean("suspended"),
        object.getString("storage"),
        RoleAssignment.fromJson(object.get("role")),
        JsonArrays.listFromJson(object.get("roles"), RoleJson::fromJson),
        JsonArrays.listFromJson(object.get("securityCategories"), SecurityCategoryJson::fromJson),
        JsonArrays.listFromJson(object.get("resources"), Resource::fromJson),
        JsonArrays.listFromJson(object.get("locks"), RecordLockJson::fromJson),
        JsonArrays.listFromJson(object.get("grants"), DatabaseUserGrant::fromJson),
        (long)object.getNumber("billingAccountId"),
        object.getBoolean("publishedTemplate"));
  }
}
