package org.activityinfo.model.form;

import java.lang.String;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Optional;
import javax.annotation.Nonnull;
import org.activityinfo.model.database.UserRef;

public final class RecordHistoryEntryBuilder {
  private String formId = "";

  private String recordId = "";

  private int time;

  private String subFieldId = "";

  private String subFieldLabel = "";

  private String subRecordKey = "";

  private String changeType = "";

  Optional<UserRef> user = Optional.empty();

  List<FieldValueChange> values = new ArrayList();

  public RecordHistoryEntryBuilder() {
    formId = "";
    recordId = "";
    subFieldId = "";
    subFieldLabel = "";
    subRecordKey = "";
    changeType = "";
    this.values = new ArrayList();
  }

  public RecordHistoryEntryBuilder(RecordHistoryEntry toCopy) {
    this.formId = toCopy.getFormId();
    this.recordId = toCopy.getRecordId();
    this.time = toCopy.getTime();
    this.subFieldId = toCopy.getSubFieldId();
    this.subFieldLabel = toCopy.getSubFieldLabel();
    this.subRecordKey = toCopy.getSubRecordKey();
    this.changeType = toCopy.getChangeType();
    this.user = toCopy.getUser();
    this.values = new ArrayList(toCopy.getValues());
  }

  @Nonnull
  public RecordHistoryEntryBuilder formId(@Nonnull String formId) {
    this.formId = formId;
    return this;
  }

  @Nonnull
  public RecordHistoryEntryBuilder recordId(@Nonnull String recordId) {
    this.recordId = recordId;
    return this;
  }

  @Nonnull
  public RecordHistoryEntryBuilder time(int time) {
    this.time = time;
    return this;
  }

  @Nonnull
  public RecordHistoryEntryBuilder subFieldId(@Nonnull String subFieldId) {
    this.subFieldId = subFieldId;
    return this;
  }

  @Nonnull
  public RecordHistoryEntryBuilder subFieldLabel(@Nonnull String subFieldLabel) {
    this.subFieldLabel = subFieldLabel;
    return this;
  }

  @Nonnull
  public RecordHistoryEntryBuilder subRecordKey(@Nonnull String subRecordKey) {
    this.subRecordKey = subRecordKey;
    return this;
  }

  @Nonnull
  public RecordHistoryEntryBuilder changeType(@Nonnull String changeType) {
    this.changeType = changeType;
    return this;
  }

  @Nonnull
  public RecordHistoryEntryBuilder user(@Nonnull UserRef value) {
    this.user = Optional.of(value);
    return this;
  }

  @Nonnull
  public RecordHistoryEntryBuilder user(@Nonnull Optional<UserRef> value) {
    this.user = value;
    return this;
  }

  @Nonnull
  public RecordHistoryEntryBuilder addValues(@Nonnull Collection<FieldValueChange> collection) {
    this.values.addAll(collection);
    return this;
  }

  @Nonnull
  public RecordHistoryEntryBuilder addValue(@Nonnull FieldValueChange element) {
    this.values.add(element);
    return this;
  }

  @Nonnull
  public RecordHistoryEntryBuilder removeValue(@Nonnull FieldValueChange element) {
    this.values.remove(element);
    return this;
  }

  public RecordHistoryEntryBuilder addValues(@Nonnull FieldValueChange... array) {
    this.values.addAll(Arrays.asList(array));
    return this;
  }

  @Nonnull
  public RecordHistoryEntryBuilder values(@Nonnull FieldValueChange... array) {
    return values(Arrays.asList(array));
  }

  @Nonnull
  public RecordHistoryEntryBuilder values(@Nonnull Collection<FieldValueChange> collection) {
    this.values.clear();
    this.values.addAll(collection);
    return this;
  }

  @Nonnull
  public RecordHistoryEntry build() {
    assert this.formId != null : "formId is missing";
    assert this.recordId != null : "recordId is missing";
    assert this.subFieldId != null : "subFieldId is missing";
    assert this.subFieldLabel != null : "subFieldLabel is missing";
    assert this.subRecordKey != null : "subRecordKey is missing";
    assert this.changeType != null : "changeType is missing";
    return new RecordHistoryEntry(formId,recordId,time,subFieldId,subFieldLabel,subRecordKey,changeType,user,values);
  }
}
