/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.form;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.promise.HttpError;

import java.util.Collections;
import java.util.List;

@AutoJson
public class UpdateFormError {

    public enum Code implements HttpError {
        DATABASE_DELETED(BAD_REQUEST_400),
        DATABASE_NOT_FOUND(BAD_REQUEST_400),
        INVALID_SCHEMA(BAD_REQUEST_400),
        PERMISSION_DENIED(FORBIDDEN_403);

        private final int status;

        Code(int status) {
            this.status = status;
        }

        @Override
        public int getHttpStatusCode() {
            return status;
        }
    }

    private final Code code;
    private final List<FormValidationError> errors;

    @Builder
    public UpdateFormError(Code code, List<FormValidationError> errors) {
        this.code = code;
        this.errors = errors;
    }

    public UpdateFormError(List<FormValidationError> errors) {
        this.code = Code.INVALID_SCHEMA;
        this.errors = errors;
    }

    public UpdateFormError(Code code) {
        this.code = code;
        this.errors = Collections.emptyList();
    }

    public int getHttpStatusCode() {
        return code.status;
    }

    public List<FormValidationError> getErrors() {
        return errors;
    }

    public Code getCode() {
        return code;
    }


    @Override
    public String toString() {
        return "FormUpdateError{" +
            "code=" + code +
            ", errors=" + errors +
            '}';
    }
}
