/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.form;

import org.activityinfo.model.database.Role;
import org.activityinfo.model.database.RoleParameter;
import org.activityinfo.model.database.VisibleDatabaseTree;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.Cardinality;
import org.activityinfo.model.type.FieldType;
import org.activityinfo.model.type.ReferenceType;
import org.activityinfo.model.type.enumerated.EnumItem;
import org.activityinfo.model.type.enumerated.EnumType;
import org.activityinfo.model.type.primitive.TextType;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

public class UserFormBuilder {
    public static final ResourceId NAME_FIELD_ID = ResourceId.valueOf("name");
    public static final ResourceId ROLE_FIELD_ID = ResourceId.valueOf("role");

    public static FormClass userForm(ResourceId databaseId, List<Role> roles, long userFormSchemaVersion) {
        FormClass schema = new FormClass(VisibleDatabaseTree.getUserFormIdFromDatabaseId(databaseId));
        schema.setDatabaseId(databaseId);
        schema.setLabel("Users");
        schema.setSchemaVersion(userFormSchemaVersion);
        schema.addElement(new FormField(UserFormBuilder.NAME_FIELD_ID)
            .setType(TextType.SIMPLE)
            .setLabel("Name")
            .setKey(true)
            .setRequired(true));

        if(!roles.isEmpty()) {
            schema.addElement(new FormField(UserFormBuilder.ROLE_FIELD_ID)
                .setType(roleFieldType(roles))
                .setLabel("Role")
                .setRequired(true));

            Set<ResourceId> parameters = new HashSet<>();
            for (Role role : roles) {
                for (RoleParameter parameter : role.getParameters()) {
                    if(parameters.add(parameter.getParameterId())) {
                        schema.addElement(roleParameterField(parameter));
                    }
                }
            }
        }
        return schema;
    }

    private static FormField roleParameterField(RoleParameter parameter) {
        FormField field = new FormField(parameter.getParameterId());
        field.setLabel(parameter.getLabel());
        field.setType(new ReferenceType(Cardinality.SINGLE, parameter.getRange()));
        field.setRequired(true);
        return field;
    }

    private static FieldType roleFieldType(List<Role> roles) {
        return new EnumType(Cardinality.SINGLE,
            roles
                .stream()
                .map(r -> new EnumItem(r.getId(), r.getLabel()))
                .collect(Collectors.toList()));
    }

    public static FormClass userForm(VisibleDatabaseTree tree, long schemaVersion) {
        return userForm(tree.getDatabaseId(), tree.getRoles(), schemaVersion);
    }
}
