/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formTree;

import org.activityinfo.model.database.RecordLockSet;
import org.activityinfo.model.database.VisibleDatabaseTree;
import org.activityinfo.model.form.FormClass;
import org.activityinfo.model.form.FormMetadata;
import org.activityinfo.model.permission.FormPermissions;
import org.activityinfo.model.permission.PermissionOracle;
import org.activityinfo.model.resource.ResourceId;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class FormMetadataProviderStub implements FormMetadataProvider {

    public static final FormPermissions OWNER = FormPermissions.owner(RecordLockSet.EMPTY, Collections.emptySet());

    private Map<ResourceId, FormMetadata> map = new HashMap<>();

    public FormMetadataProviderStub(FormClass... forms) {
        for (FormClass form : forms) {

            map.put(form.getId(), FormMetadata.of(form, OWNER));
        }
    }

    private void add(VisibleDatabaseTree databaseTree, FormClass formClass) {
        if(!formClass.getDatabaseId().equals(databaseTree.getDatabaseId())) {
            throw new IllegalArgumentException("DatabaseIds of the tree and the form do not match.");
        }
        FormPermissions permissions = PermissionOracle.formPermissions(formClass.getId(), databaseTree);
        if(permissions.isVisible()) {
            map.put(formClass.getId(), FormMetadata.of(formClass, permissions));
        } else {
            map.put(formClass.getId(), FormMetadata.forbidden(formClass.getId()));
        }
    }

    @Override
    public FormMetadata getFormMetadata(ResourceId formId) {
        return map.getOrDefault(formId, FormMetadata.notFound(formId));
    }

    public static FormTree of(FormClass formClass) {
        return new FormTreeBuilder(new FormMetadataProviderStub(formClass)).queryTree(formClass.getId());
    }

    public static FormTree of(FormClass formClass, FormClass... otherForms) {
        FormMetadataProviderStub stub = new FormMetadataProviderStub(otherForms);
        stub.map.put(formClass.getId(), FormMetadata.of(formClass, OWNER));

        return new FormTreeBuilder(stub).queryTree(formClass.getId());
    }

    public static FormTree of(VisibleDatabaseTree tree, FormClass formClass, FormClass... otherForms) {
        FormMetadataProviderStub stub = new FormMetadataProviderStub();
        stub.add(tree, formClass);
        for (FormClass otherForm : otherForms) {
            stub.add(tree, otherForm);
        }

        return new FormTreeBuilder(stub).queryTree(formClass.getId());
    }
}
