/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formTree;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.form.FormMetadata;
import org.activityinfo.model.resource.ResourceId;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Serializes/deserializes a FormTree to JSON
 */
public class JsonFormTreeBuilder {


    public static JsonValue toJson(FormTree formTree) {

        JsonValue forms = Json.createObject();
        for (FormMetadata formMetadata : formTree.getForms()) {
            forms.put(formMetadata.getId().asString(), formMetadata.toJson());
        }

        JsonValue object = Json.createObject();
        object.put("root", formTree.getRootFormId().asString());
        object.put("forms", forms);

        return object;
    }

    public static List<FormMetadata> fromJsonAsList(JsonValue object) {
        JsonValue formObject = object.get("forms");
        List<FormMetadata> forms = new ArrayList<>();
        for (Map.Entry<String, JsonValue> form : formObject.entrySet()) {
            forms.add(FormMetadata.fromJson(form.getValue()));
        }
        return forms;
    }

    public static FormTree fromJson(JsonValue object) {
        Map<ResourceId, FormMetadata> map = new HashMap<>();
        for (FormMetadata formMetadata : fromJsonAsList(object)) {
            map.put(formMetadata.getId(), formMetadata);
        }
        FormMetadataProvider provider = new FormMetadataProvider() {
            @Override
            public FormMetadata getFormMetadata(ResourceId formId) {
                FormMetadata form = map.get(formId);
                if(form == null) {
                    return FormMetadata.notFound(formId);
                } else {
                    return form;
                }
            }
        };
        FormTreeBuilder builder = new FormTreeBuilder(provider);
        ResourceId rootFormId = ResourceId.valueOf(object.getString("root"));
        return builder.queryTree(rootFormId);
    }

}
