/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formTree;

import org.activityinfo.model.form.TypedFormRecord;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.RecordRef;
import org.activityinfo.promise.Maybe;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * A FormRecord, it's FormTree, and all related records
 */
public class RecordTree {


    public static class ParentKey {
        private final RecordRef parentRef;
        private final ResourceId subFormId;

        public ParentKey(RecordRef parentRef, ResourceId subFormId) {
            this.parentRef = parentRef;
            this.subFormId = subFormId;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            ParentKey parentKey = (ParentKey) o;

            if (!parentRef.equals(parentKey.parentRef)) return false;
            return subFormId.equals(parentKey.subFormId);

        }

        @Override
        public int hashCode() {
            int result = parentRef.hashCode();
            result = 31 * result + subFormId.hashCode();
            return result;
        }
    }

    private final FormTree formTree;
    private final TypedFormRecord root;

    private final Map<RecordRef, Maybe<TypedFormRecord>> relatedRecords;

    public RecordTree(FormTree formTree,
                      RecordRef rootRecordRef,
                      Map<RecordRef, Maybe<TypedFormRecord>> records) {
        this.formTree = formTree;
        this.relatedRecords = records;
        this.root = records.get(rootRecordRef).get();
    }

    public TypedFormRecord getRoot() {
        return root;
    }

    public List<TypedFormRecord> getRelatedRecords() {
        List<TypedFormRecord> records = new ArrayList<>();
        for (Maybe<TypedFormRecord> record : relatedRecords.values()) {
            if(record.isVisible()) {
                records.add(record.get());
            }
        }
        return records;
    }

    public RecordRef getRootRef() {
        return root.getRef();
    }

    public Maybe<TypedFormRecord> getRecord(RecordRef ref) {
        Maybe<TypedFormRecord> record = this.relatedRecords.get(ref);
        if(record == null) {
            return Maybe.notFound();
        }
        return record;
    }

    public boolean contains(RecordRef ref) {
        return this.relatedRecords.containsKey(ref);
    }

    public FormTree getFormTree() {
        return formTree;
    }

    public RecordTree subTree(RecordRef ref) {
        return new RecordTree(formTree.tree(ref.getFormId()), ref, this.relatedRecords);
    }

}
