/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula;

import org.activityinfo.model.resource.ResourceId;

import javax.annotation.Nonnull;
import java.util.function.Function;

/**
 * symbol.symbol
 */
public class CompoundExpr extends FormulaNode {
    @Nonnull
    private final FormulaNode value;

    @Nonnull
    private final SymbolNode field;

    public CompoundExpr(@Nonnull ResourceId value, @Nonnull String field) {
        this.value = new SymbolNode(value);
        this.field = new SymbolNode(field);
    }
    
    public CompoundExpr(@Nonnull FormulaNode value, @Nonnull SymbolNode field) {
        this.value = value;
        this.field = field;
    }

    public CompoundExpr(@Nonnull FormulaNode value, @Nonnull ResourceId field) {
        this.value = value;
        this.field = new SymbolNode(field.asString());
    }

    public CompoundExpr(@Nonnull FormulaNode value, @Nonnull String field) {
        this.value = value;
        this.field = new SymbolNode(field);
    }

    public CompoundExpr(@Nonnull FormulaNode value, @Nonnull SymbolNode field, SourceRange range) {
        this.value = value;
        this.field = field;
        this.sourceRange = range;
    }


    @Nonnull
    public FormulaNode getValue() {
        return value;
    }

    @Nonnull
    public SymbolNode getField() {
        return field;
    }

    @Override
    public String deparse() {
        return value.deparse() + "." + field.deparse();
    }

    @Override
    public <T> T accept(FormulaVisitor<T> visitor) {
        return visitor.visitCompoundExpr(this);
    }

    @Override
    public FormulaNode transform(Function<FormulaNode, FormulaNode> function) {
        return function.apply(new CompoundExpr(
                value.transform(function),
                field));
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        CompoundExpr that = (CompoundExpr) o;

        if (!field.equals(that.field)) return false;
        if (!value.equals(that.value)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = value.hashCode();
        result = 31 * result + field.hashCode();
        return result;
    }

    @Override
    public String toString() {
        return value + "." + field;
    }
}
