/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula;

/**
 * Wraps a {@link FormulaLexer}, allowing peeking from the next token, and filtering out
 * whitespace tokens.
 */
class FilteredFormulaLexer {
    private FormulaLexer lexer;
    private Token next;
    private boolean hasNext;
    private Token last;

    public FilteredFormulaLexer(FormulaLexer lexer) {
        this.lexer = lexer;
        computeNext();
    }

    /**
     * Returns the next non-whitespace token in the stream, without advancing the stream.
     */
    public Token peek() {
        return next;
    }


    /**
     * Returns true if there is another non-whitespace token in the stream.
     */
    public boolean hasNext() {
        return hasNext;
    }

    /**
     * Returns the next non-whitespace token in the stream, and advances to the next non-whitespace token in the stream.
     */
    public Token next() {
        Token result = next;
        computeNext();
        return result;
    }


    /**
     * Returns the last-read token from the stream, which may be a whitespace token.
     */
    public Token last() {
        return last;
    }

    private void computeNext() {
        hasNext = false;
        while(true) {
            if(lexer.isEndOfInput()) {
                return;
            }
            last = lexer.next();
            if(last.getType() != TokenType.WHITESPACE) {
                hasNext = true;
                next = last;
                return;
            }
        }
    }
}
