/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula;

import java.util.function.Function;

/**
 * An expression group ()
 */
public class GroupNode extends FormulaNode {

    private FormulaNode expr;

    public GroupNode(FormulaNode expr) {
        super();
        this.expr = expr;
    }

    public GroupNode(FormulaNode expr, SourceRange sourceRange) {
        super();
        this.expr = expr;
        this.sourceRange = sourceRange;
    }

    @Override
    public String toString() {
        return deparse();
    }

    public String deparse() {
        return "(" + expr.deparse() + ")";
    }

    @Override
    public <T> T accept(FormulaVisitor<T> visitor) {
        return visitor.visitGroup(this);
    }

    @Override
    public FormulaNode transform(Function<FormulaNode, FormulaNode> function) {
        return function.apply(new GroupNode(expr.transform(function)));
    }

    public FormulaNode getExpr() {
        return expr;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((expr == null) ? 0 : expr.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        GroupNode other = (GroupNode) obj;
        return other.expr.equals(expr);
    }
}
