/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula;

import org.activityinfo.model.formula.resolved.*;

public class SupportCheckVisitor implements ResolvedNodeVisitor<Boolean>, ReferenceVisitor<Boolean> {

    public static final SupportCheckVisitor INSTANCE = new SupportCheckVisitor();

    private SupportCheckVisitor() {}

    @Override
    public Boolean visitRecordId(RecordIdNode recordIdNode) {
        return true;
    }

    @Override
    public Boolean visitFormId(FormIdNode formIdNode) {
        return true;
    }

    @Override
    public Boolean visitLastEditTime(LastEditTimeNode node) {
        return true;
    }

    @Override
    public Boolean visitField(FieldReferenceNode field) {
        return true;
    }

    @Override
    public Boolean visitCalculatedField(CalculatedFieldNode field) {
        return field.getResolved().accept(this);
    }

    @Override
    public Boolean visitEnumItem(EnumItemReference enumItemReference) {
        return true;
    }

    @Override
    public Boolean visitJoin(JoinQueryNode node) {
        return node.getLeft().accept(this) && node.getRight().accept(this);
    }

    @Override
    public Boolean visitUnion(UnionNode node) {
        return true;
    }

    @Override
    public Boolean visitAggregation(AggregateNode node) {
        return false;
    }

    @Override
    public Boolean visitConstant(ConstantNode node) {
        return true;
    }

    @Override
    public Boolean visitInvalidSyntax(InvalidSyntaxNode node) {
        return true;
    }

    @Override
    public Boolean visitReferenceNode(ResolvedReference node) {
        return node.getRoot().accept(this);
    }

    @Override
    public Boolean visitInvalidReferenceNode(InvalidReferenceNode node) {
        return true;
    }

    @Override
    public Boolean visitCallNode(ResolvedCallNode node) {
        for (ResolvedNode child : node.getChildren()) {
            if(!child.accept(this)) {
                return false;
            }
        }
        return true;
    }

    @Override
    public Boolean visitInvalidCall(InvalidCallNode invalidCallNode) {
        return true;
    }

    @Override
    public Boolean visitInvalidArgumentNode(InvalidArgumentNode invalidArgumentNode) {
        return true;
    }
}



