/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula;

import org.activityinfo.model.resource.ResourceId;

import javax.annotation.Nonnull;

/**
 * @author yuriyz on 6/2/14.
 */
public class SymbolNode extends FormulaNode {

    @Nonnull
    private String name;

    public SymbolNode(@Nonnull String name) {
        this.name = name;
    }

    public SymbolNode(ResourceId id) {
        this(id.asString());
    }

    public SymbolNode(Token token) {
        assert token.getType() == TokenType.SYMBOL;
        this.name = token.getString();
        this.sourceRange = new SourceRange(token);
    }


    public String getName() {
        return name;
    }

    public void setName(@Nonnull String name) {
        this.name = name;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        SymbolNode that = (SymbolNode) o;
        return name.equals(that.name);
    }

    @Override
    public int hashCode() {
        return name.hashCode();
    }

    public ResourceId asResourceId() {
        return ResourceId.valueOf(name);
    }
    
    public String deparse() {
        return deparse(name);
    }

    public static String deparse(String name) {
        if(needsEscaping(name)) {
            return "[" + name.replace("]", "\\]") + "]";
        } else {
            return name;
        }
    }

    public static String deparse(ResourceId id) {
        return deparse(id.asString());
    }
    
    public static boolean needsEscaping(String name) {
        assert name.length() != 0;

        char firstChar = name.charAt(0);
        if(firstChar != '@' && !Character.isLetter(firstChar) && firstChar != '_') {
            return true;
        }
        for(int i=1;i<name.length();++i) {
            char c = name.charAt(i);
            if(!Character.isLetterOrDigit(c) && c != '_' && c != '@') {
                return true;
            }
        }
        return false;
    }

    @Override
    public <T> T accept(FormulaVisitor<T> visitor) {
        return visitor.visitSymbol(this);
    }

    @Override
    public String toString() {
        return deparse();
    }
}
