/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.diagnostic;

import org.activityinfo.model.formula.FormulaType;

import java.util.List;
import java.util.Set;

/**
 * Signals an invalid argument to a function.
 */
public class ArgumentException extends FormulaException {

    private int argumentIndex;

    public ArgumentException(int argumentIndex, String message) {
        super(message);
        this.argumentIndex = argumentIndex;
    }

    public ArgumentException(int argumentIndex, FormulaType expectedType, FormulaType actualType) {
        super("Found type " + actualType + ", expected " + expectedType);
        this.argumentIndex = argumentIndex;
    }

    public ArgumentException(int argumentIndex, Set<FormulaType> expectedTypes, FormulaType actualType) {
        super("Found type " + actualType + ", expected " + expectedTypes);
        this.argumentIndex = argumentIndex;
    }

    public static void check(List<FormulaType> argumentTypes, int index, FormulaType expectedType) {
        FormulaType type = argumentTypes.get(index);
        if(type != expectedType) {
            throw new ArgumentException(index, expectedType, type);
        }
    }

    public int getArgumentIndex() {
        return argumentIndex;
    }
}
