/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.diagnostic;

import org.activityinfo.model.formula.functions.FormulaFunction;

import java.util.List;

public class ArityException extends FormulaSyntaxException {

    private final FormulaFunction function;
    private final int minExpectedArguments;
    private final int maxExpectedArguments;
    private final int actualArguments;

    public ArityException(FormulaFunction function, int minExpectedArguments, int maxExpectedArguments, int actualArguments) {
        super("Expected between " + minExpectedArguments + " and " + maxExpectedArguments + " for " + function.getId() + ", found " + actualArguments);
        this.function = function;
        this.minExpectedArguments = minExpectedArguments;
        this.maxExpectedArguments = maxExpectedArguments;
        this.actualArguments = actualArguments;
    }

    public ArityException(FormulaFunction function, int expectedArguments, int actualArguments) {
        super("Expected " + expectedArguments + " for " + function.getId() + ", found " + actualArguments);
        this.function = function;
        this.minExpectedArguments = expectedArguments;
        this.maxExpectedArguments = expectedArguments;
        this.actualArguments = actualArguments;
    }

    public static void check(FormulaFunction function, int expected, List<?> arguments) {
        if(arguments.size() != expected) {
            throw new ArityException(function, expected, arguments.size());
        }
    }

    public static void check(FormulaFunction function, int min, int max, List<?> arguments) {
        int num = arguments.size();
        if(num < min || num > max) {
            throw new ArityException(function, min, max, num);
        }
    }
}
