/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.eval;

import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldValue;
import org.activityinfo.model.type.RecordRef;
import org.activityinfo.observable.Observable;

import java.util.Optional;
import java.util.function.Predicate;

/**
 * Combines two field value providers, using the given {@code provider} only for records which
 * pass the {@code predicate}. All other field values comes from the fallback provider.
 */
public class CompositeFieldValueProvider implements FieldValueProvider {
    private final FieldValueProvider fallback;
    private final Predicate<RecordRef> predicate;
    private final FieldValueProvider provider;

    public CompositeFieldValueProvider(FieldValueProvider fallback, Predicate<RecordRef> predicate, FieldValueProvider provider) {
        this.fallback = fallback;
        this.predicate = predicate;
        this.provider = provider;
    }
    public CompositeFieldValueProvider(FieldValueProvider fallback, RecordRef recordRef, FieldValueProvider provider) {
        this(fallback, recordRef::equals, provider);
    }

    @Override
    public Observable<Optional<RecordRef>> getParentRef(RecordRef ref) {
        if(predicate.test(ref)) {
            return provider.getParentRef(ref);
        } else {
            return fallback.getParentRef(ref);
        }
    }

    @Override
    public Observable<Optional<FieldValue>> getFieldValue(RecordRef recordRef, ResourceId fieldId) {
        if(predicate.test(recordRef)) {
            return provider.getFieldValue(recordRef, fieldId);
        } else {
            return fallback.getFieldValue(recordRef, fieldId);
        }
    }

    @Override
    public Observable<Double> getLastEditTime(RecordRef recordRef) {
        if(predicate.test(recordRef)) {
            return provider.getLastEditTime(recordRef);
        } else {
            return fallback.getLastEditTime(recordRef);
        }
    }
}
