/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.eval;

import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldValue;
import org.activityinfo.model.type.RecordRef;
import org.activityinfo.observable.Observable;

import java.util.Optional;

/**
 * Provides an interface to accessing properties of records, which need to be
 * retrieved in a context-specific way.
 *
 * <p>A record might need to be fetched from the server, from the current HRD transaction,
 * or from the user's current input, etc.</p>
 */
public interface FieldValueProvider {

    Observable<Optional<RecordRef>> getParentRef(RecordRef ref);

    Observable<Optional<FieldValue>> getFieldValue(RecordRef recordRef, ResourceId fieldId);

    /**
     * Returns an Observable of the record's last edit time, or {@code Double.NaN} if the record does not exist,
     * or if the last edit time is not available, or the record has not yet been saved to the server.
     */
    Observable<Double> getLastEditTime(RecordRef recordRef);

}
