/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.eval;

import org.activityinfo.model.form.FormClass;
import org.activityinfo.model.form.TypedFormRecord;
import org.activityinfo.model.formTree.FormTree;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldValue;
import org.activityinfo.model.type.RecordRef;
import org.activityinfo.observable.Observable;

import java.util.Arrays;
import java.util.Optional;

public class FieldValueProviders {

    public static FieldValueProvider of(FormTree formTree, TypedFormRecord... records) {
        return new FieldValueProvider() {

            private Optional<TypedFormRecord> get(RecordRef ref) {
                return Arrays.stream(records).filter(r -> r.getRef().equals(ref)).findFirst();
            }

            @Override
            public Observable<Optional<RecordRef>> getParentRef(RecordRef ref) {
                return Observable.just(
                    get(ref)
                        .flatMap(r -> r.getParentRecordId())
                        .map(parentId -> {
                            FormClass formClass = formTree.getFormClass(ref.getFormId());
                            return new RecordRef(formClass.getParentFormId().get(), parentId);
                        }));
            }

            @Override
            public Observable<Optional<FieldValue>> getFieldValue(RecordRef recordRef, ResourceId fieldId) {
                return Observable.just(get(recordRef).flatMap(record -> Optional.ofNullable(record.get(fieldId))));
            }


            @Override
            public Observable<Double> getLastEditTime(RecordRef recordRef) {
                return Observable.just(get(recordRef).map(record -> record.getLastEditTime()).orElse(Double.NaN));
            }
        };
    }

    public static FieldValueProvider empty() {
        return new FieldValueProvider() {
            @Override
            public Observable<Optional<RecordRef>> getParentRef(RecordRef ref) {
                return Observable.just(Optional.empty());
            }

            @Override
            public Observable<Optional<FieldValue>> getFieldValue(RecordRef recordRef, ResourceId fieldId) {
                return Observable.just(Optional.empty());
            }

            @Override
            public Observable<Double> getLastEditTime(RecordRef recordRef) {
                return Observable.just(Double.NaN);
            }
        };
    }
}
