/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.diagnostic.ArgumentException;
import org.activityinfo.model.formula.diagnostic.FormulaSyntaxException;
import org.activityinfo.model.query.ColumnView;

import java.util.List;

public abstract class AbstractUnaryFunction extends FormulaFunction implements ColumnFunction {

    private final String name;

    private static final int MAXARGS = 1;

    protected AbstractUnaryFunction(String name) {
        this.name = name;
    }

    @Override
    public final String getId() {
        return name;
    }

    @Override
    public final ColumnView apply(int numRows, List<ColumnView> arguments) {
        checkArity(arguments, MAXARGS);
        ColumnView unaryArgument = arguments.get(0);
        return columnApply(numRows,unaryArgument);
    }

    @Override
    public final FormulaType resolveResultType(List<FormulaType> argumentTypes) {
        if(argumentTypes.size() != 1) {
            throw new FormulaSyntaxException("Expected single argument");
        }
        FormulaType unaryArgType = argumentTypes.get(0);
        return resolveUnaryResultType(unaryArgType);
    }

    public abstract FormulaType resolveUnaryResultType(FormulaType argumentType);

    public abstract ColumnView columnApply(int numRows, ColumnView argument);

    protected final void checkArgumentType(FormulaType actual, FormulaType expectedType) {
        if(actual != expectedType) {
            throw new ArgumentException(0, expectedType, actual);
        }
    }

}
