/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.BooleanColumnView;
import org.activityinfo.model.query.ColumnView;

import java.util.List;

public abstract class BinaryBooleanOperator extends FormulaFunction implements ColumnFunction {

    private final String name;

    protected BinaryBooleanOperator(String name) {
        this.name = name;
    }

    @Override
    public final String getId() {
        return name;
    }

    @Override
    public final ColumnView apply(int numRows, List<ColumnView> arguments) {
        checkArity(arguments,2);

        ColumnView a = arguments.get(0);
        ColumnView b = arguments.get(1);

        int[] result = new int[a.numRows()];
        for (int i = 0; i < result.length; i++) {
            int ai = a.getBoolean(i);
            int bi = b.getBoolean(i);
            result[i] = apply(ai, bi);
        }
        return new BooleanColumnView(result);
    }

    @Override
    public final FormulaType resolveResultType(List<FormulaType> argumentTypes) {
        checkArity(argumentTypes, 2);
        checkArgumentType(argumentTypes, 0, FormulaType.BOOLEAN);
        checkArgumentType(argumentTypes, 1, FormulaType.BOOLEAN);

        return FormulaType.BOOLEAN;
    }

    public abstract boolean apply(boolean a, boolean b);

    /**
     * Apply the function to {@code a} and {@code b}, which must have the values 
     * {@link ColumnView#TRUE}, {@link ColumnView#FALSE} or {@link ColumnView#BLANK}
     */
    public abstract int apply(int a, int b);

}
