/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.diagnostic.ArgumentException;
import org.activityinfo.model.type.geo.Extents;

import java.util.List;

/**
 * Returns min/max lat/lng from a geographic
 */
public abstract class BoundingBoxFunction extends FormulaFunction {


    public static final BoundingBoxFunction XMIN = new BoundingBoxFunction("ST_XMIN") {

        @Override
        public double apply(Extents envelope) {
            return envelope.getX1();
        }
    };

    public static final BoundingBoxFunction XMAX = new BoundingBoxFunction("ST_XMAX") {

        @Override
        public double apply(Extents envelope) {
            return envelope.getX2();
        }
    };

    public static final BoundingBoxFunction YMIN = new BoundingBoxFunction("ST_YMIN") {

        @Override
        public double apply(Extents envelope) {
            return envelope.getY1();
        }
    };

    public static final BoundingBoxFunction YMAX = new BoundingBoxFunction("ST_YMAX") {

        @Override
        public double apply(Extents envelope) {
            return envelope.getY2();
        }
    };


    private String id;

    private BoundingBoxFunction(String id) {
        this.id = id;
    }

    @Override
    public String getId() {
        return id;
    }

    public abstract double apply(Extents envelope);

    @Override
    public FormulaType resolveResultType(List<FormulaType> argumentTypes) {
        checkArity(argumentTypes,1);
        ArgumentException.check(argumentTypes, 0, FormulaType.GEOMETRY);

        return FormulaType.NUMBER;
    }

    @Override
    public FunctionCategory getCategory() {
        return FunctionCategory.GEOGRAPHIC;
    }

    @Override
    public int getReferenceTopicId() {
        return 0;
    }
}
