/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import org.activityinfo.model.query.ColumnView;

import java.util.ArrayList;
import java.util.List;

/**
 * Function which can be applied to an entire column of values at once
 */
public interface ColumnFunction {

    /**
     * Apply this function to all rows in the provided arguments.
     *
     * <p>As a precondition, all columns in the {@code arguments} list must have the
     * same number of rows. This should be checked before calling this method, so
     * implementors should not check again.</p>
     */
    ColumnView apply(int numRows, List<ColumnView> arguments);

    default ColumnView apply(ColumnView firstArgument, ColumnView... moreArguments) {
        List<ColumnView> arguments = new ArrayList<>(1 + moreArguments.length);
        arguments.add(firstArgument);
        for (int i = 0; i < moreArguments.length; i++) {
            arguments.add(moreArguments[i]);
        }
        return apply(firstArgument.numRows(), arguments);
    }

}
