/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import com.google.common.base.Strings;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.diagnostic.ArgumentException;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.StringArrayColumnView;

import java.util.List;

public class ConcatenateFunction extends FormulaFunction implements ColumnFunction {

    public static final ConcatenateFunction INSTANCE = new ConcatenateFunction();

    private ConcatenateFunction() {
    }

    @Override
    public String getId() {
        return "CONCAT";
    }

    @Override
    public FunctionCategory getCategory() {
        return FunctionCategory.TEXT;
    }

    @Override
    public int getReferenceTopicId() {
        return 1002806;
    }

    @Override
    public ColumnView apply(int numRows, List<ColumnView> arguments) {
        String[] concatenation = new String[numRows];
        StringBuilder builder = new StringBuilder();

        for (int i=0; i<numRows; i++) {
            concatenation[i] = concatenateRow(builder, i, arguments);
        }

        return new StringArrayColumnView(concatenation);
    }

    private String concatenateRow(StringBuilder builder, int row, List<ColumnView> arguments) {
        builder.delete(0, builder.length());
        for (ColumnView column : arguments) {
            builder.append(Strings.nullToEmpty(column.getString(row)));
        }
        return builder.toString();
    }

    @Override
    public FormulaType resolveResultType(List<FormulaType> argumentTypes) {
        for (int i=0; i<argumentTypes.size(); i++) {
            if (argumentTypes.get(i) != FormulaType.STRING) {
                throw new ArgumentException(i, FormulaType.STRING, argumentTypes.get(i));
            }
        }
        return FormulaType.STRING;
    }
}