/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import java.util.Arrays;

public class CountDistinctFunction extends StatFunction {

    public static final CountDistinctFunction INSTANCE = new CountDistinctFunction();

    private CountDistinctFunction() {}

    @Override
    public String getId() {
        return "countdistinct";
    }

    @Override
    public int getReferenceTopicId() {
        return 889882;
    }

    @Override
    public double compute(double[] values, int start, int end) {
        Arrays.sort(values, start, end);
        int count = 0;
        double lastValue = Double.NaN;
        for (int i = start; i < end; i++) {
            double value = values[i];
            if(!Double.isNaN(value)) {
                if(value != lastValue) {
                    count++;
                }
                lastValue = value;
            }
        }
        return count;
    }

    @Override
    public double emptyValue() {
        return 0;
    }
}
