/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.BitSetColumnView;
import org.activityinfo.model.query.ColumnView;

import java.util.BitSet;
import java.util.List;

public class IsBlankFunction extends FormulaFunction implements ColumnFunction {

    public static final IsBlankFunction INSTANCE = new IsBlankFunction();

    private IsBlankFunction() {
    }

    @Override
    public String getId() {
        return "ISBLANK";
    }

    @Override
    public FunctionCategory getCategory() {
        return FunctionCategory.LOGICAL;
    }

    @Override
    public int getReferenceTopicId() {
        return 0;
    }

    @Override
    public ColumnView apply(int numRows, List<ColumnView> arguments) {
        checkArity(arguments, 1);

        ColumnView argument = arguments.get(0);
        BitSet blank = new BitSet();
        for (int i = 0; i < argument.numRows(); i++) {
            blank.set(i, argument.isBlank(i));
        }
        return new BitSetColumnView(argument.numRows(), blank);
    }

    @Override
    public FormulaType resolveResultType(List<FormulaType> argumentTypes) {
        checkArity(argumentTypes, 1);
        return FormulaType.BOOLEAN;
    }
}
