/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.BitSetColumnView;
import org.activityinfo.model.query.ColumnType;
import org.activityinfo.model.query.ColumnView;

import java.util.BitSet;

public class IsNumberFunction extends AbstractUnaryFunction {

    public static final IsNumberFunction INSTANCE = new IsNumberFunction();

    protected IsNumberFunction() {
        super("ISNUMBER");
    }

    @Override
    public FormulaType resolveUnaryResultType(FormulaType argumentType) {
        return FormulaType.BOOLEAN;
    }

    @Override
    public ColumnView columnApply(int numRows, ColumnView argument) {
        BitSet result = new BitSet();
        if(argument.getType() == ColumnType.NUMBER) {
            for (int i = 0; i < numRows; i++) {
                if (!Double.isNaN(argument.getDouble(i))) {
                    result.set(i);
                }
            }
        }
        return new BitSetColumnView(numRows, result);
    }

    @Override
    public FunctionCategory getCategory() {
        return FunctionCategory.LOGICAL;
    }

    @Override
    public int getReferenceTopicId() {
        return 880286;
    }
}
