/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import java.util.Arrays;

/**
 * Computes the median of its arguments.
 */
public class MedianFunction extends StatFunction {

    public static final MedianFunction INSTANCE = new MedianFunction();

    @Override
    public String getId() {
        return "median";
    }

    @Override
    public int getReferenceTopicId() {
        return 889869;
    }

    @Override
    public double compute(double[] values, int start, int end) {

        Arrays.sort(values, start, end);

        // Exclude missing (NaN) values, which are sorted to the end
        // of the array section.
        while(end > start && Double.isNaN(values[end - 1])) {
            end--;
        }

        int length = end - start;
        if(length == 0) {
            return Double.NaN;
        }

        int midpoint = start + (length / 2);
        if(length % 2 == 0) {
            return (values[midpoint] + values[midpoint-1]) / 2.0;
        } else {
            return values[midpoint];
        }
    }
}
