/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.BitSetColumnView;
import org.activityinfo.model.query.BitSetWithMissingView;
import org.activityinfo.model.query.ColumnView;

import java.util.BitSet;

public class NotFunction extends AbstractUnaryFunction implements ColumnFunction {

    public static final NotFunction INSTANCE = new NotFunction();

    private NotFunction() {
        super("!");
    }

    @Override
    public FunctionCategory getCategory() {
        return FunctionCategory.LOGICAL;
    }

    @Override
    public String getHelpLabel() {
        return "NOT";
    }

    @Override
    public int getReferenceTopicId() {
        return 878533;
    }

    @Override
    public FormulaType resolveUnaryResultType(FormulaType argumentType) {
        checkArgumentType(argumentType, FormulaType.BOOLEAN);
        return FormulaType.BOOLEAN;
    }

    @Override
    public ColumnView columnApply(int numRows, ColumnView argument) {

        BitSet bitSet = new BitSet();
        BitSet missing = new BitSet();

        for (int i = 0; i < argument.numRows(); i++) {
            int value = argument.getBoolean(i);
            if(value == ColumnView.BLANK) {
                missing.set(i);
            } else {
                bitSet.set( i, value != ColumnView.TRUE );
            }
        }

        if(missing.isEmpty()) {
            return new BitSetColumnView(numRows, bitSet);
        } else {
            return new BitSetWithMissingView(numRows, bitSet, missing);
        }
    }
}
