/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import com.google.common.base.Strings;
import com.google.gwt.regexp.shared.MatchResult;
import com.google.gwt.regexp.shared.RegExp;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.StringArrayColumnView;

import java.util.List;

public class RegexExtractFunction extends FormulaFunction implements ColumnFunction {

    public static final FormulaFunction INSTANCE = new RegexExtractFunction();

    private RegexExtractFunction() {
    }

    @Override
    public ColumnView apply(int numRows, List<ColumnView> arguments) {
        checkArity(arguments, 2);

        ColumnView textColumn = arguments.get(0);
        RegexpVector regexColumn = RegexpVector.of(arguments.get(1));

        String[] result = new String[numRows];
        for (int i = 0; i < numRows; i++) {
            String text = textColumn.getString(i);
            RegExp regex = regexColumn.get(i);
            if(text != null && regex != null) {
                MatchResult match = regex.exec(text);
                if(match != null) {
                    result[i] = Strings.emptyToNull(match.getGroup(0));
                }
            }
        }
        return new StringArrayColumnView(result);
    }

    @Override
    public String getId() {
        return "REGEXEXTRACT";
    }

    @Override
    public FunctionCategory getCategory() {
        return FunctionCategory.TEXT;
    }

    @Override
    public int getReferenceTopicId() {
        return 1284823;
    }

    @Override
    public FormulaType resolveResultType(List<FormulaType> argumentTypes) {
        checkArity(argumentTypes, 2);
        checkArgumentType(argumentTypes, 0, FormulaType.STRING);
        checkArgumentType(argumentTypes, 1, FormulaType.STRING);

        return FormulaType.STRING;
    }
}
