/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import com.google.common.base.Strings;
import com.google.gwt.regexp.shared.RegExp;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.BitSetColumnView;
import org.activityinfo.model.query.ColumnView;

import java.util.BitSet;
import java.util.List;

public class RegexMatchFunction extends FormulaFunction implements ColumnFunction {

    public static final RegexMatchFunction INSTANCE = new RegexMatchFunction();

    private RegexMatchFunction() {
    }

    @Override
    public ColumnView apply(int numRows, List<ColumnView> arguments) {
        checkArity(arguments, 2);

        ColumnView textColumn = arguments.get(0);
        RegexpVector regexColumn = RegexpVector.of(arguments.get(1));

        BitSet bitSet = new BitSet();
        for (int i = 0; i < numRows; i++) {
            String text = Strings.nullToEmpty(textColumn.getString(i));
            RegExp regex = regexColumn.get(i);
            if(regex != null) {
                bitSet.set(i, regex.test(text));
            }
        }

        return new BitSetColumnView(numRows, bitSet);
    }

    @Override
    public String getId() {
        return "REGEXMATCH";
    }

    @Override
    public FunctionCategory getCategory() {
        return FunctionCategory.TEXT;
    }

    @Override
    public int getReferenceTopicId() {
        return 1284819;
    }

    @Override
    public FormulaType resolveResultType(List<FormulaType> argumentTypes) {
        checkArity(argumentTypes, 2);
        checkArgumentType(argumentTypes, 0, FormulaType.STRING);
        checkArgumentType(argumentTypes, 1, FormulaType.STRING);

        return FormulaType.BOOLEAN;
    }
}
