/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import com.google.common.base.Strings;
import com.google.gwt.regexp.shared.RegExp;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.StringArrayColumnView;

import java.util.List;

public class RegexReplaceFunction extends FormulaFunction implements ColumnFunction {

    public static final FormulaFunction INSTANCE = new RegexReplaceFunction();

    private RegexReplaceFunction() {
    }

    @Override
    public ColumnView apply(int numRows, List<ColumnView> arguments) {
        checkArity(arguments, 3);

        ColumnView textColumn = arguments.get(0);
        RegexpVector regexColumn = RegexpVector.of(arguments.get(1));
        ColumnView replacementColumn = arguments.get(2);

        String[] result = new String[numRows];
        for (int i = 0; i < numRows; i++) {
            String text = Strings.nullToEmpty(textColumn.getString(i));
            RegExp regex = regexColumn.get(i);
            String replacement = Strings.nullToEmpty(replacementColumn.getString(i));
            if(regex != null) {
                result[i] = Strings.emptyToNull(regex.replace(text, replacement));
            }
        }
        return new StringArrayColumnView(result);
    }

    @Override
    public String getId() {
        return "REGEXREPLACE";
    }

    @Override
    public FunctionCategory getCategory() {
        return FunctionCategory.TEXT;
    }

    @Override
    public int getReferenceTopicId() {
        return 1284821;
    }

    @Override
    public FormulaType resolveResultType(List<FormulaType> argumentTypes) {
        checkArity(argumentTypes, 3);
        checkArgumentType(argumentTypes, 0, FormulaType.STRING);
        checkArgumentType(argumentTypes, 1, FormulaType.STRING);
        checkArgumentType(argumentTypes, 2, FormulaType.STRING);

        return FormulaType.STRING;
    }
}
