/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import com.google.common.base.Strings;
import com.google.gwt.regexp.shared.RegExp;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.ConstantColumnView;


public interface RegexpVector {

    /**
     * Returns the compiled RegExp at row {@code rowIndex}, or {@code null} if the RegEx was invalid.
     *
     * Note: blank regexs match anything.
     *
     */
    RegExp get(int rowIndex);

    public static RegexpVector of(ColumnView columnView) {
        if(columnView instanceof ConstantColumnView) {
            RegExp regExp = compile(columnView.getString(0));
            return rowIndex -> regExp;
        } else {
            return rowIndex -> compile(columnView.getString(rowIndex));
        }
    }

    static RegExp compile(String string) {
        try {
            return RegExp.compile(Strings.nullToEmpty(string), "g");
        } catch (Exception e) {
            return null;
        }
    }
}
