/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.DoubleArrayColumnView;

public abstract class RoundingOperator extends AbstractUnaryFunction {

    protected RoundingOperator(String name) {
        super(name);
    }

    @Override
    public final FunctionCategory getCategory() {
        return FunctionCategory.MATH;
    }

    @Override
    public FormulaType resolveUnaryResultType(FormulaType argumentType) {
        checkArgumentType(argumentType, FormulaType.NUMBER);
        return FormulaType.NUMBER;
    }

    @Override
    public ColumnView columnApply(int numRows, ColumnView argument) {
        double[] result = new double[numRows];
        for(int i=0;i<numRows;i++) {
            result[i] = apply(argument.getDouble(i));
        }
        return new DoubleArrayColumnView(result);
    }

    public abstract double apply(double argument);

}
