/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.ConstantColumnView;
import org.activityinfo.model.type.time.LocalDate;

import java.util.Date;
import java.util.List;

/**
 * Returns today's current date as {@link org.activityinfo.model.type.time.LocalDate} value
 */
public class TodayFunction extends FormulaFunction implements ColumnFunction {

    public static final TodayFunction INSTANCE = new TodayFunction();

    private TodayFunction() {}

    @Override
    public String getId() {
        return "today";
    }

    @Override
    public FunctionCategory getCategory() {
        return FunctionCategory.DATE_TIME;
    }

    @Override
    public int getReferenceTopicId() {
        return 889440;
    }

    @Override
    public FormulaType resolveResultType(List<FormulaType> argumentTypes) {
        checkArity(argumentTypes, 0);
        return FormulaType.LOCAL_DATE;
    }

    @Override
    public ColumnView apply(int numRows, List<ColumnView> arguments) {
        return new ConstantColumnView(numRows, new LocalDate(new Date()).toString());
    }
}
