/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions.date;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.functions.ColumnFunction;
import org.activityinfo.model.formula.functions.FormulaFunction;
import org.activityinfo.model.formula.functions.FunctionCategory;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.StringArrayColumnView;
import org.activityinfo.model.type.time.LocalDate;

import java.util.List;

public class AddDateFunction extends FormulaFunction implements ColumnFunction {

    public static final AddDateFunction INSTANCE = new AddDateFunction();

    private AddDateFunction() {
    }

    @Override
    public String getId() {
        return "adddate";
    }

    @Override
    public FunctionCategory getCategory() {
        return FunctionCategory.DATE_TIME;
    }

    @Override
    public int getReferenceTopicId() {
        return 889457;
    }

    @Override
    public FormulaType resolveResultType(List<FormulaType> argumentTypes) {
        checkArity(argumentTypes, 2);
        checkArgumentType(argumentTypes, 0, FormulaType.LOCAL_DATE);
        checkArgumentType(argumentTypes, 1, FormulaType.NUMBER);

        return FormulaType.LOCAL_DATE;
    }

    @Override
    public ColumnView apply(int numRows, List<ColumnView> arguments) {
        ColumnView dateView = arguments.get(0);
        ColumnView daysView = arguments.get(1);
        String[] result = new String[dateView.numRows()];
        for (int i = 0; i < dateView.numRows(); i++) {
            String dateString = dateView.getString(i);
            int days = (int)daysView.getDouble(i);
            if(dateString != null) {
                LocalDate date = LocalDate.parse(dateString);
                result[i] = date.plusDays(days).toString();
            }
        }
        return new StringArrayColumnView(result);
    }
}
