/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions.date;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.functions.ColumnFunction;
import org.activityinfo.model.formula.functions.FormulaFunction;
import org.activityinfo.model.formula.functions.FunctionCategory;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.DoubleArrayColumnView;
import org.activityinfo.model.type.time.LocalDate;

import java.util.List;

public abstract class DateComponentFunction extends FormulaFunction implements ColumnFunction {


    @Override
    public final FormulaType resolveResultType(List<FormulaType> argumentTypes) {
        checkArity(argumentTypes, 1);
        checkArgumentType(argumentTypes, 0, FormulaType.LOCAL_DATE);
        return FormulaType.NUMBER;
    }

    protected abstract int apply(LocalDate date);

    protected int apply(String string) {
        return apply(LocalDate.parse(string));
    }

    @Override
    public ColumnView apply(int numRows, List<ColumnView> arguments) {
        checkArity(arguments, 1);

        ColumnView view = arguments.get(0);
        double result[] = new double[view.numRows()];

        for (int i = 0; i < view.numRows(); i++) {
            String date = view.getString(i);
            if(date == null) {
                result[i] = Double.NaN;
            } else {
                result[i] = apply(date);
            }
        }
        return new DoubleArrayColumnView(result);
    }

    @Override
    public final FunctionCategory getCategory() {
        return FunctionCategory.DATE_TIME;
    }
}

