/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions.date;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.functions.ColumnFunction;
import org.activityinfo.model.formula.functions.FormulaFunction;
import org.activityinfo.model.formula.functions.FunctionCategory;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.StringArrayColumnView;
import org.activityinfo.model.type.time.LocalDate;

import java.util.List;

/**
 * DATE() function implementation.
 *
 * <p>Can you be used to create a LocalDate value, for example: DATE(2015,1,1)</p>
 */
public class DateFunction extends FormulaFunction implements ColumnFunction {

    public static final DateFunction INSTANCE = new DateFunction();

    private DateFunction() {
    }

    @Override
    public String getId() {
        return "DATE";
    }

    @Override
    public FunctionCategory getCategory() {
        return FunctionCategory.DATE_TIME;
    }

    @Override
    public int getReferenceTopicId() {
        return 889445;
    }

    @Override
    public FormulaType resolveResultType(List<FormulaType> argumentTypes) {
        checkArity(argumentTypes, 3);
        checkArgumentType(argumentTypes, 0, FormulaType.NUMBER);
        checkArgumentType(argumentTypes, 1, FormulaType.NUMBER);
        checkArgumentType(argumentTypes, 2, FormulaType.NUMBER);

        return FormulaType.LOCAL_DATE;
    }

    @Override
    public ColumnView apply(int numRows, List<ColumnView> arguments) {
        checkArity(arguments, 3);

        ColumnView year = arguments.get(0);
        ColumnView month = arguments.get(1);
        ColumnView day = arguments.get(2);

        String[] dates = new String[numRows];
        for (int i = 0; i < numRows; i++) {
            dates[i] = apply(
                year.getDouble(i),
                month.getDouble(i),
                day.getDouble(i)).toString();
        }

        return new StringArrayColumnView(dates);
    }

    private static LocalDate apply(double year, double month, double day) {
        return new LocalDate((int)year, (int)month, (int)day);
    }
}
