/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.functions.date;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.functions.ColumnFunction;
import org.activityinfo.model.formula.functions.FormulaFunction;
import org.activityinfo.model.formula.functions.FunctionCategory;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.DoubleArrayColumnView;
import org.activityinfo.model.type.time.LocalDate;

import java.util.List;

public class DaysFunction extends FormulaFunction implements ColumnFunction {

    public static final DaysFunction INSTANCE = new DaysFunction();

    private DaysFunction() {
    }

    @Override
    public ColumnView apply(int numRows, List<ColumnView> arguments) {
        checkArity(arguments, 2);
        ColumnView x = arguments.get(0);
        ColumnView y = arguments.get(1);

        double[] days = new double[numRows];
        for (int i = 0; i < numRows; i++) {
            LocalDate xd = x.getLocalDate(i);
            LocalDate yd = y.getLocalDate(i);
            if(xd == null || yd == null) {
                days[i] = Double.NaN;
            } else {
                days[i] = LocalDate.daysBetween(yd, xd);
            }
        }
        return new DoubleArrayColumnView(days);
    }

    @Override
    public String getId() {
        return "DAYS";
    }

    @Override
    public FunctionCategory getCategory() {
        return FunctionCategory.DATE_TIME;
    }

    @Override
    public int getReferenceTopicId() {
        return 889846;
    }

    @Override
    public FormulaType resolveResultType(List<FormulaType> argumentTypes) {
        checkArity(argumentTypes, 2);
        checkArgumentType(argumentTypes, 0, FormulaType.LOCAL_DATE);
        checkArgumentType(argumentTypes, 1, FormulaType.LOCAL_DATE);

        return FormulaType.NUMBER;
    }
}
