/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.functions.CountFunction;
import org.activityinfo.model.formula.functions.StatFunction;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldType;

import java.util.Objects;
import java.util.Optional;
import java.util.Set;

public class AggregateNode implements ReferenceNode {

    private FieldReferenceNode subFormField;
    private StatFunction statFunction;
    private ReferenceNode valueField;

    public AggregateNode(FieldReferenceNode subFormField, StatFunction statFunction, ReferenceNode valueField) {
        this.subFormField = subFormField;
        this.statFunction = statFunction;
        this.valueField = valueField;
    }

    @Override
    public <T> T accept(ReferenceVisitor<T> visitor) {
        return visitor.visitAggregation(this);
    }

    public StatFunction getStatFunction() {
        return statFunction;
    }

    public FieldReferenceNode getSubFormField() {
        return subFormField;
    }

    public ReferenceNode getValueField() {
        return valueField;
    }

    @Override
    public Optional<FormulaType> getResolvedFormulaType() {
        return Optional.of(FormulaType.NUMBER);
    }

    @Override
    public String label(ResourceId rootFormId) {
        return subFormField.label(rootFormId);
    }

    @Override
    public Set<ResourceId> getReferencedForms() {
        return valueField.getReferencedForms();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        AggregateNode that = (AggregateNode) o;
        return subFormField.equals(that.subFormField) &&
            statFunction.equals(that.statFunction) &&
            valueField.equals(that.valueField);
    }

    @Override
    public int hashCode() {
        return Objects.hash(subFormField, statFunction, valueField);
    }

    @Override
    public Optional<FieldType> getFieldType() {
        if(valueField instanceof RecordIdNode && statFunction == CountFunction.INSTANCE) {
            return subFormField.getFieldType();
        } else {
            return Optional.empty();
        }
    }
}
