/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import org.activityinfo.model.formula.FormulaNode;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.enumerated.EnumItem;

import java.util.Objects;
import java.util.Optional;
import java.util.Set;

public class EnumItemReference implements ReferenceNode {

    private final FormulaNode formulaNode;
    private final FieldReferenceNode field;
    private final EnumItem enumItem;

    public EnumItemReference(FormulaNode formulaNode, FieldReferenceNode field, EnumItem enumItem) {
        this.formulaNode = formulaNode;
        this.field = field;
        this.enumItem = enumItem;
    }

    public FieldReferenceNode getField() {
        return field;
    }

    public EnumItem getEnumItem() {
        return enumItem;
    }

    @Override
    public <T> T accept(ReferenceVisitor<T> visitor) {
        return visitor.visitEnumItem(this);
    }

    @Override
    public Optional<FormulaType> getResolvedFormulaType() {
        return Optional.of(FormulaType.BOOLEAN);
    }

    @Override
    public String label(ResourceId rootFormId) {
        return field.label(rootFormId);
    }

    @Override
    public Set<ResourceId> getReferencedForms() {
        return field.getReferencedForms();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        EnumItemReference that = (EnumItemReference) o;
        return field.equals(that.field) &&
            enumItem.getId().equals(that.enumItem.getId());
    }

    @Override
    public int hashCode() {
        return Objects.hash(field, enumItem.getId());
    }

    public FormulaNode toFormula() {
        return formulaNode;
    }
}
