/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import org.activityinfo.model.form.FormClass;
import org.activityinfo.model.form.FormField;
import org.activityinfo.model.formTree.FormTree;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.ColumnModel;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldType;

import java.util.Collections;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;

public class FieldReferenceNode implements ReferenceNode {

    protected final FormClass formClass;
    protected final FormField field;

    public FieldReferenceNode(FormTree.Node fieldNode) {
        this.formClass = fieldNode.getDefiningFormClass();
        this.field = fieldNode.getField();
    }

    public FieldReferenceNode(FormClass formClass, FormField field) {
        this.formClass = formClass;
        this.field = field;
    }

    @Override
    public <T> T accept(ReferenceVisitor<T> visitor) {
        return visitor.visitField(this);
    }

    @Override
    public Optional<FormulaType> getResolvedFormulaType() {
        return Optional.of(field.getType().getFormulaType());
    }

    @Override
    public final String label(ResourceId rootFormId) {
        if (formClass.getId().equals(rootFormId)) {
            return field.getLabel();
        } else {
            return formClass.getLabel() + " " + field.getLabel();
        }
    }

    public final FormField getField() {
        return field;
    }

    public final Optional<FieldType> getFieldType() {
        return Optional.of(field.getType());
    }


    @Override
    public Set<ResourceId> getReferencedForms() {
        return Collections.singleton(formClass.getId());
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        FieldReferenceNode that = (FieldReferenceNode) o;
        return formClass.getId().equals(that.formClass.getId()) &&
            field.getId().equals(that.field.getId());
    }

    @Override
    public int hashCode() {
        return Objects.hash(formClass.getId(), field.getId());
    }

    public boolean isParentField() {
        return field.getId().asString().equals(ColumnModel.PARENT_SYMBOL);
    }

}
