/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;


import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.FunctionCallNode;
import org.activityinfo.model.formula.functions.AndFunction;
import org.activityinfo.model.formula.functions.OrFunction;

import java.util.Arrays;

public class FormulaFixer {

    public static ResolvedNode fix(ResolvedNode node) {
        if (node instanceof InvalidCallNode) {
            InvalidCallNode callNode = (InvalidCallNode) node;
            if (callNode.getFunctionName().equals(AndFunction.NAME) || callNode.getFunctionName().equals(OrFunction.NAME)) {
                ResolvedNode a = fix(callNode.getChildren().get(0));
                ResolvedNode b = fix(callNode.getChildren().get(1));
                if(a.isValid() && b.isValid()) {
                    FunctionCallNode ast = new FunctionCallNode(callNode.getFunctionName(), a.toFormula(), b.toFormula());
                    return new ResolvedCallNode(ast, ast.getFunction(), Arrays.asList(a, b), FormulaType.BOOLEAN);
                } else if(a.isValid()) {
                    return a;
                } else if(b.isValid()) {
                    return b;
                }
            }
        }
        return node;
    }
}
