/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import org.activityinfo.model.formula.FormulaNode;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.SourceRange;

import java.util.Collections;
import java.util.Optional;

public class InvalidArgumentNode implements ResolvedNode {

    private ResolvedNode argument;
    private String errorMessage;

    public InvalidArgumentNode(ResolvedNode argument, String errorMessage) {
        this.argument = argument;
        this.errorMessage = errorMessage;
    }

    @Override
    public boolean isValid() {
        return false;
    }

    @Override
    public SourceRange getSourceRange() {
        return argument.getSourceRange();
    }

    @Override
    public Optional<FormulaType> getResolvedType() {
        return argument.getResolvedType();
    }

    @Override
    public <T> T accept(ResolvedNodeVisitor<T> visitor) {
        return visitor.visitInvalidArgumentNode(this);
    }

    @Override
    public FormulaNode toFormula() {
        return argument.toFormula();
    }

    public String getErrorMessage() {
        return errorMessage;
    }

    @Override
    public Iterable<ResolvedNode> getChildren() {
        return Collections.singleton(argument);
    }
}
