/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import org.activityinfo.model.formula.FormulaNode;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.FunctionCallNode;
import org.activityinfo.model.formula.SourceRange;

import java.util.List;
import java.util.Optional;

public class InvalidCallNode implements ResolvedNode {

    private final FunctionCallNode call;
    private final boolean validFunction;
    private final List<ResolvedNode> arguments;
    private final String errorMessage;

    public InvalidCallNode(FunctionCallNode call, boolean validFunction, List<ResolvedNode> arguments) {
        this(call, validFunction, arguments, "");
    }

    public InvalidCallNode(FunctionCallNode call, boolean validFunction, List<ResolvedNode> arguments, String errorMessage) {
        this.call = call;
        this.validFunction = validFunction;
        this.arguments = arguments;
        this.errorMessage = errorMessage;
    }

    @Override
    public boolean isValid() {
        return false;
    }

    @Override
    public SourceRange getSourceRange() {
        return call.getSourceRange();
    }

    @Override
    public Optional<FormulaType> getResolvedType() {
        return Optional.empty();
    }

    @Override
    public <T> T accept(ResolvedNodeVisitor<T> visitor) {
        return visitor.visitInvalidCall(this);
    }

    @Override
    public FormulaNode toFormula() {
        return call;
    }

    @Override
    public List<ResolvedNode> getChildren() {
        return arguments;
    }

    /**
     * Returns true if the function referenced is valid, or false if the function name itself is garbage. E.g.:
     * FOOBAR(3, 4)
     */
    public boolean isValidFunction() {
        return validFunction;
    }

    /**
     * Returns an error message that applies to the whole function call, or empty if the error
     * only relates to a single invalid argument.
     */
    public String getErrorMessage() {
        return errorMessage;
    }

    public String getFunctionName() {
        return call.getFunctionName();
    }
}
