/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import org.activityinfo.model.formula.FormulaNode;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.SourceRange;

import java.util.Collections;
import java.util.Optional;

public class InvalidReferenceNode implements ResolvedNode {

    private final FormulaNode formulaNode;
    private final String message;

    public InvalidReferenceNode(FormulaNode formulaNode, String message) {
        this.formulaNode = formulaNode;
        this.message = message;
    }

    @Override
    public boolean isValid() {
        return false;
    }

    @Override
    public SourceRange getSourceRange() {
        return formulaNode.getSourceRange();
    }

    public FormulaNode getFormulaNode() {
        return formulaNode;
    }

    @Override
    public Optional<FormulaType> getResolvedType() {
        return Optional.empty();
    }

    @Override
    public <T> T accept(ResolvedNodeVisitor<T> visitor) {
        return visitor.visitInvalidReferenceNode(this);
    }

    @Override
    public FormulaNode toFormula() {
        return formulaNode;
    }

    @Override
    public Iterable<ResolvedNode> getChildren() {
        return Collections.emptyList();
    }
}
