/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import com.google.common.base.Strings;
import org.activityinfo.model.formula.ErrorFormulaNode;
import org.activityinfo.model.formula.FormulaNode;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.SourceRange;

import java.util.Collections;
import java.util.Optional;

public class InvalidSyntaxNode implements ResolvedNode {

    private final String formula;
    private final String errorMessage;
    private final SourceRange sourceRange;

    public InvalidSyntaxNode(String formula, String errorMessage, SourceRange sourceRange) {
        this.formula = formula;
        this.errorMessage = errorMessage;
        this.sourceRange = sourceRange;
    }

    @Override
    public boolean isValid() {
        return false;
    }

    public String getErrorMessage() {
        return errorMessage;
    }

    @Override
    public SourceRange getSourceRange() {
        return sourceRange;
    }

    @Override
    public Optional<FormulaType> getResolvedType() {
        return Optional.empty();
    }

    @Override
    public <T> T accept(ResolvedNodeVisitor<T> visitor) {
        return visitor.visitInvalidSyntax(this);
    }

    @Override
    public FormulaNode toFormula() {
        return new ErrorFormulaNode(Strings.nullToEmpty(formula));
    }

    @Override
    public Iterable<ResolvedNode> getChildren() {
        return Collections.emptyList();
    }
}
