/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldType;

import java.util.Objects;
import java.util.Optional;
import java.util.Set;

public class JoinQueryNode implements ReferenceNode {

    private final FieldReferenceNode left;
    private final ResourceId rightFormId;
    private final ReferenceNode right;

    public JoinQueryNode(FieldReferenceNode left, ResourceId rightFormId, ReferenceNode right) {
        this.left = left;
        this.rightFormId = rightFormId;
        this.right = right;
    }

    @Override
    public <T> T accept(ReferenceVisitor<T> visitor) {
        return visitor.visitJoin(this);
    }

    public FieldReferenceNode getLeft() {
        return left;
    }

    public ReferenceNode getRight() {
        return right;
    }

    public ResourceId getRightFormId() {
        return rightFormId;
    }

    @Override
    public Optional<FormulaType> getResolvedFormulaType() {
        return right.getResolvedFormulaType();
    }

    @Override
    public String label(ResourceId rootFormId) {
        return right.label(rootFormId);
    }

    @Override
    public Set<ResourceId> getReferencedForms() {
        return right.getReferencedForms();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        JoinQueryNode that = (JoinQueryNode) o;
        return left.equals(that.left) &&
            rightFormId.equals(that.rightFormId) &&
            right.equals(that.right);
    }

    @Override
    public Optional<FieldType> getFieldType(){
        return right.getFieldType();
    }

    @Override
    public int hashCode() {
        return Objects.hash(left, rightFormId, right);
    }
}
