/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import org.activityinfo.i18n.shared.I18N;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.resource.ResourceId;

import java.util.Collections;
import java.util.Optional;
import java.util.Set;

public class RecordIdNode implements ReferenceNode {

    private ResourceId formId;

    public RecordIdNode(ResourceId formId) {
        this.formId = formId;
    }

    @Override
    public <T> T accept(ReferenceVisitor<T> visitor) {
        return visitor.visitRecordId(this);
    }

    @Override
    public Optional<FormulaType> getResolvedFormulaType() {
        return Optional.of(FormulaType.STRING);
    }

    @Override
    public String label(ResourceId rootFormId) {
        return I18N.CONSTANTS.recordId();
    }

    @Override
    public Set<ResourceId> getReferencedForms() {
        return Collections.singleton(formId);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        RecordIdNode that = (RecordIdNode) o;
        return formId.equals(that.formId);
    }

    @Override
    public int hashCode() {
        return formId.hashCode();
    }
}
