/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldType;

import java.util.Optional;
import java.util.Set;

public interface ReferenceNode {

    <T> T accept(ReferenceVisitor<T> visitor);

    Optional<FormulaType> getResolvedFormulaType();

    /**
     * Returns true if this node is valid. This will only be false for the case of
     * calculated fields which have problems with their formula.
     */
    default boolean isValid() {
        return false;
    }

    /**
     * Constructs a human-readable label for this resolved node, relative
     * to the given {@code rootFormId}.
     */
    String label(ResourceId rootFormId);

    /**
     * Return the resolved {@link FieldType}, if this node resolves to a single field type. Otherwise,
     * if this node resolves to a formula, or to multiple fields with different types, then return {@code Optional.empty()}.
     */
    default Optional<FieldType> getFieldType() {
        return Optional.empty();
    }


    Set<ResourceId> getReferencedForms();

    /**
     * Returns true if this {@code ReferenceNode} refers to the same field as {@code node}, even
     * if they have different formulas.
     *
     */
    boolean equals(Object node);

}
