/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.FunctionCallNode;
import org.activityinfo.model.formula.SourceRange;
import org.activityinfo.model.formula.functions.FormulaFunction;

import java.util.List;
import java.util.Optional;

public class ResolvedCallNode implements ResolvedNode {

    private final FunctionCallNode formula;
    private final FormulaFunction function;
    private final List<ResolvedNode> arguments;
    private final FormulaType resultType;

    public ResolvedCallNode(FunctionCallNode formula, FormulaFunction function, List<ResolvedNode> arguments, FormulaType type) {
        this.formula = formula;
        this.function = function;
        this.arguments = arguments;
        this.resultType = type;
    }

    public FormulaFunction getFunction() {
        return function;
    }

    @Override
    public boolean isValid() {
        return true;
    }

    @Override
    public SourceRange getSourceRange() {
        return formula.getSourceRange();
    }

    public FormulaType getResultType() {
        return resultType;
    }

    @Override
    public Optional<FormulaType> getResolvedType() {
        return Optional.of(resultType);
    }

    @Override
    public <T> T accept(ResolvedNodeVisitor<T> visitor) {
        return visitor.visitCallNode(this);
    }

    @Override
    public FunctionCallNode toFormula() {
        return formula;
    }

    @Override
    public List<ResolvedNode> getChildren() {
        return arguments;
    }

    public ResolvedNode getArgument(int i) {
        return arguments.get(i);
    }

    public int getArgumentCount() {
        return arguments.size();
    }
}
