/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import org.activityinfo.model.formula.FormulaNode;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.formula.SourceRange;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldType;

import java.util.Collections;
import java.util.List;
import java.util.Optional;

public class ResolvedReference implements ResolvedNode {

    private final FormulaNode formula;
    private ReferenceNode root;

    public ResolvedReference(FormulaNode formula, List<ReferenceNode> matches) {
        assert !matches.isEmpty();
        this.formula = formula;
        if(matches.size() == 1) {
            this.root = matches.iterator().next();
        } else {
            this.root = new UnionNode(matches);
        }
    }

    public ResolvedReference(FormulaNode formula, ReferenceNode root) {
        this.formula = formula;
        this.root = root;
    }

    public ReferenceNode getRoot() {
        return root;
    }

    @Override
    public boolean isValid() {
        return getResolvedType().isPresent();
    }

    @Override
    public SourceRange getSourceRange() {
        return formula.getSourceRange();
    }

    @Override
    public Optional<FormulaType> getResolvedType() {
        return root.getResolvedFormulaType();
    }

    @Override
    public Optional<FieldType> getFieldType() {
        return root.getFieldType();
    }

    @Override
    public <T> T accept(ResolvedNodeVisitor<T> visitor) {
        return visitor.visitReferenceNode(this);
    }

    @Override
    public FormulaNode toFormula() {
        return formula;
    }

    @Override
    public Iterable<ResolvedNode> getChildren() {
        return Collections.emptyList();
    }

    @Override
    public String label(ResourceId rootFormId) {
        return root.label(rootFormId);
    }

    @Override
    public int hashCode() {
        return root.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        ResolvedReference other = (ResolvedReference) obj;
        return root.equals(other.root);
    }

    private JoinQueryNode findLastRight(ReferenceNode node) {
        while (node instanceof JoinQueryNode) {
            JoinQueryNode joinQueryNode = (JoinQueryNode) node;
            if (!(joinQueryNode.getRight() instanceof JoinQueryNode)) {
                return joinQueryNode;
            }
            node = joinQueryNode.getRight();
        }
        return (JoinQueryNode) node;
    }
}