/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import org.activityinfo.model.form.FormField;
import org.activityinfo.model.formula.ConstantNode;
import org.activityinfo.model.resource.ResourceId;

import java.util.HashSet;
import java.util.Set;
import java.util.function.Predicate;

public class RootFieldReferenceFinder implements ResolvedNodeVisitor<Void>, ReferenceVisitor<Void> {

    private static final Void NOTHING = null;

    private Set<ResourceId> fields = new HashSet<>();

    private Predicate<FormField> filter;

    public RootFieldReferenceFinder(Predicate<FormField> filter) {
        this.filter = filter;
    }

    /**
     * Finds all the ids of fields referenced by this formula in the root form.
     */
    public static Set<ResourceId> find(ResolvedNode root, Predicate<FormField> filter) {
        RootFieldReferenceFinder finder = new RootFieldReferenceFinder(filter);
        root.accept(finder);
        return finder.fields;
    }

    public static Set<ResourceId> find(ResolvedNode root) {
        return find(root, f -> true);
    }

    @Override
    public Void visitConstant(ConstantNode node) {
        return NOTHING;
    }

    @Override
    public Void visitInvalidSyntax(InvalidSyntaxNode node) {
        return NOTHING;
    }

    @Override
    public Void visitReferenceNode(ResolvedReference node) {
        return node.getRoot().accept(this);
    }

    @Override
    public Void visitInvalidReferenceNode(InvalidReferenceNode node) {
        return NOTHING;
    }

    @Override
    public Void visitCallNode(ResolvedCallNode node) {
        for (ResolvedNode child : node.getChildren()) {
            child.accept(this);
        }
        return NOTHING;
    }

    @Override
    public Void visitInvalidCall(InvalidCallNode invalidCallNode) {
        for (ResolvedNode child : invalidCallNode.getChildren()) {
            child.accept(this);
        }
        return NOTHING;
    }

    @Override
    public Void visitInvalidArgumentNode(InvalidArgumentNode invalidArgumentNode) {
        return NOTHING;
    }

    @Override
    public Void visitRecordId(RecordIdNode recordIdNode) {
        return NOTHING;
    }

    @Override
    public Void visitFormId(FormIdNode formIdNode) {
        return NOTHING;
    }

    @Override
    public Void visitLastEditTime(LastEditTimeNode node) {
        return NOTHING;
    }

    @Override
    public Void visitField(FieldReferenceNode field) {
        if(filter.test(field.getField())) {
            fields.add(field.getField().getId());
        }
        return NOTHING;
    }

    @Override
    public Void visitCalculatedField(CalculatedFieldNode field) {
        return visitField(field);
    }

    @Override
    public Void visitEnumItem(EnumItemReference enumItemReference) {
        return enumItemReference.getField().accept(this);
    }

    @Override
    public Void visitJoin(JoinQueryNode node) {
        return node.getLeft().accept(this);
    }

    @Override
    public Void visitUnion(UnionNode node) {
        for (ReferenceNode child : node.getChildren()) {
            child.accept(this);
        }
        return NOTHING;
    }

    @Override
    public Void visitAggregation(AggregateNode node) {
        fields.add(node.getSubFormField().getField().getId());

        return NOTHING;
    }
}
