/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.formula.resolved;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldType;

import java.util.*;

public class UnionNode implements ReferenceNode {


    private List<ReferenceNode> references;
    private Optional<FieldType> fieldType;

    public UnionNode(List<ReferenceNode> references) {
        assert references.size() > 1;
        this.references = references;
        this.fieldType = findUniqueFieldType(references);
    }

    private static Optional<FieldType> findUniqueFieldType(List<ReferenceNode> references) {
        Iterator<ReferenceNode> it = references.iterator();
        Optional<FieldType> firstType = it.next().getFieldType();
        if(!firstType.isPresent()) {
            return firstType;
        }
        // Make sure the other nodes have the same type.
        while(it.hasNext()) {
            if(!it.next().getFieldType().equals(firstType)) {
                return Optional.empty();
            }
        }
        return firstType;
    }

    public List<ReferenceNode> getChildren() {
        return references;
    }

    @Override
    public <T> T accept(ReferenceVisitor<T> visitor) {
        return visitor.visitUnion(this);
    }

    @Override
    public Optional<FormulaType> getResolvedFormulaType() {
        return references.get(0).getResolvedFormulaType();
    }

    @Override
    public String label(ResourceId rootFormId) {
        return references.get(0).label(rootFormId);
    }

    @Override
    public Set<ResourceId> getReferencedForms() {
        Set<ResourceId> forms = new HashSet<>();
        for (ReferenceNode reference : references) {
            forms.addAll(reference.getReferencedForms());
        }
        return forms;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        UnionNode unionNode = (UnionNode) o;
        return references.equals(unionNode.references);
    }

    @Override
    public int hashCode() {
        return Objects.hash(references);
    }

}
