/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.job;

import org.activityinfo.json.AutoJson;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.resource.ResourceId;

@AutoJson
public class DuplicateDatabaseJob implements JobDescriptor<DuplicateDatabaseResult> {

    public static final String TYPE = "duplicateDatabase";

    private final ResourceId templateDatabaseId;
    private final String databaseLabel;
    private final boolean duplicateRecords;

    @Builder
    public DuplicateDatabaseJob(ResourceId templateDatabaseId, String databaseLabel, boolean duplicateRecords) {
        this.templateDatabaseId = templateDatabaseId;
        this.databaseLabel = databaseLabel;
        this.duplicateRecords = duplicateRecords;
    }

    public ResourceId getTemplateDatabaseId() {
        return templateDatabaseId;
    }

    public String getDatabaseLabel() {
        return databaseLabel;
    }

    @Override
    public String getType() {
        return TYPE;
    }

    @Override
    public DuplicateDatabaseResult parseResult(JsonValue resultObject) {
        return DuplicateDatabaseResultJson.fromJson(resultObject);
    }

    public boolean isDuplicateRecords() {
        return duplicateRecords;
    }

    @Override
    public <R> R accept(JobVisitor<R> visitor) {
        return visitor.visit(this);
    }

    @Override
    public boolean isReadOnly() {
        return false;
    }

    @Override
    public JsonValue toJson() {
        return DuplicateDatabaseJobJson.toJson(this);
    }
}
