/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.job;

import org.activityinfo.json.AutoJson;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.resource.ResourceId;

import java.util.Optional;

@AutoJson
public class ExportAttachmentsJob implements JobDescriptor<ExportResult> {

    public static final String TYPE = "exportAttachments";

    private final ResourceId formId;
    private final Optional<String> filter;

    public ExportAttachmentsJob(ResourceId formId, Optional<String> filter) {
        this.formId = formId;
        this.filter = filter;
    }

    @Override
    public String getType() {
        return TYPE;
    }

    @Override
    public ExportResult parseResult(JsonValue resultObject) {
        return ExportResult.fromJson(resultObject);
    }

    @Override
    public <R> R accept(JobVisitor<R> visitor) {
        return visitor.visit(this);
    }

    @Override
    public boolean isReadOnly() {
        return true;
    }

    @Override
    public JsonValue toJson() {
        return ExportAttachmentsJobJson.toJson(this);
    }

    public ResourceId getFormId() {
        return formId;
    }

    public Optional<String> getFilter() {
        return filter;
    }
}
