/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.job;


import org.activityinfo.json.AutoJson;
import org.activityinfo.json.JsonValue;

/**
 * Export of all forms in a database or folder to a CSV table
 */
@AutoJson
public class ExportBillingAccountStatsJob implements JobDescriptor<ExportResult> {

    public static final String TYPE = "exportBillingAccount";

    /**
     * The id of the billing account
     */
    private final long billingAccountId;

    private final int utcOffset;


    public ExportBillingAccountStatsJob(long billingAccountId, int utcOffset) {
        this.billingAccountId = billingAccountId;
        this.utcOffset = utcOffset;
    }

    public long getBillingAccountId() {
        return billingAccountId;
    }

    public int getUtcOffset() {
        return utcOffset;
    }

    @Override
    public String getType() {
        return TYPE;
    }

    @Override
    public ExportResult parseResult(JsonValue resultObject) {
        return ExportResult.fromJson(resultObject);
    }

    @Override
    public <R> R accept(JobVisitor<R> visitor) {
        return visitor.visit(this);
    }

    @Override
    public boolean isReadOnly() {
        return true;
    }

    @Override
    public JsonValue toJson() {
        return ExportBillingAccountStatsJobJson.toJson(this);
    }

    public static ExportBillingAccountStatsJob fromJson(JsonValue object) {
        return ExportBillingAccountStatsJobJson.fromJson(object);
    }

}
