/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.job;

public enum  ExportFileFormat {

    /**
     * Office Open XML .xlsx file.
     */
    XLSX(".xlsx", "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"),

    /**
     * UTF-8 encoded text file, with rows seperated by newline characters ('\n') and columns
     * seperated by the INFORMATION SEPARATOR control character (U+001F). Can only be used for
     * a single form, or for database forms exported using the LONG format.
     */
    TEXT(".txt", "text/plain");

    private final String extension;
    private final String mimeType;

    ExportFileFormat(String extension, String mimeType) {
        this.extension = extension;
        this.mimeType = mimeType;
    }

    /**
     * @return the extension for the file format, including an initial "."
     */
    public String getExtension() {
        return extension;
    }

    public String getMimeType() {
        return mimeType;
    }
}
