/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.job;

import org.activityinfo.json.AutoJson;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.resource.ResourceId;

@AutoJson
public class ExportUsersJob implements JobDescriptor<ExportResult> {

    public static final String TYPE = "exportUsers";

    private final ResourceId databaseId;
    private final int utcOffset;

    public ExportUsersJob(ResourceId databaseId, int utcOffset) {
        this.databaseId = databaseId;
        this.utcOffset = utcOffset;
    }

    @Override
    public String getType() {
        return TYPE;
    }

    public ResourceId getDatabaseId() {
        return databaseId;
    }

    /**
     * The requested offset from Universal Coordinated Time (UTC) in minutes.
     *
     * <p>Excel times do not include timezones, so we need to decide which timezone to write the
     * invitation and last login times.</p>
     */
    public int getUtcOffset() {
        return utcOffset;
    }

    @Override
    public ExportResult parseResult(JsonValue resultObject) {
        return ExportResult.fromJson(resultObject);
    }

    @Override
    public <R> R accept(JobVisitor<R> visitor) {
        return visitor.visit(this);
    }

    @Override
    public boolean isReadOnly() {
        return true;
    }

    @Override
    public JsonValue toJson() {
        return ExportUsersJobJson.toJson(this);
    }

    public static ExportUsersJob fromJson(JsonValue jsonValue) {
        return ExportUsersJobJson.fromJson(jsonValue);
    }
}
