/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.job;

import org.activityinfo.i18n.shared.I18N;
import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;

@AutoJson
public class JobError {
    private String code;
    private String message;
    private String localizedMessage;

    @Builder
    public JobError(String code, String message, String localizedMessage) {
        this.code = code;
        this.message = message;
        this.localizedMessage = localizedMessage;
    }

    public JobError(Enum<?> enumItem) {
        this.code = enumItem.name();
        this.message = null;
        this.localizedMessage = null;
    }

    public String getCode() {
        return code;
    }

    public String getMessage() {
        return message;
    }

    public String getLocalizedMessage() {
        return localizedMessage;
    }

    public static JobError permissionDenied() {
        return new JobError(ExportError.PERMISSION_DENIED.name(),
            "Permission denied", I18N.CONSTANTS.permissionDeniedHeading());
    }

    public static JobError serverError() {
        return new JobError("SERVER_ERROR",
            "Server error", I18N.CONSTANTS.errorOnServer());
    }

    @Override
    public String toString() {
        return "JobError{" +
            "code='" + code + '\'' +
            ", message='" + message + '\'' +
            ", localizedMessage='" + localizedMessage + '\'' +
            '}';
    }
}
