/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.job;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonArrays;
import org.activityinfo.json.JsonValue;

import java.io.IOException;
import java.util.List;

/**
 * Writes a list of records as line delimited JSON.
 */
public class LdJsonRecordWriter {
    public static final String HEADER = "LINE DELIMITED JSON RECORDS";

    private final StringBuilder out;

    public LdJsonRecordWriter(int recordCount, List<String> fieldIdList)  {
        this.out = new StringBuilder();
        this.out.append(HEADER + "\n");
        this.out.append(Integer.toString(recordCount)).append("\n");
        this.out.append(JsonArrays.toJsonArray(fieldIdList, Json::create).toJson()).append("\n");
    }

    /**
     * Appends a record to the output.
     *
     * {@code record} must be a JSON array, with the first value the record ID, or {@code null} if
     * the record is new and an ID can be generated on the server.
     *
     * <p>Following the record id, the remaining values of the array should correspond to the values of
     * the fields specified in the header, in the same order as specificied in {@code fieldIdList}</p>
     *
     * @param record a JSON array of the record id followed by the field values.
     */
    public void write(JsonValue record) {
        this.out.append(Json.stringify(record));
        this.out.append('\n');
    }

    @Override
    public String toString() {
        return out.toString();
    }
}
