/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.permission;

import org.activityinfo.model.database.DatabaseUserGrant;
import org.activityinfo.model.database.Resource;
import org.activityinfo.model.form.FormRecord;
import org.activityinfo.model.formula.FormulaNode;

/**
 * <p>
 *     Defines an operation that can be granted to a User on a specific {@link Resource} via a {@link DatabaseUserGrant}.
 * </p>
 * <p>
 *     {@code Operation}s define {@code Resource}-level permissions. {@code Operation}s can be combined with filters,
 *     defined by {@link FormulaNode}s, which allow the Database Owner to specify further {@link FormRecord}-level
 *     restrictions which must be applied.
 * </p>
 */
public enum Operation {

    /**
     * View the resource, whether a form, folder, or database.
     */
    VIEW,

    /**
     * Create a record within a form contained by this
     * folder or form.
     */
    ADD_RECORD,

    /**
     * Edit a record's values within a form contained by this folder or form.
     */
    EDIT_RECORD,


    /**
     * Add, edit, and delete records within reference forms
     */
    MANAGE_REFERENCE_DATA,

    /**
     * Delete a record within this form.
     */
    DELETE_RECORD,

    /**
     * Bulk record delete within this form
     */
    BULK_DELETE,

    /**
     * Export Records from a form, folder or database.
     */
    EXPORT_RECORDS,

    /**
     * Grant permissions to a user to this database, folder, or form.
     */
    MANAGE_USERS,


    /**
     * Add, modify, or remove locks on records.
     */
    LOCK_RECORDS,

    /**
     * Create a new Resource (Form or Folder)
     */
    ADD_RESOURCE,


    /**
     * Edit a Resource's schema, structure, attributes or data.
     */
    EDIT_RESOURCE,

    /**
     * Delete a Resource (Form or Folder)
     */
    DELETE_RESOURCE,


    /**
     * Manage (open/close) collection links for the given form
     */
    MANAGE_COLLECTION_LINKS,


    /**
     * Access the Audit logs for a database (or a subset)
     */
    AUDIT,

    /**
     * Allow the user to share reports with other roles in the database.
     */
    SHARE_REPORTS,

    /**
     * Allows the user to publish reports.
     */
    PUBLISH_REPORTS,

    /**
     * Add, modify and delete roles
     */
    MANAGE_ROLES

}
