/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.query;

import org.activityinfo.model.util.HeapsortColumn;

/**
 * Boolean column view backed by a int[] array
 */
public class BooleanColumnView extends ColumnView {

    private int values[];

    public BooleanColumnView() {
    }

    public BooleanColumnView(int... values) {
        this.values = values;
    }

    @Override
    public ColumnType getType() {
        return ColumnType.BOOLEAN;
    }

    @Override
    public int numRows() {
        return values.length;
    }

    @Override
    public Object get(int row) {
        int v = values[row];
        if(v == ColumnView.BLANK) {
            return null;
        } else {
            return v != 0;
        }
    }

    @Override
    public double getDouble(int row) {
        int v = values[row];
        if(v == ColumnView.BLANK) {
            return Double.NaN;
        } else {
            return v;
        }
    }

    @Override
    public String getString(int row) {
        return null;
    }

    @Override
    public int getBoolean(int row) {
        return values[row];
    }

    @Override
    public boolean isBlank(int row) {
        return values[row] == BLANK;
    }

    @Override
    public ColumnView select(int[] selectedRows) {
        int filteredValues[] = new int[selectedRows.length];
        for (int i = 0; i < filteredValues.length; i++) {
            int selectedRow = selectedRows[i];
            if(selectedRow == -1) {
                filteredValues[i] = BLANK;
            } else {
                filteredValues[i] = values[selectedRow];
            }

        }
        return new BooleanColumnView(filteredValues);
    }

    @Override
    public int[] order(int[] sortVector, SortDir direction, int[] range) {

        int numRows = values.length;
        if (range == null || range.length == numRows) {
            HeapsortColumn.heapsortInt(values, sortVector, numRows,
                    HeapsortColumn.withIntDirection(BooleanColumnView::isLessThan, direction));
        } else {

            HeapsortColumn.heapsortInt(values, sortVector, range.length, range,
                    HeapsortColumn.withIntDirection(BooleanColumnView::isLessThan, direction));
        }
        return sortVector;
    }


    private static boolean isLessThan(int a, int b) {
        if (a == ColumnView.TRUE) {
            return false;
        } else if (a == ColumnView.FALSE) {
            return b == ColumnView.TRUE;
        } else {
            // a == ColumnView.NA
            return b != ColumnView.BLANK;
        }
    }

}
