/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.query;

import com.google.gwt.core.client.JavaScriptObject;


@SuppressWarnings("GwtInconsistentSerializableClass")
public class ColumnArrayView extends ColumnView {
   
    private ColumnType columnType;
    private JavaScriptObject array;

    public ColumnArrayView(ColumnType columnType, JavaScriptObject array) {
        this.columnType = columnType;
        this.array = array;
    }

    @Override
    public ColumnType getType() {
        return columnType;
    }

    @Override
    public int numRows() {
        return getLength(array);
    }

    @Override
    public Object get(int row) {
        switch (columnType) {
            case STRING:
                return getString(row);
            case NUMBER:
                return getDouble(row);
            case BOOLEAN:
                return getBoolean(row);
        }
        throw new UnsupportedOperationException();
    }

    @Override
    public double getDouble(int row) {
        return getDouble(array, row);
    }

    @Override
    public String getString(int row) {
        return getString(array, row);
    }

    @Override
    public int getBoolean(int row) {
        return getBoolean(array, row);
    }

    @Override
    public boolean isBlank(int row) {
        switch (columnType) {
            case STRING:
                return getString(row) == null;
            case NUMBER:
                return Double.isNaN(getDouble(row));
            case BOOLEAN:
                return getBoolean(row) == BLANK;
        }
        return true;
    }

    @Override
    public ColumnView select(int[] selectedRows) {
        throw new UnsupportedOperationException();
    }

    private static native int getLength(JavaScriptObject array) /*-{
        return array.length;
    }-*/;
    
    private static native String getString(JavaScriptObject array, int index) /*-{
        var value = array[index];
        return value ? "" + value : null;
    }-*/;

    private static native double getDouble(JavaScriptObject array, int index) /*-{
        var value = array[index];
        return value == null ? NaN : +value;
    }-*/;
    
    private static native int getBoolean(JavaScriptObject array, int index) /*-{
        var value = array[index];
        return value == null ? @org.activityinfo.model.query.ColumnView::BLANK : (value ? 1 : 0);
    }-*/;

    @Override
    public int[] order(int[] sortVector, SortDir direction, int[] range) {
        // TODO: Enum Sorting
        // Do not sort on column
        return sortVector;
    }

}
