/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.query;

import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.type.time.LocalDate;

public class ColumnViews {

    public static final ColumnView TRUE = new BooleanColumnView(ColumnView.TRUE);
    public static final ColumnView FALSE = new BooleanColumnView(ColumnView.FALSE);

    public static final ColumnView BLANK_NUMBER = new DoubleArrayColumnView(Double.NaN);
    public static final ColumnView BLANK_BOOLEAN = new BooleanColumnView(ColumnView.BLANK);
    public static final ColumnView BLANK_STRING = new BlankColumnView(ColumnType.STRING, 1);
    public static final ColumnView BLANK_DATE = new BlankColumnView(ColumnType.STRING, 1);
    public static final ColumnView BLANK_GEOMETRY = new BlankColumnView(ColumnType.STRING, 1);

    public static ColumnView of(boolean value) {
        return value ? TRUE : FALSE;
    }

    public static ColumnView of(double value) {
        return new DoubleArrayColumnView(value);
    }

    public static ColumnView blank(FormulaType resultType) {
        switch (resultType) {
            case NUMBER:
                return new BlankColumnView(ColumnType.NUMBER, 1);
            case STRING:
            case LOCAL_DATE:
                return new BlankColumnView(ColumnType.STRING, 1);
            default:
            case BOOLEAN:
                return new BlankColumnView(ColumnType.BOOLEAN, 1);
        }
    }

    public static ColumnView of(String string) {
        if(string == null || string.isEmpty()) {
            return BLANK_STRING;
        } else {
            return new StringArrayColumnView(string);
        }
    }

    public static ColumnView of(LocalDate date) {
        return new StringArrayColumnView(date.toString());
    }

}
