/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.query;

import org.activityinfo.model.util.HeapsortColumn;

public class DoubleArrayColumnView extends ColumnView {
    private double[] values;
    private int numRows;

    protected DoubleArrayColumnView() {
    }

    public DoubleArrayColumnView(double... values) {
        this(values, values.length);
    }

    public DoubleArrayColumnView(double[] values, int numRows) {
        this.values = values;
        this.numRows = numRows;
    }

    @Override
    public ColumnType getType() {
        return ColumnType.NUMBER;
    }

    @Override
    public int numRows() {
        return numRows;
    }

    @Override
    public Object get(int row) {
        double value = values[row];
        if(Double.isNaN(value)) {
            return null;
        } else {
            return value;
        }
    }

    @Override
    public double getDouble(int row) {
        return values[row];
    }

    @Override
    public String getString(int row) {
        return isBlank(row) ? null : Double.toString(values[row]);
    }



    @Override
    public boolean isBlank(int row) {
        return Double.isNaN(values[row]);
    }

    @Override
    public ColumnView select(int[] selectedRows) {
        double filteredValues[] = new double[selectedRows.length];
        for (int i = 0; i < filteredValues.length; i++) {

            int selectedRow = selectedRows[i];
            if(selectedRow == -1) {
                filteredValues[i] = Double.NaN;
            } else {
                filteredValues[i] = values[selectedRow];
            }
        }
        return new DoubleArrayColumnView(filteredValues);
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("[");
        for(int i=0;i!=Math.min(10, values.length);++i) {
            if(i > 0) {
                sb.append(", ");
            }
            if(Double.isNaN(values[i])) {
                sb.append("NaN");
            } else {
                sb.append(values[i]);
            }
        }
        if(numRows() > 10) {
            sb.append("... total rows = ").append(numRows());
        }
        sb.append(" ]");
        return sb.toString();
    }

    @Override
    public int[] order(int[] sortVector, SortDir direction, int[] range) {
        if (range == null || range.length == numRows) {
            HeapsortColumn.heapsortDouble(values, sortVector, numRows, direction == SortDir.ASC);
        } else {
            HeapsortColumn.heapsortDouble(values, sortVector, range.length, range, direction == SortDir.ASC);
        }
        return sortVector;
    }

}
