/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.query;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonArrays;
import org.activityinfo.json.JsonSerializable;
import org.activityinfo.json.JsonValue;

import java.util.Set;

/**
 * Applies a filter based on a set of strings.
 *
 */
public class FilterSet implements JsonSerializable {
    private final String columnFormula;
    private final Set<String> values;

    public FilterSet(String columnFormula, Set<String> values) {
        this.columnFormula = columnFormula;
        this.values = values;
    }

    /**
     * The formula that produces the column view to filter
     */
    public String getColumnFormula() {
        return columnFormula;
    }

    /**
     * The set of string values that should be included in the filter.
     */
    public Set<String> getValues() {
        return values;
    }

    @Override
    public JsonValue toJson() {
        JsonValue object = Json.createObject();
        object.put("columnFormula", columnFormula);
        object.put("values", Json.createStringArray(values));
        return object;
    }

    public static FilterSet fromJson(JsonValue value) {
        return new FilterSet(value.getString("columnFormula"),
            JsonArrays.setFromJson(value.get("values"), json -> json.asString()));
    }
}
