/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.resource;

import com.fasterxml.jackson.annotation.JsonProperty;
import jsinterop.annotations.JsOverlay;
import jsinterop.annotations.JsPackage;
import jsinterop.annotations.JsType;
import org.activityinfo.json.Json;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.form.FormRecord;
import org.activityinfo.model.type.FieldValue;
import org.activityinfo.model.type.RecordRef;

import java.util.Optional;

/**
 * An update to specific {@code Record}
 */
@JsType(isNative = true, namespace = JsPackage.GLOBAL, name = "Object")
public final class RecordUpdate {

    @JsonProperty(required = true)
    private String formId;

    @JsonProperty(required = true)
    private String recordId;

    private String parentRecordId;

    private boolean deleted;

    private JsonValue fields;

    @JsOverlay
    public ResourceId getFormId() {
        return ResourceId.valueOf(formId);
    }

    @JsOverlay
    public RecordUpdate setRecordId(ResourceId id) {
        this.recordId = id.asString();
        return this;
    }

    @JsOverlay
    public RecordUpdate setRecordId(String id) {
        this.recordId = id;
        return this;
    }

    @JsOverlay
    public ResourceId getRecordId() {
        return ResourceId.valueOf(recordId);
    }

    @JsOverlay
    public void setFormId(String id) {
        this.formId = id;
    }

    @JsOverlay
    public RecordUpdate setFormId(ResourceId id) {
        this.formId = id.asString();
        return this;
    }


    @JsOverlay
    public boolean isDeleted() {
        return deleted;
    }


    @JsOverlay
    public RecordUpdate setDeleted(boolean deleted) {
        this.deleted = deleted;
        return this;
    }

    @JsOverlay
    public String getParentRecordId() {
        return parentRecordId;
    }

    @JsOverlay
    public void setParentRecordId(String parentRecordId) {
        this.parentRecordId = parentRecordId;
    }
    @JsOverlay
    public void setParentRecordId(Optional<String> parentRecordId) {
        this.parentRecordId = parentRecordId.orElse(null);
    }

    @JsOverlay
    public RecordUpdate delete() {
        this.deleted = true;
        return this;
    }

    @JsOverlay
    public void setFields(JsonValue object) {
        this.fields = object;
    }

    @JsOverlay
    public JsonValue getFields() {
        return fields;
    }

    @JsOverlay
    public RecordUpdate setFieldValue(ResourceId fieldId, FieldValue value) {
        if(value == null) {
            setFieldValue(fieldId.asString(), Json.createNull());
        } else {
            setFieldValue(fieldId.asString(), value.toJson());
        }
        return this;
    }

    @JsOverlay
    public void setFieldValue(ResourceId fieldId, Optional<FieldValue> value) {
        setFieldValue(fieldId, value.orElse(null));
    }

    @JsOverlay
    public void setFieldValue(String fieldId, FieldValue value) {
        if(value == null) {
            setFieldValue(fieldId, Json.createNull());
        } else {
            setFieldValue(fieldId, value.toJson());
        }
    }

    @JsOverlay
    public void setFieldValue(String fieldId, String value) {
        setFieldValue(fieldId, Json.create(value));
    }

    @JsOverlay
    public void setFieldValue(String fieldId, double value) {
        setFieldValue(fieldId, Json.create(value));
    }

    @JsOverlay
    public void setFieldValue(String fieldId, JsonValue value) {
        if(fields == null) {
            fields = Json.createObject();
        }
        fields.put(fieldId, value);
    }

    @JsOverlay
    public RecordRef getRecordRef() {
        return new RecordRef(ResourceId.valueOf(formId), ResourceId.valueOf(recordId));
    }

    @JsOverlay
    public static RecordUpdate fromRecord(FormRecord record) {
        RecordUpdate update = new RecordUpdate();
        update.setFormId(record.getFormId());
        update.setRecordId(record.getRecordId());
        update.setFields(record.getFields());
        return update;
    }

}
