/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.resource;

import com.fasterxml.jackson.annotation.JsonProperty;
import jsinterop.annotations.JsOverlay;
import jsinterop.annotations.JsPackage;
import jsinterop.annotations.JsType;
import org.activityinfo.model.type.RecordRef;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

/**
 * Describes one or more changes to records.
 */
@JsType(isNative = true, namespace = JsPackage.GLOBAL, name = "Object")
public final class RecordUpdateSet {

    @JsonProperty(required = true)
    RecordUpdate[] changes;

    public RecordUpdateSet() {
    }

    @JsOverlay
    public Iterable<RecordUpdate> getChanges() {
        return Arrays.asList(changes);
    }

    @JsOverlay
    public RecordUpdate[] getChangeArray() {
        return changes;
    }

    @JsOverlay
    public int getChangeCount() {
        return changes.length;
    }

    @JsOverlay
    public static RecordUpdateSetBuilder builder() {
        return new RecordUpdateSetBuilder();
    }

    @JsOverlay
    public boolean isEmpty() {
        return changes.length == 0;
    }

    @JsOverlay
    public Set<ResourceId> getAffectedFormIds() {
        Set<ResourceId> forms = new HashSet<>();
        for (RecordUpdate update : getChanges()) {
            forms.add(update.getFormId());
        }
        return forms;
    }

    @JsOverlay
    public Set<RecordRef> getRecordRefs() {
        Set<RecordRef> refs = new HashSet<>();
        for (RecordUpdate change : getChanges()) {
            refs.add(change.getRecordRef());
        }
        return refs;
    }
}
