/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.resource;

import org.activityinfo.json.JsonValue;
import org.activityinfo.model.form.FormRecord;
import org.activityinfo.model.form.TypedFormRecord;
import org.activityinfo.model.type.RecordRef;

import java.util.ArrayList;
import java.util.List;

import static org.activityinfo.json.Json.toJson;

/**
 *  Constructs a set of updates to be applied atomically
 */
public class RecordUpdateSetBuilder {

    private List<RecordUpdate> updates = new ArrayList<>();

    public Iterable<RecordUpdate> getUpdates() {
        return updates;
    }

    public int getUpdateCount() {
        return updates.size();
    }

    public RecordUpdate create(ResourceId formId, ResourceId resourceId)  {
        RecordUpdate update = new RecordUpdate();
        update.setRecordId(resourceId);
        update.setFormId(formId);
        updates.add(update);
        return update;
    }


    public RecordUpdateSetBuilder create(TypedFormRecord record) {
        RecordUpdate update = new RecordUpdate();
        update.setFormId(record.getFormId());
        update.setRecordId(record.getId());
        update.setFields(FormRecord.fromInstance(record).getFields());
        updates.add(update);
        return this;
    }

    public RecordUpdateSetBuilder add(RecordUpdate update) {
        updates.add(update);
        return this;
    }

    public RecordUpdateSetBuilder add(Iterable<RecordUpdate> updates) {
        for (RecordUpdate update : updates) {
            add(update);
        }
        return this;
    }

    public RecordUpdateSetBuilder delete(RecordRef ref) {
        return delete(ref.getFormId(), ref.getRecordId());
    }

    public RecordUpdateSetBuilder delete(List<RecordRef> refList) {
        for (RecordRef ref : refList) {
            RecordUpdate update = new RecordUpdate();
            update.setFormId(ref.getFormId());
            update.setRecordId(ref.getRecordId());
            update.delete();
            updates.add(update);
        }
        return this;
    }

    public RecordUpdateSetBuilder delete(ResourceId formId, ResourceId id) {
        RecordUpdate update = new RecordUpdate();
        update.setFormId(formId);
        update.setRecordId(id);
        update.delete();
        add(update);
        return this;
    }

    public RecordUpdate update(ResourceId formId, ResourceId id) {
        RecordUpdate update = new RecordUpdate();
        update.setFormId(formId);
        update.setRecordId(id);
        updates.add(update);
        return update;
    }

    public RecordUpdateSet build() {
        RecordUpdateSet updateSet = new RecordUpdateSet();
        updateSet.changes = this.updates.toArray(new RecordUpdate[this.updates.size()]);
        return updateSet;
    }

    public JsonValue toJsonObject() {
        return toJson(build());
    }

}
