/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.search;

import com.google.gwt.safehtml.shared.SafeHtmlUtils;
import org.activityinfo.i18n.shared.I18N;
import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.database.DatabaseHeader;
import org.activityinfo.model.database.Resource;
import org.activityinfo.model.database.ResourceType;
import org.activityinfo.model.database.VisibleDatabaseTree;

@AutoJson
public class SearchResult {
    private final String surtitleHtml;
    private final String uri;
    private final String label;
    private final String type;

    public SearchResult(VisibleDatabaseTree database) {
        this.surtitleHtml = I18N.CONSTANTS.database();
        this.label = database.getLabel();
        this.uri = "/app#database/" + database.getDatabaseId();
        this.type = ResourceType.DATABASE.name().toLowerCase();
    }

    public SearchResult(DatabaseHeader database) {
        this.surtitleHtml = I18N.CONSTANTS.database();
        this.label = database.getLabel();
        this.uri = "/app#database/" + database.getDatabaseId();
        this.type = ResourceType.DATABASE.name().toLowerCase();
    }

    public SearchResult(VisibleDatabaseTree database, Resource resource) {
        this.surtitleHtml = surtitle(database, resource);
        this.label = resource.getLabel();
        this.type = resource.getType().name().toLowerCase();
        switch (resource.getType()) {
            default:
            case SUB_FORM:
            case FORM:
                this.uri = "/app#form/" + resource.getId() + "/table";
                break;
            case FOLDER:
                this.uri = "/app#database/" + database.getDatabaseId() + "/" + resource.getId();
                break;
        }
    }

    @Builder
    public SearchResult(String surtitleHtml, String uri, String label, String type) {
        this.surtitleHtml = surtitleHtml;
        this.uri = uri;
        this.label = label;
        this.type = type;
    }


    public String getSurtitleHtml() {
        return surtitleHtml;
    }

    public String getType() {
        return type;
    }

    public String getUri() {
        return uri;
    }

    public String getPath() {
        int hashStart = uri.indexOf('#');
        if(hashStart == -1) {
            return uri;
        } else {
            return uri.substring(0, hashStart);
        }
    }

    public String getFragment() {
        int hashStart = uri.indexOf('#');
        if(hashStart == -1) {
            return "";
        } else {
            return uri.substring(hashStart + 1);
        }
    }

    public String getLabel() {
        return label;
    }

    public static String surtitle(VisibleDatabaseTree tree, Resource resource) {
        switch (resource.getType()) {
            case DATABASE:
                return "<strong>" + SafeHtmlUtils.htmlEscape(I18N.CONSTANTS.database()) + "</strong>";
            case FORM:
                return I18N.MESSAGES.formDatabaseLabel(SafeHtmlUtils.htmlEscape(tree.getLabel()));
            case FOLDER:
                return I18N.MESSAGES.folderDatabaseLabel(SafeHtmlUtils.htmlEscape(tree.getLabel()));
        }
        return "";
    }

    @Override
    public String toString() {
        return label;
    }



    public static boolean matches(String loweredSearchString, VisibleDatabaseTree tree) {
        return tree.getLabel().toLowerCase().contains(loweredSearchString);
    }

    public static boolean matches(String loweredSearchString, Resource resource) {
        if(resource.getType() == ResourceType.FOLDER || resource.getType() == ResourceType.FORM) {
            return resource.getLabel().toLowerCase().contains(loweredSearchString);
        } else {
            return false;
        }
    }
}
