/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.search;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;

import java.util.Collections;
import java.util.List;

@AutoJson
public class SearchResults {

    public static final SearchResults COLLAPSED = new SearchResults("", Collections.emptyList(), false, false);

    public static final int MAX_RESULTS = 20;

    private final String searchString;

    /**
     * The list of results matching the search string
     */
    private final List<SearchResult> results;

    /**
     * True if the number of results exceeded the maximum and have been truncated.
     */
    private final boolean truncated;

    /**
     * True if these results are partial pending the full results.
     */
    private final boolean partial;

    @Builder
    public SearchResults(String searchString, List<SearchResult> results, boolean truncated, boolean partial) {
        this.searchString = searchString;
        this.results = results;
        this.truncated = truncated;
        this.partial = partial;
    }

    public String getSearchString() {
        return searchString;
    }

    public List<SearchResult> getMatching() {
        return results;
    }

    public boolean isEmpty() {
        return results.isEmpty();
    }

    public boolean isCollapsed() {
        return this == COLLAPSED;
    }

    public boolean isTruncated() {
        return truncated;
    }

    public boolean isPartial() {
        return partial;
    }

    public List<SearchResult> getResults() {
        return results;
    }
}
