/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.search;

import com.google.common.base.Strings;
import com.google.common.collect.Iterables;
import org.activityinfo.model.database.DatabaseHeader;
import org.activityinfo.model.database.Resource;
import org.activityinfo.model.database.VisibleDatabaseTree;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class Searcher {
    private final String searchString;
    private String loweredSearchString;
    private List<SearchResult> results = new ArrayList<>();
    private boolean truncated;
    private boolean partial;

    public Searcher(String searchString) {
        this.searchString = Strings.nullToEmpty(searchString).trim();
        this.loweredSearchString = this.searchString.toLowerCase();
    }

    public Searcher partial() {
        this.partial = true;
        return this;
    }

    public boolean isSearchStringEmpty() {
        return searchString.isEmpty();
    }

    public void visit(VisibleDatabaseTree tree) {
        if(SearchResult.matches(loweredSearchString, tree)) {
            results.add(new SearchResult(tree));
        }
        for (Resource resource : tree.getResources()) {
            if(SearchResult.matches(loweredSearchString, resource)) {
                results.add(new SearchResult(tree, resource));
            }
        }
    }

    public void visit(DatabaseHeader database) {
        results.add(new SearchResult(database));
    }

    public int size() {
        return results.size();
    }

    public boolean exceedsMaxResults() {
        if(results.size() > SearchResults.MAX_RESULTS) {
            truncated = true;
            return true;
        } else {
            return false;
        }
    }

    public SearchResults build() {
        return new SearchResults(searchString, results, truncated, partial);
    }

    public static SearchResults merge(SearchResults a, SearchResults b) {


        Set<String> uris = new HashSet<>();
        List<SearchResult> merged = new ArrayList<>();
        for (SearchResult result : Iterables.concat(a.getResults(), b.getResults())) {
            if(uris.add(result.getUri())) {
                merged.add(result);
            }
        }
        return new SearchResults(a.getSearchString(), merged,
            a.isTruncated() || b.isTruncated(),
            a.isPartial() || b.isPartial());
    }

    public void add(SearchResult searchResult) {
        results.add(searchResult);
    }
}

