package org.activityinfo.model.setup;

import java.lang.String;
import java.util.Optional;
import javax.annotation.Nonnull;

public final class CertificateConfigurationBuilder {
  private boolean httpsProxyEnabled;

  private String domain = "";

  private boolean licensed;

  private int httpPort;

  Optional<PendingCertificateOrder> pendingOrder = Optional.empty();

  Optional<CertificateStatus> current = Optional.empty();

  public CertificateConfigurationBuilder() {
    domain = "";
  }

  public CertificateConfigurationBuilder(CertificateConfiguration toCopy) {
    this.httpsProxyEnabled = toCopy.isHttpsProxyEnabled();
    this.domain = toCopy.getDomain();
    this.licensed = toCopy.isLicensed();
    this.httpPort = toCopy.getHttpPort();
    this.pendingOrder = toCopy.getPendingOrder();
    this.current = toCopy.getCurrent();
  }

  @Nonnull
  public CertificateConfigurationBuilder httpsProxyEnabled(boolean httpsProxyEnabled) {
    this.httpsProxyEnabled = httpsProxyEnabled;
    return this;
  }

  @Nonnull
  public CertificateConfigurationBuilder domain(@Nonnull String domain) {
    this.domain = domain;
    return this;
  }

  @Nonnull
  public CertificateConfigurationBuilder licensed(boolean licensed) {
    this.licensed = licensed;
    return this;
  }

  @Nonnull
  public CertificateConfigurationBuilder httpPort(int httpPort) {
    this.httpPort = httpPort;
    return this;
  }

  @Nonnull
  public CertificateConfigurationBuilder pendingOrder(@Nonnull PendingCertificateOrder value) {
    this.pendingOrder = Optional.of(value);
    return this;
  }

  @Nonnull
  public CertificateConfigurationBuilder pendingOrder(
      @Nonnull Optional<PendingCertificateOrder> value) {
    this.pendingOrder = value;
    return this;
  }

  @Nonnull
  public CertificateConfigurationBuilder current(@Nonnull CertificateStatus value) {
    this.current = Optional.of(value);
    return this;
  }

  @Nonnull
  public CertificateConfigurationBuilder current(@Nonnull Optional<CertificateStatus> value) {
    this.current = value;
    return this;
  }

  @Nonnull
  public CertificateConfiguration build() {
    assert this.domain != null : "domain is missing";
    return new CertificateConfiguration(httpsProxyEnabled,domain,licensed,httpPort,pendingOrder,current);
  }
}
