package org.activityinfo.model.setup;

import java.lang.String;
import javax.annotation.Nonnull;

public final class SmtpConfigurationBuilder {
  private boolean enabled;

  private String senderAddress = "";

  private String host = "";

  private int port;

  private String username = "";

  private String password = "";

  public SmtpConfigurationBuilder() {
    senderAddress = "";
    host = "";
    username = "";
    password = "";
  }

  public SmtpConfigurationBuilder(SmtpConfiguration toCopy) {
    this.enabled = toCopy.isEnabled();
    this.senderAddress = toCopy.getSenderAddress();
    this.host = toCopy.getHost();
    this.port = toCopy.getPort();
    this.username = toCopy.getUsername();
    this.password = toCopy.getPassword();
  }

  @Nonnull
  public SmtpConfigurationBuilder enabled(boolean enabled) {
    this.enabled = enabled;
    return this;
  }

  @Nonnull
  public SmtpConfigurationBuilder senderAddress(@Nonnull String senderAddress) {
    this.senderAddress = senderAddress;
    return this;
  }

  @Nonnull
  public SmtpConfigurationBuilder host(@Nonnull String host) {
    this.host = host;
    return this;
  }

  @Nonnull
  public SmtpConfigurationBuilder port(int port) {
    this.port = port;
    return this;
  }

  @Nonnull
  public SmtpConfigurationBuilder username(@Nonnull String username) {
    this.username = username;
    return this;
  }

  @Nonnull
  public SmtpConfigurationBuilder password(@Nonnull String password) {
    this.password = password;
    return this;
  }

  @Nonnull
  public SmtpConfiguration build() {
    assert this.senderAddress != null : "senderAddress is missing";
    assert this.host != null : "host is missing";
    assert this.username != null : "username is missing";
    assert this.password != null : "password is missing";
    return new SmtpConfiguration(enabled,senderAddress,host,port,username,password);
  }
}
