/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.sync;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.resource.ResourceId;

import java.util.Collections;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;

/**
 * The set of databases the user has requested to keep offline.
 */
@AutoJson
public class OfflineSet {

    public static final OfflineSet EMPTY = new OfflineSet(Collections.emptySet());

    private final Set<ResourceId> databaseIds;

    public OfflineSet(Set<ResourceId> databaseIds) {
        this.databaseIds = databaseIds;
    }

    public Set<ResourceId> getDatabaseIds() {
        return databaseIds;
    }

    public OfflineSet enableOffline(ResourceId databaseId, boolean offline) {
        HashSet<ResourceId> newSet = new HashSet<>(databaseIds);
        if(offline) {
            newSet.add(databaseId);
        } else {
            newSet.remove(databaseId);
        }
        return new OfflineSet(newSet);
    }

    public boolean isEmpty() {
        return databaseIds.isEmpty();
    }


    public boolean contains(ResourceId databaseId) {
        return databaseIds.contains(databaseId);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof OfflineSet)) {
            return false;
        }
        OfflineSet that = (OfflineSet) o;
        return databaseIds.equals(that.databaseIds);
    }

    @Override
    public int hashCode() {
        return Objects.hash(databaseIds);
    }

}
