/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.sync;

import jsinterop.annotations.JsOverlay;
import jsinterop.annotations.JsPackage;
import jsinterop.annotations.JsType;
import org.activityinfo.json.Json;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.form.FormRecord;
import org.activityinfo.model.type.RecordRef;

/**
 * The value of the IndexedDB record ObjectStore
 *
 */
@JsType(isNative = true, namespace = JsPackage.GLOBAL, name = "Object")
public final class RecordObject {

    @JsOverlay
    public static final String LAST_EDIT_TIME_NAME = "lastEditTime";

    @JsOverlay
    public static final String PARENT_RECORD_ID_NAME = "parentRecordId";

    @JsOverlay
    public static final String FIELDS_NAME = "fields";

    private double lastEditTime;
    private String parentRecordId;
    private JsonValue fields;


    @JsOverlay
    public static RecordObject from(FormRecord record) {
        RecordObject object = new RecordObject();
        object.lastEditTime = record.getLastEditTime();
        object.parentRecordId = record.getParentRecordId();
        object.fields = record.getFields();
        return object;
    }

    @JsOverlay
    public static RecordObject from(UpdatedRecord record) {
        RecordObject object = new RecordObject();
        object.lastEditTime = record.getLastEditTime();
        object.parentRecordId = record.getParentRecordId();
        object.fields = record.getFields();
        return object;
    }

    @JsOverlay
    public String getParentRecordId() {
        return parentRecordId;
    }

    @JsOverlay
    public void setParentRecordId(String parentRecordId) {
        this.parentRecordId = parentRecordId;
    }

    @JsOverlay
    public JsonValue getFields() {
        return fields;
    }

    @JsOverlay
    public void setFields(JsonValue fields) {
        this.fields = fields;
    }

    @JsOverlay
    public void setField(String fieldName, JsonValue value) {
        if(fields == null) {
            fields = Json.createObject();
        }
        fields.put(fieldName, value);
    }

    @JsOverlay
    public JsonValue getField(String fieldName) {
        if(fields == null) {
            return null;
        }
        return fields.get(fieldName);
    }

    @JsOverlay
    public double getLastEditTime() {
        return lastEditTime;
    }

    @JsOverlay
    public void setLastEditTime(double lastEditTime) {
        this.lastEditTime = lastEditTime;
    }

    @JsOverlay
    public FormRecord toFormRecord(RecordRef recordRef) {
        return new FormRecord(recordRef, parentRecordId, lastEditTime, fields);
    }

}
