/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.sync;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.resource.ResourceId;

import java.util.Optional;

@AutoJson
public class RecordSyncRequest {

    private final ResourceId formId;
    private final String localVersion;
    private final String version;
    private final Optional<String> cursor;

    @Builder
    public RecordSyncRequest(ResourceId formId, String localVersion, String version, Optional<String> cursor) {
        this.formId = formId;
        this.localVersion = localVersion;
        this.version = version;
        this.cursor = cursor;
    }

    public RecordSyncRequest(OfflineSnapshot current, UpdatedForm form) {
        this.formId = form.getFormId();
        this.localVersion = current.getLocalFormVersion(form.getFormId());
        this.version = form.getUpdatedVersion();
        this.cursor = Optional.empty();
    }

    public String getLocalVersion() {
        return localVersion;
    }

    public ResourceId getFormId() {
        return formId;
    }

    public String getVersion() {
        return version;
    }

    public Optional<String> getCursor() {
        return cursor;
    }

    public RecordSyncRequest withCursorPosition(String cursor) {
        return new RecordSyncRequest(formId, localVersion, version, Optional.of(cursor));
    }
}
