/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.sync;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.resource.ResourceId;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;


/**
 * Data structure which contains the deltas to update the local snapshot to a new version set.
 *
 */
@AutoJson
public class SyncDelta {

    private final List<UpdatedDatabase> updatedDatabases;
    private final List<UpdatedForm> updatedForms;
    private final List<ResourceId> removedDatabases;
    private final List<ResourceId> removedForms;

    public SyncDelta(
            List<UpdatedDatabase> updatedDatabases,
            List<UpdatedForm> updatedForms,
            List<ResourceId> removedDatabases,
            List<ResourceId> removedForms) {
        this.updatedDatabases = updatedDatabases;
        this.updatedForms = updatedForms;
        this.removedDatabases = removedDatabases;
        this.removedForms = removedForms;
    }

    public static SyncDelta empty() {
        return new SyncDelta(
                Collections.emptyList(),
                Collections.emptyList(),
                Collections.emptyList(),
                Collections.emptyList());
    }

    public List<UpdatedDatabase> getUpdatedDatabases() {
        return updatedDatabases;
    }

    public List<UpdatedForm> getUpdatedForms() {
        return updatedForms;
    }

    public List<ResourceId> getRemovedDatabases() {
        return removedDatabases;
    }

    public List<ResourceId> getRemovedForms() {
        return removedForms;
    }

    public boolean isEmpty() {
        return updatedDatabases.isEmpty() &&
               updatedForms.isEmpty() &&
               removedDatabases.isEmpty() &&
               removedForms.isEmpty();
    }

    @Override
    public String toString() {
        return new StringBuilder("Sync{")
            .append("\nupdatedDatabases=[")
            .append(updatedDatabases.stream().map(d -> d.getDatabaseId().asString()).collect(Collectors.joining(", ")))
                .append("], ")
            .append("\nupdatedForms=[")
                .append(updatedForms.stream().map(f -> f.getFormId() + "->" + f.getUpdatedVersion()).collect(Collectors.joining(", ")))
                .append("], ")
            .append("\nremovedDatabases=").append(removedDatabases).append(", ")
            .append("\nremovedForms=").append(removedForms).append("\n}")
            .toString();
    }
}
