/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.sync;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.database.VisibleDatabaseTree;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.promise.Maybe;

import java.util.Optional;

@AutoJson
public class UpdatedDatabase {
    private ResourceId databaseId;
    private String version;
    private Maybe.State state;
    private Optional<VisibleDatabaseTree> tree;

    @Builder
    public UpdatedDatabase(ResourceId databaseId, String version, Maybe.State state, Optional<VisibleDatabaseTree> tree) {
        this.databaseId = databaseId;
        this.version = version;
        this.state = state;
        this.tree = tree;
    }

    public UpdatedDatabase(ResourceId databaseId, String latestVersion, Maybe<VisibleDatabaseTree> latestTree) {
        this.databaseId = databaseId;
        this.version = latestVersion;
        this.state = latestTree.getState();
        this.tree = latestTree.getIfVisible();
    }

    public UpdatedDatabase(VisibleDatabaseTree databaseMeta) {
        this(databaseMeta.getDatabaseId(), databaseMeta.getVersion(), Maybe.of(databaseMeta));
    }

    public ResourceId getDatabaseId() {
        return databaseId;
    }

    public String getVersion() {
        return version;
    }

    public Maybe.State getState() {
        return state;
    }

    public Optional<VisibleDatabaseTree> getTree() {
        return tree;
    }

    public Maybe<VisibleDatabaseTree> toMaybeTree() {
        switch (state) {
            case NOT_FOUND:
                return Maybe.notFound();
            case FORBIDDEN:
                return Maybe.forbidden();
            case DELETED:
                return Maybe.deleted();
            default:
            case VISIBLE:
                return Maybe.fromOptional(tree);
        }
    }
}
