/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.sync;

import org.activityinfo.json.AutoJson;
import org.activityinfo.model.annotation.AutoBuilder;
import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.form.FormClass;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.promise.Maybe;

import java.util.Optional;

@AutoJson
@AutoBuilder
public class UpdatedForm {
    private final ResourceId formId;
    private final String updatedVersion;
    private final Optional<ResourceId> databaseId;
    private final Maybe.State state;
    private final Optional<FormClass> updatedSchema;
    private final boolean syncable;

    @Builder
    public UpdatedForm(ResourceId formId, String updatedVersion, Optional<ResourceId> databaseId, Maybe.State state, Optional<FormClass> updatedSchema, boolean syncable) {
        this.formId = formId;
        this.updatedVersion = updatedVersion;
        this.databaseId = databaseId;
        this.state = state;
        this.updatedSchema = updatedSchema;
        this.syncable = syncable;
    }

    public ResourceId getFormId() {
        return formId;
    }

    public Optional<ResourceId> getDatabaseId() {
        return databaseId;
    }

    public Maybe.State getState() {
        return state;
    }

    public String getUpdatedVersion() {
        return updatedVersion;
    }

    public Optional<FormClass> getUpdatedSchema() {
        return updatedSchema;
    }

    public boolean isSchemaUpdated() {
        return updatedSchema.isPresent();
    }

    public boolean isSyncable() {
        return syncable;
    }
}
