package org.activityinfo.model.sync;

import java.lang.String;
import java.util.Optional;
import javax.annotation.Nonnull;
import org.activityinfo.model.form.FormClass;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.promise.Maybe;

public final class UpdatedFormBuilder {
  private ResourceId formId;

  private String updatedVersion = "";

  Optional<ResourceId> databaseId = Optional.empty();

  private Maybe.State state = Maybe.State.VISIBLE;

  Optional<FormClass> updatedSchema = Optional.empty();

  private boolean syncable;

  public UpdatedFormBuilder() {
    updatedVersion = "";
    state = Maybe.State.VISIBLE;
  }

  public UpdatedFormBuilder(UpdatedForm toCopy) {
    this.formId = toCopy.getFormId();
    this.updatedVersion = toCopy.getUpdatedVersion();
    this.databaseId = toCopy.getDatabaseId();
    this.state = toCopy.getState();
    this.updatedSchema = toCopy.getUpdatedSchema();
    this.syncable = toCopy.isSyncable();
  }

  @Nonnull
  public UpdatedFormBuilder formId(@Nonnull ResourceId formId) {
    this.formId = formId;
    return this;
  }

  @Nonnull
  public UpdatedFormBuilder updatedVersion(@Nonnull String updatedVersion) {
    this.updatedVersion = updatedVersion;
    return this;
  }

  @Nonnull
  public UpdatedFormBuilder databaseId(@Nonnull ResourceId value) {
    this.databaseId = Optional.of(value);
    return this;
  }

  @Nonnull
  public UpdatedFormBuilder databaseId(@Nonnull Optional<ResourceId> value) {
    this.databaseId = value;
    return this;
  }

  @Nonnull
  public UpdatedFormBuilder state(@Nonnull Maybe.State state) {
    this.state = state;
    return this;
  }

  @Nonnull
  public UpdatedFormBuilder updatedSchema(@Nonnull FormClass value) {
    this.updatedSchema = Optional.of(value);
    return this;
  }

  @Nonnull
  public UpdatedFormBuilder updatedSchema(@Nonnull Optional<FormClass> value) {
    this.updatedSchema = value;
    return this;
  }

  @Nonnull
  public UpdatedFormBuilder syncable(boolean syncable) {
    this.syncable = syncable;
    return this;
  }

  @Nonnull
  public UpdatedForm build() {
    assert this.formId != null : "formId is missing";
    assert this.updatedVersion != null : "updatedVersion is missing";
    assert this.state != null : "state is missing";
    return new UpdatedForm(formId,updatedVersion,databaseId,state,updatedSchema,syncable);
  }
}
