package org.activityinfo.model.sync;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonArrays;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.form.FormClass;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.promise.Maybe;

public final class UpdatedFormJson {
  public static JsonValue toJson(UpdatedForm model) {
    JsonValue object = Json.createObject();
    object.put("formId", model.getFormId().asString());
    object.put("updatedVersion", model.getUpdatedVersion());
    object.put("databaseId", JsonArrays.optionalToJson(model.getDatabaseId(), ResourceId::toJson));
    object.put("state", model.getState().name());
    object.put("updatedSchema", JsonArrays.optionalToJson(model.getUpdatedSchema(), FormClass::toJson));
    object.put("syncable", model.isSyncable());
    return object;
  }

  public static UpdatedForm fromJson(JsonValue object) {
    return new UpdatedForm(ResourceId.valueOf(object.getString("formId")),
        object.getString("updatedVersion"),
        JsonArrays.optionalFromJson(object.get("databaseId"), ResourceId::fromJson),
        JsonArrays.enumFromJson(object.get("state"), Maybe.State::valueOf, Maybe.State.VISIBLE),
        JsonArrays.optionalFromJson(object.get("updatedSchema"), FormClass::fromJson),
        object.getBoolean("syncable"));
  }
}
