/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.sync;

import jsinterop.annotations.JsOverlay;
import jsinterop.annotations.JsPackage;
import jsinterop.annotations.JsType;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.form.FormRecord;


@JsType(isNative = true, name = "Object", namespace = JsPackage.GLOBAL)
public final class UpdatedRecord {

    private double lastEditTime;
    private String recordId;
    private String parentRecordId;
    private JsonValue fields;

    public UpdatedRecord() {
    }

    @JsOverlay
    public static UpdatedRecord create(String recordId, double lastEditTime, String parentRecordId, JsonValue fieldValues) {
        UpdatedRecord update = new UpdatedRecord();
        update.recordId = recordId;
        update.lastEditTime = lastEditTime;
        update.parentRecordId = parentRecordId;
        update.fields = fieldValues;
        return update;
    }

    @JsOverlay
    public static UpdatedRecord create(FormRecord record) {
        UpdatedRecord update = new UpdatedRecord();
        update.lastEditTime = record.getLastEditTime();
        update.parentRecordId = record.getParentRecordId();
        update.recordId = record.getRecordId();
        update.fields = record.getFields();
        return update;
    }

    @JsOverlay
    public String getRecordId() {
        return recordId;
    }

    @JsOverlay
    public String getParentRecordId() {
        return parentRecordId;
    }

    @JsOverlay
    public JsonValue getFields() {
        return fields;
    }

    @JsOverlay
    public double getLastEditTime() {
        return lastEditTime;
    }
}
