/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.template;

import org.activityinfo.model.database.*;
import org.activityinfo.model.permission.Operation;
import org.activityinfo.model.resource.ResourceId;

import java.util.Collections;


public class BlankDatabase extends DatabaseTemplate {
    private final ResourceId databaseId;

    public BlankDatabase(ResourceId databaseId) {
        this.databaseId = databaseId;
    }

    public UpdateDatabaseRequest build() {

        UpdateDatabaseRequestBuilder req = new UpdateDatabaseRequestBuilder();
        RoleBuilder admBuilder = new RoleBuilder()
                .id(ResourceId.valueOf("admin"))
                .label("Administrator");
        for (Operation operation : Operation.values()) {
            if (operation == Operation.EDIT_RECORD) {
                admBuilder.addPermission(new FilteredPermission(Operation.EDIT_RECORD, Collections.singleton(SecurityCategory.REVIEWER.getId())));
            } else {
                admBuilder.addPermission(new FilteredPermission(operation));
            }
        }

        req.addRoleUpdate(admBuilder.build());

        req.addRoleUpdate(new RoleBuilder()
            .id(ResourceId.valueOf("readonly"))
            .label("Read only")
            .addPermission(new FilteredPermission(Operation.VIEW))
            .build());

        req.addRoleUpdate(new RoleBuilder()
            .id(ResourceId.valueOf("dataentry"))
            .label("Data Entry")
            .addPermission(new FilteredPermission(Operation.VIEW))
            .addPermission(new FilteredPermission(Operation.EDIT_RECORD))
            .addPermission(new FilteredPermission(Operation.ADD_RECORD))
            .addPermission(new FilteredPermission(Operation.DELETE_RECORD))
            .addPermission(new FilteredPermission(Operation.EXPORT_RECORDS))
            .build());


        return req.build();
    }


}
