/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type;

import org.activityinfo.json.JsonValue;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.ColumnViews;

/**
 * Value type that represents a FieldType containing paragraph-like text.
 *
 */
public class NarrativeType implements FieldType {


    public static final NarrativeType INSTANCE = new NarrativeType();

    public static final FieldTypeClass TYPE_CLASS = new RecordFieldTypeClass() {
        @Override
        public String getId() {
            return "NARRATIVE";
        }

        @Override
        public FieldType createType() {
            return INSTANCE;
        }

    };

    private NarrativeType() {
    }

    @Override
    public FieldTypeClass getTypeClass() {
        return TYPE_CLASS;
    }

    @Override
    public FieldValue parseJsonValue(JsonValue value) {
        return NarrativeValue.valueOf(value.asString());
    }

    @Override
    public <T> T accept(FieldTypeVisitor<T> visitor) {
        return visitor.visitNarrative(this);
    }

    @Override
    public boolean isUpdatable() {
        return true;
    }

    @Override
    public FormulaType getFormulaType() {
        return FormulaType.STRING;
    }

    @Override
    public ColumnView toFormulaValue(FieldValue fieldValue) {
        if(fieldValue instanceof NarrativeValue) {
            return ColumnViews.of(((NarrativeValue) fieldValue).asString());
        } else {
            return ColumnViews.BLANK_STRING;
        }
    }

    /**
     *
     * @return the singleton instance for this type
     */
    private Object readResolve() {
        return INSTANCE;
    }


}
