/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonSerializable;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.resource.ResourceId;

import javax.annotation.Nonnull;

/**
 * Reference to a Record that includes the id of the Form to which the record belongs and
 * as well as the Record's id.
 */
public class RecordRef implements JsonSerializable {

    public static final char SEPARATOR = ':';

    @Nonnull
    private ResourceId formId;

    @Nonnull
    private ResourceId recordId;

    public RecordRef(ResourceId formId, ResourceId recordId) {
        assert formId != null : "formId is null";
        assert recordId != null : "recordId is null";
        this.formId = formId;
        this.recordId = recordId;
    }

    public RecordRef(ResourceId formId, String string) {
        this(formId, ResourceId.valueOf(string));
    }

    public ResourceId getFormId() {
        return formId;
    }

    public ResourceId getRecordId() {
        return recordId;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        RecordRef recordRef = (RecordRef) o;

        if (!formId.equals(recordRef.formId)) return false;
        return recordId.equals(recordRef.recordId);

    }

    public String toQualifiedString() {
        return formId.asString() + SEPARATOR + recordId.asString();
    }

    @Override
    public JsonValue toJson() {
        return Json.create(toQualifiedString());
    }

    public static RecordRef fromJson(JsonValue value) {
        return fromQualifiedString(value.asString());
    }

    public static RecordRef fromQualifiedString(@Nonnull  String string) {
        int separatorPos = string.indexOf(SEPARATOR);
        if(separatorPos == -1) {
            throw new IllegalStateException("Malformed record ref: " + string);
        }
        ResourceId formId = ResourceId.valueOf(string.substring(0, separatorPos));
        ResourceId recordId = ResourceId.valueOf(string.substring(separatorPos+1));

        return new RecordRef(formId, recordId);
    }

    @Override
    public int hashCode() {
        int result = formId.hashCode();
        result = 31 * result + recordId.hashCode();
        return result;
    }

    @Override
    public String toString() {
        return "RecordRef<" + formId.asString() + ":" + recordId.asString() + ">";
    }

}
