/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type;

import com.google.common.base.Strings;
import org.activityinfo.json.Json;
import org.activityinfo.json.JsonValue;

import static org.activityinfo.json.Json.createObject;

/**
 * A sequential file number, generated by the server when first saved.
 */
public class SerialNumber implements FieldValue {

    private final String prefix;
    private int number;

    public SerialNumber(int serialNumber) {
        number = serialNumber;
        prefix = null;
    }

    public SerialNumber(String prefix, int number) {
        this.prefix = prefix;
        this.number = number;
    }


    @Override
    public FieldTypeClass getTypeClass() {
        return SerialNumberType.TYPE_CLASS;
    }

    @Override
    public JsonValue toJson() {
        if(prefix == null) {
            return Json.create(number);
        } else {
            JsonValue object = createObject();
            object.put("prefix", prefix);
            object.put("number", number);
            return object;
        }
    }

    public String getPrefix() {
        return prefix;
    }

    public int getNumber() {
        return number;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        SerialNumber that = (SerialNumber) o;

        if (number != that.number) return false;
        return prefix != null ? prefix.equals(that.prefix) : that.prefix == null;

    }

    @Override
    public int hashCode() {
        int result = prefix != null ? prefix.hashCode() : 0;
        result = 31 * result + number;
        return result;
    }

    @Override
    public String toString() {
        return "SerialNumber{" +
                "prefix='" + prefix + '\'' +
                ", number=" + number +
                '}';
    }

    public static SerialNumber parse(String string) {
        int numberStart = string.lastIndexOf("-");
        if(numberStart == -1) {
            return new SerialNumber(Integer.parseInt(string));
        } else {
            return new SerialNumber(string.substring(0, numberStart), Integer.parseInt(string.substring(numberStart + 1)));
        }
    }

    public boolean hasPrefix() {
        return !Strings.isNullOrEmpty(prefix);
    }
}
