/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type.attachment;

import com.google.common.collect.Lists;
import org.activityinfo.json.Json;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.type.FieldTypeClass;
import org.activityinfo.model.type.FieldValue;

import java.util.ArrayList;
import java.util.List;

/**
 * @author yuriyz on 8/6/14.
 */
public class AttachmentValue implements FieldValue {

    private final List<Attachment> values = Lists.newArrayList();

    @Override
    public FieldTypeClass getTypeClass() {
        return AttachmentType.TYPE_CLASS;
    }

    public AttachmentValue() {
    }

    public AttachmentValue(Attachment imageRowValue) {
        values.add(imageRowValue);
    }

    public AttachmentValue(List<Attachment> list) {
        values.addAll(list);
    }

    public List<Attachment> getValues() {
        return values;
    }

    public boolean hasValues() {
        return !values.isEmpty();
    }

    @Override
    public JsonValue toJson() {
        JsonValue array = Json.createArray();
        for (Attachment value : values) {
            array.add(value.toJsonElement());
        }
        return array;
    }

    public static AttachmentValue fromJson(String json) {
        return fromJson(Json.parse(json));
    }

    public static AttachmentValue fromJson(JsonValue root) {
        JsonValue array;
        if(root.isJsonObject()) {
            array = root.get("values");
        } else {
            array = root;
        }

        List<Attachment> list = new ArrayList<>();
        for (JsonValue value : array.values()) {
            list.add(Attachment.fromJson(value));
        }

        return new AttachmentValue(list);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        AttachmentValue that = (AttachmentValue) o;

        return !(values != null ? !values.equals(that.values) : that.values != null);

    }

    @Override
    public int hashCode() {
        return values != null ? values.hashCode() : 0;
    }

    public static AttachmentValue fromJsonSilently(String json) {
        try {
            return fromJson(json);
        } catch (Exception e) {
            return null;
        }
    }


}
