/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type.expr;

import org.activityinfo.json.JsonValue;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.type.*;

import static org.activityinfo.json.Json.createObject;

/**
 * A Value Type that represents a value calculated from a symbolic expression,
 * such as "A + B"
 */
public class CalculatedFieldType implements ParametrizedFieldType {

    public static final ParametrizedFieldTypeClass TYPE_CLASS = new ParametrizedFieldTypeClass() {
        @Override
        public String getId() {
            return "calculated";
        }

        @Override
        public FieldType createType() {
            return new CalculatedFieldType();
        }

        @Override
        public FieldType deserializeType(JsonValue parametersObject) {
            JsonValue exprElement = parametersObject.get("formula");
            if(exprElement.isJsonNull()) {
                exprElement = parametersObject.get("expression");
            }
            if(exprElement.isJsonNull()) {
                return new CalculatedFieldType();
            } else if(exprElement.isJsonObject()) {
                return new CalculatedFieldType(exprElement.getString("value"));
            } else {
                return new CalculatedFieldType(exprElement.asString());
            }
        }
    };


    private String formula;

    public CalculatedFieldType() {
    }

    public CalculatedFieldType(String formula) {
        this.formula = formula;
    }

    public String getFormula() {
        return formula;
    }

    public void setFormula(String formula) {
        this.formula = formula;
    }

    public CalculatedFieldType withFormula(String formula) {
        return new CalculatedFieldType(formula);
    }

    @Override
    public ParametrizedFieldTypeClass getTypeClass() {
        return TYPE_CLASS;
    }

    @Override
    public FieldValue parseJsonValue(JsonValue value) {
        throw new UnsupportedOperationException();
    }

    @Override
    public <T> T accept(FieldTypeVisitor<T> visitor) {
        return visitor.visitCalculated(this);
    }

    @Override
    public boolean isUpdatable() {
        return false;
    }

    @Override
    public FormulaType getFormulaType() {
        // Not correct: the value of formula types depend
        // their formula.
        return FormulaType.OPAQUE;
    }

    @Override
    public ColumnView toFormulaValue(FieldValue fieldValue) {
        throw new UnsupportedOperationException();
    }

    @Override
    public JsonValue getParametersAsJson() {
        JsonValue object = createObject();
        if (formula != null) {
            object.put("formula", formula);
        }
        return object;
    }

    @Override
    public boolean isValid() {
        return true;
    }

}
