/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type.geo;

public class ExtentsBuilder {
    private double minLat = Extents.LAT_MAX;
    private double maxLat = Extents.LAT_MIN;
    private double minLon = Extents.LNG_MAX;
    private double maxLon = Extents.LNG_MIN;

    public ExtentsBuilder() {
    }

    public ExtentsBuilder grow(double lat, double lon) {
        minLat = Math.min(minLat, lat);
        maxLat = Math.max(maxLat, lat);
        minLon = Math.min(minLon, lon);
        maxLon = Math.max(maxLon, lon);
        return this;
    }

    public Extents build() {
        return new Extents(minLat, maxLat, minLon, maxLon);
    }

    public ExtentsBuilder buffer(int percent) {
        double bx;
        double by;
        if(minLon == maxLon) {
            // If there is only point, avoid infinite zoom
            by = 1;
        } else {
            by = Math.max(0, (maxLon - minLon) * (percent / 100d));
        }
        if(minLat == maxLat) {
            // If there is only point, avoid infinite zoom
            bx = 2;
        } else {
            bx = Math.max(0, (maxLon - minLon) * (percent / 100d));
        }

        this.maxLon += bx;
        this.minLon -= bx;
        this.maxLat = Math.min(maxLat + by, 90);
        this.minLat = Math.max(minLat - by, -90);
        return this;
    }
}
