/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type.geo;

import org.activityinfo.json.JsonValue;
import org.activityinfo.model.type.FieldTypeClass;

import java.util.Objects;

import static org.activityinfo.json.Json.createObject;

/* A Field Value describing a geographic area on the Earth's surface
* in the WGS84 geographic reference system.
*/
public class GeoArea implements GeoFieldValue {

    private Extents envelope;

    public GeoArea(Extents envelope, String blobId) {
        this.envelope = envelope;
    }

    public GeoArea(Extents envelope) {
        this.envelope = envelope;
    }


    public Extents getEnvelope() {
        return envelope;
    }

    @Override
    public FieldTypeClass getTypeClass() {
        return GeoAreaType.TYPE_CLASS;
    }


    @Override
    public JsonValue toJson() {
        JsonValue object = createObject();
        object.put("bbox", envelope.toJsonElement());
        return object;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof GeoArea)) {
            return false;
        }
        GeoArea geoArea = (GeoArea) o;
        return Objects.equals(envelope, geoArea.envelope);
    }

    @Override
    public int hashCode() {
        return Objects.hash(envelope);
    }
}
