/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type.geo;

import org.activityinfo.json.JsonValue;
import org.activityinfo.model.type.FieldTypeClass;
import org.activityinfo.model.type.RecordFieldValue;

import static org.activityinfo.json.Json.createObject;

/**
 * A Field Value containing a geographic point in the WGS84 geographic
 * reference system.
 */
public class GeoPoint implements GeoFieldValue, RecordFieldValue {

    private double latitude;
    private double longitude;

    public GeoPoint(double latitude, double longitude) {
        this.latitude = latitude;
        this.longitude = longitude;
    }

    public double getLatitude() {
        return latitude;
    }

    public double getLongitude() {
        return longitude;
    }

    @Override
    public FieldTypeClass getTypeClass() {
        return GeoPointType.TYPE_CLASS;
    }

    @Override
    public JsonValue toJson() {
        JsonValue object = createObject();
        object.put("latitude", latitude);
        object.put("longitude", longitude);
        return object;
    }

    public static GeoPoint fromJson(JsonValue jsonValue) {
        return GeoPointType.INSTANCE.parseJsonValue(jsonValue);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        GeoPoint geoPoint = (GeoPoint) o;

        if (Double.compare(geoPoint.latitude, latitude) != 0) return false;
        if (Double.compare(geoPoint.longitude, longitude) != 0) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result;
        long temp;
        temp = Double.doubleToLongBits(latitude);
        result = (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(longitude);
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        return result;
    }

    @Override
    public String toString() {
        return "GeoPoint[" + latitude + "," + longitude + "]";
    }

    @Override
    public Extents getEnvelope() {
        return Extents.fromLatLng(latitude, longitude);
    }

}
