/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type.geo;

import org.activityinfo.json.JsonValue;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.ColumnViews;
import org.activityinfo.model.type.*;

/**
 * A value type describing a point within the WGS84 Geographic Reference System.
 */
public class GeoPointType implements FieldType {

    public static final String LATITUDE = "latitude";
    public static final String LONGITUDE = "longitude";

    public static final String TYPE_ID = "geopoint";

    public static final GeoPointType INSTANCE = new GeoPointType();

    public static final FieldTypeClass TYPE_CLASS = new RecordFieldTypeClass() {
        @Override
        public String getId() {
            return TYPE_ID;
        }

        @Override
        public FieldType createType() {
            return INSTANCE;
        }

    };

    private GeoPointType() {  }

    @Override
    public FieldTypeClass getTypeClass() {
        return TYPE_CLASS;
    }

    @Override
    public GeoPoint parseJsonValue(JsonValue value) {
        if(value.isJsonArray()) {
            return new GeoPoint(value.getNumber(0), value.getNumber(1));
        } else {
            return new GeoPoint(
                    value.getNumber("latitude"),
                    value.getNumber("longitude"));
        }
    }

    @Override
    public <T> T accept(FieldTypeVisitor<T> visitor) {
        return visitor.visitGeoPoint(this);
    }

    @Override
    public boolean isUpdatable() {
        return true;
    }

    @Override
    public FormulaType getFormulaType() {
        return FormulaType.GEOMETRY;
    }

    @Override
    public ColumnView toFormulaValue(FieldValue fieldValue) {
        if(fieldValue instanceof GeoPoint) {
            return ColumnViews.TRUE;
        } else {
            return ColumnViews.BLANK_BOOLEAN;
        }
    }

    /**
     *
     * @return the singleton instance for this type
     */
    private Object readResolve() {
        return INSTANCE;
    }

}
