/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type.time;

import org.activityinfo.json.JsonValue;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.ColumnViews;
import org.activityinfo.model.type.*;

public class FortnightType implements PeriodType {


    public static final String TYPE_ID = "fortnight";

    public static final FieldTypeClass TYPE_CLASS = new RecordFieldTypeClass() {
        @Override
        public String getId() {
            return TYPE_ID;
        }

        @Override
        public FieldType createType() {
            return INSTANCE;
        }
    };

    public static final FortnightType INSTANCE = new FortnightType();

    private FortnightType() {
    }

    @Override
    public FieldTypeClass getTypeClass() {
        return TYPE_CLASS;
    }

    @Override
    public FortnightValue parseJsonValue(JsonValue value) {
        EpiWeek startWeek = EpiWeekType.INSTANCE.parseJsonValue(value);
        return new FortnightValue(startWeek);
    }

    @Override
    public <T> T accept(FieldTypeVisitor<T> visitor) {
        return visitor.visitFortnight(this);
    }

    @Override
    public boolean isUpdatable() {
        return true;
    }

    @Override
    public FormulaType getFormulaType() {
        return FormulaType.STRING;
    }

    @Override
    public ColumnView toFormulaValue(FieldValue fieldValue) {
        if (fieldValue instanceof FortnightValue) {
            return ColumnViews.of(((FortnightValue) fieldValue).toString());
        } else {
            return ColumnViews.BLANK_STRING;
        }
    }

    @Override
    public PeriodValue parseString(String string) {
        int dashIndex = string.indexOf("-");
        if(dashIndex != -1) {
            string = string.substring(0, dashIndex);
        }
        EpiWeek week = EpiWeek.parse(string);
        if(week.getWeekInYear() % 2 == 0) {
            return new FortnightValue(week.previous());
        } else {
            return new FortnightValue(week);
        }
    }

    @Override
    public FortnightValue fromSubFormKey(RecordRef ref) {
        // s0417565614-2017W2-3

        String subRecordId = ref.getRecordId().asString();
        int endWeekDelimiter = subRecordId.lastIndexOf('-');
        int delimiter = subRecordId.lastIndexOf('-', endWeekDelimiter-1);

        EpiWeek firstWeek = EpiWeek.parse(subRecordId.substring(delimiter+1, endWeekDelimiter));
        return new FortnightValue(firstWeek);
    }

    @Override
    public PeriodValue containingDate(LocalDate localDate) {
        EpiWeek week = EpiWeek.weekOf(localDate);
        if(week.getWeekInYear() % 2 == 0) {
            return new FortnightValue(week.previous());
        } else {
            return new FortnightValue(week);
        }
    }

}
