/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type.time;

import org.activityinfo.json.JsonValue;
import org.activityinfo.model.type.FieldTypeClass;

public class FortnightValue implements PeriodValue {

    private final EpiWeek startWeek;

    public FortnightValue(EpiWeek startWeek) {
        this.startWeek = startWeek;
    }

    public FortnightValue(int year, int weekNum) {
        this(EpiWeek.of(year, weekNum));
    }

    @Override
    public FieldTypeClass getTypeClass() {
        return FortnightType.TYPE_CLASS;
    }

    @Override
    public JsonValue toJson() {
        return startWeek.toJson();
    }

    public int getYear() {
        return startWeek.getYear();
    }

    public Month toMonth(){
        return startWeek.toMonth();
    }

    @Override
    public LocalDateInterval asInterval() {
        LocalDate startDate = startWeek.asInterval().getStartDate();
        LocalDate endDate = startDate.plusDays(13);

        return new LocalDateInterval(startDate, endDate);
    }

    @Override
    public PeriodValue previous() {
        return new FortnightValue(startWeek.plus(-2));
    }

    @Override
    public PeriodValue next() {
        return new FortnightValue(startWeek.plus(+2));
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        FortnightValue that = (FortnightValue) o;

        return startWeek.equals(that.startWeek);

    }

    @Override
    public int hashCode() {
        return startWeek.hashCode();
    }

    @Override
    public String toString() {
        return startWeek.toString();
    }

    public static String toString(int year, int startWeekNumber) {
        return EpiWeek.toString(year, startWeekNumber);
    }

    public int getWeekInYear() {
        return startWeek.getWeekInYear();
    }

}
