/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type.time;

import org.activityinfo.json.JsonValue;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.ColumnViews;
import org.activityinfo.model.type.FieldType;
import org.activityinfo.model.type.FieldTypeClass;
import org.activityinfo.model.type.FieldTypeVisitor;
import org.activityinfo.model.type.FieldValue;

/**
 * Value type that represents a continuous interval between two {@link org.activityinfo.model.type.time.LocalDate}s,
 * starting on {@code startDate}, inclusive, and ending on {@code endDate}, inclusive.
 */
public class LocalDateIntervalType implements FieldType {

    public static final FieldTypeClass TYPE_CLASS = new FieldTypeClass() {
        @Override
        public String getId() {
            return "localDateInterval";
        }

        @Override
        public FieldType createType() {
            return LocalDateIntervalType.INSTANCE;
        }
    };

    public static final LocalDateIntervalType INSTANCE = new LocalDateIntervalType();

    private LocalDateIntervalType() {
    }

    @Override
    public FieldTypeClass getTypeClass() {
        return TYPE_CLASS;
    }

    @Override
    public FieldValue parseJsonValue(JsonValue value) {
        return LocalDateInterval.fromJson(value);
    }

    @Override
    public <T> T accept(FieldTypeVisitor<T> visitor) {
        return visitor.visitLocalDateInterval(this);
    }

    @Override
    public boolean isUpdatable() {
        return true;
    }

    @Override
    public FormulaType getFormulaType() {
        throw new UnsupportedOperationException("TODO");
    }

    @Override
    public ColumnView toFormulaValue(FieldValue fieldValue) {
        if(fieldValue instanceof LocalDateInterval) {
            return ColumnViews.of(((LocalDateInterval) fieldValue).getStartDate());
        } else {
            return ColumnViews.BLANK_DATE;
        }
    }

    /**
     *
     * @return the singleton instance for this type
     */
    private Object readResolve() {
        return INSTANCE;
    }


}
