/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type.time;

import org.activityinfo.json.JsonValue;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.ColumnViews;
import org.activityinfo.model.type.*;

/**
 * Value type that represents a calendar month in the ISO-8601 calendar.
 * There is no representation of time-of-day or time-zone.
 */
public class MonthType implements FieldType, PeriodType {


    public interface TypeClass extends SingletonTypeClass, RecordFieldTypeClass {
    }

    public static final TypeClass TYPE_CLASS = new TypeClass() {

        @Override
        public String getId() {
            return "month";
        }

        @Override
        public FieldType createType() {
            return INSTANCE;
        }
    };

    public static final MonthType INSTANCE = new MonthType();

    private MonthType() {}

    @Override
    public FieldTypeClass getTypeClass() {
        return TYPE_CLASS;
    }

    @Override
    public FieldValue parseJsonValue(JsonValue value) {
        return Month.parseMonth(value.asString());
    }

    @Override
    public <T> T accept(FieldTypeVisitor<T> visitor) {
        return visitor.visitMonth(this);
    }

    @Override
    public boolean isUpdatable() {
        return true;
    }

    @Override
    public FormulaType getFormulaType() {
        return FormulaType.STRING;
    }

    @Override
    public ColumnView toFormulaValue(FieldValue fieldValue) {
        if(fieldValue instanceof Month) {
            return ColumnViews.of(((Month) fieldValue).toString());
        } else {
            return ColumnViews.BLANK_STRING;
        }
    }


    @Override
    public PeriodValue parseString(String string) {
        return Month.parseMonth(string);
    }

    @Override
    public PeriodValue fromSubFormKey(RecordRef ref) {
        String recordId = ref.getRecordId().asString();
        int monthSepIndex = recordId.lastIndexOf('-');
        int yearSepIndex = recordId.lastIndexOf('-', monthSepIndex - 1);
        String monthKey = recordId.substring(yearSepIndex + 1);
        return Month.parseMonth(monthKey);
    }

    @Override
    public PeriodValue containingDate(LocalDate localDate) {
        return new Month(localDate.getYear(), localDate.getMonthOfYear());
    }
}