/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type.time;

import org.activityinfo.json.JsonValue;
import org.activityinfo.model.formula.FormulaType;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.ColumnViews;
import org.activityinfo.model.type.*;


/**
 * Value type that represents a calendar year in the ISO-8601 calendar.
 * There is no representation of time-of-day or time-zone.
 */
public class YearType implements FieldType, PeriodType {


    public interface TypeClass extends SingletonTypeClass, RecordFieldTypeClass {
    }

    public static final TypeClass TYPE_CLASS = new TypeClass() {

        @Override
        public String getId() {
            return "year";
        }

        @Override
        public FieldType createType() {
            return INSTANCE;
        }
    };

    public static final YearType INSTANCE = new YearType();

    private YearType() {}

    @Override
    public FieldTypeClass getTypeClass() {
        return TYPE_CLASS;
    }

    @Override
    public FieldValue parseJsonValue(JsonValue value) {
        return new YearValue(value.asInt());
    }

    @Override
    public <T> T accept(FieldTypeVisitor<T> visitor) {
        return visitor.visitYear(this);
    }

    @Override
    public boolean isUpdatable() {
        return true;
    }

    @Override
    public FormulaType getFormulaType() {
        return FormulaType.NUMBER;
    }

    @Override
    public ColumnView toFormulaValue(FieldValue fieldValue) {
        if(fieldValue instanceof YearValue) {
            return ColumnViews.of(((YearValue) fieldValue).getYear());
        } else {
            return ColumnViews.BLANK_NUMBER;
        }
    }

    @Override
    public PeriodValue parseString(String string) {
        return new YearValue(Integer.parseInt(string));
    }

    @Override
    public PeriodValue fromSubFormKey(RecordRef ref) {
        String subRecordId = ref.getRecordId().asString();
        String yearKey = subRecordId.substring(subRecordId.length() - 4);
        return new YearValue(Integer.parseInt(yearKey));
    }

    @Override
    public PeriodValue containingDate(LocalDate localDate) {
        return new YearValue(localDate.getYear());
    }
}