/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.model.type.time;

import org.activityinfo.json.Json;
import org.activityinfo.json.JsonValue;
import org.activityinfo.model.type.FieldTypeClass;
import org.activityinfo.model.type.FieldValue;

/**
 * Represents a specific calendar year in the ISO-8601 calendar.
 */
public class YearValue implements FieldValue, PeriodValue {

    private final int year;

    public YearValue(int year) {
        this.year = year;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        YearValue yearValue = (YearValue) o;

        if (year != yearValue.year) return false;

        return true;
    }

    @Override
    public int hashCode() {
        return year;
    }

    @Override
    public FieldTypeClass getTypeClass() {
        return YearType.TYPE_CLASS;
    }

    @Override
    public JsonValue toJson() {
        return Json.create(year);
    }

    @Override
    public LocalDateInterval asInterval() {
        return new LocalDateInterval(new LocalDate(year, 1, 1), new LocalDate(year, 12, 31));
    }

    @Override
    public PeriodValue next() {
        return new YearValue(year + 1);
    }

    @Override
    public PeriodValue previous() {
        return new YearValue(year - 1);
    }

    public int getYear() {
        return year;
    }

    public static boolean isLeapYear(int year) {
        return !((year % 4 != 0) || ((year % 100 == 0) && (year % 400 != 0)));
    }
}
