/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.observable;

import com.google.gwt.core.shared.GwtIncompatible;
import org.checkerframework.checker.nullness.qual.Nullable;

class CachedObservable<T> extends Observable<T> {

    private final Observable<T> source;
    private final CachePredicate<T> cachePredicate;

    private @Nullable Subscription sourceSubscription;

    public CachedObservable(Observable<T> source, CachePredicate<T> cachePredicate) {
        this.source = source;
        this.cachePredicate = cachePredicate;
    }

    @Override
    protected void onConnect() {
        this.sourceSubscription = source.subscribe(new Observer<T>() {
            @Override
            public void onChange(@Nullable T newValue) {
                if (cachedValue == null) {
                    if(newValue != null) {
                        fireChange(newValue);
                    }
                } else if(newValue == null) {
                    fireChange(newValue);
                } else if(!cachePredicate.isSame(cachedValue, newValue)) {
                    fireChange(newValue);
                }
            }
        });
    }

    @Override
    protected void onDisconnect() {
        assert sourceSubscription != null : "onConnect not called";
        sourceSubscription.unsubscribe();
        sourceSubscription = null;
    }


    @Override
    @GwtIncompatible
    public T waitFor() {
        return source.waitFor();
    }
}
