/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.observable;

import org.checkerframework.checker.nullness.qual.Nullable;

public class ChainedObservable<T> extends Observable<T> {

    private final Observable<Observable<T>> outer;
    private @Nullable Observable<T> inner;
    private @Nullable Subscription outerSubscription = null;
    private @Nullable Subscription innerSubscription = null;

    public ChainedObservable(Observable<Observable<T>> value) {
        this.outer = value;
    }

    @Override
    protected void onConnect() {
        assert outerSubscription == null;

        outerSubscription = outer.subscribe(new Observer<Observable<T>>() {
            @Override
            public void onChange(@Nullable Observable<T> newInner) {
                if(inner != newInner) {
                    unsubscribeFromOldValue();
                    if (newInner == null) {
                        fireChange(null);
                    } else {
                        subscribeToNewValue(newInner);
                    }
                }
            }
        });
    }

    private void unsubscribeFromOldValue() {
        // Unsubscribe from the old value if we had previously been listening
        inner = null;
        if(innerSubscription != null) {
            innerSubscription.unsubscribe();
            innerSubscription = null;
        }
    }

    private void subscribeToNewValue(Observable<T> newInner) {
        inner = newInner;
        innerSubscription = newInner.subscribe(new Observer<T>() {
            @Override
            public void onChange(@Nullable T value) {
                ChainedObservable.this.fireChange(value);
            }
        });
    }

    @Override
    protected void onDisconnect() {
        inner = null;
        cachedValue = null;
        if(outerSubscription != null) {
            outerSubscription.unsubscribe();
            outerSubscription = null;
        }
        if(innerSubscription != null) {
            innerSubscription.unsubscribe();
            innerSubscription = null;
        }
    }
}
