/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.observable;

import com.google.gwt.core.shared.GwtIncompatible;
import com.google.gwt.user.client.Timer;
import org.checkerframework.checker.nullness.qual.Nullable;

class DebouncedObservable<T> extends Observable<T> {

    private final Observable<T> source;
    private final int delay;
    private @Nullable Timer timer;
    private @Nullable Subscription sourceSubscription;
    private @Nullable T pendingValue;

    public DebouncedObservable(Observable<T> source, int delay) {
        this.source = source;
        this.delay = delay;
    }

    @Override
    protected void onConnect() {

        sourceSubscription = source.subscribe(new Observer<T>() {
            @Override
            public void onChange(@Nullable T newValue) {
                if(isConnecting()) {
                    DebouncedObservable.this.fireChange(newValue);
                } else {
                    if(timer == null) {
                        timer = new Timer() {
                            @Override
                            public void run() {
                                DebouncedObservable.this.fireChange(pendingValue);
                            }
                        };
                    }
                    pendingValue = newValue;
                    if (timer.isRunning()) {
                        timer.cancel();
                    }
                    timer.schedule(delay);
                }
            }
        });
    }

    @Override
    protected void onDisconnect() {
        cachedValue = null;
        pendingValue = null;
        timer = null;
        if(sourceSubscription != null) {
            sourceSubscription.unsubscribe();
            sourceSubscription = null;
        }
    }

    @Override
    @GwtIncompatible
    public T waitFor() {
        return source.waitFor();
    }
}
