/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.observable;

import com.google.gwt.core.shared.GWT;
import org.checkerframework.checker.nullness.qual.Nullable;

public class DebouncingEventLoopScheduler implements Scheduler {

    private @Nullable Runnable scheduled = null;

    @Override
    public void schedule(Runnable runnable) {
        // If running in a unit test, behave like a synchronous scheduler
        if(!GWT.isScript()) {
            runnable.run();
            return;
        }

        // Otherwise, check if something is already scheduled
        if(scheduled == null) {
            scheduled = runnable;
            com.google.gwt.core.client.Scheduler.get().scheduleDeferred(() -> execute());
        } else {
            // We are already scheduled-- update the task and wait for it to execute
            scheduled = runnable;
        }
    }

    private void execute() {
        if(scheduled != null) {
            scheduled.run();
        }
        scheduled = null;
    }
}
