/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.observable;

import org.checkerframework.checker.nullness.qual.Nullable;

public class ExplicitlyOptimisticObservable<T> extends Observable<MaybeStale<T>> {

    private final Observable<T> observable;
    private @Nullable Subscription subscription;

    public ExplicitlyOptimisticObservable(Observable<T> observable) {
        this.observable = observable;
    }

    @Override
    protected void onConnect() {
        subscription = observable.subscribe(new Observer<T>() {
            @Override
            public void onChange(@Nullable T value) {
                maybeUpdateValue(value);
            }
        });
    }

    @Override
    protected void onDisconnect() {
        if(cachedValue != null) {
            cachedValue = cachedValue.outdated();
        }
        assert subscription != null : "onConnect not called";
        subscription.unsubscribe();
        subscription = null;
    }

    private void maybeUpdateValue(@Nullable T value) {
        if(value != null) {
            fireChange(new MaybeStale<>(value, false));

        } else if(cachedValue != null) {
            fireChange(cachedValue.outdated());
        }
    }
}
