/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.observable;

public class IncrementalObservable<T> extends Observable<T> {

    private final IncrementalTask<T> task;
    private final Scheduler scheduler;
    private boolean completed = false;

    public IncrementalObservable(IncrementalTask<T> task, Scheduler scheduler) {
        this.task = task;
        this.scheduler = scheduler;
    }

    @Override
    protected void onConnect() {
        if(!completed) {
            scheduleNextSlice();
        }
    }

    private void scheduleNextSlice() {
        scheduler.schedule(() -> {
            if(!isConnected() && !isConnecting()) {
                return;
            }

            fireChange(task.execute());

            if(task.isDone()) {
                completed = true;
            } else {
                scheduleNextSlice();
            }
        });
    }
}
