/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.observable;

import org.checkerframework.checker.nullness.qual.Nullable;

public interface IncrementalTask<T> {

    /**
     * Executes some part of the work to be done.
     *
     * <p>Some tasks may return intermediate results, even if the task is not completely finished.</p>
     *
     * <p>If no result, even an intermediate result, is available, this method should return {@code null}</p>
     *
     * <p>If an intermediate result is to be returned, it must be immutable.</p>
     *
     */
    @Nullable T execute();

    /**
     *
     * @return true if this task is complete, or {@code false} if {@code execute()} should be invoked again
     */
    boolean isDone();

}
