/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.observable;

import org.checkerframework.checker.nullness.qual.Nullable;

public class MockObserver<T> implements Observer<T> {
    private int changeCount;
    private @Nullable T lastValue;
    private boolean lastLoading;
    
    @Override
    public void onChange(@Nullable T value) {
        changeCount++;
        if(value != null) {
            this.lastValue = value;
            this.lastLoading = false;
        } else {
            this.lastValue = null;
            this.lastLoading = true;
        }
    }
    
    public void resetCount() {
        changeCount = 0;
    }

    public int getChangeCount() {
        return changeCount;
    }

    public void assertChangeFiredOnce() {
        assertFired(1);
    }

    public void assertFired(int expectedCount) {
        if(changeCount == 0) {
            throw new AssertionError("onChange() has not been called.");
        }
        if(changeCount != expectedCount) {
            throw new AssertionError("onChange() was called " + changeCount + " times, expected " + expectedCount);
        }
        changeCount = 0;
    }

    public void assertChangeNotFired() {
        if(changeCount > 0) {
            throw new AssertionError("No call to onChange() expected; there have been " + changeCount + " call(s).");
        }
    }

    public @Nullable T getLastValue() {
        return lastValue;
    }

    public boolean isLoading() {
        return lastLoading;
    }

    public boolean isLoaded() {
        return !isLoading();
    }

    public void assertValueEquals(T value) {
        if(lastValue == null) {
            throw new AssertionError("Expected: " + value + " but was not loaded");
        }
        if(!lastValue.equals(value)) {
            throw new AssertionError("Expected: " + value + " but was " + lastValue);
        }
    }

    public void assertLoading() {
        if(lastValue != null) {
            throw new AssertionError("Expected loading, but found: " + lastValue);
        }
    }
}
