/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.observable;

import com.google.gwt.core.shared.GwtIncompatible;
import org.checkerframework.checker.nullness.qual.Nullable;

class OptimisticObservable<T> extends Observable<T> {

    private final Observable<T> observable;
    private @Nullable Subscription subscription;


    OptimisticObservable(Observable<T> observable, @Nullable T defaultValue) {
        this.observable = observable;
        this.cachedValue = defaultValue;
    }

    @Override
    protected void onConnect() {
        subscription = observable.subscribe(new Observer<T>() {
            @Override
            public void onChange(@Nullable T newValue) {
                // Ignore loading events during refreshes, just keep the previous value
                // until we get an updated value.
                if(newValue != null && newValue != cachedValue) {
                    fireChange(newValue);
                }
            }
        });
    }

    @Override
    protected void onDisconnect() {
        assert subscription != null : "onConnect not called";
        subscription.unsubscribe();
        subscription = null;
    }

    @Override
    @GwtIncompatible
    public T waitFor() {
        return observable.waitFor();
    }
}
