/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.observable;

import org.checkerframework.checker.nullness.qual.Nullable;

import java.util.Optional;

class OptionalObservable<T> extends Observable<Optional<T>> {

    private final Observable<T> source;
    private @Nullable Subscription sourceSubscription;

    OptionalObservable(Observable<T> source) {
        this.source = source;
        this.cachedValue = Optional.empty();
    }

    @Override
    protected void onConnect() {
        sourceSubscription = source.subscribe(this::onSourceChange);
    }

    private void onSourceChange(@Nullable T value) {
        fireChange(Optional.ofNullable(value));
    }

    @Override
    protected void onDisconnect() {
        super.onDisconnect();
        if(sourceSubscription != null) {
            sourceSubscription.unsubscribe();
            sourceSubscription = null;
        }
    }
}
