/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.observable;

import com.google.gwt.core.shared.GwtIncompatible;

import java.util.Objects;
import java.util.function.Consumer;
import java.util.function.Function;

public class StatefulValue<T> extends Observable<T> implements Consumer<T> {
    
    private T value;

    public StatefulValue(T value) {
        this.value = value;
        this.cachedValue = value;
    }

    public T update(Function<T, T> function) {
        updateValue(function.apply(value));
        return value;
    }

    /**
     * Updates the reference to this state's value and notifies subscribers.
     */
    public void updateValue(T value) {
        this.value = value;
        fireChange(value);
    }

    @Override
    protected void onConnect() {
        fireChange(value);
    }

    @Override
    @GwtIncompatible
    public T waitFor() {
        return value;
    }

    /**
     * If the given {@code newValue} is not equal to the current value,
     * update the current value, fire a changed event, and return {@code true}.
     *
     * <p>If the given {@code newValue} <em>is</em> equal to the current value, do nothing and
     * return {@code false}</p>
     * @param newValue
     * @return
     */
    public boolean updateIfNotEqual(T newValue) {
        if (!Objects.equals(this.value, newValue)) {
            this.value = newValue;
            fireChange(newValue);
            return true;
        } else {
            return false;
        }
    }

    public boolean updateIfNotSame(T value) {
        if(this.value != value) {
            this.value = value;
            fireChange(value);
            return true;
        } else {
            return false;
        }
    }

    public T get() {
        return value;
    }

    @Override
    public void accept(T t) {
        updateValue(t);
    }
}
