<#--

    ActivityInfo 4.0
    Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved

    Unauthorized copying of this file, via any medium is strictly prohibited
    Proprietary and confidential

-->

<!DOCTYPE html>
<#-- @ftlvariable name="" type="org.activityinfo.server.page.AppPageModel" -->
<#--

    This page "hosts" the Collection link application.

    There is a very limited amount of CSS, HTML, and JavaScript in this initial page
    that provides some basic feedback to the user about the process of starting
    the application:

    - The page directly contains the CSS and HTML to display the loading indicator.
      When the application loads, it calls Base::hideLoader() which removes the loading
      indicator from the dom.

    - The initial selection script is included with a <script> tag below. This script
      detects the language and loads the appropriate permutation script.

    - This page includes CSS rules that will hide the loading indicator and show the error message
      if the body has the class "app-load-failed".

    - If the ServiceWorker is NOT installed, then we have no way to detect whether or not the selection
      script or the application permutation fail to load. Instead, this page sets a timeout of 15s
      after which we display a warning and prompt the user to reload.

  -->
<html lang="${locale}">
<head>
    <meta http-equiv="content-type" content="text/html; charset=UTF-8">
    <meta name="application-name" content="ActivityInfo"/>
    <meta name="description" content="ActivityInfo"/>
    <meta name="application-url" content="${appUrl}"/>
    <meta http-equiv="X-UA-Compatible" content="edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0">
    <meta name="robots" content="noindex">
    <meta name='gwt:property' content="locale=${locale}">

    <link rel="icon" href="/App/logo72.png">
    <link rel="apple-touch-icon" sizes="72x72" href="/App/logo72.png">
    <link rel="apple-touch-icon" sizes="196x196" href="/App/logo196.png">
    <link rel="apple-touch-icon" sizes="300x300" href="/App/logo300.png">

<#--    <link rel="manifest" href="/App/App.webmanifest">-->

    <title>${title}</title>

    <#include "AppStyle.ftl">

    <script type="application/javascript" src="/CollectionLink/CollectionLink.nocache.js"></script>

    <script>
        <#-- if the script fails to load after a reasonable amount of time, show a warning message -->
        <#-- this must be cleared by Base::hideLoader() if the application does eventually load -->
        window.setTimeout(function() {
            var loading = document.getElementById('initial-loader');
            var loadFailed = document.body.classList.contains('app-load-failed')
            if(loading && !loadFailed) {
                document.getElementById('load-timeout-warning').style.display = 'block';
            }
        }, 15000);
    </script>
</head>
<body role="application" >
<div id="initial-loader">
    <div class="loader">
        <div class="loader__blob"></div>
        <div class="loader__blob"></div>
        <div class="loader__blob"></div>
    </div>
</div>
<div id="load-failed-message">
    <div>
        <span>${i18n.applicationLoadFailed}</span>
        <a href="javascript:location.reload();">${i18n.retry}</a>
    </div>
</div>
<div id="load-timeout-warning">
    <span>${i18n.applicationLoadTimeout}</span>
    <a href="javascript:location.reload();">${i18n.retry}</a>
</div>
<noscript>
    <div>${i18n.javascriptRequired}</div>
</noscript>
<iframe src="javascript:''" id="__gwt_historyFrame" tabIndex='-1' style="position:absolute;width:0;height:0;border:0"></iframe>
</body>
</html>