<#--

    ActivityInfo 4.0
    Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved

    Unauthorized copying of this file, via any medium is strictly prohibited
    Proprietary and confidential

-->
<#include "Scaffolding.ftl">
<@scaffolding title="Subscribe">

    <form id="completeTrialForm">

        <h1>Complete your subscription</h1>


        <div id="errorAlert" class="alert alert-error" style="display: none">
        </div>

        <p>Complete the following form to complete your ActivityInfo subscription.</p>

        <p>Need to pay by bank transfer or invoice? <a href="https://www.activityinfo.org/about/quote.html">Generate a quote now.</a></p>

        <h2>Your contact details</h2>

        <label class="field">
            Name
            <input type="text" required name="name" autocomplete="name" readonly value="${name?html}">
        </label>

        <label class="field">
            E-mail address
            <input type="email" required name="email" readonly value="${email?html}">
        </label>

        <h2>About your organization</h2>

        <label class="field">
            Organization
            <input type="text" required name="organization" autocomplete="organization">
        </label>

        <label class="field">
            Street address
            <input type="text" name="address" required autocomplete="street-address"></input>
        </label>

        <p class="tooltip" id="tooltip-address">
            We need your address for the invoice.
        </p>

        <div style="display: flex; justify-content: space-between; align-items: flex-start;">
            <label class="field" style="flex-grow: 1;">
                City
                <input type="text" name="city" autocomplete="address-level2" required></input>
            </label>
            <label class="field" style="width: 15ch; margin-top: 0; margin-left: 1rem;">
                Postal Code
                <input type="text" name="postalCode" autocomplete="postal-code" required></input>
            </label>
        </div>

        <label class="field">
            Country
            <select name="country" id="countrySelect" autocomplete="country">
                <option value="">Choose a country</option>
                <#list countries as billingCountry>
                    <option value="${billingCountry.code}"<#if country == billingCountry.code> selected</#if>>${billingCountry.name}</option>
                </#list>
            </select>
        </label>

        <div id="vatField" style="display: none;">
            <label class="field">
                EU VAT Number
                <input type="text" aria-described-by="vatTooltip" name="vatNumber" id="vatNumberInput">
            </label>

            <p class="tooltip" id="vatTooltip">
                To invoice an entity in the European Union, we need an EU <abbr title="Value Added Tax">VAT</abbr> number.
                If you don't have a <abbr>VAT</abbr> number, we will need to invoice an additional 21% Dutch VAT.
            </p>
        </div>

        <p class="tooltip" id="nlVatTooltip" style="display: none;">
            When we invoice entities based in the Netherlands, we must invoice an additional
            21% <abbr title="Value Added Tax">VAT</abbr>. If you have a question, <a href="/about/contact.html">contact us</a>.
        </p>

        <h2>Subscription</h2>

        <div class="subscription-choice">
            <input type="radio" name="frequency" id="monthlyRadio" value="monthly" checked>
            <label class="field" for="monthlyRadio">
                <span>Monthly plan</span><br>
                <span class="subscription-choice__conditions">€35/month per user</span><br>
                <span class="subscription-choice__conditions">Pay by credit card <em>only</em>, cancel any time.</span>
                <select name="monthlyUsers" id="monthlySelect" style="display: block; margin-top: 0.5em;">
                    <#list userChoices as userChoice>
                        <option value="${userChoice.count?c}">${userChoice.label?html}</option>
                    </#list>
                </select>
            </label>
        </div>

        <div class="subscription-choice">
            <input type="radio" name="frequency" id="annualRadio" value="annual">
            <label class="field" for="annualRadio">
                <span>Annual plan</span><br>
                <span class="subscription-choice__conditions">Pay by bank transfer <em>or</em> online with credit card.</span>
                <select name="annualPlan" id="annualSelect" style="display: block; margin-top: 0.5em;">
                    <option value="level1">Level 1 - 75 users - €4&#x202f;800/year</option>
                    <option value="level2">Level 2 - 150 users - €9&#x202f;000/year</option>
                    <option value="level3">Level 3 - 1000 users - €15&#x202f;000/year</option>
                </select>
            </label>
        </div>

        <p class="tooltip" id="tooltip-users">
            The user count includes all users given access to the system, across all databases
            you own.
        </p>

        <button class="button button--primary" id="payButton">Pay online now</button>
    </form>
    <script src="https://js.stripe.com/v3/"></script>
    <script>


        var stripe = Stripe('${stripePublishableKey}');

        var euVAT = { "AT":"U12345678",
            "BE":"1234567890",
            "BG":"123456789",
            "HR":"12345678901",
            "CY":"12345678X",
            "CZ":"12345678",
            "DK":"12345678",
            "EE":"123456789",
            "FI":"12345678",
            "FR":"12345678901",
            "DE":"123456789",
            "EL":"123456789",
            "HU":"12345678",
            "IE":"1234567AA",
            "IT":"12345678901",
            "LV":"12345678901",
            "LT":"123456789",
            "LU":"12345678",
            "MT":"12345678",
            "NO":"123456789MVA",
            "PL":"1234567890",
            "PT":"123456789",
            "RO":"1234567890",
            "SK":"1234567890",
            "SI":"12345678",
            "ES":"X12345678",
            "SE":"123456789012",
            "GB":"123456789" };


        var countrySelect = document.getElementById("countrySelect");
        var vatField = document.getElementById("vatField");
        var vatFieldInput = document.getElementById('vatNumberInput');
        var nlVatTooltip = document.getElementById('nlVatTooltip');

        var toggleVAT = function(event) {
            var country = countrySelect.value;
            var eu = euVAT[country];
            vatField.style.display = eu ? 'block' : 'none';
            vatFieldInput.placeholder = country + ' ' + eu;
            nlVatTooltip.style.display = (country === 'NL') ? 'block' : 'none';
        };

        var showErrorMessage = function(message) {
            var alertDiv = document.getElementById('errorAlert');
            alertDiv.style.display = !!message ? 'block' : 'none';
            alertDiv.innerText = message;
        }

        countrySelect.addEventListener("change", toggleVAT);
        window.addEventListener("load", toggleVAT);

        var monthlyRadio = document.getElementById('monthlyRadio');
        var monthlySelect = document.getElementById('monthlySelect');
        monthlySelect.addEventListener('click', function(e) {
            monthlyRadio.checked = true;
        });
        var annualRadio = document.getElementById('annualRadio');
        var annualSelect = document.getElementById('annualSelect');
        annualSelect.addEventListener('click', function(e) {
            annualRadio.checked = true;
        });

        toggleVAT();

        var payButton = document.getElementById('payButton');
        var enablePayButton = function(enabled) {
            payButton.disabled = !enabled;
            payButton.innerText = enabled ? 'Pay online now' : 'Processing...';
        }

        var completeTrialForm = document.getElementById('completeTrialForm');
        completeTrialForm.addEventListener('submit', function(event) {

            event.preventDefault();

            showErrorMessage(null);

            if(!countrySelect.value) {
                alert("Please select your organization's country.");
                countrySelect.scrollIntoView();
                countrySelect.focus();
            }

            enablePayButton(false);

            var xhr = new XMLHttpRequest();
            xhr.open("POST", "/billing/convertTrial/checkout", true);
            xhr.send(new FormData(completeTrialForm));
            xhr.onload = function() {

                enablePayButton(true);

                var data;
                try {
                    data = JSON.parse(this.responseText);
                } catch(err) {
                    console.log(err);
                    alert('There was an error reaching the server. Please check your connection and try again.');
                }
                console.log(data);

                if(this.status === 200) {
                    stripe.redirectToCheckout({
                        sessionId: data.sessionId
                    }).then(function(result) {
                        if(result.error) {
                            alert(result.error.message);
                        }

                    })
                } else {
                    alert(data.errorMessage);
                }
            };
            xhr.onerror = function() {
                enablePayButton(true);
                alert('There was an error reaching the server. Please check your connection and try again.');
            }
        });

    </script>
</@scaffolding>