<#--

    ActivityInfo 4.0
    Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved

    Unauthorized copying of this file, via any medium is strictly prohibited
    Proprietary and confidential

-->
<#include "Scaffolding.ftl">
<@scaffolding title="${i18n.login}">

    <form action="" method="POST" id="loginForm">

        <h1>${i18n.login}</h1>

        <noscript>
            <div class="alert alert--error">
                ${i18n.javascriptRequired}
            </div>
        </noscript>

        <div id="errorAlert" class="alert alert--error" style="display: none;">
        </div>

        <label class="field">
            <span>${i18n.email}</span>
            <div id="emailField">
                <input id="emailInput" name="email" type="email" autocomplete="username" required value="${email?html}" autofocus>
            </div>
            <div id="emailDisplay" style="display: none;">
                <span id="emailSpan">${email?html}</span>
                <button id="changeEmailButton" class="button button--subtle" type="button">Change</button>
            </div>
        </label>

        <label id="passwordField" class="field" style="display: none">
            <span>${i18n.password}</span>
            <a id="problemLink" href="/loginProblem">Forgot your password?</a>
            <input id="password" type="password" name="password" autocomplete="current-password">
        </label>

        <footer>
            <button id="submitButton" type="submit" disabled class="button button--primary">
                <@forwardIcon/>
                <span class="button__label">${i18n.continue_}</span>
            </button>
        </footer>
    </form>

    <div id="ios-prompt" style="display: none;" role="dialog" >
        <div>
            <svg class="ios-illustration" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 112 113">
                <path d="M56.05 112.72c30.928 0 56-25.072 56-56s-25.072-56-56-56-56 25.072-56 56 25.072 56 56 56z" fill="#00cf79" fill-opacity=".4"/>
                <path d="M50.88 56.74a5 5 0 015-5h28.75a5 5 0 015 5v28.75a5 5 0 01-5 5H55.88a5 5 0 01-5-5z" fill="#333"/>
                <path d="M59.6 63.45a3 3 0 013-3h15.313a3 3 0 013 3v15.313a3 3 0 01-3 3H62.6a3 3 0 01-3-3z" fill="none" stroke="#fff" stroke-width="2"/>
                <path d="M70.259 66.75v8.719M74.619 71.109H65.9" fill="none" stroke-linecap="square" stroke="#fff" stroke-width="2"/>

                <path d="M21.05 26.22a5 5 0 015-5H54.8a5 5 0 015 5v28.75a5 5 0 01-5 5H26.05a5 5 0 01-5-5z" fill="#333"/>
                <path d="M43.223 36.18h3.197a2 2 0 012 2v10.471a3 3 0 01-3 3h-9.471a3 3 0 01-3-3V38.18a2 2 0 012-2h3.171" fill="none" stroke="#fff" stroke-width="2"/>
                <path d="M40.415 42.322v-14" fill="none" stroke-linecap="square" stroke="#fff" stroke-width="2"/>
                <path d="M34.5 32l5.952-5.95 5.95 5.95" fill="none" stroke="#fff" stroke-width="2"/>
            </svg>
            <h3>${i18n.installAppText}</h3>
            <p>${i18n.installAppIosInstructions}</p>

            <div>
                <button id="ios-dismiss" type="button" class="button">
                    <@closeIcon/>
                    <span class="button__label">${i18n.dontAskAgain}</span>
                </button>
            </div>
        </div>
    </div>

    <script type="application/javascript">

        var hash = window.location.hash.replace("#", "");
        var page = hash ? "/app#" + hash : "${page?js_string}";

        var loginForm = document.getElementById('loginForm');
        var errorAlert = document.getElementById('errorAlert');
        var emailField = document.getElementById('emailField');
        var emailInput = document.getElementById('emailInput');
        var emailDisplay = document.getElementById('emailDisplay');
        var emailSpan = document.getElementById('emailSpan');
        var passwordField = document.getElementById('passwordField');
        var passwordInput = document.getElementById('password');
        var submitButton = document.getElementById('submitButton');
        var submitButtonLabel = document.querySelector('#submitButton .button__label');

        var passwordReset = document.getElementById('loginProblem');
        var changeEmailButton = document.getElementById('changeEmailButton');


        function render() {
            emailField.readOnly = (state.step === 'password') || state.submitting;
            emailField.style.display = state.step === 'password' ? 'none' : 'block';
            emailDisplay.style.display = state.step === 'password' ? 'block' : 'none';
            emailSpan.innerText = state.email;
            passwordField.style.display = (state.step === 'password') ? 'block' : 'none';
            // passwordReset.style.display = (state.step == 'password') ? 'block' : 'none';
            passwordInput.required = (state.step === 'password');

            errorAlert.style.display = !!state.error ? 'block' : 'none';
            errorAlert.innerHTML = state.error;
            if(state.step === 'email') {
                submitButtonLabel.innerText = state.submitting ? "${i18n.loggingIn}" : "${i18n.continue_}";
            } else {
                submitButtonLabel.innerText = state.submitting ? "${i18n.loggingIn}" : "${i18n.login}";
            }
        }


        <#if email?has_content>
        var state = {
            step: 'password',
            email: "${email?js_string}"
        }
        render();
        window.localStorage.setItem('lastLogin', state.email);
        <#else>
        var state = {
            step: 'email'
        };
        </#if>


        function emailChecked(email, r) {

            if(r === 'sso') {
                window.location.assign('/login/startSSO?email=' + encodeURIComponent(email) + '&page=' + page);
                return;
            }

            if(r === 'password') {
                state.step = 'password';
                state.email = email;
                state.error = null;
            } else if(r == 'pending') {
                state.error = "${i18n.accountPending?html} <a href='/loginProblem'>${i18n.resendActivationEmail?html}</a>";
            } else {
                state.error = "${i18n.noAccount?html}";
            }
            state.submitting = false;
            render();
            if(state.step == 'password') {
                passwordInput.focus();
            }
        }

        function errorHandler(e) {
            console.log('request failed', e);
            state.error = "${i18n.saveFailedConnectionProblem?html}"
            state.submitting = false;
            render();
        }

        function submitEmail() {
            var email = emailInput.value;
            if(!email || !email.match(/.+\@.+\..+/)) {
                state.error = "${i18n.invalidEmail?html}";
                state.submitting = false;
                return render();
            }

            var xhr = new XMLHttpRequest();
            xhr.addEventListener('error', errorHandler);
            xhr.addEventListener('load', function(e) {
                emailChecked(email, xhr.responseText);
            });
            xhr.open("POST", "/login/checkEmail");
            xhr.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
            xhr.send(JSON.stringify({ email: email }));

            state.submitting = true;
            state.error = null;

            render();

            window.localStorage.setItem('lastLogin', email);
        }

        function passwordSubmitted(xhr) {
            if(xhr.status === 200) {
                window.location.assign(page);
                return;
            }
            state.submitting = false;

            if(xhr.status === 400) {
                var r = JSON.parse(xhr.responseText);
                if (r.lockedOut) {
                    state.error = "${i18n.lockedOut?html}";
                    state.lockedOut = r.lockedOut;
                } else {
                    state.error = "${i18n.incorrectLogin?html}";
                }
            } else {
                state.error = "${i18n.errorOnServer?html}";
            }
            render();

            if(xhr.status === 400) {
                passwordInput.value = '';
                passwordInput.focus();
            }
        }

        function submitPassword() {
            var xhr = new XMLHttpRequest();
            xhr.addEventListener('error', errorHandler);
            xhr.addEventListener('load', function(e) {
                passwordSubmitted(xhr);
            });
            xhr.open("POST", "/login");
            xhr.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
            xhr.send(JSON.stringify({ email: state.email, password: passwordInput.value }));

            state.submitting = true;
            state.error = null;
            render();
        }

        loginForm.addEventListener('submit', function(e) {
            e.preventDefault();

            if(state.step === 'email') {
                submitEmail();
            } else if(state.step === 'password') {
                submitPassword();
            }
        });

        changeEmailButton.addEventListener('click', function(e) {
            state.step = 'email';
            state.error = null;
            render();
        });

        <#-- Check for cookies. We require this to support login -->
        if(!navigator.cookieEnabled) {
            errorAlert.style.display = 'block'
            errorAlert.innerHTML = "${i18n.cookiesRequired?js_string}";
        }

        <#-- Only enable the form submission javascript is running and cookies are enabled -->
        submitButton.disabled = !navigator.cookieEnabled;

        var lastEmail = window.localStorage.getItem('lastLogin');
        if(lastEmail) {
            emailInput.value = lastEmail;
        }

        <#-- install the service worker already -->
        if(navigator.serviceWorker && !navigator.serviceWorker.controller) {
            navigator.serviceWorker.register('/sw.js', {scope: '/'});
        }

        <#-- IOS PWAs, when launched in standalone mode, share NO context with the safari web browser -->
        <#-- So ideally we get the user to install the app BEFORE they login and choose which databases to take -->
        <#-- offline, otherwise they have to do it all over again -->

        var iOS = /iPad|iPhone/.test(navigator.userAgent) || (navigator.userAgent.match(/Mac/) && navigator.maxTouchPoints && navigator.maxTouchPoints > 2);
        var stillInBrowser = !navigator.standalone;
        var dismissed = !!localStorage.getItem('ios-dismissed');
        if(stillInBrowser && iOS && !dismissed) {
            var prompt = document.getElementById('ios-prompt');
            prompt.style.display = 'flex';
            emailInput.disabled = true;
            document.getElementById('ios-dismiss').addEventListener('click', function() {
                prompt.style.display = 'none';
                emailInput.disabled = false;
                emailInput.focus();
                localStorage.setItem('ios-dismissed', true);
            });
            var vv = window.visualViewport;
            var ove = document.querySelector("#ios-prompt > div");
            if(vv && ove) {
                ove.style.height = vv.height + "px";
                ove.style.maxHeight = "unset";
                vv.addEventListener('resize', function() {
                    ove.style.height = vv.height + "px";
                });
            }
        }

        <#-- Avoid banner from android at this stage, it will be handled by the application -->
        window.addEventListener("beforeinstallprompt", function(e) {
            e.preventDefault();
        });
    </script>
</@scaffolding>