/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.query.server;

import org.activityinfo.model.form.FormMetadata;
import org.activityinfo.model.formTree.FormMetadataProvider;
import org.activityinfo.model.permission.FormPermissions;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.store.spi.FormSupervisor;

import java.util.Optional;
import java.util.logging.Logger;

public class FormMetadataProviderAdapter implements FormMetadataProvider {

    private static final Logger LOGGER = Logger.getLogger(FormMetadataProviderAdapter.class.getName());

    private FormStorageProvider catalog;
    private FormSupervisor supervisor;

    public FormMetadataProviderAdapter(FormStorageProvider catalog, FormSupervisor supervisor) {
        this.catalog = catalog;
        this.supervisor = supervisor;
    }

    @Override
    public FormMetadata getFormMetadata(ResourceId formId) {
        Optional<FormStorage> storage = catalog.getForm(formId);
        if(storage.isPresent()) {
            FormPermissions permissions = supervisor.getFormPermissions(formId);
            if(permissions == null) {
                LOGGER.severe("Missing form permissions for " + formId);
                permissions = FormPermissions.none();
            }
            if(permissions.isVisible()) {
                return FormMetadata.of(storage.get().getFormClass(), permissions);
            } else {
                return FormMetadata.forbidden(formId);
            }
        } else {
            return FormMetadata.notFound(formId);
        }
    }
}
