/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.query.server;

import org.activityinfo.model.form.FormClass;
import org.activityinfo.model.form.FormRecord;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.store.query.shared.ColumnQueryBuilder;

import java.util.Optional;

/**
 * Internal interface used to construct table queries.
 */
public interface FormStorage {

    /**
     * Retrieves a single resource from the ResourceStore.
     * @param recordId the id of the resource to retrieve
     * @return the Resource
     */
    Optional<FormRecord> get(ResourceId recordId);


    /**
     * @return this collection's schema
     */
    FormClass getFormClass();


    ColumnQueryBuilder newColumnQuery();


    /**
     * 
     * @return the current version of this form to lookup.
     */
    long cacheVersion();

    /**
     * @return the current version of the cache for whole columns
     */
    default long wholeColumnCacheVersion() {
        return cacheVersion();
    }


    default long approximateRecordCount() {
        return 0;
    }

}
