/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.query.server.columns;

import com.google.common.annotations.VisibleForTesting;
import com.google.common.collect.Maps;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldValue;
import org.activityinfo.model.type.enumerated.EnumItem;
import org.activityinfo.model.type.enumerated.EnumType;
import org.activityinfo.model.type.enumerated.EnumValue;
import org.activityinfo.store.query.shared.CursorObserver;
import org.activityinfo.store.query.shared.PendingSlot;
import org.activityinfo.store.query.shared.columns.MultiDiscreteStringColumnView;

import java.util.BitSet;
import java.util.Map;

/**
 * Constructs ColumnViews for fields with multi-select enumerated type
 *
 */
public class MultiEnumColumnBuilder implements CursorObserver<FieldValue> {

    private final PendingSlot<ColumnView> result;

    private final Map<ResourceId, Integer> labelIndexMap = Maps.newHashMap();
    private final String[] ids;
    private final String[] labels;
    private BitSet[] selections;
    private int n = 0;

    public MultiEnumColumnBuilder(PendingSlot<ColumnView> result, EnumType enumType) {
        this.result = result;

        int index = 0;
        this.ids = new String[enumType.getItems().size()];
        this.labels = new String[enumType.getItems().size()];
        this.selections = new BitSet[enumType.getItems().size()];

        for (EnumItem item : enumType.getItems()) {
            this.ids[index] = item.getId().asString();
            this.labels[index] = item.getLabel();
            this.labelIndexMap.put(item.getId(), index);
            this.selections[index] = new BitSet();
            index++;
        }
    }

    @Override
    public void onNext(FieldValue value) {
        if (!(value instanceof EnumValue)) {
            clearAll(n);
            n++;
            return;
        }

        EnumValue fieldValue = (EnumValue) value;
        labelIndexMap.forEach((id, index) -> {
            if (fieldValue.getResourceIds().contains(id)) {
                selections[index].set(n);
            } else {
                selections[index].clear(n);
            }
        });

        n++;
    }

    private void clearAll(int rowIndex) {
        for (int i=0; i < selections.length; i++) {
            selections[i].clear(rowIndex);
        }
    }

    @Override
    public void done() {
        result.set(build());
    }

    @VisibleForTesting
    ColumnView build() {
        return new MultiDiscreteStringColumnView(n, labels, selections);
    }

}
