/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.query.server.columns;

import it.unimi.dsi.fastutil.ints.Int2DoubleOpenHashMap;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.FilteredColumnView;
import org.activityinfo.model.query.SortDir;
import org.activityinfo.model.util.HeapsortColumn;

public class SparseNumberColumnView extends AbstractNumberColumn {

    private final int numRows;
    private final Int2DoubleOpenHashMap map;

    public SparseNumberColumnView(double[] elements, int numRows, int numMissing) {

        this.numRows = numRows;
        this.map = new Int2DoubleOpenHashMap(numRows - numMissing);
        this.map.defaultReturnValue(Double.NaN);

        for (int i = 0; i < numRows; i++) {
            double value = elements[i];
            if(!Double.isNaN(value)) {
                map.put(i, value);
            }
        }
    }

    @Override
    public int numRows() {
        return numRows;
    }

    @Override
    public double getDouble(int row) {
        return map.get(row);
    }

    @Override
    public boolean isBlank(int row) {
        return !map.containsKey(row);
    }

    @Override
    public ColumnView select(int[] rows) {
        return new FilteredColumnView(this, rows);
    }

    @Override
    public int[] order(int[] sortVector, SortDir direction, int[] range) {
        if (range == null || range.length == numRows) {
            HeapsortColumn.heapsortSparseDouble(map, sortVector, numRows, direction == SortDir.ASC);
        } else {
            HeapsortColumn.heapsortSparseDouble(map, sortVector, range.length, range, direction == SortDir.ASC);
        }
        return sortVector;
    }

}
