/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.query.shared;

import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldValue;


/**
 * Builds a query that returns the requested fields as column streams.
 */
public interface ColumnQueryBuilder {

    void only(ResourceId resourceId);
    
    /**
     * Adds the {@code resourceId} to the list of columns to fetch.
     * 
     * @param observer interface to the object that will receive the stream
     *                 of {@code resourceIds} when {@link #execute()} is called.                 
     */
    void addRecordId(CursorObserver<ResourceId> observer);

    void addLastEditTime(CursorObserver<Double> observer);

    /**
     * Adds the {@code fieldId} to the list of columns to fetch.
     * * 
     * @param fieldId the id of the column to fetch
     * @param observer interface to the object that will receive the stream
     *                 of column values when {@link #execute()} is called.                 
     */
    void addField(ResourceId fieldId, CursorObserver<FieldValue> observer);


    /**
     * Fetches the requested columns.
     */
    void execute();
}
