/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.query.shared;

import java.util.function.Function;

/**
 * Called by the cursor on each new Resource
 */
public interface CursorObserver<T> {

    /**
     * Called when the Cursor advances to the next Resource
     *
     * @param value the value of the field at the Cursor's new position
     */
    void onNext(T value);

    /**
     * Called when the cursor is closed.
     */
    void done();

    default <R> CursorObserver<R> map(Function<R, T> function) {
        CursorObserver<T> outer = this;
        return new CursorObserver<R>() {
            @Override
            public void onNext(R value) {
                outer.onNext(function.apply(value));
            }

            @Override
            public void done() {
                outer.done();
            }
        };
    }

}
