/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.query.shared;

import org.activityinfo.model.query.BitSetColumnView;
import org.activityinfo.model.query.BitSetWithMissingView;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldValue;
import org.activityinfo.model.type.enumerated.EnumValue;

import java.util.BitSet;

class EnumItemReader implements CursorObserver<FieldValue> {

    private final ResourceId itemId;
    private final PendingSlot<ColumnView> result;
    private BitSet values = new BitSet();
    private BitSet missing = new BitSet();
    private int index = 0;


    public EnumItemReader(String component, PendingSlot<ColumnView> result) {
        this.itemId = ResourceId.valueOf(component);
        this.result = result;
    }

    @Override
    public void onNext(FieldValue value) {
        if(value instanceof EnumValue) {
            EnumValue enumValue = (EnumValue) value;
            values.set(index, enumValue.getResourceIds().contains(itemId));
        } else {
            missing.set(index, true);
        }
        index++;
    }

    @Override
    public void done() {
        int numRows = index;
        if (missing.isEmpty()) {
            result.set(new BitSetColumnView(numRows, values));
        } else {
            result.set(new BitSetWithMissingView(numRows, values, missing));
        }
    }
}
