/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.query.shared.columns;

import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldValue;
import org.activityinfo.model.type.enumerated.EnumType;
import org.activityinfo.store.query.shared.CursorObserver;
import org.activityinfo.store.query.shared.ForeignKeyBuilder;
import org.activityinfo.store.query.shared.PendingSlot;
import org.activityinfo.store.query.shared.join.PrimaryKeyMap;

/**
 * Creates optimizing column builder.
 *
 * <p>We use different implementations on the server and on the client
 * that use different types of optimizations.</p>
 */
public interface ColumnFactory {

    CursorObserver<FieldValue> newStringBuilder(PendingSlot<ColumnView> result, StringReader reader);

    CursorObserver<FieldValue> newDoubleBuilder(PendingSlot<ColumnView> result, DoubleReader reader);

    CursorObserver<FieldValue> newEnumBuilder(PendingSlot<ColumnView> result, EnumType enumType);

    ForeignKeyBuilder newForeignKeyBuilder(ResourceId rightFormId, PendingSlot<ForeignKey> value);

    PrimaryKeyMap newPrimaryKeyMap(ColumnView columnView);
}
