/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.query.shared.columns;

import org.activityinfo.model.query.ColumnType;
import org.activityinfo.model.query.ColumnView;
import org.activityinfo.model.query.SortDir;
import org.activityinfo.model.util.HeapsortColumn;

public class DiscreteStringColumnView extends ColumnView {

    private String[] labels;
    private int[] values;

    public DiscreteStringColumnView() {
    }

    public DiscreteStringColumnView(String[] labels, int[] values) {
        this.labels = labels;
        this.values = values;
    }

    public DiscreteStringColumnView(String[] ids, String[] labels, int[] values) {
        this(labels, values);
    }

    @Override
    public ColumnType getType() {
        return ColumnType.STRING;
    }

    @Override
    public int numRows() {
        return values.length;
    }

    @Override
    public String get(int row) {
        return getString(row);
    }

    @Override
    public double getDouble(int row) {
        return Double.NaN;
    }

    @Override
    public String getString(int row) {
        int labelIndex = values[row];
        if(labelIndex < 0) {
            return null;
        } else {
            return labels[labelIndex];
        }
    }

    @Override
    public int getBoolean(int row) {
        return BLANK;
    }

    @Override
    public boolean isBlank(int row) {
        return getString(row) == null;
    }

    @Override
    public ColumnView select(int[] selectedRows) {
        int filteredValues[] = new int[selectedRows.length];
        for (int i = 0; i < selectedRows.length; i++) {
            int selectedRow = selectedRows[i];
            if(selectedRow < 0) {
                filteredValues[i] = -1;
            } else {
                filteredValues[i] = values[selectedRow];
            }
        }
        return new DiscreteStringColumnView(labels, filteredValues);
    }

    @Override
    public String toString() {
        return getClass().getSimpleName() + "{numRows=" + numRows() + "}";
    }

    @Override
    public int[] order(int[] sortVector, SortDir direction, int[] range) {
        int numRows = values.length;
        if (range == null || range.length == numRows) {
            HeapsortColumn.heapsortInt(values, sortVector, numRows,
                    HeapsortColumn.withIntDirection(this::isLessThan, direction));
        } else {
            HeapsortColumn.heapsortInt(values, sortVector, range.length, range,
                    HeapsortColumn.withIntDirection(this::isLessThan, direction));
        }
        return sortVector;
    }

    private boolean isLessThan(int a, int b) {
        if (a == b) {
            return false;
        } else if (a < 0 && b >= 0) {
            return true;
        } else if (b < 0) {
            return false;
        } else {
            return labels[a].compareToIgnoreCase(labels[b]) < 0;
        }
    }

}
