/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.spi;

import com.google.common.base.Strings;
import org.activityinfo.json.JsonValue;
import org.activityinfo.store.spi.ClientIpAddress;

import java.time.Instant;
import java.time.format.DateTimeFormatter;

public class ClientIpAddressProfile {


    private final ClientIpAddress ipAddress;
    private double abuseConfidenceScore;
    private int totalReports;
    private final boolean whitelisted;
    private final Instant lastReportedTime;
    private final String countryCode;

    public ClientIpAddressProfile(ClientIpAddress ipAddress) {
        this.ipAddress = ipAddress;
        this.abuseConfidenceScore = -1;
        this.totalReports = 0;
        this.whitelisted = false;
        this.lastReportedTime = Instant.ofEpochMilli(0);
        this.countryCode = "";
    }

    public ClientIpAddressProfile(ClientIpAddress ipAddress, JsonValue data) {
        this.ipAddress = ipAddress;
        this.abuseConfidenceScore = data.getNumber("abuseConfidenceScore");
        this.totalReports = (int) data.getNumber("totalReports");
        this.whitelisted = data.getBoolean("isWhitelisted");
        this.countryCode = Strings.nullToEmpty(data.getString("countryCode"));
        this.lastReportedTime = tryParseLastReportedTime(data);
    }

    private Instant tryParseLastReportedTime(JsonValue data) {
        try {
            return Instant.from(DateTimeFormatter.ISO_OFFSET_DATE_TIME.parse(data.getString("lastReportedAt")));
        } catch (Exception e) {
            return Instant.ofEpochMilli(0);
        }
    }

    public ClientIpAddress getIpAddress() {
        return ipAddress;
    }

    public String getCountryCode() {
        return countryCode;
    }

    /**
     * Returns a number between 0 and 100 indicating the confidence that this is an IP address actively engaging
     * in abuse. 100 indicates full confidence that this IP address is engaged in abuse.
     */
    public double getAbuseConfidenceScore() {
        return abuseConfidenceScore;
    }

    public int getTotalReports() {
        return totalReports;
    }

    public boolean isWhitelisted() {
        return whitelisted;
    }

    public Instant getLastReportedTime() {
        return lastReportedTime;
    }

    @Override
    public String toString() {
        return "ClientIpAddressProfile{" +
            "ipAddress=" + ipAddress +
            ", abuseConfidenceScore=" + abuseConfidenceScore +
            ", totalReports=" + totalReports +
            ", whitelisted=" + whitelisted +
            ", lastReportedTime=" + lastReportedTime +
            ", countryCode='" + countryCode + '\'' +
            '}';
    }
}

