/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.spi;

import com.google.common.io.ByteSink;

import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

public final class GeneratedResource {
    private final String jobId;
    private final String exportId;
    private final String filename;
    private final ByteSink byteSink;

    public GeneratedResource(String jobId, String exportId, String filename, ByteSink byteSink) {
        this.jobId = jobId;
        this.exportId = exportId;
        this.filename = filename;
        this.byteSink = byteSink;
    }

    /**
     * @return the publicly-accessible path for this temporary resource
     */
    public String getDownloadPath() {
        return "resources/jobs/" + jobId + "/" + exportId + "/" + getUrlEncodedFilename();
    }

    public String getFilename() {
        return filename;
    }

    public String getUrlEncodedFilename()  {
        try {
            return URLEncoder.encode(filename, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("JDK is broken", e);
        }
    }

    public OutputStream openOutputStream() throws IOException {
        return byteSink.openStream();
    }
}
