/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.spi;

import java.security.SecureRandom;
import java.time.Duration;
import java.time.Instant;
import java.util.Optional;

public class OneTimeCode {
    public static final int DIGITS = 6;

    public static final int EXPIRE_AFTER_MINUTES = 5;

    private final String code;
    private final Instant issueTime;

    public OneTimeCode(String code, Instant issueTime) {
        this.code = code;
        this.issueTime = issueTime;
    }

    public String getCode() {
        return code;
    }

    public Instant getIssueTime() {
        return issueTime;
    }

    public static OneTimeCode generate() {
        SecureRandom random = new SecureRandom();
        int number = random.nextInt(999999);
        String code = String.format("%06d", number);

        return new OneTimeCode(code, Instant.now());
    }


    private boolean isValidAt(Instant instant) {
        return Duration.between(issueTime, instant).getSeconds() < (EXPIRE_AFTER_MINUTES * 60);
    }

    public static boolean isValid(String suppliedCode, Optional<OneTimeCode> optional) {
        return optional
                .map(otc -> otc.isValid(suppliedCode))
                .orElse(false);
    }

    public boolean isValid(String suppliedCode) {
        if(suppliedCode == null) {
            return false;
        }
        String trimmedCode = suppliedCode.replaceAll("\\s+", "");

        return this.code.equals(trimmedCode) && isValidAt(Instant.now());
    }
}
