/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.spi;

import org.activityinfo.model.annotation.Builder;
import org.activityinfo.model.form.RecordChangeType;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldValue;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * Records a change to the record
 */
public class RecordVersion {
    
    private ResourceId recordId;
    private Optional<ResourceId> userId;
    private long time;
    private long version;
    private long formVersion;
    
    private RecordChangeType type;

    private boolean delta;

    private final Map<ResourceId, FieldValue> values;

    public RecordVersion() {
        this.values = new HashMap<>();
    }

    @Builder
    public RecordVersion(ResourceId recordId,
                         Optional<ResourceId> userId,
                         long time,
                         long version,
                         long formVersion,
                         RecordChangeType type,
                         boolean delta,
                         Map<ResourceId, FieldValue> values) {
        this.recordId = recordId;
        this.userId = userId;
        this.time = time;
        this.version = version;
        this.formVersion = formVersion;
        this.type = type;
        this.delta = delta;
        this.values = values;
    }

    /**
     * @return the id of the record changed.
     */
    public ResourceId getRecordId() {
        return recordId;
    }

    public void setRecordId(ResourceId recordId) {
        this.recordId = recordId;
    }

    /**
     * @return the id of the user changed.
     */
    public Optional<ResourceId> getUserId() {
        return userId;
    }

    public void setUserId(Optional<ResourceId> userId) {
        this.userId = userId;
    }

    /**
     * 
     * @return the time, in milliseconds since the epoch, of the change.
     */
    public long getTime() {
        return time;
    }

    public void setTime(long time) {
        this.time = time;
    }

    /**
     * @return the version that this change correspons to.
     */
    public long getVersion() {
        return version;
    }

    public void setVersion(long version) {
        this.version = version;
    }

    public long getFormVersion() {
        return formVersion;
    }

    public void setFormVersion(long formVersion) {
        this.formVersion = formVersion;
    }

    /**
     * @return the values of the fields at this change.
     */
    public Map<ResourceId, FieldValue> getValues() {
        return values;
    }


    public RecordChangeType getType() {
        return type;
    }

    public void setType(RecordChangeType type) {
        this.type = type;
    }

    /**
     * Returns true if this contains only changes to fields.
     */
    public boolean isDelta() {
        return delta;
    }

    public void setDelta(boolean delta) {
        this.delta = delta;
    }
}
