/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.spi;

import org.activityinfo.model.account.ServingDomain;

import java.util.Optional;

public interface ServingConfiguration {


    String getGoogleAnalyticsPropertyId();

    boolean isNewsletterEnabled();

    ServingDomain getDomain();

    default String getTermsOfServiceLink() {
        return "https://www.activityinfo.org/about/terms.html";
    }

    default String getPrivacyPolicyLink() {
        return "https://www.activityinfo.org/about/privacy-policy.html";
    }

    /**
     * Returns true if HTTP Strict Transport Security is enabled.
     *
     * Note: this should be very carefully enabled as by design it is not easy to revoke if
     * there is a problem with certificate
     */
    default boolean isStrictTransportSecurityEnabled() {
        return false;
    }

    /**
     * Returns the value of the "X-Robots-Tag" header
     */
    default String getRobotsTag() {
        return "all";
    }

    default Optional<String> getExternalDashboardLink() {
        return Optional.empty();
    }

    boolean isDevelopment();

    /**
     * True if this server is behind a proxy server which is terminating the
     * SSL connection.
     */
    default boolean isHttpsProxyEnabled() {
        return false;
    }
}
