/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.spi;

import org.activityinfo.model.account.AuthenticatedUser;
import org.activityinfo.model.account.BillingAccount;
import org.activityinfo.model.account.BillingAccountRole;
import org.activityinfo.model.account.UserAccount;
import org.activityinfo.model.database.DatabaseHeader;
import org.activityinfo.model.resource.ResourceId;

import java.util.Optional;

public interface TransferTransaction {

    /**
     * Retrieve a user by email, within this transaction.
     *
     * @param email the user's email
     * @return the user account, if one exists
     */
    Optional<UserAccount> getUserByEmail(String email);

    /**
     * Retrieve a user by system id number, within this transaction.
     *
     * @param id the user's id
     * @return the user account, if one exists
     */
    Optional<UserAccount> getUserById(ResourceId id);

    /**
     * Update a user's billing account role
     * @param userId
     * @param billingAccountId
     * @param role
     */
    void updateUserBillingAccount(ResourceId userId, long billingAccountId, BillingAccountRole role);

    /**
     * Retrieves a list of databases owned by the user with the given {@code id}
     * @param id the user id
     * @return a list of databases
     */
    Iterable<DatabaseHeader> getDatabasesOwnedBy(ResourceId id);

    /**
     * Updates the owner of the database and removes the new owner from the list of database users
     * if they were present.
     */
    void updateDatabaseOwner(AuthenticatedUser user, ResourceId databaseId, ResourceId newOwnerId);

    /**
     * Retrieves the billing account with the given {@code billingAccountId}, if one exists.
     */
    Optional<BillingAccount> getBillingAccount(long billingAccountId);

    /**
     * Retrieves the database with the given {@code databaseId}, if one exists.
     */
    Optional<DatabaseHeader> getDatabase(ResourceId databaseId);

    void enqueueMail(Mail mail);
}
