/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.spi;

import org.activityinfo.model.form.TypedFormRecord;
import org.activityinfo.model.resource.ResourceId;
import org.activityinfo.model.type.FieldValue;
import org.activityinfo.model.type.RecordRef;

import java.time.Instant;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * Describes an update to a Record
 */
public class TypedRecordUpdate {
    
    private Optional<ResourceId> userId;
    private ResourceId formId;
    private ResourceId recordId;
    private Optional<ResourceId> parentId;
    private boolean deleted = false;
    private Map<ResourceId, FieldValue> changedFieldValues = new HashMap<>();
    private Instant lastEditTime;
    private boolean definitelyKnownToBeANewRecord = false;

    public TypedRecordUpdate() {
        this.parentId = Optional.empty();
    }

    public TypedRecordUpdate(Optional<ResourceId> userId, TypedFormRecord record) {
        this.userId = userId;
        this.formId = record.getFormId();
        this.recordId = record.getRef().getRecordId();
        this.parentId = record.getParentRecordId();
        this.changedFieldValues.putAll(record.getFieldValueMap());
        this.lastEditTime = Instant.now();
    }

    /**
     * The record id for this record was generated on the server with sufficient
     * randomness that a check for an existing record can be skipped.
     */
    public boolean isDefinitelyKnownToBeANewRecord() {
        return definitelyKnownToBeANewRecord;
    }

    /**
     * Set this to true ONLY IF the id was generated on the server and we can skip the check
     * for a new record. DO NOT SET THIS if the id has been provided by the client.
     */
    public void setDefinitelyKnownToBeANewRecordAfterReadingTheDocumentation(boolean definitelyKnownToBeANewRecord) {
        this.definitelyKnownToBeANewRecord = definitelyKnownToBeANewRecord;
    }

    public RecordRef getRef() {
        return new RecordRef(formId, recordId);
    }

    public Optional<ResourceId> getUserId() {
        return userId;
    }

    public void setUserId(Optional<ResourceId> userId) {
        this.userId = userId;
    }

    public void setRecordId(ResourceId recordId) {
        this.recordId = recordId;
    }
    
    public ResourceId getRecordId() {
        return recordId;
    }

    public ResourceId getFormId() {
        return formId;
    }

    public void setFormId(ResourceId formId) {
        this.formId = formId;
    }

    public Optional<ResourceId> getParentId() {
        return parentId;
    }

    public void setParentId(ResourceId parentId) {
        this.parentId = Optional.of(parentId);
    }

    public void setParentId(Optional<ResourceId> parentId) {
        this.parentId = parentId;
    }

    public void set(ResourceId fieldId, FieldValue value) {
        changedFieldValues.put(fieldId, value);
    }

    public Map<ResourceId, FieldValue> getChangedFieldValues() {
        return changedFieldValues;
    }

    public boolean isDeleted() {
        return deleted;
    }

    public void setDeleted(boolean deleted) {
        this.deleted = deleted;
    }

    public Instant getLastEditTime() {
        return lastEditTime;
    }

    public void setLastEditTime(Instant lastEditTime) {
        this.lastEditTime = lastEditTime;
    }

}
