/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.spi;

import org.activityinfo.model.account.AuthenticatedUser;
import org.activityinfo.model.account.BillingAccount;
import org.activityinfo.model.account.UserAccount;
import org.activityinfo.model.database.DatabaseHeader;
import org.activityinfo.model.resource.ResourceId;

import java.time.Instant;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;

public interface UpdateBillingAccountTransaction {

    BillingAccount getAccount();

    void suspend();

    void reactivate();

    void close();

    Optional<UserAccount> getUserById(ResourceId id);

    <T> T updateOrAddUser(String email, Function<UpdateUserTransaction, T> mutator);

    void updateEndTime(Instant instant);

    Iterable<UserAccount> getAccountOwners();

    /**
     * Retrieves the list of databases that are owned or shared with this user. This can be retrieved
     * outside of the curent transaction and may not reflect the changes made in the current transaction.
     */
    List<DatabaseHeader> getOwnedOrSharedDatabases(ResourceId userId);

    /**
     * Retrieves the list of databases that are owned by this billling account. This can be retrieved
     * outside of the curent transaction and may not reflect the changes made in the current transaction.
     */
    List<DatabaseHeader> getDatabases();

    void updateName(String name);

    void updateTrial(boolean trial);

    void updateUserLimit(int limit);

    String getStripeSubscriptionId();

    void updateStripeSubscriptionId(String id);

    void updateAutomaticCollecton(boolean automatic);

    void updatePlanName(String displayName);

    void addInvite(String token, AuthenticatedUser invitingUser, long userId, Instant time);

    void enqueueMail(Mail mail);
}
