/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.spi;

import org.activityinfo.model.account.Action;
import org.activityinfo.model.account.AuthenticatedUser;
import org.activityinfo.model.analysis.*;
import org.activityinfo.model.resource.ResourceId;

import java.util.List;

public interface UpdateReportTransaction {

    void updateOwner(AuthenticatedUser user);

    void updateComponents(List<ReportComponent> componentList);

    void updateAnalysis(Analysis analysis);

    void updateLayout(ReportLayout layout);

    void updateIcon(String icon);

    void updateSources(ReportSources sources);

    void updateLabel(String label);

    void updateGrants(ReportGrantSet grantSet);

    boolean exists();

    Report getReport();

    ReportLayout getLayout();

    void updatePublished(boolean publish);

    QueryTransaction query();

    /**
     * Adds a callback to be invoked after the transaction successfully commits.
     */
    void onCommit(Runnable runnable);

    void publishEvent(AuthenticatedUser user, Action action);

    /**
     * Cancels the transaction and rolls back the transaction. Calling any other methods after
     * {@code rollback()} may throw an {@link IllegalStateException}
     */
    void rollback();

    /**
     * Deletes the report. Calling any other methods after
     * {@code delete()} may throw an {@link IllegalStateException}
     */
    void delete(ResourceId userId);

    void invalidPublishedReportCacheOnCommit(String analysisId);

}
