/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.spi;

import org.activityinfo.i18n.shared.ApplicationLocale;
import org.activityinfo.model.account.*;
import org.activityinfo.model.resource.ResourceId;

import javax.ws.rs.core.HttpHeaders;
import java.time.Instant;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.Optional;

public interface UpdateUserTransaction {

    /**
     * Returns true if a user account exists for user already.
     */
    boolean exists();

    void updateName(String name);

    void updatePasswordResetToken(String token, Instant issueTime);

    void updateLocale(ApplicationLocale locale);

    void updateHashedPassword(String hash);

    void updateOneTimeCode(OneTimeCode code);

    Optional<OneTimeCode> getOneTimeCode();

    void clearOneTimeCode();

    BillingAccount startOrExtendTrial(Instant expirationTime);

    int getPasswordResetCount();

    void updatePasswordResetCount(int count);

    String getName();

    ResourceId getId();

    String getEmail();

    String getLocale();

    String getPasswordResetToken();

    Instant getPasswordResetStartTime();

    UserAccount getUserAccount();

    AccountStatus getAccountStatus();

    void clearPasswordResetToken();

    void updateDeliveryStatus(DeliveryStatus status, Instant time);

    DeliveryStatus getDeliveryStatus();

    String getHashedPassword();

    ActivationStatus getActivationStatus();

    void updateActivationStatus(ActivationStatus status);

    void updatePlatformRole(PlatformRole platformRole);

    void updateLastLoginTime(Instant instant);

    Instant getLastLoginTime();

    void publishEvent(Action action);

    void updateBillingAccountRole(long billingAccountId, BillingAccountRole role);

    void updateIdp(String id);

    void updateNewsletter(boolean newsletter);

    String getIdp();
    void enqueueMail(Mail mail);

    default void startOrExtendTrial() {
        ZoneId zoneId = ZoneId.of( "Europe/Paris" );
        ZonedDateTime now = ZonedDateTime.ofInstant( Instant.now() , zoneId );
        Instant expireTime = now.plusDays(30).toInstant();

        startOrExtendTrial(expireTime);
    }

    void addSession(String token, AuthMethod method, HttpHeaders headers, Instant expireTime);

    void updateIdpLoginHint(String loginHint);

    Optional<String> getLoginHint();

}
