/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.store.spi;

import org.activityinfo.model.database.AuditEvent;
import org.activityinfo.model.database.DatabaseUserGrant;
import org.activityinfo.model.database.RoleAssignment;

import java.time.Instant;
import java.util.List;
import java.util.Optional;

public class UserUpdate {
    private final long updateId;
    private final long userId;
    private final long executingUserId;
    private final Instant time;
    private final UserUpdateType type;
    private final Optional<RoleAssignment> previousRole;
    private final List<DatabaseUserGrant> previousGrants;
    private final Optional<RoleAssignment> newRole;
    private final List<DatabaseUserGrant> newGrants;

    public UserUpdate(long updateId, long userId, long executingUserId, Instant time, UserUpdateType type, Optional<RoleAssignment> previousRole, List<DatabaseUserGrant> previousGrants, Optional<RoleAssignment> newRole, List<DatabaseUserGrant> newGrants) {
        this.updateId = updateId;
        this.userId = userId;
        this.executingUserId = executingUserId;
        this.time = time;
        this.type = type;
        this.previousRole = previousRole;
        this.previousGrants = previousGrants;
        this.newRole = newRole;
        this.newGrants = newGrants;
    }

    public String getAuditLogEventId() {
        return AuditEvent.USER_PERMISSION_UPDATE_PREFIX + updateId;
    }

    public long getUpdateId() {
        return updateId;
    }

    public Instant getTime() {
        return time;
    }

    public long getUserId() {
        return userId;
    }

    public long getExecutingUserId() {
        return executingUserId;
    }

    public UserUpdateType getType() {
        return type;
    }

    public Optional<RoleAssignment> getPreviousRole() {
        return previousRole;
    }

    public List<DatabaseUserGrant> getPreviousGrants() {
        return previousGrants;
    }

    public Optional<RoleAssignment> getNewRole() {
        return newRole;
    }

    public List<DatabaseUserGrant> getNewGrants() {
        return newGrants;
    }

}
