/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.ui.vdom.client;

import com.google.gwt.core.client.Scheduler;
import elemental2.core.JsArray;
import org.activityinfo.ui.vdom.shared.tree.VComponentInstance;

public class RenderQueue {

    /**
     * Managed queue of dirty components to be re-rendered
     */
    private static final JsArray<VComponentInstance> ITEMS = new JsArray<>();

    private static final JsArray<Runnable> finishedRenderingCallbacks = new JsArray<>();

    private static boolean scheduled = false;

    /**
     * Enqueue a rerender of a component
     * @param component The component to rerender
     */
    public static void enqueueRender(VComponentInstance component) {
        if (!component.dirty && (component.dirty = true)) {
            ITEMS.push(component);
            scheduleRerender();
        }
    }

    public static void scheduleRerender() {
        if(!scheduled) {
            scheduled = true;
            Scheduler.get().scheduleDeferred(() -> rerender());
        }
    }

    public static void whenFinishedRendering(Runnable runnable) {
        finishedRenderingCallbacks.push(runnable);
        scheduleRerender();
    }

    /**
     *  Rerender all enqueued dirty components
     */
    private static void rerender() {
        scheduled = false;
        VComponentInstance p;
        while ( (p = ITEMS.pop()) != null) {
            if (p.dirty) {
                Diff.renderComponent(p, null, false);
            }
        }

        Runnable callback;
        while ( (callback = finishedRenderingCallbacks.pop()) != null) {
            callback.run();
        }
    }
}
