/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.ui.vdom.client;

import elemental2.dom.Element;
import jsinterop.base.Js;
import org.activityinfo.ui.vdom.shared.html.H;
import org.activityinfo.ui.vdom.shared.tree.VNode;

public class VDom {

    /**
     * Updates the browser's DOM to match the supplied vnode.
     *
     * <p>If the {@code domParent} already has a child, it is updated to match the
     * supplied {@code vnode}</p>
     *
     * @param domParent The DOM element to which the vnode should be added/updated.
     * @param vdomNode the virtual DOM element
     */
    public static void updateDom(Element domParent, VNode vnode) {

        Element child = domParent.firstElementChild;

        Diff.diff(Js.uncheckedCast(child), vnode, Js.uncheckedCast(domParent), false);
    }

    /**
     * Unmounts all components and removes all children.
     */
    public static void tearDown(Element domParent) {
        updateDom(domParent, H.div());
        domParent.remove();
    }

    public static boolean isDevMode() {
        return System.getProperty("superdevmode").equals("on");
    }
}
