/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.ui.vdom.shared.tree;

import org.activityinfo.observable.Observable;
import org.activityinfo.ui.vdom.client.VDom;

import java.util.function.Function;

public class Reactive {

    private static String findDebugId() {
        if(VDom.isDevMode()){
            return new Error().getStackTrace()[4].getMethodName();
        }
        return null;
    }

    public static VNode create(Observable<VNode> observable) {
        return new VComponent(ReactiveComponent.CONSTRUCTOR, new ReactiveProps<>(findDebugId(), observable, Function.identity(), null));
    }

    /**
     * Creates a new Reactive component.
     *
     * <p>This factory method accepts an observable value of any type, and a rendering function which can transform
     * the Observable of type {@code T} to a {@code VNode}. If the value does not change, as judged by referential equality,
     * then {@code renderer} will not be called again.</p>
     *
     * @param observable an Observable value
     * @param renderer a function to render the observable value as a VNode.
     * @param <T> the type of Observable value
     * @return a {@code VComponent} node.
     */
    public static <T> VNode create(Observable<T> observable, Function<T, VNode> renderer) {
        return new VComponent(ReactiveComponent.CONSTRUCTOR, new ReactiveProps<>(findDebugId(), observable, renderer, null));
    }

    /**
     * Creates a new Reactive component.
     *
     * <p>This factory method accepts an observable value of any type, and a rendering function which can transform
     * the Observable of type {@code T} to a {@code VNode}. If the value does not change, as judged by referential equality,
     * then {@code renderer} will not be called again.</p>
     *
     * @param observable an Observable value
     * @param renderer a function to render the observable value as a VNode.
     * @param loadingIndicator a loading indicator to use while waiting for the observable value.
     * @param <T> the type of Observable value
     * @return a {@code VComponent} node.
     */
    public static <T> VNode create(Observable<T> observable, Function<T, VNode> renderer, VNode loadingIndicator) {
        return new VComponent(ReactiveComponent.CONSTRUCTOR, new ReactiveProps<>(findDebugId(), observable, renderer, loadingIndicator));
    }

    public static VNode create(Observable<VNode> observable, VNode loadingIndicator) {
        return new VComponent(ReactiveComponent.CONSTRUCTOR, new ReactiveProps<>(null, observable, Function.identity(), loadingIndicator));
    }

    public static VNode create(String debugId, Observable<VNode> observable) {
        return new VComponent(ReactiveComponent.CONSTRUCTOR, new ReactiveProps<>(debugId, observable, Function.identity(), null));
    }

    public static <T> VNode create(String debugId, Observable<T> observable, Function<T, VNode> renderer) {
        return new VComponent(ReactiveComponent.CONSTRUCTOR, new ReactiveProps<>(debugId, observable, renderer, null));
    }

    public static VNode create(String debugId, Observable<VNode> observable, VNode loadingIndicator) {
        return new VComponent(ReactiveComponent.CONSTRUCTOR, new ReactiveProps<>(debugId, observable, Function.identity(), loadingIndicator));
    }
}
