/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.ui.vdom.shared.tree;

import org.activityinfo.observable.Observer;
import org.activityinfo.observable.Subscription;
import org.activityinfo.ui.vdom.shared.html.H;
import org.checkerframework.checker.nullness.qual.Nullable;

import java.util.logging.Logger;

public class ReactiveComponent<T> extends VComponentInstance {

    public static final VComponentConstructor CONSTRUCTOR = ReactiveComponent::new;

    private static final Logger LOGGER = Logger.getLogger(ReactiveComponent.class.getName());

    private ReactiveProps<T> props;

    private Subscription subscription;

    private T value = null;

    private VNode content = null;


    public ReactiveComponent(VComponentProps props) {
        this.props = (ReactiveProps<T>) props;
        this.content = ((ReactiveProps<T>) props).getLoadingIndicator();
        if(this.content == null) {
            this.content = H.div();
        }
    }

    @Override
    public boolean canAcceptProps(VComponent vnode) {
        return vnode.getConstructor() == CONSTRUCTOR;
    }

    @Override
    public boolean updateProps(VComponentProps newPropsObj) {
        ReactiveProps<T> oldProps = this.props;
        this.props = (ReactiveProps<T>) newPropsObj;

        if((oldProps.getValue() != props.getValue() ||
                oldProps.getRenderer() != props.getRenderer()) && subscription != null) {
            subscription.unsubscribe();
            subscribe();
        }

        // Always return false, because subscribe() will trigger a refresh if needed.
        return false;
    }

    @Override
    public VNode render() {
        return content;
    }

    @Override
    public void componentDidMount() {
        subscribe();
    }

    private void subscribe() {
        subscription = props.getValue().subscribe(new Observer<T>() {
            @Override
            public void onChange(@Nullable T newValue) {
                if(newValue == null) {
                    if(props.getLoadingIndicator() != null && content != props.getLoadingIndicator()) {
                        content = props.getLoadingIndicator();
                        value = null;
                        ReactiveComponent.this.refresh();
                    }
                } else {
                    if(newValue != value) {
                        value = newValue;
                        content = props.getRenderer().apply(value);
                        ReactiveComponent.this.refresh();
                    }
                }
            }
        });
    }

    @Override
    public void componentWillUnmount() {
        if(subscription != null) {
            subscription.unsubscribe();
            subscription = null;
        }
    }

    @Override
    public String toString() {
        return "Reactive[" + props.getDebugId() + "]";
    }
}
