/*
 * ActivityInfo 4.0
 * Copyright © 2014-2021 BeDataDriven Groep B.V. - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */
package org.activityinfo.ui.vdom.shared.tree;

import elemental2.dom.Element;
import elemental2.dom.HTMLElement;
import jsinterop.base.Js;
import org.activityinfo.ui.vdom.client.DomNode;
import org.activityinfo.ui.vdom.client.RenderQueue;

public abstract class VComponentInstance {

    public DomNode domNode;
    public DomNode nextBase;

    public boolean disabled;
    public VComponentInstance childComponent;
    public VComponentInstance parentComponent;

    /**
     * True if this component needs to be re-rendered on the next pass
     */
    public boolean dirty = false;

    /**
     * Marks this node as dirty
     */
    public final void refresh() {
        RenderQueue.enqueueRender(this);
    }

    /**
     * @return {@code true} if this component can be updated with the given component node.
     */
    public abstract boolean canAcceptProps(VComponent vnode);

    /**
     *
     * Updates this component instance in place with the given properties.
     *
     * @return {@code true} if a re-render is necessary
     *
     */
    public abstract boolean updateProps(VComponentProps props);

    public final boolean isDirty() {
        return dirty;
    }


    /**
     * Renders this component to a concrete {@code VNode}.
     */
    public abstract VNode render();

    /**
     * Called immediately after the component is newly added to the real
     * DOM tree.
     */
    public void componentDidMount() {
    }

    /**
     * Called before the Component is to be unmounted from the dom.
     */
    public void componentWillUnmount() {
    }

    public final Element getDomNode() {
        assert domNode != null : "component has not been mounted";
        return Js.uncheckedCast(domNode);
    }

    public boolean isMounted() {
        return domNode != null;
    }


    public HTMLElement getHTMLElement() {
        return Js.uncheckedCast(domNode);
    }

    public String getKey() {
        return null;
    }


}
